#! /bin/bash

source /opt/fibaro/scripts/config

HC2_Serial=`sqlQuery "SELECT Serial_Number FROM HC_Data"`
HW_Key=`sqlQuery "SELECT HW_Key FROM HC_Data"`
RSA_KEY_LOCATION="/opt/fibaro/id_rsa"

is_valid_port()
{
	echo $1 | egrep "^[0-9]+$"
}

is_valid_host()
{
	echo $1 | egrep "^[0-9a-zA-Z.-]+$"
}

start_remote_access()
{
  remote=`sqlQuery "SELECT Remote_Access FROM HC_Data" `
  remoteSupport=`sqlQuery "SELECT Remote_Access_Support FROM HC_Data" `

  if [[ "$remote" == "1" ]]; then
      openssl rsa -inform PEM -in $RSA_KEY_LOCATION -noout &> /dev/null
      keyVerify=$?
      # get RSA key if it doesn't exist
      if [[ $keyVerify != 0 ]]
      then
        echo "RSA key maflormed or not found. Trying to get key from the internet ..."

        rm $RSA_KEY_LOCATION

        GET_KEY_URL="https://getkeys.fibaro.com/get_key_hc2.php?PK_AccessPoint=$HC2_Serial&HW_Key=$HW_Key"
        wget --secure-protocol="TLSv1" -t 3 --output-document /tmp/id_rsa "$GET_KEY_URL"

        if [[ `ls /tmp/id_rsa -s | cut -d ' ' -f 1` -eq 0 ]]
        then
          echo "Error while trying to download RSA key."
          exit 1
       fi

       cp /tmp/id_rsa $RSA_KEY_LOCATION
       rm /tmp/id_rsa

       echo "RSA key has been successfuly downloaded."
      else
       echo "RSA key valid"
      fi

      chmod 600 $RSA_KEY_LOCATION

      # get IP
      GET_IP_URL="https://dom.fibaro.com/get_ssh_ip.php?PK_AccessPoint=$HC2_Serial&HW_Key=$HW_Key"
      IP_Response=`curl -k -f -s -S --retry 3 --connect-timeout 100 --max-time 100 "$GET_IP_URL"`

      # get PORT
      GET_PORT_URL="https://dom.fibaro.com/get_ssh_port.php?PK_AccessPoint=$HC2_Serial&HW_Key=$HW_Key"
      PORT_Response=`curl -k -f -s -S --retry 3 --connect-timeout 100 --max-time 100 "$GET_PORT_URL"`

      is_valid_port $PORT_Response
      if [ "$?" != "0" ]; then
          echo "Invalid port: $PORT_Response"
          exit 1
      fi

      is_valid_host $IP_Response
      if [ "$?" != "0" ]; then
          echo "Invalid host: $IP_Response"
          exit 1
      fi

      screen -d -m -S RemoteAccess /usr/bin/autossh -o StrictHostKeyChecking=no -M 20000 -i $RSA_KEY_LOCATION -R $PORT_Response:localhost:80 remote2@$IP_Response


      now=`date +%s`
      if [[ "$remoteSupport" -gt "$now" ]]; then
        echo "Starting remote access for support"
        screen -d -m -S SupportRemoteAccess /opt/fibaro/LuaEnvironment -b "HWKey='$HW_Key' serialNumber='$HC2_Serial'" /opt/fibaro/scripts/remote-support.lua
      fi
  fi
  exit 0
}

stop_remote_access()
{
    killall -w -9 autossh
    echo "RemoteAccess killed"
}

case $1 in
	start)
	   if [[ `pgrep autossh; echo $?` == "0" ]]
	   then
	     echo "RemoteAccess is already running"
	     exit 0
	   fi
	   
		start_remote_access
		
		if [[ "$!" == "0" ]]
		then
		  echo "RemoteAccess started"
		else
		  echo "RemoteAccess - error while starting"
		fi
		;;
		
	stop)
	    stop_remote_access
		;;
		
	restart)
      stop_remote_access
      start_remote_access
      
      if [[ "$!" == "0" ]]
		then
		  echo "RemoteAccess started"
		else
		  echo "RemoteAccess - error while starting"
		fi

esac

