<?php
header('Content-Type: text/html; charset=utf-8');
date_default_timezone_set('UTC');
set_time_limit(300);

require_once dirname(__FILE__) . '/../includes/KLogger.php';

require_once dirname(__FILE__) . '/../includes/config.php';
require_once dirname(__FILE__) . '/../includes/tools.php';
require_once dirname(__FILE__) . '/../db/DbConnect.php';
require_once dirname(__FILE__) . '/../OldDevIdGroup/DevicesOld.php';
require_once dirname(__FILE__) . '/../Device/Devices.php';
require_once dirname(__FILE__) . '/../RPCClient/JsonRPCClient.php';
require_once dirname(__FILE__) . '/Report.php';

require_once dirname(__FILE__) . '/../NoTemplate/CommandClassesLoader.php';
require_once dirname(__FILE__) . '/../NoTemplate/NewDeviceType.php';

require_once dirname(__FILE__) . '/../Device/Reconfiguration.php';
require_once dirname(__FILE__) . '/../HTTPClient/HTTPClient.php';


class Convert
{

    public $devOld;
    public $devNew;
    private $typeToId;
    private $typeToIdNoRewrite;
    private $idToType;
    private $rewrittenIDs;
    private $nodesRooms;
    public $report;

    function __construct()
    {
        global $logger;
        
        $this->devOld = array();

        $dev = new DevicesOld();
        $this->devOld = $dev->getZwaveDevice();
        $logger->logDebug('Old Z-Wave devices: ', $this->devOld);
        
        $this->devNew = new stdClass();
        $this->typeToId = array();
        $this->typeToIdNoRewrite = array();
        $this->idToType = array();
        $this->rewrittenIDs = array();
        $this->nodesRooms = array();
        
        $this->report = array();
        
        $this->constructTypeToIdArray();
    }

    public function getNewDevice()
    {
        $devices = new Devices($this->devOld);

        $this->devNew = $devices->listNewDevice;

        $json = '';
        foreach ($this->devNew as $newDevice)
        {
            if ($json != '')
            {
                $json .= ', ';
            }
            $json .= $newDevice->serialize();
        }

        $json = '[' . $json . ']';
        
        $this->checkBattery();

        $this->save();
        $this->setNames();
        $this->rewriteIDs();
    }

    public function save()
    {
        $jsonRPCClient = new JsonRPCClient();
        foreach ($this->devNew as &$newDevice)
        {
            //cannot create new deice as drencher without proper info in drencher panel (segmentation foult on serialize)
            if($newDevice->getPropertyValue("deviceControlType") ==  "3")
            {
                $newDevice->isDrencher = true;
                $newDevice->setPropertyValue("deviceControlType","20");
            }
            
            $jsonRPCClient->createDevice($newDevice);
        }
    }

    private function constructTypeToIdArray()
    {
        $db = new DbConnect(DB_TMP_PATH);
        $res = $db->query('select * from `' . TABLE_NEW_Device_Type . '`');
        while ($row = $res->fetch())
        {
            $this->typeToId[$row['Name']] = $row['Id'];
            $this->typeToIdNoRewrite[$row['Name']] = $row['Id'];
            $this->idToType[$row['Id']] = $row['Name'];
        }
        unset($res);

        //hardcoding new Fibaro devices types
        $this->typeToId['com.fibaro.FGFS101'] = 9;
        $this->typeToId['com.fibaro.FGWP101'] = 2;
        $this->typeToId['com.fibaro.FGRGBW441Dimmer'] = 3;
        $this->typeToId['com.fibaro.FGRGBW441M'] = 31;
        $this->typeToId['com.fibaro.FGRGBW441Sensor'] = 3;
        $this->typeToId['com.fibaro.FGRM222'] = 4;
        $this->typeToId['com.fibaro.FGSS001'] = 10;
        $this->typeToId['com.fibaro.doorWindowSensor'] = 7;
        $this->typeToId['com.fibaro.FGMS001'] = 8;
        $this->typeToId['com.fibaro.remoteController'] = 28;
        $this->typeToId['com.fibaro.thermostatHorstmann'] = 5;
    }

    /**
     * 
     * @param string $type - device type to find id
     * @param bool $nonExistenTypesRewritten - if true function returns rewritten ids for device types non-existent in v3.x
     * @return device type id
     */
    public function getTypeId($type, $nonExistenTypesRewritten = true)
    {
        if ($nonExistenTypesRewritten)
        {
            if (array_key_exists($type, $this->typeToId))
            {
                return $this->typeToId[$type];
            }
        }
        else
        {
            if (array_key_exists($type, $this->typeToIdNoRewrite))
            {
                return $this->typeToIdNoRewrite[$type];
            }
        }

        return NULL;
    }
    
    public function getTypeName($typeId)
    {
        if (array_key_exists($typeId, $this->idToType))
        {
            return $this->idToType[$typeId];
        }

        return NULL;
    }
    
    private function findRGBWParentIndexByDeviceId($deviceId)
    {
//        foreach ($this->devNew as $index => $device)
//        {
//            if(get_class($device) == 'Device' && $device->id == $deviceId)
//            {
//                return $index;
//            }
//        } 
//        
        foreach ($this->devNew as $index => $device)
        {
            if(get_class($device) == 'Device' && $device->id == $deviceId)
            {
                if ($device->hasProperty('parametersTemplate') && in_array($device->getPropertyValue('parametersTemplate'), array(231, 232, 254, 255, 256, 257, 258, 259)))
                {
                    if (!in_array($device->type, array('com.fibaro.FGRGBW441M', 'com.fibaro.colorController', 'com.fibaro.zwaveDevice')))
                    {
                        $rgbDeviceIndex = $this->findDeviceIndexForNewType('com.fibaro.FGRGBW441M', $device->getPropertyValue('nodeId'), $device->getPropertyValue('endPointId'));
                        if (!is_null($rgbDeviceIndex))
                        {
                            return $rgbDeviceIndex;
                        }
                    }
                }
            }
        }
        
        return NULL;
    }
            
    
    private function findDeviceIndexForParentType($typeId, $nodeId, $endPointId, $sourceTypeId)
    {
        global $logger, $deviceHierarchy, $newDeviceTypes, $deviceTypesToRewriteFromOldDB;
        
        $deviceType = $newDeviceTypes->get($typeId);
        if (!is_null($deviceType))
        {
            $parentType = $deviceHierarchy->getBaseType($deviceType);
            if ($parentType != '')
            {
                $parentTypeId = $this->getTypeId($parentType, false);
                if (!is_null($parentTypeId))
                {
                    $logger->logDebug('findDeviceIndexForOldParentType - searching for device type ' . $parentType . ', nodeId: ' . $nodeId . ', endPointId: ' . $endPointId);
                    foreach ($this->devNew as $index => $device)
                    {
                        if (get_class($device) == 'Device' && $device->getPropertyValue('nodeId') == $nodeId && $this->getTypeId($device->type) == $parentTypeId && $device->getPropertyValue('endPointId') == $endPointId)
                        {
                            $logger->logDebug('findDeviceIndexForOldParentType - found device: ' . $device->id . ' for device type ' . $parentType . ', nodeId: ' . $nodeId . ', endPointId: ' . $endPointId);
                            
                            if (in_array($device->type, $deviceTypesToRewriteFromOldDB))
                            {
                                $sourceType = $newDeviceTypes->get($sourceTypeId);
                                if (!is_null($sourceType))
                                {
                                    $device->type = $sourceType;
                                }
                            }
                            
                            return $index;
                        }
                    }

                    $fromParentType = $this->findDeviceIndexForParentType($parentTypeId, $nodeId, $endPointId, $sourceTypeId);
                    return $fromParentType;
                }
                
            }
        }

        return NULL;
    }
    
    private function findDeviceIndexForChildTypes($typeId, $nodeId, $endPointId, $sourceTypeId)
    {
        global $logger, $deviceHierarchy, $newDeviceTypes, $deviceTypesToRewriteFromOldDB;
        
        $logger->logDebug('Searching in child types... ' . $typeId);
        $deviceType = $newDeviceTypes->get($typeId);
        if (!is_null($deviceType))
        {
            $childTypes = $deviceHierarchy->getChildTypes($deviceType);
            if (count($childTypes) > 0)
            {
                foreach ($childTypes as $childType)
                {
                    $childTypeId = $this->getTypeId($childType, false);
                    if (!is_null($childTypeId))
                    {
                        $logger->logDebug('findDeviceIndexForChildTypes - searching for device type ' . $childType . ', nodeId: ' . $nodeId . ', endPointId: ' . $endPointId);
                        foreach ($this->devNew as $index => $device)
                        {
                            if (get_class($device) == 'Device' && $device->getPropertyValue('nodeId') == $nodeId && $this->getTypeId($device->type) == $childTypeId && $device->getPropertyValue('endPointId') == $endPointId)
                            {
                                $logger->logDebug('findDeviceIndexForChildTypes - found device: ' . $device->id . ' for device type ' . $childType . ', nodeId: ' . $nodeId . ', endPointId: ' . $endPointId);
                                
                                if (in_array($device->type, $deviceTypesToRewriteFromOldDB))
                                {
                                    $sourceType = $newDeviceTypes->get($sourceTypeId);
                                    if (!is_null($sourceType))
                                    {
                                        $device->type = $sourceType;
                                    }
                                }
                                
                                return $index;
                            }
                        }

                        $fromChildTypes = $this->findDeviceIndexForChildTypes($childTypeId, $nodeId, $endPointId, $sourceTypeId);
                        if (!is_null($fromChildTypes))
                        {
                            return $fromChildTypes;
                        }
                    }
                }
            }
        }

        return NULL;
    }

    private function findDeviceIndexForOldType($oldTypeId, $nodeId, $endPointId, $searchOther = true)
    {
        global $logger, $newDeviceTypes;
        
        $logger->logDebug('findDeviceIndexForOldType - searching for device type ' . $oldTypeId . ', nodeId: ' . $nodeId . ', endPointId: ' . $endPointId);
        foreach ($this->devNew as $index => $device)
        {
            if (get_class($device) == 'Device' && $device->getPropertyValue('nodeId') == $nodeId && $this->getTypeId($device->type) == $oldTypeId && $device->getPropertyValue('endPointId') == $endPointId)
            {
                $logger->logDebug('findDeviceIndexForOldType - found device: ' . $device->id . ' for device type ' . $oldTypeId . ', nodeId: ' . $nodeId . ', endPointId: ' . $endPointId);
                return $index;
            }
        }
        
        /**
         * hardcoded exceptions...
         */
        $exception = false;
        if ($oldTypeId == 5) //com.fibaro.thermostatDanfoss (Horstmann etc.)
        {
            $oldTypeId = 71; //com.fibaro.thermostat (find proper thermostat type in child types...)
            $exception = true;
        }
        
        if ($exception)
        {
            $logger->logDebug('findDeviceIndexForOldType - searching for device type ' . $oldTypeId . ', nodeId: ' . $nodeId . ', endPointId: ' . $endPointId);
            foreach ($this->devNew as $index => $device)
            {
                if (get_class($device) == 'Device' && $device->getPropertyValue('nodeId') == $nodeId && $this->getTypeId($device->type) == $oldTypeId && $device->getPropertyValue('endPointId') == $endPointId)
                {
                    $logger->logDebug('findDeviceIndexForOldType - found device: ' . $device->id . ' for device type ' . $oldTypeId . ', nodeId: ' . $nodeId . ', endPointId: ' . $endPointId);
                    return $index;
                }
            }
        }
        
        $fromParentType = $this->findDeviceIndexForParentType($oldTypeId, $nodeId, $endPointId, $oldTypeId);
        if (!is_null($fromParentType))
        {
            return $fromParentType;
        }
        else
        {
            $fromChildTypes = $this->findDeviceIndexForChildTypes($oldTypeId, $nodeId, $endPointId, $oldTypeId);
            if (!is_null($fromChildTypes))
            {
                return $fromChildTypes;
            }
        }
        
        /**
         * search in other types - motion/door/window/flood/smoke sensor
         */
        $otherTypes = array(
            7,  //door_sensor
            8,  //motion sensor
            9,  //water_sensor,
            10, //smoke_sensor
            32, //window_sensor
        );
        if ($searchOther && in_array($oldTypeId, $otherTypes))
        {
            $logger->logDebug('Searching for type set by user for node: ' . $nodeId . ', user type: ' . $oldTypeId);
            foreach ($otherTypes as $otherType)
            {
                $logger->logDebug('Searching ' . $otherType . '...');
                $otherIndex = $this->findDeviceIndexForOldType($otherType, $nodeId, $endPointId, false);
                if (!is_null($otherIndex))
                {
                    /**
                     * Changing type to one set by user
                     */
                    $logger->logDebug('Found  type, original type id: ' . $otherType);
                    $newTypeName = $this->getTypeName($oldTypeId);
                    if (!is_null($newTypeName))
                    {
                        $logger->logDebug('Found user type, type name: ' . $newTypeName);
                        $this->devNew[$otherIndex]->type = $newTypeName;
                    }
                    
                    return $otherIndex;
                }
            }
        }

        return NULL;
    }
    
    private function findDeviceIndexForNewType($type, $nodeId)
    {
        global $logger, $newDeviceTypes;
        
        $logger->logDebug('findDeviceIndexForNewType - searching for device type ' . $type . ', nodeId: ' . $nodeId);
        foreach ($this->devNew as $index => $device)
        {
            if (get_class($device) == 'Device' && $device->getPropertyValue('nodeId') == $nodeId && $device->type == $type)
            {
                $logger->logDebug('findDeviceIndexForNewType - found device: ' . $device->id . ' for device type ' . $type . ', nodeId: ' . $nodeId);
                return $index;
            }
        }
        
        return NULL;
    }
    
    private function findDeviceIndexForMainDevice($nodeId)
    {
        foreach ($this->devNew as $index => $device)
        {
            if (get_class($device) == 'Device' && $device->getPropertyValue('nodeId') == $nodeId && $device->type == 'com.fibaro.mainZwaveDevice')
            {
                return $index;
            }
        }
        
        return NULL;
    }
    
    private function setNames()
    {
        global $logger;
        
        $mainIDs = array();
        
        

        foreach ($this->devNew as $device)
        {
            if ($device->type == 'com.fibaro.zwaveDevice')
            {
                $mainIDs[$device->getPropertyValue('nodeId')] = $device->id;
                $device->parentId = 1;
                $device->name = 'Main_Zwave_Device';
            }
        }

        foreach ($this->devNew as $device)
        {
            if ($device->type != 'com.fibaro.zwaveDevice')
            {
                $device->parentId = intval($mainIDs[$device->getPropertyValue('nodeId')]);
                
                if ($device->name == '')
                {
                    $endPointId = $device->getPropertyValue('endPointId');
                    $virtualId = $device->virtualID;

                    $device->name = $device->parentId . '.' . $endPointId . '.' . $virtualId;
                }
            }
            echo "\n\n DEVICE SET NAMES \n" . json_encode($device) . "\n\n";
        }
    }
    
    private function cleanUpConfiguredProperty()
    {
        global $_RECONFIGURATION;
        
        $nodes = array();
        $nodeConfigured = false;
        
        /**
         * UPDATE: all devices need to be set as not configured
         */
        
        foreach ($this->devNew as $device)
        {
            // exclude Fibaro RGBW devices (child devices)
            if ($device->hasProperty('parametersTemplate') && in_array($device->getPropertyValue('parametersTemplate'), array(231, 232, 254, 255, 256, 257, 258, 259)))
            {
                if($device->hasProperty('deviceControlType') && $device->getPropertyValue('deviceControlType') == 52)
                    continue;
                    
                echo "parentId: \n";
                echo "$device->parentId"; 
                
                echo "deviceId: \n";
                echo $device->id;
                
                $parentIndex = $this->findRGBWParentIndexByDeviceId($device->id);
                                            
                if(!is_null($parentIndex))
                {
                    $parent = $this->devNew[$parentIndex];
                    
                    if(!is_null($parent))
                    {
                        echo "is not null\n";
                        
                        if($parent->hasProperty('deviceControlType') && $parent->getPropertyValue('deviceControlType') == 52)
                        {
                            echo "omit device: \n";
                            echo $device->id;
                            
                            continue;
                        }
                        
                        echo "\n";   
                    }
                }
            }

            // exclude Fibaro RGBW (com.fibaro.zwaveDevice)
            if ($device->hasProperty('productInfo') && substr($device->getPropertyValue('productInfo'), 0, 8) == '1,15,5,1')
            {
                if($device->hasProperty('deviceControlType') && $device->getPropertyValue('deviceControlType') == 52)
                    continue; 
            }

            if ($device->hasProperty('configured'))
            {
                $device->setPropertyValue('configured', $nodeConfigured);
            }
        }
    }
    
    private function setHardcodedDevicesValues()
    {
        global $logger;
        
        foreach ($this->devNew as $device)
        {
            if ($device->type == 'com.fibaro.setPoint' && $device->hasProperty('parametersTemplate') && $device->getPropertyValue('parametersTemplate') == 43)
            {
                //Horstmann thermostat setpoint
                $logger->logDebug('Setting values for Horstmann thermostat setpoint');
                $device->setPropertyValue('supportedModes', '1');
            }
            elseif ($device->type == 'com.fibaro.operatingModeHorstmann' && $device->hasProperty('parametersTemplate') && $device->getPropertyValue('parametersTemplate') == 44)
            {
                //Horstmann thermostat mode
                $logger->logDebug('Setting values for Horstmann thermostat mode');
                $device->setPropertyValue('supportedModes', '0,1');
            }
        }
    }
    
    public function rgbPatch()
    {
        global $logger;
        
        foreach ($this->devNew as $device)
        {
            $rewriteParentId = true;
            if ($device->hasProperty('parametersTemplate') && in_array($device->getPropertyValue('parametersTemplate'), array(231, 232, 254, 255, 256, 257, 258, 259)))
            {
                if (!in_array($device->type, array('com.fibaro.FGRGBW441M', 'com.fibaro.colorController', 'com.fibaro.zwaveDevice')))
                {
                    $rgbDeviceIndex = $this->findDeviceIndexForNewType('com.fibaro.FGRGBW441M', $device->getPropertyValue('nodeId'), $device->getPropertyValue('endPointId'));
                    if (!is_null($rgbDeviceIndex))
                    {
                        $device->parentId = intval($this->devNew[$rgbDeviceIndex]->id);
                        $logger->logDebug('Found parent RGB device ' . $device->parentId . ' for ' . $device->id);
                    }
                }
            }
        }
    }
    
    public function getOldDevice($deviceId)
    {
        foreach ($this->devOld as $nodeId => $oldDevices)
        {
            foreach ($oldDevices as $oldDevice)
            {
                if ($oldDevice->getID() == $deviceId)
                {
                    return $oldDevice;
                }
            }
        }
        
        return NULL;
    }
    
    public function checkBattery()
    {
        global $logger;
        
        foreach ($this->devOld as $nodeId => $oldDevices)
        {
            $firstDevice = $oldDevices[0];
            $nodeId = $firstDevice->getPropertyName('nodeID');
            
            $mainDeviceIndex = $this->findDeviceIndexForMainDevice($nodeId);
            if (!is_null($mainDeviceIndex))
            {
                $device = $this->devNew[$mainDeviceIndex];
                $logger->logDebug($device->id . ': checking battery ');
                if ($device->internal->getCheckBattery() === true)
                {
                    $logger->logDebug($device->id . ': found checkBattery option');
                    if ($firstDevice->getPropertyName('isBatteryOperated') == '1')
                    {
                        $logger->logDebug($device->id . ': adding interfaces from checkBattery()');
                        $this->devNew[$mainDeviceIndex]->addInterface('battery');
                        $this->devNew[$mainDeviceIndex]->addInterface('zwaveWakeup');
                    }
                    else
                    {
                        foreach ($this->devNew as $dev)
                        {
                            if ($dev->getPropertyValue('nodeId') == $nodeId)
                            {
                                $logger->logDebug($dev->id . ': removing interfaces from checkBattery()');
                                $dev->removeInterface('battery');
                                $dev->removeInterface('zwaveWakeup');
                            }
                        }
                    }
                }
            }
        }
    }
    
    public function addBatteryToChildren()
    {
        global $logger;
        
        foreach ($this->devNew as $device)
        {
            if ($device->type != 'com.fibaro.zwaveDevice')
            {
                $mainDeviceIndex = $this->findDeviceIndexForMainDevice($device->getPropertyValue('nodeId'));
                
                if (isset($this->devNew[$mainDeviceIndex]) && $this->devNew[$mainDeviceIndex]->hasInterface('battery'))
                {
                    $logger->logDebug($device->id . ': adding interface battery from parent');
                    $device->addInterface('battery');
                }
                
                if (isset($this->devNew[$mainDeviceIndex]) && $this->devNew[$mainDeviceIndex]->hasInterface('zwaveWakeup'))
                {
                    $device->addInterface('zwaveWakeup');
                }
            }
        }
    }
    
    public function otherPatches()
    {
        $pollingIntervals = array();
        foreach ($this->devNew as $device)
        {
            if ($device->hasProperty('productInfo'))
            {
                /**
                 * switch 3kW - hiding third child
                 */
                if (substr($device->getPropertyValue('productInfo'), 0, 8) == '1,15,4,0' && $device->hasProperty('endPointId') && $device->getPropertyValue('endPointId') == 2)
                {
                    $device->visible = false;
                    $index = $this->findDeviceIndexForOldType($this->getTypeId($device->type), $device->getPropertyValue('nodeId'), 0);
                    if (!is_null($index))
                    {
                        $id = $this->devNew[$index]->id;
                        $oldDevice = $this->getOldDevice($id);
                        if (!is_null($oldDevice))
                        {
                            if ($oldDevice->getPropertyName('showChildren') == 1)
                            {
                                $device->visible = true;
                            }
                        }
                    }
                }
                
                /**
                 * switch 3kW - hiding second child
                 */
                
                elseif (substr($device->getPropertyValue('productInfo'), 0, 8) == '1,15,4,0' && $device->hasProperty('endPointId') && $device->getPropertyValue('endPointId') == 1)
                {
                	$device->visible = false;
                	$index = $this->findDeviceIndexForOldType($this->getTypeId($device->type), $device->getPropertyValue('nodeId'), 0);
                	if (!is_null($index))
                	{
                		$id = $this->devNew[$index]->id;
                		$oldDevice = $this->getOldDevice($id);
                		if (!is_null($oldDevice))
                		{
                			if ($oldDevice->getPropertyName('showChildren') == 1)
                			{
                				$device->visible = true;
                			}
                		}
                	}
                }
                
                /**
                 * UBS - hide endPoint 0
                 */
                elseif (substr($device->getPropertyValue('productInfo'), 0, 8) == '1,15,5,1' && $device->hasProperty('endPointId') && $device->getPropertyValue('endPointId') == 0)
                {
                    $device->visible = false;
                }
            }
            
            /**
             * main Z-Wave device icon
             */
            if ($device->type == 'com.fibaro.zwaveDevice')
            {
                $device->setPropertyValue('deviceIcon', 78);
            }
            
            /**
             * Horstmann icon
             */
            if ($device->type == 'com.fibaro.thermostatHorstmann')
            {
                $device->setPropertyValue('deviceIcon', 39);
            }

            /**
             * HRT4 ASR-ZW icon
             */
            if ($device->type == 'com.fibaro.operatingModeHorstmann')
            {
                $device->setPropertyValue('deviceIcon', 38);
            }

            /**
            * com.fibaro.FGSS001 - fibaro smoke sensor icon
            */
            if ($device->type == 'com.fibaro.FGSS001' && $device->getPropertyValue('deviceIcon') > 49 && $device->getPropertyValue('deviceIcon') < 1000)
            {
                $device->setPropertyValue('deviceIcon', 69);
            }
            
            /**
             * com.fibaro.FGMS001 - fibaro motion sensor icon
             */
            if ($device->type == 'com.fibaro.FGMS001' && $device->getPropertyValue('deviceIcon') > 49 && $device->getPropertyValue('deviceIcon') < 1000)
            {
                $device->setPropertyValue('deviceIcon', 90);
            }
            
            /**
             * main Z-Wave device pollingInterval - READ
             */
            if ($device->type != 'com.fibaro.zwaveDevice' && $device->hasProperty('pollingTimeSec') && $device->hasProperty('nodeId'))
            {
                $pollingIntervals[$device->getPropertyValue('nodeId')] = $device->getPropertyValue('pollingTimeSec');
            }
        }
        
        /**
        * main Z-Wave device pollingInterval - WRITE
        */
        foreach ($this->devNew as $device)
        {
            if ($device->type == 'com.fibaro.zwaveDevice' && $device->hasProperty('nodeId') && isset($pollingIntervals[$device->getPropertyValue('nodeId')]))
            {
                if (!$device->hasProperty('pollingInterval'))
                {
                    $device->addProperty(new Property('pollingInterval', 'int'));
                }
                $device->setPropertyValue('pollingInterval', intval($pollingIntervals[$device->getPropertyValue('nodeId')]) * 1000);
            }
        }
    }

    public function rewriteIDs()
    {
        global $propertiesToCopyFromFirstDevice, $megaHardcodeArray, $logger, $_RECONFIGURATION;
        
        $this->addBatteryToChildren();
        
        foreach ($this->devOld as $nodeId => $oldDevices)
        {
            /**
             * setting main Z-Wave device
             */
            $firstDevice = $oldDevices[0];
            $nodeId = $firstDevice->getPropertyName('nodeID');
            $parametersTemplate = intval($firstDevice->getPropertyName('parametersTemplate'));
            
            $mainDeviceIndex = $this->findDeviceIndexForMainDevice($nodeId);
            if (!is_null($mainDeviceIndex))
            {
                $this->devNew[$mainDeviceIndex]->setPropertyValue('nodeId', $nodeId);
                $oldDead = $firstDevice->getPropertyName('dead');

                if ($oldDead == "0")
                {
                    $this->devNew[$mainDeviceIndex]->setPropertyValue('dead', false);
                    $this->devNew[$mainDeviceIndex]->setPropertyValue('markAsDead', true);
                } elseif ($oldDead == "1")
                {
                    $this->devNew[$mainDeviceIndex]->setPropertyValue('dead', true);
                    $this->devNew[$mainDeviceIndex]->setPropertyValue('markAsDead', true);
                } elseif ($oldDead == "2")
                {
                    $this->devNew[$mainDeviceIndex]->setPropertyValue('dead', false);
                    $this->devNew[$mainDeviceIndex]->setPropertyValue('markAsDead', false);
                }

                foreach ($firstDevice->property as $property)
                {
                    $this->devNew[$mainDeviceIndex]->setPropertyValue($property['Name'], $property['Value']);
                }

                //hardcoded changes
                $changes = array(
                    'valueSensor' => 'power',
                    'valueMeter' => 'energy'
                );
                foreach ($changes as $oldPropertyName => $newPropertyName)
                {
                    $propertyValue = $firstDevice->getPropertyName($oldPropertyName);
                    if (!is_null($propertyValue))
                    {
                        $this->devNew[$mainDeviceIndex]->setPropertyValue($newPropertyName, $propertyValue);
                    }
                }

                //mega hardcode
                $needConfigure = $firstDevice->getPropertyName('needConfigure');
                if (!is_null($needConfigure))
                {
                    if ($needConfigure == '5')
                    {
                        $this->devNew[$mainDeviceIndex]->setPropertyValue('configured', true);
                    } else
                    {
                        $this->devNew[$mainDeviceIndex]->setPropertyValue('configured', false);
                    }
                }

                /*
                 * associations, parameters, internal - cleaning up
                 */
                foreach ($propertiesToCopyFromFirstDevice as $propertyName => $propertyType)
                {
                    $oldVal = $firstDevice->getPropertyName($propertyName);
                    if (!is_null($oldVal))
                    {
                        if (!$this->devNew[$mainDeviceIndex]->hasProperty($propertyName))
                        {
                            $this->devNew[$mainDeviceIndex]->addProperty(new Property($propertyName, $propertyType));
                        }
                        $this->devNew[$mainDeviceIndex]->setPropertyValue($propertyName, $oldVal);
                    }
                }
            }
            
            /**
             * setting other devices
             */
            foreach ($oldDevices as $oldDevice)
            {
                $id = $oldDevice->getID();
                $name = $oldDevice->device['Name'];
                $typeId = $oldDevice->device['Device_Type'];
                $roomId = $oldDevice->device['Room_Id'];
                $sectionId = $oldDevice->device['Section_Id'];
                $endPointId = $oldDevice->getPropertyName('endPoint');

                if (array_key_exists($parametersTemplate, $megaHardcodeArray))
                {
                    if (array_key_exists(intval($endPointId), $megaHardcodeArray[$parametersTemplate]) && !is_null($megaHardcodeArray[$parametersTemplate][intval($endPointId)]))
                    {
                        $newDeviceIndex = $this->findDeviceIndexForOldType($typeId, $nodeId, (string)$megaHardcodeArray[$parametersTemplate][intval($endPointId)]);
                    }
                    else
                    {
                        $newDeviceIndex = NULL;
                    }
                }
                else
                {
                    $newDeviceIndex = $this->findDeviceIndexForOldType($typeId, $nodeId, $endPointId);
                }
                
                if (!is_null($newDeviceIndex))
                {
                    $this->rewrittenIDs[$this->devNew[$newDeviceIndex]->id] = $id;
                    $this->devNew[$newDeviceIndex]->id = $id;
                    $this->devNew[$newDeviceIndex]->name = $name;
                    
                    echo "\n\n ID CHANGE " . $id . " \n";
                    echo "NAME CHANGE " . $name . " \n\n";
                    
                    $this->devNew[$newDeviceIndex]->roomID = $roomId;
                    $this->devNew[$newDeviceIndex]->sectionID = $sectionId;
                    
                    if ($endPointId == 0)
                    {
                        //remember room for main device to copy it to new devices
                        $this->nodesRooms[$nodeId] = $roomId;
                    }
                    
                    /**
                     * code copied from Devices::updateDataInNewDevice() - here we have proper connections between new and old devices
                     */
                    $this->devNew[$newDeviceIndex]->setPropertyValue('nodeId', $nodeId);
                    $oldDead = $firstDevice->getPropertyName('dead');

                    if ($oldDead == "0")
                    {
                        $this->devNew[$newDeviceIndex]->setPropertyValue('dead', false);
                        $this->devNew[$newDeviceIndex]->setPropertyValue('markAsDead', true);
                    } elseif ($oldDead == "1")
                    {
                        $this->devNew[$newDeviceIndex]->setPropertyValue('dead', true);
                        $this->devNew[$newDeviceIndex]->setPropertyValue('markAsDead', true);
                    } elseif ($oldDead == "2")
                    {
                        $this->devNew[$newDeviceIndex]->setPropertyValue('dead', false);
                        $this->devNew[$newDeviceIndex]->setPropertyValue('markAsDead', false);
                    }
                    
                    /**
                     * set enabled and visible - these are messes up in some templates...
                     */
                    $oldDisabled = $oldDevice->getPropertyName('disabled');
                    if ($oldDisabled == '1')
                    {
                        $this->devNew[$newDeviceIndex]->enabled = false;
                        $this->devNew[$newDeviceIndex]->visible = false;
                    }
                    else {
                        $this->devNew[$newDeviceIndex]->enabled = true;
                        $this->devNew[$newDeviceIndex]->visible = true;
                    }
                    
                    /**
                     * virtual power consumption patch...
                     */
                    $powerConsumption = $oldDevice->getPropertyName('powerConsumption');
                    if (!is_null($powerConsumption))
                    {
                        $logger->logDebug('Adding powerConsumption to device ' . $this->devNew[$newDeviceIndex]->id);
                        $this->devNew[$newDeviceIndex]->addProperty(new Property('powerConsumption', 'real'));
                        $this->devNew[$newDeviceIndex]->propertiesOutOfTemplate[] = 'powerConsumption';
                        if (!$this->devNew[$newDeviceIndex]->hasInterface('power'))
                        {
                            $logger->logDebug('Adding power interface to device ' . $this->devNew[$newDeviceIndex]->id);
                            $this->devNew[$newDeviceIndex]->addInterface('power');
                            $this->devNew[$newDeviceIndex]->propertiesOutOfTemplate[] = 'showEnergy';
                            $this->devNew[$newDeviceIndex]->propertiesOutOfTemplate[] = 'power';
                        }
                    }

                    foreach ($oldDevice->property as $property)
                    {
                        /*
                         * RGB patch...
                         */
                        if (!is_null($firstDevice->getPropertyName('parametersTemplate')) && in_array($firstDevice->getPropertyName('parametersTemplate'), array(231, 232, 254, 255, 256, 257, 258, 259)))
                        {
                            $rgbChildrenProperties = array(
                                'unit' => 'string', 
                                'minValue' => 'real', 
                                'maxValue' => 'real', 
                                'minVoltage' => 'real', 
                                'maxVoltage' => 'real', 
                                'buttonType' => 'int', 
                                'deviceControlType' => 'int'
                            );
                            if (array_key_exists($property['Name'], $rgbChildrenProperties) && !$this->devNew[$newDeviceIndex]->hasProperty($property['Name']))
                            {
                                $this->devNew[$newDeviceIndex]->propertiesOutOfTemplate[] = $property['Name'];
                                $this->devNew[$newDeviceIndex]->addProperty(new Property($property['Name'], $rgbChildrenProperties[$property['Name']]));
                            }
                        }
                        $this->devNew[$newDeviceIndex]->setPropertyValue($property['Name'], $property['Value']);
                    }

                    //hardcoded changes
                    $changes = array(
                        'valueSensor' => 'power',
                        'valueMeter' => 'energy'
                    );
                    foreach ($changes as $oldPropertyName => $newPropertyName)
                    {
                        $propertyValue = $oldDevice->getPropertyName($oldPropertyName);
                        if (!is_null($propertyValue))
                        {
                            $this->devNew[$newDeviceIndex]->setPropertyValue($newPropertyName, $propertyValue);
                        }
                    }

                    //mega hardcode
                    $needConfigure = $firstDevice->getPropertyName('needConfigure');
                    if (!is_null($needConfigure))
                    {
                        if ($needConfigure == '5')
                        {
                            $this->devNew[$newDeviceIndex]->setPropertyValue('configured', true);
                        } else
                        {
                            $this->devNew[$newDeviceIndex]->setPropertyValue('configured', false);
                        }
                    }

                    /*
                     * associations, parameters, internal - cleaning up
                     */
                    foreach ($propertiesToCopyFromFirstDevice as $propertyName => $propertyType)
                    {
                        $oldVal = $firstDevice->getPropertyName($propertyName);
                        if (!is_null($oldVal))
                        {
                            if (!$this->devNew[$newDeviceIndex]->hasProperty($propertyName))
                            {
                                $this->devNew[$newDeviceIndex]->addProperty(new Property($propertyName, $propertyType));
                            }
                            $this->devNew[$newDeviceIndex]->setPropertyValue($propertyName, $oldVal);
                        }
                    }
                }
            }
        }

        //copy new IDs to internal
        $endPointDevIDs = array();
        foreach ($this->devNew as $tmpDev)
        {
            $endPointDevIDs[$tmpDev->getPropertyValue('nodeId')][$tmpDev->getPropertyValue('endPointId')][$tmpDev->virtualID] = $tmpDev->id;
        }

        $jsonRPCClient = new JsonRPCClient();
        foreach ($this->devNew as $tmpDev)
        {
            if ($tmpDev->type == 'com.fibaro.zwaveDevice')
            {
                $tmpDev->internal->setDevParentID($tmpDev->id);
                foreach ($tmpDev->internal->endPoint as $endPoint)
                {
                    if (isset($endPointDevIDs[$tmpDev->getPropertyValue('nodeId')][$endPoint->getEndPointID()][0]))
                    {
                        $endPoint->setDevIdEndPoint($endPointDevIDs[$tmpDev->getPropertyValue('nodeId')][$endPoint->getEndPointID()][0]);
                    }
                    if (count($endPoint->getValue()) > 0)
                    {
                        foreach ($endPoint->getValue() as $value)
                        {
                            if (isset($endPointDevIDs[$tmpDev->getPropertyValue('nodeId')][$endPoint->getEndPointID()][$value->data['virtual']]))
                            {
                                $value->data['devID'] = intval($endPointDevIDs[$tmpDev->getPropertyValue('nodeId')][$endPoint->getEndPointID()][$value->data['virtual']]);
                            }
                        }
                    }
                }

                $jsonRPCClient->insertInternal($tmpDev->id, $tmpDev->getInternal());
            }
        }

        $this->cleanUpConfiguredProperty();
        $this->setHardcodedDevicesValues();
        $this->rgbPatch();
        $this->otherPatches();
        
        $this->saveNewIDs();
    }

    public function stopHCServer()
    {   
        echo 'Stopping services...' . PHP_EOL;
        exec('killall -w -9 HCServer');
        echo 'Services stopped' . PHP_EOL;
    }

    public function startHCServer()
    {
        $hcSeverLocation='/opt/fibaro/HCServer'; 

        if (isset($_SERVER['PHP_CONVERSION_HCSERVER_LOCATION']))
        {
            $hcSeverLocation = trim($_SERVER['PHP_CONVERSION_HCSERVER_LOCATION']);
        }

        exec('screen -dmS HCServer_conversion ' . $hcSeverLocation . ' --db \'' . DB_TMP_PATH . '\' --omit scene,externalHttpRequests,zwaveServicePing');
    }

    private function saveNewIDs()
    {
        global $logger;
        $logger->logDebug('Convert::saveNewIDs');

        
        $this->stopHCServer();
        
        $mainIDs = array();
        
        $db = new DbConnect(DB_TMP_PATH);
        $db->beginTransaction();

        $db->exec('update NEW_Device set Parent_Id=0 where Id in (1,2,3)');
        
        if (count($this->rewrittenIDs) > 0)
        {
            foreach ($this->rewrittenIDs as $oldId => $newId)
            {
                if (is_numeric($oldId) && is_numeric($newId) && $oldId > 0 && $newId > 0)
                {
                	echo "\n\n REPLACE IDs newId:" . $newId . " oldId:" . $oldId . " \n\n";
                	
                    if (!$db->exec('UPDATE NEW_Device set Id=' . $newId . ' WHERE Id=' . $oldId))
                    {
                        echo "\n\n ERROR UPDATE NEW DEVICE id: $newId -> oldId: $oldId \n\n";
                        $error = true;
                    }

                    if (!$db->exec('UPDATE NEW_Property set Device_Id=' . $newId . ' WHERE Device_Id=' . $oldId))
                    {
                        echo "\n\n ERROR UPDATE NEW PROPERTY id: $newId -> oldId: $oldId \n\n";
                        $error = true;
                    }

                    if (!$db->exec('UPDATE NEW_Action set Device_Id=' . $newId . ' WHERE Device_Id=' . $oldId))
                    {
                        echo "\n\n ERROR UPDATE NEW ACTION id: $newId -> oldId: $oldId \n\n";
                        $error = true;
                    }

                    if (!$db->exec('UPDATE NEW_Interface set Device_Id=' . $newId . ' WHERE Device_Id=' . $oldId))
                    {
                        echo "\n\n ERROR UPDATE NEW INTERFACE id: $newId -> oldId: $oldId \n\n";
                        $error = true;
                    }

                    if (!$db->exec('UPDATE NEW_Internal set Device_Id=' . $newId . ' WHERE Device_Id=' . $oldId))
                    {
                        echo "\n\n ERROR UPDATE NEW INTERNAL id: $newId -> oldId: $oldId \n\n";
                        $error = true;
                    }
                }
            }
        }

        foreach ($this->devNew as $device)
        {
            $device->updateSQL($db);
            $db->exec('update NEW_Device set Socket_Id=1 where Id=?', array($device->id));
        }

        $error = false;
        
        $db->commit();

        $this->convertScenes();
        $this->convertVirtualDevices();

        
        $logger->logDebug('saveNewIDs done');

        //$this->startHCServer();
    }

    
    //drencher patch 
    //dimmer pacth 
    public function anotherPatch()
    {
        $db = new DbConnect(DB_TMP_PATH);

        $db->beginTransaction();
        $error = false;  
        
        foreach ($this->devNew as $device)
        {
            if($device->isDrencher == true && $device->type != "com.fibaro.zwaveDevice")
            {
                //deviceControlType
                if (!$db->exec('UPDATE NEW_Property SET Value="3" WHERE Name=\'deviceControlType\' AND Device_Id=' . $device->id))
                {
                    $error = true;
                }
                if (!$db->exec('INSERT INTO NEW_Interface ("Device_Id", "InterfaceName") VALUES(' . $device->id . ',"drencher")'))
                {
                    $error = true;
                }
            }
        }
        
        /*
         * moving device 1 pollingTimeSec to zwave_settings
         */
        $zwaveSettings = new stdClass();
        $res = $db->query('select Value from New_Property where Device_Id=1 and Name=\'pollingTimeSec\'');
        $row = $res->fetch();
        $zwaveSettings->globalPollingInterval = intval($row['Value']) * 1000;
        if ($zwaveSettings->globalPollingInterval > 0)
        {
            if (!$db->exec('UPDATE zwave_settings set data=?', array(json_encode($zwaveSettings))))
            {
                $error = true;
            }
        }
        
        if (!$db->exec('UPDATE NEW_Property SET Value="0" WHERE Name=\'value\' AND Device_Id IN (SELECT id FROM NEW_Device WHERE Device_Type_Id=3)'))
        {
            $error = true;
        }

        /*
         * remove virtual device properties (ui.*) - they will be recreated by HCServer and initialized with default values
         */
        if (!$db->exec("DELETE FROM NEW_Property WHERE Name LIKE 'ui.%' AND Device_Id IN (SELECT Id FROM NEW_Device WHERE Device_Type = 'virtual_device')"))
        {
            $error = true;
        }
        
        if ($error)
        {
            echo 'ROLLBACK' . PHP_EOL;
            $db->rollBack();
        } 
        else
        {
            $db->commit();
        }
   
    }
    
    private function convertScenes()
    {
        if ($dh = opendir(SCENES_PATH))
        {
            $db = new DbConnect(DB_TMP_PATH);

            while (($filename = readdir($dh)) !== false)
            {
                $ext = pathinfo($filename, PATHINFO_EXTENSION);
                $name = pathinfo($filename, PATHINFO_FILENAME);
                $sceneID = intval($name);

                if ($ext == 'lua' && is_file(SCENES_PATH . '/' . $filename))
                {
                    $code = file_get_contents(SCENES_PATH . '/' . $filename);

                    $ret = $this->convertLua($code);
                    $code = $ret['code'];
                    $status = $ret['status'];

                    if ($fh = fopen(SCENES_PATH . '/' . $filename, 'w'))
                    {
                        fwrite($fh, $code);
                        fclose($fh);
                    }

                    $this->addToReport($sceneID, 'scene', $status);
                    if ($status === false)
                    {
                        $db->exec('update NEW_Scene set Enabled=0 where Id=' . $sceneID);
                    }
                }
            }

            closedir($dh);
        }
    }

    private function convertLua($code)
    {
        $oldProperties = array('valueSensor', 'valueMeter', 'endPoint', 'nodeID', 'timeStamp', 'unitMeter', 'unitSensor');
        $newProperties = array('power', 'energy', 'endPointId', 'nodeId', 'timestamp', 'unit', 'unit');

        $errorProperties = array('disabled', 'needConfigure', 'tamperNotificationID', 'tamperNotificationTypelog', 'tamperTimestamp', 
            'pollingRetryError', 'pollingTime', 'pollingTimeNext', 'pollingTimeSec', 'parentID', 'status', 'needConfigure');

        $methods = array('getValue', 'get');

        $status = true;

        foreach ($methods as $method)
        {
            $glue = 'fibaro:' . $method . '(';
            $temp = explode($glue, $code);
            if (count($temp) > 1)
            {
                for ($i = 1; $i < count($temp); $i++)
                {
                    $temp2 = explode(')', $temp[$i]);
                    $temp2[0] = str_replace($oldProperties, $newProperties, $temp2[0]);

                    if ($status) //one found is enough
                    {
                        foreach ($errorProperties as $property)
                        {
                            if (strpos($temp2[0], $property) !== false)
                            {
                                $status = false;
                            }
                        }
                    }

                    $temp[$i] = implode(')', $temp2);
                }
            }
            $code = implode($glue, $temp);
        }

        $ret = array(
            'code' => $code,
            'status' => $status
        );

        return $ret;
    }

    private function convertVirtualDevices()
    {
        $db = new DbConnect(DB_TMP_PATH);

        $statuses = array();

        /*
         * buttons
         */
        $res = $db->query('select * from N_Virtual_Device_Button where HasSlider=2');
        while ($row = $res->fetch())
        {
            $code = $row['ButtonString'];
            $id = $row['PK_N_Virtual_Device_Button'];
            $deviceId = intval($row['FK_N_Virtual_Device']);

            $ret = $this->convertLua($code);
            
            /* 23.09.2014 - BEGIN - Pawe� Wo�ny */
            echo "\n\n ALL VIRTUAL DEVICES BEGIN \n";
            
            $code = escapeString($ret['code']);
            $status = escapeString($ret['status']);
            
            echo "ALL VIRTUAL DEVICES END \n\n";
            /* 23.09.2014 - END - Pawe� Wo�ny */

            $db->exec('update N_Virtual_Device_Button set ButtonString=? where FK_N_Virtual_Device=? AND PK_N_Virtual_Device_Button=?', array($code, $deviceId, $id));

            $statuses[$deviceId] = $status;
        }
        unset($res);

        /*
         * mainLoop
         */
        $res = $db->query('select * from NEW_Property where `Name`=\'mainLoop\' and Device_Id in (select Id from NEW_Device where Device_Type_Id=' . DEVICE_TYPE_VIRTUAL_DEVICE . ')');
        while ($row = $res->fetch())
        {
            $code = $row['Value'];
            $deviceId = intval($row['Device_Id']);

            $ret = $this->convertLua($code);
            
            /* 23.09.2014 - BEGIN - Pawe� Wo�ny */
            echo "\n\n ALL VIRTUAL DEVICES MAIN LOOP BEGIN \n";
            
            $code = escapeString($ret['code']);
            $status = escapeString($ret['status']);
            
            echo "ALL VIRTUAL DEVICES MAIN LOOP END \n\n";
            /* 23.09.2014 - END - Pawe� Wo�ny */

            $db->exec('update NEW_Property set `Value`=? where `Name`=? and Device_Id=' . $deviceId, array($code, $mainLoop, $deviceId));

            if ($status === false || !array_key_exists($deviceId, $statuses))
            {
                $statuses[$deviceId] = $status;
            }
        }
        unset($res);

        if (count($statuses) > 0)
        {
            foreach ($statuses as $status)
            {
                $this->addToReport($deviceId, 'device', $status);
            }
        }
    }

    public function addToReport($id, $type = 'device', $status = true)
    {
        $report = new Report($id, $type, $status);
        $this->report[] = $report;
    }

}

define('DB_TMP_PATH', DEFAULT_DB_TMP_PATH);


$logsDir='/var'; 

if (isset($_SERVER['PHP_CONVERSION_LOG_DIR']))
{
    $logsDir = trim($_SERVER['PHP_CONVERSION_LOG_DIR']);
}

$logger = KLogger::instance($logsDir, KLogger::DEBUG);

$_RECONFIGURATION = new Reconfiguration();

$newDeviceTypes = new NewDeviceType();
$ccLoader = new CommandClassesLoader();
$deviceHierarchy = new DeviceHierarchy();

$convert = new Convert();
$convert->getNewDevice();

/**
 * converting Satel alarm devices
 */
$db = new DbConnect(DB_TMP_PATH);
$res = $db->query('select d.Id, p.Value '
        . ' from NEW_Device d left join NEW_Property p on d.Id=p.Device_Id and p.Name=\'parentID\''
        . ' where d.Device_Type_Id in (25, 26, 27)');
$sql = '';
while ($row = $res->fetch())
{
    if (!is_null($row['Value']) && is_numeric($row['Value']))
    {
        $parentId = intval($row['Value']);
        $sql .= 'update NEW_Device set Parent_Id=' . $parentId . ' where Id=' . $row['Id'] . ';' . PHP_EOL;
        $sql .= 'update NEW_Property set `Value`=\'true\' where Device_Id=' . $row['Id'] . ' and `Value`=1 and `Name` in (\'armed\', \'alarm\', \'alarmMemory\', \'fireAlarm\', '
                . '\'fireAlarmMemory\', \'entryTime\', \'exitTimeMoreThan10\', \'exitTimeLessThan10\', \'disabled\', \'tamper\', '
                . '\'tamperAlarm\', \'tamperAlarmMemory\', \'noViolationTrouble\', \'longViolationTrouble\', \'value\');' . PHP_EOL;
        $sql .= 'update NEW_Property set `Value`=\'false\' where Device_Id=' . $row['Id'] . ' and `Value`=0 and `Name` in (\'armed\', \'alarm\', \'alarmMemory\', \'fireAlarm\', '
                . '\'fireAlarmMemory\', \'entryTime\', \'exitTimeMoreThan10\', \'exitTimeLessThan10\', \'disabled\', \'tamper\', '
                . '\'tamperAlarm\', \'tamperAlarmMemory\', \'noViolationTrouble\', \'longViolationTrouble\', \'value\');' . PHP_EOL;
    }
}
unset($res);



$sql .= 'update NEW_Device set Visible=0 where Device_Type_Id in (25, 26, 27) and Id in (select Device_Id from NEW_Property where `Name`=\'disabled\' and (`Value`=\'1\' or `Value`=\'true\'));' . PHP_EOL;
$sql .= 'update NEW_Device set Visible=1 where Device_Type_Id in (25, 26, 27) and Id in (select Device_Id from NEW_Property where `Name`=\'disabled\' and (`Value`=\'0\' or `Value`=\'false\'));' . PHP_EOL;

$sQuery = $db->query('select * from NEW_Device where Device_Type_Id in (24, 25, 26, 27)');
while ($s = $sQuery->fetch())
{
    $sql .= 'insert or replace into NEW_Property (Device_Id, `Name`, `Value`) values (' . $s['Id'] . ', \'log\', \'\');' . PHP_EOL;
    $sql .= 'insert or replace into NEW_Property (Device_Id, `Name`, `Value`) values (' . $s['Id'] . ', \'logTemp\', \'\');' . PHP_EOL;
    if ($s['Device_Type_Id'] == 25)
    {
        $sql .= 'insert into NEW_Property (Device_Id, `Name`, `Value`) values (' . $s['Id'] . ', \'deviceIcon\', \'21\');' . PHP_EOL;
        $sql .= 'update NEW_Property set `Name`=\'zoneId\' where `Name`=\'satelID\' and Device_Id=' . $s['Id'] . ';' . PHP_EOL;
    }
    elseif ($s['Device_Type_Id'] == 26)
    {
        $sql .= 'insert or replace into NEW_Property (Device_Id, `Name`, `Value`) values (' . $s['Id'] . ', \'exitTimeLessThan10\', \'false\');' . PHP_EOL;
        $sql .= 'insert or replace into NEW_Property (Device_Id, `Name`, `Value`) values (' . $s['Id'] . ', \'exitTimeMoreThan10\', \'false\');' . PHP_EOL;
        $sql .= 'insert into NEW_Property (Device_Id, `Name`, `Value`) values (' . $s['Id'] . ', \'deviceIcon\', \'28\');' . PHP_EOL;
        $sql .= 'update NEW_Property set `Name`=\'partitionId\' where `Name`=\'satelID\' and Device_Id=' . $s['Id'] . ';' . PHP_EOL;
        $sql .= 'insert into NEW_Property (Device_Id, `Name`, `Value`) values (' . $s['Id'] . ', \'satelId\', \'0\');' . PHP_EOL;
        
    }
    elseif ($s['Device_Type_Id'] == 27)
    {
        $sql .= 'insert into NEW_Property (Device_Id, `Name`, `Value`) values (' . $s['Id'] . ', \'deviceIcon\', \'2\');' . PHP_EOL;
        $sql .= 'update NEW_Property set `Name`=\'outputId\' where `Name`=\'satelID\' and Device_Id=' . $s['Id'] . ';' . PHP_EOL;
    }
}
unset($sQuery);

echo $sql;
$db->beginTransaction();
$db->exec($sql);
$db->commit();

echo "----> ANOTHER PATCH";
$convert->anotherPatch();

$convert->startHCServer();
/**
 * waiting for HCServer
 */
sleep(20);
$httpClient = new HTTPClient();
$hcserverWorking = false;
while (!$hcserverWorking)
{
    $logger->logDebug('waiting for hcserver');
    $hcserverWorking = $httpClient->get('settings/info');
    sleep(1);
}
/**
 * sending soft reconfiguration action before HCServer is being stopped in saveNewIDs() method
 */

$excludedFromReconfiguration = array();
foreach ($convert->devNew as $device)
{
    if ($device->type == 'com.fibaro.FGRGBW441M' && $device->getPropertyValue('deviceControlType') == 52)
    {
        $excludedFromReconfiguration[] = $device->parentId;
    }
}

/**
 * list of devices that should be fully reconfigured
 */
$fullReconfigurationProductInfoList = array(
    "0,134,0,2,0,28" // Home Energy Meter DSB28-ZWEU E2
);

function shouldBeMarkForSoftReconfiguration($device)
{
    global $fullReconfigurationProductInfoList;

    $softReconfiguration = true;

    $productInfo = $device->getPropertyValue("productInfo");

    foreach ($fullReconfigurationProductInfoList as $entry)
    {
        if (substr($productInfo, 0, strlen($entry)) == $entry)
        {
            $softReconfiguration = false;
            break;
        }
    }

    return $softReconfiguration;
}

foreach ($convert->devNew as $device)
{
    if ($device->type == 'com.fibaro.zwaveDevice' && !$device->hasInterface('battery') 
            && !in_array($device->id, $excludedFromReconfiguration))
    {
        $softReconfiguration = shouldBeMarkForSoftReconfiguration($device);
        $_RECONFIGURATION->addDevice($device->id, $softReconfiguration);
    }
}
foreach ($convert->devNew as $device)
{
    if ($device->type == 'com.fibaro.zwaveDevice' && $device->hasInterface('battery') 
            && !in_array($device->id, $excludedFromReconfiguration))
    {
        $softReconfiguration = shouldBeMarkForSoftReconfiguration($device);
        $_RECONFIGURATION->addDevice($device->id, $softReconfiguration);
    }
}

$_RECONFIGURATION->reconfigureDevices();
$_RECONFIGURATION->createSystemMigrationNotification();

$convert->stopHCServer();
