<?php
require_once dirname(__FILE__) . '/Property.php';
require_once dirname(__FILE__) . '/Action.php';
require_once dirname(__FILE__) . '/../internal/Node.php';

class Device
{

    public $id;
    public $name;
    public $roomID;
    public $sectionID;
    public $type;
    public $baseType;
    public $enabled;
    public $visible;
    public $parentId;
    private $interfaces;
    public $internal;
    private $properties;
    private $actions;
    public $created;
    public $modified;
    public $sortOrder;
    public $virtualID;
    public $propertiesOutOfTemplate;
    public $isDrencher;

    function __construct($id)
    {
        $this->id = $id;
        $this->name = "";
        $this->roomID = 0;
        $this->sectionID = 0;
        $this->type = "";
        $this->baseType = "";
        $this->enabled = true;
        $this->visible = true;
        $this->parentId = 0;
        $this->interfaces = array();
        $this->internal = new Node();
        $this->properties = array();

        $this->actions = array();
        $this->created = 0;
        $this->modified = 0;
        $this->sortOrder = 0;

        $this->virtualID = 0;
        $this->propertiesOutOfTemplate = array();
        $this->isDrencher = false;
    }

    public function getActions()
    {
        return $this->actions;
    }

    public function addActions($actions)
    {
        if (is_array($actions))
        {
            foreach ($actions as $action)
            {
                $this->addAction($action);
            }
        }
    }

    public function addAction($action)
    {
        if (get_class($action) == 'Action' && $action->valid)
        {
            if (!array_key_exists($action->getName(), $this->actions))
            {
                $this->actions[$action->getName()] = $action;
            }
        }
    }
    
    public function removeActions($actions)
    {
        if (is_array($actions))
        {
            foreach ($actions as $action)
            {
                $this->removeAction($action->getName());
            }
        }
    }
    
    public function removeAction($actionName)
    {
        if (isset($this->actions[$actionName]))
        {
            unset($this->actions[$actionName]);
        }
    }

    public function addInterfaces($interfaces)
    {
        if (is_array($interfaces))
        {
            foreach ($interfaces as $interface)
            {
                $this->addInterface($interface);
            }
        }
    }

    public function addInterface($interface)
    {
        global $deviceHierarchy;

        if (!in_array($interface, $this->interfaces))
        {
            $this->interfaces[] = $interface;
            $this->addProperties($deviceHierarchy->getInterfaceProperties($interface));
            $this->addActions($deviceHierarchy->getInterfaceActions($interface));
        }
    }

    public function getInterfaces()
    {
        return $this->interfaces;
    }

    public function hasInterface($interface)
    {
        return (in_array($interface, $this->interfaces));
    }
    
    public function removeInterfaces($interfaces)
    {
        if (is_array($interfaces))
        {
            foreach ($interfaces as $interface)
            {
                $this->removeInterface($interface);
            }
        }
    }
    
    public function removeInterface($interface)
    {
        global $deviceHierarchy;

        if (in_array($interface, $this->interfaces))
        {
            $interfaceKeys = array();
            foreach ($this->interfaces as $index => $iName)
            {
                if ($iName == $interface)
                {
                    $interfaceKeys[] = $index;
                }
            }
            if (count($interfaceKeys) > 0)
            {
                foreach ($interfaceKeys as $index)
                unset($this->interfaces[$index]);
            }
            $this->removeProperties($deviceHierarchy->getInterfaceProperties($interface));
            $this->removeActions($deviceHierarchy->getInterfaceActions($interface));
        }
    }

    public function getProperties()
    {
        return $this->properties;
    }

    public function addProperties($properties)
    {
        if (is_array($properties))
        {
            foreach ($properties as $property)
            {
                $this->addProperty($property);
            }
        }
    }
    
    public function addProperty($property)
    {
        if (get_class($property) == 'Property' && $property->valid)
        {
            $this->properties[$property->getName()] = $property;
        }
    }
    
    public function removeProperties($properties)
    {
        if (is_array($properties))
        {
            foreach ($properties as $property)
            {
                $this->removeProperty($property->getName());
            }
        }
    }
    
    public function removeProperty($propertyName)
    {
        if (isset($this->properties[$propertyName]))
        {
            unset($this->properties[$propertyName]);
        }
    }

    public function setPropertyValue($name, $value)
    {
        if (array_key_exists($name, $this->properties) && get_class($this->properties[$name]) == 'Property')
        {
            if (!$this->properties[$name]->setValue($value))
            {
                echo 'Wrong property type. Trying to set value ' . $value . ' for property ' . $name . ' (type: ' . $this->properties[$name]->getType() . ').' . PHP_EOL;
            }
        }
    }

    public function getPropertyValue($name)
    {
        if (array_key_exists($name, $this->properties) && get_class($this->properties[$name]) == 'Property')
        {
            return $this->properties[$name]->getValue();
        }

        return NULL;
    }

    public function hasProperty($name)
    {
        if (array_key_exists($name, $this->properties) && get_class($this->properties[$name]) == 'Property')
        {
            return true;
        }

        return false;
    }

    public function getInternalData(){
        return $this->internal;
    }
    
    public function getInternal()
    {
        $tmp["node"] = $this->internal;
        return $tmp;
    }

    public function serialize()
    {
        $obj = new stdClass();
        $obj->id = $this->id;
        $obj->name = $this->name;
        $obj->roomID = $this->roomID;
        $obj->type = $this->type;
        $obj->baseType = $this->baseType;
        $obj->enabled = $this->enabled;
        $obj->visible = $this->visible;
        $obj->parentId = $this->parentId;
        $obj->interfaces = $this->interfaces;

        if ($this->type == 'com.fibaro.zwaveDevice')
        {
            $obj->internal = $this->getInternal();
        } else
        {
            $obj->internal = new stdClass();
        }

        $obj->created = $this->created;
        $obj->modified = $this->modified;
        $obj->sortOrder = $this->sortOrder;

        $obj->properties = array();
        foreach ($this->properties as $property)
        {
            if (get_class($property) == 'Property')
            {
                $obj->properties[$property->getName()] = $property->getValue();
            }
        }

        $obj->actions = array();
        foreach ($this->actions as $action)
        {
            if (get_class($action) == 'Action')
            {
                $obj->actions[$action->getName()] = $action->getArgc();
            }
        }

        return json_encode($obj);
    }

    public function getForCreate()
    {
        $obj = new stdClass();
        $obj->id = $this->id;
        $obj->name = $this->name;
        $obj->roomID = $this->roomID;
        $obj->type = $this->type;
        $obj->baseType = $this->baseType;
        $obj->enabled = $this->enabled;
        $obj->visible = $this->visible;
        $obj->parentId = $this->parentId;
        $obj->interfaces = $this->interfaces;
        if ($this->type == 'com.fibaro.zwaveDevice')
        {
            $obj->internal = $this->getInternal();
        } else
        {
            $obj->internal = new stdClass();
        }
        $obj->created = $this->created;
        $obj->modified = $this->modified;
        $obj->sortOrder = $this->sortOrder;

        $obj->properties = array();
        foreach ($this->properties as $property)
        {
            if (get_class($property) == 'Property')
            {
            	$str = json_encode($obj->id).' '.json_encode($property->getName()).' '.json_encode($property->getValue()).PHP_EOL.PHP_EOL;
                $obj->properties[$property->getName()] = $property->getValue();
            }
        }

        $obj->actions = array();
        foreach ($this->actions as $action)
        {
            if (get_class($action) == 'Action')
            {
                $obj->actions[$action->getName()] = $action->getArgc();
            }
        }

        return $obj;
    }
    
    public function updateSQL(&$db)
    {
        global $convert, $logger;
        
        $sql = '';
        $sql .= 'UPDATE NEW_Device SET '
                . 'Name=?, '
                . 'Room_Id=?, '
                . 'Section_Id=?, '
                . 'Parent_Id=?';
        $values = array($this->name, $this->roomID, $this->sectionID, $this->parentId);
        
        $deviceTypeId = $convert->getTypeId($this->type, false);
        if (!is_null($deviceTypeId))
        {
            $logger->logDebug('deviceType: ' . $this->type . ', typeId: ' . $deviceTypeId);
            $sql .= ', Device_Type_Id=?';
            $values[] = intval($deviceTypeId);
        }
        
        if ($this->enabled)
        {
            $sql .= ', Enabled=1';
        }
        else
        {
            $sql .= ', Enabled=0';
        }
        
        if ($this->visible)
        {
            $sql .= ', Visible=1';
        }
        else
        {
            $sql .= ', Visible=0';
        }
        
        $sql .= ' WHERE Id=?';
        $values[] = $this->id;
        
        $db->exec($sql, $values);
        
        if (count($this->interfaces) > 0)
        {
            foreach ($this->interfaces as $interfaceName)
            {
                $sql = 'INSERT OR REPLACE INTO NEW_Interface (Device_Id, InterfaceName) VALUES (?, ?)';
                $db->exec($sql, array($this->id, $interfaceName));
            }
        }
        
        if (count($this->propertiesOutOfTemplate) > 0)
        {
            foreach ($this->propertiesOutOfTemplate as $propertyName)
            {
                $sql = 'INSERT OR REPLACE INTO NEW_Property (Device_Id, Name, Value) VALUES (?, ?, ?)';
                $db->exec($sql, array($this->id, $propertyName, ''));
                
            }
        }
        
        if (count($this->properties) > 0)
        {
            
            foreach ($this->properties as $property)
            {
                if ($property->getName() == 'productInfo'){
                    $property->updateProductInfo($this->internal->productInfo);
                }
                $propertyValue = $property->getUpdateSQLValue();
                
                // dla kartonu nie ustawiamy ikony
                
                if (!is_null($propertyValue) && !($property->getName() == "deviceIcon" && $propertyValue == 28))
                {
                    $sql = 'INSERT OR REPLACE INTO NEW_Property (Device_Id, Name, Value) VALUES (?, ?, ?)';
                    $db->exec($sql, array($this->id, $property->getName(), $propertyValue));
                }
            }
        }
    }

}

?>
