<?php

/**
 * Represents Device property.
 *
 * @author Wojciech Wichrowski
 */

require_once dirname(__FILE__) . '/../includes/config.php';
require_once dirname(__FILE__) . '/../includes/tools.php';

class Property
{
    private $name;
    private $type;
    public $valid;
    private $value;
    
    public function __construct($name, $type, $value = NULL)
    {
        $this->name = '';
        $this->type = '';
        $this->value = NULL;
        $this->valid = false;
        
        if (is_string($name) && is_string($type) && $name != '' && $type != '')
        {
            $this->name = $name;
            $this->type = $type;
            $this->valid = true;
            
            if (!is_null($value))
            {
                if (is_string($value))
                {
                    if ($type == 'bool')
                    {
                        $value = xmlStringToBool($value);
                    }
                    else if ($type == 'int' || $type == 'real')
                    {
                        $value = intval($value);
                    }
                }
                
                $this->value = $value;
            }
            else
            {
                switch ($type)
                {
                    case 'int':
                    case 'real':
                        $this->value = 0;
                        break;
                    case 'bool':
                        $this->value = false;
                        break;
                    case 'object':
                        $this->value = new stdClass();
                        break;
                    case 'array':
                        $this->value = array();
                        break;
                    default:
                        $this->value = '';
                        break;
                }
            }
        }
    }
    
    public function setValue($value)
    {
        if (!is_null($value))
        {
            if ($this->type == 'string')
            {
                $this->value = strval($value);
                return true;
            }
            elseif ($this->type == 'int')
            {
                $this->value = intval($value);
                return true;
            }
            elseif ($this->type == 'real')
            {
                $this->value = floatval($value);
                return true;
            }
            elseif ($this->type == 'bool')
            {
                $this->value = (bool)$value;
                return true;
            }
            elseif ($this->type == 'array')
            {
                if (is_array($value))
                {
                    $this->value = $value;
                    return true;
                }
                elseif (is_string($value))
                {
                    $var = json_decode($value);
                    if (is_array($var))
                    {
                        $this->value = $var;
                        return true;
                    }
                    elseif ($value == '')
                    {
                        $this->value = array();
                        return true;
                    }
                }

                return false;
            }
            elseif ($this->type == 'object')
            {
                if (is_object($value))
                {
                    $this->value = $value;
                    return true;
                }
                elseif (is_string($value))
                {
                    $var = json_decode($value);
                    if (is_object($var))
                    {
                        $this->value = $var;
                        return true;
                    }
                    elseif ($value == '')
                    {
                        $this->value = new stdClass();
                        return true;
                    }
                }

                return false;
            }

            $this->value = $value;
            return true;
        }
        
        return false;
    }
    
    public function getName()
    {
        return $this->name;
    }
    
    public function getValue()
    {
        return $this->value;
    }
    
    public function getType()
    {
        return $this->type;
    }
    
    /**
     * 
     * @return \Property - a copy of the property object
     */
    public function get()
    {
        $property = new Property($this->name, $this->type);
        $property->setValue($this->value);
        return $property;
    }
    
    public function updateProductInfo($givenProductInfo){
        $this->value = $givenProductInfo;
    }
    
    public function getUpdateSQLValue()
    {        
        $sqlValue = $this->value;
        
        if ($this->type == 'string')
        {
            $sqlValue = strval($this->value);
        }
        elseif ($this->type == 'int')
        {
            $sqlValue = intval($this->value);
        }
        elseif ($this->type == 'real')
        {
            $sqlValue = floatval($this->value);
        }
        elseif ($this->type == 'bool')
        {
            if ($this->value === false)
            {
                $sqlValue = 'false';
            }
            else
            {
                $sqlValue = 'true';
            }
        }
        else
        {
            return NULL; //don't update properties like parameters/associations directly in db
        }

        return $sqlValue;
    }
}
