<?php

/**
 * Class handling soft reconfiguration of devices
 *
 * @author Wojciech Wichrowski
 */

require_once dirname(__FILE__) . '/../includes/config.php';
require_once dirname(__FILE__) . '/../includes/tools.php';
require_once dirname(__FILE__) . '/../HTTPClient/HTTPClient.php';

class Reconfiguration
{
    public $nodes;
    public $devices;
    
    public function __construct()
    {
        $this->nodes = array();
        $this->devices = array();
    }
    
    public function addNode($nodeId)
    {
        global $logger;
        
        $nodeId = intval($nodeId);
        if ($nodeId > 1)
        {
            if (!in_array($nodeId, $this->nodes))
            {
                $this->nodes[] = $nodeId;
                $logger->logDebug('RECONFIGURATION: adding node ' . $nodeId);
            }
        }
    }
    
    public function addDevice($deviceId, $softReconfiguration)
    {
        global $logger;
        
        $deviceId = intval($deviceId);
        if ($deviceId > 3)
        {
            $reconfigurationTask = new stdClass();
            $reconfigurationTask->deviceId = $deviceId;
            $reconfigurationTask->softReconfiguration = $softReconfiguration;

            $this->devices[] =  $reconfigurationTask;
            $logger->logDebug('RECONFIGURATION: adding device ' . $reconfigurationTask->deviceId);
        }
    }
    
    /**
     * converts nodeIds to deviceIds and adds them to reconfiguration list
     */
    public function convertNodesToDevices()
    {
        global $logger;
        
        if (count($this->nodes) > 0)
        {
            $nodeToDevice = array();
            $sql = 'select p.Device_Id, p.Value from NEW_Property p inner join NEW_Device d on d.Id=p.Device_Id where p.Name="nodeId" and d.Device_Type="com.fibaro.zwaveDevice";';
            $db = new DbConnect(DB_TMP_PATH);
            $res = $db->query($sql);
            while ($row = $res->fetch())
            {
                $nodeToDevice[intval($row['Value'])] = intval($row['Device_Id']);
                if (in_array(intval(intval($row['Value'])), $this->nodes))
                {
                    $this->addDevice($row['Device_Id']);
                }
            }
            unset($res);
            
            
        }
    }
    
    /**
     * sends soft reconfiguration action to HCServer
     */
    public function reconfigureDevices()
    {
        global $logger;
        
        $httpClient = new HTTPClient();

        foreach ($this->devices as $task)
        {
            $logger->logDebug('RECONFIGURATION: sending soft reconfiguration action for device ' . $task->deviceId);
            
            $payload = new stdClass();
            $payload->deviceId = $task->deviceId;
            $payload->softReconfiguration = $task->softReconfiguration;

            $httpClient->post('zwaveReconfigurationTasks', json_encode($payload));
        }
    }

     /**
     * create SystemMigrationNotification in the system
     */
    public function createSystemMigrationNotification()
    {
        global $logger;
        $logger->logDebug('Creating SystemMigrationNotification');

        $httpClient = new HTTPClient();

        $payload = new stdClass();
        $payload->type = 'SystemMigrationNotification';
        $httpClient->post('notificationCenter', json_encode($payload));
    }
}
