<?php

/**
 * HTTP client communicating with HC webservice.
 *
 * @author Wojciech Wichrowski
 */

require_once dirname(__FILE__) . '/../includes/config.php';
require_once dirname(__FILE__) . '/../includes/tools.php';

class HTTPClient
{
    const MAIN_URL = 'http://127.0.0.1:11111/api/';
    
    public function __construct()
    {
        
    }
    
    public function get($path, $uriParams = array())
    {
        return $this->sendRequest(HTTPClient::MAIN_URL . $path, 'GET', $uriParams);
    }
    
    public function post($path, $rawData = '', $uriParams = array())
    {
        return $this->sendRequest(HTTPClient::MAIN_URL . $path, 'POST', $uriParams, $rawData);
    }
    
    public function put($path, $rawData = '', $uriParams = array())
    {
        return $this->sendRequest(HTTPClient::MAIN_URL . $path, 'PUT', $uriParams, $rawData);
    }
    
    public function delete($path, $uriParams = array())
    {
        return $this->sendRequest(HTTPClient::MAIN_URL . $path, 'DELETE', $uriParams);
    }
    
    private function sendRequest($url, $method = 'GET', $uriParams = array(), $rawData = '')
    {
        if (is_array($uriParams) && count($uriParams) > 0)
        {
            $url = $url . '?' . http_build_query($uriParams, '', '&');
        }
        
        $c = curl_init();
        curl_setopt($c, CURLOPT_URL, $url);
        curl_setopt($c, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($c, CURLOPT_CONNECTTIMEOUT, 5);
        
        switch ($method)
        {
            case 'POST':
                curl_setopt($c, CURLOPT_POST, true);
                curl_setopt($c, CURLOPT_POSTFIELDS, $rawData);
                break;
            case 'PUT':
                curl_setopt($c, CURLOPT_CUSTOMREQUEST, 'PUT');
                curl_setopt($c, CURLOPT_POSTFIELDS, $rawData);
                break;
            case 'DELETE':
                curl_setopt($c, CURLOPT_CUSTOMREQUEST, 'DELETE');
                break;
            default:
                break;
        }
        
        $response = curl_exec($c);
        $info = curl_getinfo($c);
        
        curl_close($c);
        
        if ($info['http_code'] != 200)
        {
            return false;
        }
        
        return $response;
    }
}

