<?php

require_once dirname(__FILE__) . '/../includes/config.php';
require_once dirname(__FILE__) . '/../includes/tools.php';
require_once dirname(__FILE__) . '/../Device/Device.php';

class LoadXmlTemplate
{

    public $templateId;
    public $devices;
    public $xml;
    public $valid;
    
    //converting without template
    private $oldNodeId;
    private $oldDevices;

    function __construct()
    {
        $this->templateId = 0;
        $this->devices = array();
        $this->xml = NULL;
        $this->valid = false;
        
        $this->oldNodeId = 0;
        $this->oldDevices = array();
    }

    function loadTemplateFromFileName($fileName)
    {
        if (($this->xml = simplexml_load_file($fileName)) !== false)
        {
            $this->valid = true;
        } else
        {
            echo 'Can\'t load template file ' . $filename . PHP_EOL;
        }
    }

    function loadTemplateFromParametersTemplate($templateId)
    {
        $this->templateId = intval($templateId);

        if ($dh = opendir(ZWAVE_TEMPLATES_PATH))
        {
            while (($filename = readdir($dh)) !== false)
            {
                $ext = pathinfo($filename, PATHINFO_EXTENSION);
                $name = pathinfo($filename, PATHINFO_FILENAME);

                if ($ext == 'xml' && is_file(ZWAVE_TEMPLATES_PATH . '/' . $filename))
                {
                    $temp = explode('-', $name);
                    if ($temp[0] == $templateId)
                    {
                        $path = ZWAVE_TEMPLATES_PATH . '/' . $filename;
                        $this->loadTemplateFromFileName($path);
                    }
                }
            }

            closedir($dh);
        }
    }

    function createDevices($fromTemplate = true, $oldNodeId = 0, $oldDevices = array())
    {
        global $logger;
        
        $this->oldNodeId = $oldNodeId;
        $this->oldDevices = $oldDevices;
        
        if ($this->valid || !$fromTemplate) //TODO: split this class into two separate classes or change the name of the class - if we'll have some free time
        {
            $mainZwaveDevice = $this->createMainDevice();
            if (!is_null($mainZwaveDevice))
            {
                $this->devices[] = $mainZwaveDevice;
                $this->devices = mergeArrayValues($this->devices, $this->createChildren($mainZwaveDevice->getInternal()));
            }
            else
            {
                $logger->logWarn('Main Z-Wave Device is NULL');
            }
        }
    }

    function createMainDevice()
    {
        global $deviceHierarchy;

        $device = new Device(666);
        $device->name = 'Main_Zwave_Device';
        $device->type = 'com.fibaro.zwaveDevice';
        $device->baseType = 'com.fibaro.device';
        $device->visible = false;
        $device->parentId = 1;
        $device->internal = $this->createInternal();
        
        if (is_null($device->internal))
        {
            return NULL;
        }

        $device->addProperties($deviceHierarchy->getProperties($device->type));
        $device->addActions($deviceHierarchy->getActions($device->type));
        $device->addInterfaces($deviceHierarchy->getInterfaces($device->type));

        switch ($device->internal->getClassBasic())
        {
            //BASIC_TYPE_CONTROLLER 0x01 
            case 1:

                $device->addInterface((string) "zwaveControler");
                break;

            // BASIC_TYPE_ROUTING_SLAVE 0x04
            case 4:
                $device->addInterface((string) "zwaveSlaveRouting");
                break;

            //BASIC_TYPE_SLAVE 0x03 
            case 3:
                $device->addInterface((string) "zwaveSlave");
                break;

            //BASIC_TYPE_STATIC_CONTROLLER 0x02 
            case 2:
                $device->addInterface((string) "zwaveStaticControler");
                break;
            default:
                echo "WRONG INTERFACE " . PHP_EOL;
                break;
        }

        $device->addInterface('zwave');
        $device->setPropertyValue('endPointId', 0);

        return $device;
    }
    
    function createInternal()
    {
        global $logger;
        
        $node = new Node();

        if ($this->templateId > 0)
        {
            foreach ($this->xml->endPoint->endPointID[0]->commandClasses->commandClass as $cmdClass)
            {
                if ((string) $cmdClass->attributes()->id == '80')
                { //COMMAND_CLASS_BATTERY
                    if ((string) $cmdClass->attributes()->checkIsExist == 'true')
                    {
                        $node->setCheckBattery(true);
                        $node->setBattery(false);
                    }
                    else
                    {
                        $node->setBattery(true);
                    }
                    break;
                }
            }

            $node->setConfigure(true);


            $node->setClassBasic((string) intval($this->xml->zwaveData->basicClass));
            $node->setDevParentID(1);
            $node->setFlirs(xmlStringToBool((string) $this->xml->zwaveData->flirs));
            $node->setParametersTemplate((int) $this->templateId);

            $manufacturerID = xmlHexToDecArray((string) $this->xml->zwaveData->manufacturerID);
            $productType = xmlHexToDecArray((string) $this->xml->zwaveData->productType);
            $productID = xmlHexToDecArray((string) $this->xml->zwaveData->productID);
            $applicationVersion = xmlHexToDec((string) $this->xml->zwaveData->applicationVersion);
            $applicationSubVersion = xmlHexToDec((string) $this->xml->zwaveData->applicationSubVersion);


            $node->setProductInfo(implode(',', $manufacturerID) . ',' . implode(',', $productType) . ',' . implode(',', $productID) . ',' . $applicationVersion . ',' . $applicationSubVersion);
            $node->setZwaveCompany((string) $this->xml->deviceInfo->company);
            $node->setZwaveVersion($applicationVersion . ',' . $applicationSubVersion);
            
            $endPoints = $this->xml->endPoint->endPointID;
            foreach ($endPoints as $endPoint)
            {
                /**
                 * another fucking rgb patch...
                 */
                $createEndPoint = true;
                if (in_array($this->templateId, array(231, 232, 254, 255, 256, 257, 258, 259)))
                {
                    //create only endPoint 0 - we don't know what types of children are currently in device and can't use template...
                    if (intval($endPoint->attributes()->id) > 0)
                    {
                        $createEndPoint = false;
                    }
                }
                
                if ($createEndPoint)
                {
                    $node->endPoint[] = $this->createEndPoint($endPoint);
                }
            }
            
            /**
             * creating devices missing in template (e.g. multilevel sensors for UBS, RGBW children) from old devices
             */
            if (in_array($this->templateId, array(60, 78, 85, 86, 87, 88, 240, 231, 232, 254, 255, 256, 257, 258, 259)) && count($this->oldDevices) > 0)
            {
                foreach ($this->oldDevices as $oldDevice)
                {
                    $endPointIdToFind = intval($oldDevice->getPropertyName('endPoint'));
                    $endPointFound = false;
                    foreach ($node->endPoint as $endPoint)
                    {
                        if ($endPoint->getEndPointID() == $endPointIdToFind)
                        {
                            $endPointFound = true;
                            break;
                        }
                    }
                    
                    if (!$endPointFound)
                    {
                        $logger->logDebug('Creating additional device for ' . $oldDevice->getID());
                        $newEndPoint = $this->createEndPointFromOldDevice($oldDevice);
                        if (!is_null($newEndPoint))
                        {
                            $node->endPoint[] = $newEndPoint;
                        }
                        else {
                            $logger->logWarn('EndPoint is NULL');
                        }
                    }
                }
            }
            
            /**
             * Removes endpoints created from XML, which don`t have corresponding old device
             */
            
            $newEndpoints = array();
            
            foreach ($node->endPoint as $endPoint){
            	foreach ($this->oldDevices as $oldDevice){
            		if($oldDevice->getPropertyName('endPoint') == $endPoint->getEndPointID()){
            			$newEndpoints[] = $endPoint;
            			break;
            		}
            	}
            }
            $node->endPoint = $newEndpoints;
            
            
        }
        else //without template
        {
            $commandClasses = explode(',', $this->oldDevices[0]->getPropertyName('classSupport'));
            if (in_array(strval(hexdec(COMMAND_CLASS_BATTERY)), $commandClasses))
            {
                $node->setBattery(true);
            }

            $node->setConfigure(true);

            $node->setClassBasic('4');
            $node->setDevParentID(1);
            $node->setFlirs(false);
            $node->setParametersTemplate((int) $this->templateId);
            
            $productInfo = $this->oldDevices[0]->getPropertyName('productInfo');
            $productInfoArray = explode(',', $productInfo);

            $node->setProductInfo($productInfo);
            $node->setZwaveCompany('');
            
            if (count($productInfoArray) > 7)
            {
                $node->setZwaveVersion($productInfoArray[6] . ',' . $productInfoArray[7]);
            }
            else
            {
                $node->setZwaveVersion('0,0');
            }

            if (count($this->oldDevices) > 0)
            {
                foreach ($this->oldDevices as $oldDevice)
                {
                    $newEndPoint = $this->createEndPointFromOldDevice($oldDevice);
                    if (!is_null($newEndPoint))
                    {
                        $node->endPoint[] = $newEndPoint;
                    }
                    else {
                        $logger->logWarn('EndPoint is NULL');
                    }
                }
            }
            
            $logger->logDebug('Internal created from device without template: ', $node);
        }

        return $node;
    }

    function createEndPoint($endPointXMLElement)
    {
        $id = intval($endPointXMLElement->attributes()->id);
        $enabled = xmlStringToBool($endPointXMLElement->attributes()->enabled);
        $visible = xmlStringToBool($endPointXMLElement->attributes()->visible);

        $endPoint = new endPointID($id);
        $endPoint->setConfigure(true);
        $endPoint->setEnabled($enabled);
        $endPoint->setVisible($visible);
        $endPoint->setGeneric((string) $endPointXMLElement->genericClass);
        $endPoint->setSpecific((string) $endPointXMLElement->specificClass);
        $endPoint->setTypeFibaro((string) trim($endPointXMLElement->typeFibaro));
        
        foreach ($endPointXMLElement->interfacesFibaro->interface as $interfaceFibaro){
            $endPoint->addInterfaceFibaro((string) $interfaceFibaro);
        }
        
        $configurationInterfaceFound = false;
        foreach ($endPointXMLElement->interfaces->interface as $interface)
        {
            $endPoint->addInterface((string) $interface, true);
            if ((string) $interface == 'Configuration')
            {
                $configurationInterfaceFound = true;
            }
        }
        /**
         * patch for Configuration interface...
         */
        if ($id == 0 && !$configurationInterfaceFound)
        {
            $endPoint->addInterface('Configuration', true);
        }

        foreach ($endPointXMLElement->commandClasses->commandClass as $commandClass)
        {
            /**
             * not using CCSecurity in conversion
             */
            if ($commandClass->attributes()->cmdClass == 'COMMAND_CLASS_SECURITY')
            {
                continue;
            }
            $ccId = substr(eval("return ".$commandClass->attributes()->cmdClass.";"),2);
            $ccVersion = intval($commandClass->attributes()->version);
            if (is_null($ccVersion) || $ccVersion == 0)
                $ccVersion = 1;

            $nowConfigure = (string) $commandClass->attributes()->nowConfigure;
            if (is_null($nowConfigure))
                $nowConfigure = false;
            else
                $nowConfigure = xmlStringToBool($nowConfigure);

            $controller = (string) $commandClass->attributes()->controlled;
            if (is_null($controller))
                $controller = false;
            else
                $controller = xmlStringToBool($controller);

            $security = (string) $commandClass->attributes()->seciurity;
            if (is_null($security))
                $security = false;
            else
                $security = xmlStringToBool($security);

            $endPoint->addSupportCmd($ccId, $nowConfigure, $controller, $security, $ccVersion);
            
            //*************
            
            foreach ($commandClass->valueHashes->value as $value)
            {
            	$cmdClass = $ccId;
            	$index = intval($value->attributes()->index);
            	$valueType = (string) $value->attributes()->type;
            
            	$valueData = new ValueData();
            	$valueData->setInterfaceFibaro((string) $value->interfaceFibaro);
            	if (isset($value->max))
            	{
            		$valueData->setMax(intval($value->max));
            	}
            	else
            	{
            		if ($valueType == 'real' || $valueType == 'int')
            		{
            			$valueData->setMax(2147483647);
            		}
            		else {
            			$valueData->setMax(255);
            		}
            
            	}
            	if (isset($value->min))
            	{
            		$valueData->setMin(intval($value->min));
            	}
            	else
            	{
            		if ($valueType == 'real' || $valueType == 'int')
            		{
            			$valueData->setMin(-2147483648);
            		}
            		else {
            			$valueData->setMin(0);
            		}
            	}
            	$valueData->setNameValue((string) $value->name);
            
            	if (isset($value->pollingInterval))
            	{
            		$valueData->setPollingInterval(intval($value->pollingInterval));
            		if (intval($value->pollingInterval) > 0)
            			$valueData->setPollingEnable(true);
            		else
            			$valueData->setPollingEnable(false);
            	}
            	else
            	{
            		$valueData->setPollingInterval(0);
            		$valueData->setPollingEnable(false);
            	}
            	if (isset($value->pollingWakeUpDeviceHowOffen))
            	{
            		$valueData->setPollingWakeUpDeviceHowOffen(intval($value->pollingWakeUpDeviceHowOffen));
            	}
            	else
            	{
            		$valueData->setPollingWakeUpDeviceHowOffen(0);
            	}
            
            	$valueData->setTypeFibaro((string) $value->typeFibaro);
            	if (isset($value->unit))
            	{
            		$valueData->setUnit((string) $value->unit);
            	}
            	else
            	{
            		$valueData->setUnit('');
            	}
            	$valueData->setVirtual(intval($value->virtual));
            
            	$endPoint->addValue($cmdClass, $index, $valueType, $valueData);
            }
            
            //*************
            
        }

        /*
         * hardcoding Command Class Configuration - not present in template...
         */
        if ($id == 0) //endPointId
        {
            $cmdClass = '70';
            $index = 1;
            $valueType = 'configuration';

            $valueData = new ValueData();
            $valueData->setInterfaceFibaro('');
            $valueData->setMax(255);
            $valueData->setMin(0);
            $valueData->setNameValue('parameters');

            $valueData->setPollingInterval(0);
            $valueData->setPollingEnable(false);
            $valueData->setPollingWakeUpDeviceHowOffen(0);
            $valueData->setTypeFibaro('');
            $valueData->setUnit('');
            $valueData->setValue(array());
            $valueData->setVirtual(0);

            $endPoint->addValue($cmdClass, $index, $valueType, $valueData);
        }

        return $endPoint;
    }
    
    function createEndPointFromOldDevice($oldDevice)
    {
        global $ccLoader, $newDeviceTypes, $logger;
        
        $id = intval($oldDevice->getPropertyName('endPoint'));
        if ($oldDevice->getPropertyName('disabled') == '1')
        {
            $enabled = false;
            $visible = false;
        }
        else
        {
            $enabled = true;
            $visible = true;
        }

        $endPoint = new endPointID($id);
        $endPoint->setConfigure(true);
        $endPoint->setEnabled($enabled);
        $endPoint->setVisible($visible);
        
        $commandClasses = array();
        if ($oldDevice->getPropertyName('classSupport') == '')
        {
            if ($oldDevice->getPropertyName('classGeneric') == '')
            {
                $logger->logDebug('EndPoint ' . $id . ': classGeneric is NULL');
                return NULL;
            }
            else
            {
                $commandClasses[] = $oldDevice->getPropertyName('classGeneric');
            }
        }
        else
        {
            $commandClasses = explode(',', $oldDevice->getPropertyName('classSupport'));
        }
        
        $endPoint->setGeneric($oldDevice->getPropertyName('classGeneric'));
        $endPoint->setSpecific('1'); //TODO: to check in QA testing
        
        $newDeviceType = $newDeviceTypes->get($oldDevice->device['Device_Type']);
        if (!is_null($newDeviceType))
        {
            $endPoint->setTypeFibaro($newDeviceType);
        }
        else
        {
            $logger->logDebug('Empty type. Setting type to com.fibaro.zwaveDevice.');
            $endPoint->setTypeFibaro('com.fibaro.zwaveDevice');
        }
        

        if (!in_array(hexdec(COMMAND_CLASS_BASIC), $commandClasses))
        {
            $commandClasses[] = hexdec(COMMAND_CLASS_BASIC);
        }
        
        foreach ($commandClasses as $ccId)
        {
            if ($ccLoader->isSupported($ccId, 'dec'))
            {
                $cmdClass = $ccLoader->get($ccId, 'dec');
                
                $interfacesArray = $ccLoader->getInterfaces($ccId, 'dec');
                if (count($interfacesArray) > 0)
                {
                    foreach ($interfacesArray as $interface)
                    {
                        $endPoint->addInterface($interface, true);
                    }
                }
                
                $endPoint->supportCmd[] = $cmdClass->getForInternal();
                
                if (count($cmdClass->values) > 0)
                {
                    foreach ($cmdClass->values as $newValue)
                    {
                        $endPoint->value[] = $newValue;
                    }
                }
            }
        }

        /*
         * hardcoding Command Class Configuration - not present in template...
         */
        if ($id == 0) //endPointId
        {
            $cmdClass = '70';
            $index = 1;
            $valueType = 'configuration';

            $valueData = new ValueData();
            $valueData->setInterfaceFibaro('');
            $valueData->setMax(255);
            $valueData->setMin(0);
            $valueData->setNameValue('parameters');

            $valueData->setPollingInterval(0);
            $valueData->setPollingEnable(false);
            $valueData->setPollingWakeUpDeviceHowOffen(0);
            $valueData->setTypeFibaro('');
            $valueData->setUnit('');
            $valueData->setValue(array());
            $valueData->setVirtual(0);

            $endPoint->addValue($cmdClass, $index, $valueType, $valueData);
        }

        return $endPoint;
    }

    function createChildren($internal)
    {
        global $deviceHierarchy, $logger;
        
        $nodeId = $internal['node']->nodeID;
        $devices = array();
        foreach ($internal['node']->endPoint as $endPoint)
        {
            $endPointDevice = false;
            $endPointId = $endPoint->getEndPointID();
            if ($endPoint->getTypeFibaro() != '')
            {
                $endPointDevice = true;
                $device = new Device(666);
                $device->type = $endPoint->getTypeFibaro();

                if ($device->type != '')
                {
                    $device->baseType = $deviceHierarchy->getBaseType($device->type);
                }
                $device->visible = $endPoint->getVisible();
                $device->enabled = $endPoint->getEnabled();

                $device->addProperties($deviceHierarchy->getProperties($device->type));
                $device->addActions($deviceHierarchy->getActions($device->type));
                $device->addInterfaces($deviceHierarchy->getInterfaces($device->type));
                $device->addInterface('zwave');

                $device->setPropertyValue('nodeId', $nodeId);
                $device->setPropertyValue('endPointId', $endPointId);

                $devices[$endPointId . '_0'] = $device;
            }
            
            foreach ($endPoint->interfaceFibaro as $interface)
            {
                if ($endPointDevice)
                {
                    $device->addInterface($interface);
                }
                //hardcode - copy battery and wakeup to zwaveDevice
                if ($interface == 'battery' || $interface == 'zwaveWakeup')
                {
                    $this->devices[0]->addInterface($interface);
                }
            }

            //interfaces from all virtuals
            foreach ($endPoint->getValue() as $value)
            {
                $valueData = $value->getData();
                $interface = $valueData->getInterfaceFibaro();
                if ($interface != '')
                {
                    if ($endPointDevice)
                    {
                        $device->addInterface($interface);
                    }
                    //hardcode - copy battery and wakeup to zwaveDevice
                    if ($interface == 'battery' || $interface == 'zwaveWakeup')
                    {
                        $this->devices[0]->addInterface($interface);
                    }
                }
            }

            if (count($endPoint->getValue()) > 0)
            {
                foreach ($endPoint->getValue() as $value)
                {
                    $valueData = $value->getData();
                    if (($valueData->getVirtual() > 0 || $endPoint->getTypeFibaro() == '') && $valueData->getTypeFibaro() != '')
                    {
                        if (!array_key_exists($endPointId . '_' . $valueData->getVirtual(), $devices))
                        {
                            
                            $device = new Device(666);
                            $device->type = $valueData->getTypeFibaro();
                            $device->baseType = $deviceHierarchy->getBaseType($device->type);
                            /**
                             * hack for hiding seismometer in motion sensor
                             */
                            if ($device->type === 'com.fibaro.seismometer'){
                            	$device->visible = false;
                            } else {
                            	$device->visible = true;
                            }
                            $device->enabled = true;

                            $device->addProperties($deviceHierarchy->getProperties($device->type));
                            $device->addActions($deviceHierarchy->getActions($device->type));
                            $device->addInterfaces($deviceHierarchy->getInterfaces($device->type));
                            $device->addInterface('zwave');
                            $device->setPropertyValue('endPointId', $endPoint->getEndPointID());

                            if ($valueData->getInterfaceFibaro() != '')
                            {
                                $device->addInterface($valueData->getInterfaceFibaro());
                            }

                            $device->virtualID = $valueData->getVirtual();
                            
                            $device->setPropertyValue('nodeId', $nodeId);
                            $device->setPropertyValue('endPointId', $endPointId);

                            $devices[$endPointId . '_' . $valueData->getVirtual()] = $device;
                        }
                    }
                }
            }
        }
        
        return $devices;
    }

    function loadZwaveData()
    {
        print "Flirs: " . $this->xml->zwaveData->flirs . PHP_EOL;
    }

}
?>
