<?php

/**
 * Description of CommandClassesLoader
 *
 * @author Wojciech Wichrowski
 */

require_once dirname(__FILE__) . '/../includes/config.php';
require_once dirname(__FILE__) . '/../includes/tools.php';
require_once dirname(__FILE__) . '/CommandClass.php';
require_once dirname(__FILE__) . '/../internal/Value.php';
require_once dirname(__FILE__) . '/../internal/ValueData.php';
require_once dirname(__FILE__) . '/../includes/defineCmdClasses.php';

//hardcoding - these values are not in any xml - need to be copied from Zwave project every time...........
define('INTERFACE_MULTI_SWITCH', "Dimmer");
define('INTERFACE_BINARY_SWITCH', "Switch");
define('INTERFACE_ASSOCIATION', "Association");
define('INTERFACE_BASIC', "Basic");
define('INTERFACE_RGBW', "RGBW");
define('INTERFACE_ZWAVE_CONTROLLER', "ZwaveController");
define('INTERFACE_METER', "Meter");
define('INTERFACE_MULTILEVEL_SENSOR', "MultilevelSensor");
define('INTERFACE_BINARY_SENSOR', "BinarySensor");
define('INTERFACE_CONFIGURATION', "Configuration");
define('INTERFACE_BASIC_WINDOW_COVERING', "BasicWindowCovering");
define('INTERFACE_PROTECTION', "Protection");
define('INTERFACE_THERMOSTAT_MODE', "ThermostatMode");
define('INTERFACE_SCENE_ACTIVATION', "SceneActivation");
define('INTERFACE_DOOR_LOCK', "DoorLock");
define('INTERFACE_USER_CODE', "UserCode");
define('INTERFACE_FIBARO_SMOKE_SENSOR', "FibaroSmokeSensor");
define('INTERFACE_THERMOSTAT_FAN_MODE', "ThermostatFanMode");
define('INTERFACE_THERMOSTAT_SETBACK', "ThermostatSetback");
define('INTERFACE_THERMOSTAT_SETPOINT', "ThermostatSetpoint");
define('INTERFACE_FIRMWARE_UPDATE', "FirmwareUpdate");
define('INTERFACE_FIBARO_ROLLER_SHUTTER', "FibaroRollerShutter");
define('INTERFACE_BATTERY', "Battery");
define('INTERFACE_POLLING_METER', "PollingMeter");
define('INTERFACE_ZWAVE_SLAVE', "ZwaveSlave");
define('INTERFACE_ZWAVE_ROUTING_SLAVE', "ZwaveRoutingSlave");

class CommandClassesLoader
{
    //hardcoding - these values are not in any xml - need to be copied from Zwave project every time...........
    public $interfaces;
    
    public $supportedCommandClasses; //keys in hex like '0x20'
    
    public function __construct()
    {
        $this->supportedCommandClasses = array();
        $this->interfaces = array(
            COMMAND_CLASS_ASSOCIATION           => array(INTERFACE_ASSOCIATION),
            COMMAND_CLASS_BASIC                 => array(INTERFACE_BASIC),
            COMMAND_CLASS_BASIC_WINDOW_COVERING => array(INTERFACE_BASIC_WINDOW_COVERING),
            COMMAND_CLASS_COLOR_CONTROL         => array(INTERFACE_RGBW),
            COMMAND_CLASS_CONFIGURATION         => array(INTERFACE_CONFIGURATION),
            COMMAND_CLASS_DOOR_LOCK             => array(INTERFACE_DOOR_LOCK),
            COMMAND_CLASS_FIRMWARE_UPDATE_MD    => array(INTERFACE_FIRMWARE_UPDATE),
            COMMAND_CLASS_SCENE_ACTIVATION      => array(INTERFACE_BINARY_SWITCH),
            COMMAND_CLASS_SENSOR_BINARY         => array(INTERFACE_BINARY_SENSOR),
            COMMAND_CLASS_SENSOR_MULTILEVEL     => array(INTERFACE_MULTILEVEL_SENSOR),
            COMMAND_CLASS_SWITCH_BINARY         => array(INTERFACE_BINARY_SWITCH),
            COMMAND_CLASS_THERMOSTAT_FAN_MODE   => array(INTERFACE_THERMOSTAT_FAN_MODE),
            COMMAND_CLASS_THERMOSTAT_MODE       => array(INTERFACE_THERMOSTAT_MODE),
            COMMAND_CLASS_THERMOSTAT_SETBACK    => array(INTERFACE_THERMOSTAT_SETBACK),
            COMMAND_CLASS_THERMOSTAT_SETPOINT   => array(INTERFACE_THERMOSTAT_SETPOINT),
            COMMAND_CLASS_USER_CODE             => array(INTERFACE_USER_CODE),
            COMMAND_CLASS_WAKE_UP               => array(INTERFACE_BATTERY),
            COMMAND_CLASS_SWITCH_MULTILEVEL     => array(INTERFACE_MULTI_SWITCH),
            COMMAND_CLASS_SCENE_ACTIVATION      => array(INTERFACE_SCENE_ACTIVATION),
            COMMAND_CLASS_METER                 => array(INTERFACE_METER, INTERFACE_POLLING_METER),
            COMMAND_CLASS_PROTECTION            => array(INTERFACE_PROTECTION)
        );
        
        $this->loadSupportedCommandClasses();
        $this->loadValuesFromTemplates();
    }
    
    private function loadSupportedCommandClasses()
    {
        if (($xml = simplexml_load_file(ZWAVE_SUPPORTED_CMD_CLASSES_PATH)) !== false)
        {
            foreach ($xml->cmd_class as $cmdClass)
            {
                /**
                 * not using CCSecurity in conversion
                 */
                if ((string)$cmdClass->attributes()->key == '0x98')
                {
                    continue;
                }
                
                if ($cmdClass->attributes()->support == 'true')
                {
                    $id = (string)$cmdClass->attributes()->key;
                    $id = substr($id, 2); //remove '0x'
                    
                    $key = $this->prepareIndexKey($id);
                    $version = (string)$cmdClass->attributes()->version;
                    $name = (string)$cmdClass->attributes()->name;
                    $cmdClass = new CommandClass($id, $version, $name);
                    
                    if (array_key_exists($key, $this->interfaces))
                    {
                        foreach ($this->interfaces[$key] as $interface)
                        {
                            $cmdClass->interfaces[] = $interface;
                        }
                    }
                    
                    $this->supportedCommandClasses[$key] = $cmdClass;
                }
            }
        }
    }
    
    private function loadValuesFromTemplates()
    {
        if ($dh = opendir(ZWAVE_TEMPLATES_PATH))
        {
            while (($filename = readdir($dh)) !== false)
            {
                $ext = pathinfo($filename, PATHINFO_EXTENSION);
                $name = pathinfo($filename, PATHINFO_FILENAME);

                if ($ext == 'xml' && is_file(ZWAVE_TEMPLATES_PATH . '/' . $filename))
                {
                    $path = ZWAVE_TEMPLATES_PATH . '/' . $filename;
                    $this->loadValuesFromTemplate($path);
                }
            }

            closedir($dh);
        }
    }
    
    private function loadValuesFromTemplate($path)
    {
        if (($xml = simplexml_load_file($path)) !== false)
        {
            foreach ($xml->endPoint->endPointID as $endPoint)
            {
                /*
                 * COMMAND CLASSES
                 */
                foreach ($endPoint->commandClasses->commandClass as $commandClass)
                {
                    $cmdClass = substr(eval("return ".$commandClass->attributes()->cmdClass.";"),2);
                    
                    if (!is_null($this->find($cmdClass)))
                    {
                        $controller = (string) $commandClass->attributes()->controlled;
                        if (is_null($controller))
                            $controller = false;
                        else
                            $controller = xmlStringToBool($controller);

                        $security = (string) $commandClass->attributes()->seciurity;
                        if (is_null($security))
                            $security = false;
                        else
                            $security = xmlStringToBool($security);

                        $this->find($cmdClass)->controller = $controller;
                        $this->find($cmdClass)->security = $security;
                    }
                    
                    foreach ($commandClass->valueHashes->value as $value)
                    {
                    	$index = intval($value->attributes()->index);
                    
                    	if (!is_null($this->find($cmdClass)) && !$this->find($cmdClass)->hasValue($index))
                    	{
                    		$valueType = (string) $value->attributes()->type;
                    
                    		$valueData = new ValueData();
                    		$interfaceName = (string) $value->interfaceFibaro;
                    		/**
                    		 * exception: CC Sensor Multilevel doesn't have to have power
                    		 */
                    		if ($this->prepareIndexKey($cmdClass) == '0x31' && $interfaceName == 'power'){
                    			$valueData->setInterfaceFibaro("");
                    		}
                    		else
                    		{
                    			$valueData->setInterfaceFibaro($interfaceName);
                    		}
                    		if (isset($value->max))
                    		{
                    			$valueData->setMax(intval($value->max));
                    		}
                    		else
                    		{
                    			if ($valueType == 'real' || $valueType == 'int')
                    			{
                    				$valueData->setMax(2147483647);
                    			}
                    			else {
                    				$valueData->setMax(255);
                    			}
                    
                    		}
                    		if (isset($value->min))
                    		{
                    			$valueData->setMin(intval($value->min));
                    		}
                    		else
                    		{
                    			if ($valueType == 'real' || $valueType == 'int')
                    			{
                    				$valueData->setMin(-2147483648);
                    			}
                    			else {
                    				$valueData->setMin(0);
                    			}
                    		}
                    		$valueData->setNameValue((string) $value->name);
                    
                    		if (isset($value->pollingInterval))
                    		{
                    			$valueData->setPollingInterval(intval($value->pollingInterval));
                    			if (intval($value->pollingInterval) > 0)
                    				$valueData->setPollingEnable(true);
                    			else
                    				$valueData->setPollingEnable(false);
                    		}
                    		else
                    		{
                    			$valueData->setPollingInterval(0);
                    			$valueData->setPollingEnable(false);
                    		}
                    		if (isset($value->pollingWakeUpDeviceHowOffen))
                    		{
                    			$valueData->setPollingWakeUpDeviceHowOffen(intval($value->pollingWakeUpDeviceHowOffen));
                    		}
                    		else
                    		{
                    			$valueData->setPollingWakeUpDeviceHowOffen(0);
                    		}
                    
                    		$valueData->setTypeFibaro((string) $value->typeFibaro);
                    		if (isset($value->unit))
                    		{
                    			$valueData->setUnit((string) $value->unit);
                    		}
                    		else
                    		{
                    			$valueData->setUnit('');
                    		}
                    
                    		$valueData->setVirtual(0);
                    
                    		$newValue = new Value($cmdClass, $index, $valueType, $valueData);
                    
                    		/**
                    		 * fix wrong CC loading from templates (not every value needed) - put exceptions here
                    		*/
                    		$addValue = true;
                    		if ($this->prepareIndexKey($cmdClass) == '0x31' && $valueData->getNameValue() == 'power')
                    		{
                    			//CC Sensor Multilevel doesn't have to have power
                    			$addValue = false;
                    		}
                    
                    		if ($addValue)
                    		{
                    			$this->find($cmdClass)->values[$index] = $newValue;
                    		}
                    	}
                    }
                }
                
                /*
                 * VALUES
                 */

            }
        }
    }
    
    private function prepareIndexKey($key, $keyType = 'hex')
    {
        if ($keyType != 'hex')
        {
            $key = str_pad(dechex(intval($key)), 2, '0', STR_PAD_LEFT);
        }
        else
        {
            $key = strval($key);
        }
        
        if (strlen($key) < 4)
        {
            $key = '0x' . $key;
        }
        
        return $key;
    }
    
    public function isSupported($cmdClassId, $keyType = 'hex')
    {
        $cmdClassId = $this->prepareIndexKey($cmdClassId, $keyType);

        return isset($this->supportedCommandClasses[$cmdClassId]);
    }
    
    /**
     * 
     * @param type $cmdClassId
     * @param type $keyType
     * @return copy of command class object
     */
    public function get($cmdClassId, $keyType = 'hex')
    {
        $cmdClassId = $this->prepareIndexKey($cmdClassId, $keyType);
        
        if (isset($this->supportedCommandClasses[$cmdClassId]))
        {
            $cmdClass = $this->supportedCommandClasses[$cmdClassId];
            return $cmdClass->get();
        }
        
        return NULL;
    }
    
    /**
     * 
     * @param type $cmdClassId
     * @param type $keyType
     * @return command class object
     */
    public function find($cmdClassId, $keyType = 'hex')
    {
        $cmdClassId = $this->prepareIndexKey($cmdClassId, $keyType);
        
        if (isset($this->supportedCommandClasses[$cmdClassId]))
        {
            $cmdClass = $this->supportedCommandClasses[$cmdClassId];
            return $cmdClass;
        }
        
        return NULL;
    }
    
    /**
     * 
     * @param type $cmdClassId
     * @param type $keyType
     * @return interfaces array for given command class
     */
    public function getInterfaces($cmdClassId, $keyType = 'hex')
    {
        $cmdClassId = $this->prepareIndexKey($cmdClassId, $keyType);
        
        if (isset($this->interfaces[$cmdClassId]))
        {
            return $this->interfaces[$cmdClassId];
        }
        
        return array();
    }
}
