<?php

require_once dirname(__FILE__) . '/../Device/Device.php';
require_once dirname(__FILE__) . '/../includes/tools.php';

//TODO socket path stored in some config file?
define('JSON_RPC_CLIENT_SOCKET', "ipc:///tmp/fibaro/HCServer");
define('JSON_RPC_SOCKET_TIMEOUT', 5000);
define('JSON_RPC_SOCKET_LINGER', 0);

class JsonRPCResponse
{

    function __construct($status = 0, $response = "")
    {
        $this->status = $status;
        $this->response = $response;
    }

    public $status;
    public $response;

}

class JsonRPCClient
{

    //sets connection with RPC server
    function __construct($socketPath = JSON_RPC_CLIENT_SOCKET)
    {
        $context = new ZMQContext(1);
        $socket = new ZMQSocket($context, ZMQ::SOCKET_REQ);

        //set timeout 
        $socket->setSockOpt(ZMQ::SOCKOPT_SNDTIMEO, JSON_RPC_SOCKET_TIMEOUT);
        $socket->setSockOpt(ZMQ::SOCKOPT_RCVTIMEO, JSON_RPC_SOCKET_TIMEOUT);
        $socket->setSockOpt(ZMQ::SOCKOPT_LINGER, JSON_RPC_SOCKET_LINGER);

        $socket->connect($socketPath);

        $this->socket = $socket;
        $this->id = 0;
    }
    
    private function jsonEncodeNoEscaping($requestObject)
    {
        $str = json_encode($requestObject);
        $str = preg_replace_callback('/\\\\u([0-9a-f]{4})/i', 'replace_unicode_escape_sequence', $str);
        
        return $str;
    }
    
    public function call($method, $params)
    {
        global $logger;
        
        $callResult = new JsonRPCResponse();

        //$requestObject = array();
        $requestObject["id"] = $this->getRequestId();
        $requestObject["method"] = $method;
    
        if ($method == 'createDevice'){
        	
        	// dla kartonu nie ustawiamy ikony
        	
        	if ($params->initialProperties['deviceIcon'] == 28 || $params->initialProperties['deviceIcon'] == '28'){

        		unset($params->initialProperties['deviceIcon']);
        		
        	}
        }
        
        $requestObject["params"] = $params;
        
        //$logger->logDebug('JSON-RPC request:' . json_encode($requestObject));
        
        if ($this->socket->send($this->jsonEncodeNoEscaping($requestObject)) === false)
        {
            $callResult->status = 1;
            return $callResult;
        }

        $response = $this->socket->recv();
        //$logger->logDebug('JSON-RPC response:' . json_encode($response));

        //decode response string
        $response = json_decode($response);

        if (property_exists($response, "result"))
        {
            $callResult->response = $response->result;
        } else
        {
            $callResult->status = 3;
            return $callResult;
        }

        return $callResult;
    }

    public function insertInternal($id, $internal)
    {
        $params = new stdClass();

        $params->deviceId = $id;
        $params->path = "";
        $params->method = "insert";
        $params->data = $internal;

        $this->call("updateInternal", $params);
    }

    public function createDevice(&$device)
    {
        global $convert;
        
        $method = "createDevice";
        $params = new stdClass();

        $obj = $device->getForCreate();

        $params->name = $obj->name;
        $params->type = $obj->type;
        $params->parentId = $obj->parentId;
        $params->visible = $obj->visible;
        $params->interfaces = $obj->interfaces;
        $params->initialProperties = $obj->properties;

        $callResponse = $this->call($method, $params);
        if ($callResponse->status == 0)
        {
            $newDevice = $callResponse->response;
            $device->id = $newDevice->id;
        }
        
        $this->insertInternal($device->id, $obj->internal);

        return $callResponse->status;
    }

    //creates a new device with given type
    public function callCreateDevice($name, $type, $parentId, $visible, $interfaces, $initialProperties)
    {
        $method = "createDevice";
        $params = new stdClass();

        $params->name = $name;
        $params->type = $type;
        $params->parentId = $parentId;
        $params->visible = $visible;
        $params->interfaces = $interfaces;
        $params->initialProperties = $initialProperties;

        return $this->call($method, $params);
    }

    private function getRequestId()
    {
        return $this->id++;
    }

}

?>
