require('common.device')
require('net.HTTPClient')
require('common.base64')
require('common.utils')


class 'Fe200Vd' (Device)


function Fe200Vd:__init(id)
	Device.__init(self, id)

	self.http = net.HTTPClient({ timeout = 10000 })

	self.url = {
	["left"] = self.properties.moveLeftPath,
	["right"] = self.properties.moveRightPath,
	["up"] = self.properties.moveUpPath,
	["down"] = self.properties.moveDownPath,
	["stop"] = self.properties.moveStopPath,
	["jpg"] = self.properties.jpgPath,
	["mjpg"] = self.properties.mjpgPath
	}

	if(self.properties.ip == "") then
		self:updateProperty('ui.imageStream.source', 'novideo.png')
	else
		self.base64 = false
		if(self.properties.userName == "" and self.properties.password == "") then
			self:updateProperty('ui.imageStream.source', 'http://' .. self.properties.ip .. "/" .. self.url.mjpg)

			self.headers = {}
		else
			self.base64 = true
			self:updateProperty('ui.imageStream.source', 'http://' .. self.properties.username .. ":" .. utils.urlencode(self.properties.password) .. "@" .. self.properties.ip .. "/" .. self.url.mjpg)

			self.headers = {
			['Authorization'] = 'Basic '.. to_base64(self.properties.username .. ':'.. self.properties.password),
			['Content-Type'] = 'application/x-www-form-urlencoded'
			}
		end
	end
end

function Fe200Vd:moveLeft()
	self:httpRequest(self.url.left)
end

function Fe200Vd:moveRight()
	self:httpRequest(self.url.right)
end

function Fe200Vd:moveUp()
	self:httpRequest(self.url.up)
end

function Fe200Vd:moveDown()
	self:httpRequest(self.url.down)
end

function Fe200Vd:moveStop()
	self:httpRequest(self.url.stop)
end

function Fe200Vd:httpRequest(url)
		self.http:request('http://' .. self.properties.ip .. '/' .. url, {
		options = {
			method = 'GET',
			headers = self.headers
		},
		success = function() end,
		error = function(err) print(err) end
	})
end
