--! File/Class description: This is a starting point, required for of each of the plug-ins. It analyzes and executes the plugin’s code, searching the main classes and methods.

--! includes file/class WeatherProvider.lua - It’s another necessary file, required for correct plugin’s operation. The main task of this file is to create and manage a new device, representing the plugin in our system. In other words, in this file we store the most of properties and plugin’s events. Despite the fact that Lua is a scripting language, experienced programmers will recognize some classes analogy, commonly used in object-oriented programming.
require('WeatherProvider')
 
--! Creates handler for WeatherProvider (located in WeatherProvider.lua file) class.
weatherProvider = WeatherProvider(plugin.mainDeviceId)
 
--! This function is usually used for handling an event assigned to the elements (button, select, etc.) at "Advanced" tab. However this function may be called via any http client by constructing a request as described here. Users of our system may also call such function for example by scenes. Let us assume that we have remote lights plugin, which is able to switch off light. Plugin contains a method: switchOff(LightID). Scene developer may call function switchOff(LightID) by sending the POST request: POST api/devices/DEVICE_ID/action/ACTION_NAME {"args":["arg1", ..., "argN"]}
function onAction(deviceId, action)
    return weatherProvider:callAction(action.actionName, unpack(action.args))
end

--! Function that is usually used for handling an event assigned to one of the handlers (button, select, switch, etc.) at "General" tab.
function onUIEvent(deviceId, event)

end
 
--! This function is used for handling "Save" event performed by the user at plugin's "General" tab.
function configure(deviceId, config)
    weatherProvider.settingsInfo = api.get('/settings/info')
    weatherProvider.settingsLocation = api.get('/settings/location')
    if weatherProvider.properties.selectedTemperatureSource ~= config.selectedTemperatureSource then
        if config.selectedTemperatureSource ~= -1 then
            weatherProvider:updateProperty('selectedTemperatureSource', config.selectedTemperatureSource)
            local newTempDevice = api.get('/devices/' .. weatherProvider.properties.selectedTemperatureSource)
            if newTempDevice.properties.value ~= nil then
                weatherProvider:updateTemperatureProperty(newTempDevice.properties.value)
            end
            if newTempDevice.properties.Temperature ~= nil then
                weatherProvider:updateTemperatureProperty(newTempDevice.properties.Temperature)
            end
        else
            weatherProvider:updateProperty('selectedTemperatureSource', -1)
            weatherProvider:updateTemperatureProperty(0)  
        end
    end

    if weatherProvider.properties.selectedHumiditySource ~= config.selectedHumiditySource then
        if config.selectedHumiditySource ~= -1 then
            weatherProvider:updateProperty('selectedHumiditySource', config.selectedHumiditySource)
            local newHumidityDevice = api.get('/devices/' .. weatherProvider.properties.selectedHumiditySource)
            if newHumidityDevice.properties.value ~= nil then
                weatherProvider:updateProperty('Humidity', tonumber(newHumidityDevice.properties.value))
            end
            if newHumidityDevice.properties.Humidity ~= nil then
                weatherProvider:updateProperty('Humidity', tonumber(newHumidityDevice.properties.Humidity))
            end
        else
                weatherProvider:updateProperty('selectedHumiditySource', -1)
                weatherProvider:updateProperty('Humidity', "")
        end
    end
    
    if weatherProvider.properties.selectedWindSpeedSource ~= config.selectedWindSpeedSource then
        if config.selectedWindSpeedSource ~= -1 then
            weatherProvider:updateProperty('selectedWindSpeedSource', config.selectedWindSpeedSource)
            local newWindDevice = api.get('/devices/' .. weatherProvider.properties.selectedWindSpeedSource)
            if newWindDevice.properties.value ~= nil then
                weatherProvider:updateWindProperty(newWindDevice.properties.value)
            end
            if newWindDevice.properties.Wind ~= nil then
                weatherProvider:updateWindProperty(newWindDevice.properties.Wind)
            end
        else
            weatherProvider:updateProperty('selectedWindSpeedSource', -1)
            weatherProvider:updateWindProperty(0)
        end
    end

    if config.hidden == true then
        weatherProvider:setVisible(false)
    end
    if config.hidden == false then
        weatherProvider:setVisible(true)
    end
    
    weatherProvider:setCurentWeatherLabel()
end