require('common.device')
require('carrierZone')
require('carrierConfig')
require('carrierCommandQueue')
require('common/TCPListener')


class 'Carrier' (Device)

Carrier.COMMAND_TERMINATOR = string.char( 0x0D, 0x0A )

--! Array that contains all STATUS commands  stings (those we will  send to Carrier ) .
--Z1 - Means - Currently the system supports a single humidification setpoint. The zone number is intended for future upgrades
Carrier.STATUS_COMMANDS = {
    ['S_Z_HEAT_SETPOINT'] =  'HTSP?', 
    ['S_Z_ROOM_TEMPERATURE'] =  'RT?', 
    ['S_Z_DISPLAYED_HUMIDITY'] =  'RH?', 
    ['S_OUTDOR_TEMPERATURE'] =  'OAT?', 
    ['S_Z_FAN_SETTINGS'] =  'FAN?', 
    ['S_MODE'] =  'MODE?',
    ['S_Z_HOLD_STATUS'] =  'HOLD?', 
    ['S_Z_UNOCCUPIED_STATUS'] =  'UNOCC?', 
    ['S_Z_COOL_SETPOINT'] =  'CLSP?', 
    ['S_Z1_HUMIDIFICATION_TARGET'] =  'RHTG?',
    ['S_HUMIDIFIER_STATE'] =  'HUMID?', 
    ['S_DAY'] =  'DAY?', 
    ['S_TIME'] =  'TIME?',
    ['S_Z_OVERRIDE_STATE'] =  'OVR?', 
    ['S_Z_OVERRIDE_TIMER'] =  'OTMR?', 
    ['S_ZONE_NUMBER'] =  'ZONE?',
    ['S_Z_ZONE_NAME'] =  'NAME?', 
    ['S_FILTER_LIFE'] =  'FILTRLVL?',
    ['S_UVLAP_LIVE'] =  'UVLVL?',
    ['S_HUMIDIFIER_PAD_LIFE'] =  'HUMLVL?',
    ['S_FILTER_REMINDER_SETTING'] =  'FILTRRMD?',
    ['S_LAMP_REMINDER_SETTING'] =  'UVRMD?',
    ['S_HUMIDIFIER_PAD_REMINDER_SETTING'] =  'HUMRMD?',
    ['S_BACKLIGHTING_SETTING'] =  'BLIGHT?',
    ['S_VACATION_STATE'] =  'VACAT?',
    ['S_VACATION_DAYS'] =  'VACDAYS?',
    ['S_VACATION_MINIMUM_TEMPERATURE'] =  'VACMINT?',
    ['S_VACATION_MAXIMUM_TEMPERATURE'] =  'VACMAXT?',
    ['S_VACATION_MINIMUM_HUMIDITY'] =  'VACMINH?',
    ['S_VACATION_MAXIMUM_HIMIDITY'] =  'VACMAXH?',
    ['S_VACATION_FAN_SETTING'] =  'VACFAN?',
    ['S_UNITS_OF_THE_THERMOSTAT'] =  'CFGEM?',
    ['S_AUTO_CONFIGURATION_OF_THE_THERMOSTAT'] =  'CFGAUTO?',
    ['S_SYSTEM_TYPE'] =  'CFGTYPE?',
    ['S_DEADBAND_OF_THE_THERMOSTAT'] =  'CFGDEAD?',
    ['S_CYCLES_PER_HOUR_OF_THE_TERMOSTAT'] =  'CFGCPH?',
    ['S_PROGRAMMABLE_FAN_SETTING'] =  'CFGFAN?'
}

--! Array that contains all PROGRAMMING_INFO commands  stings (those we will  send to Carrier ) .
Carrier.PROGRAMMING_INFO_COMMANDS = {
    ['S_CURRENT_PROGRAM_PERIOD'] =  'PER?',
    ['S_NUMBER_OF_PERIODS_ALLOWED_FOR_PROGRAMMING'] =  'CFGPER?',
    ['S_PROGRAMMING_STATE_OF_THE_THERMOSTAT'] =  'CFGPGM?',
    ['S_Z_PROGRAMMING_INF_FOR_THE_MONDAY_WAKE_PERIOD'] =  'PGMMONWAKE?',
    ['S_Z_PROGRAMMING_INF_FOR_THE_MONDAY_DAY_PERIOD'] =  'PGMMONDAY?',
    ['S_Z_PROGRAMMING_INF_FOR_THE_MONDAY_EVE_PERIOD'] =  'PGMMONEVE?',
    ['S_Z_PROGRAMMING_INF_FOR_THE_MONDAY_SLEEP_PERIOD'] =  'PGMMONSLP?',
    ['S_Z_PROGRAMMING_INF_FOR_THE_TUESDAY_WAKE_PERIOD'] =  'PGMTUESWAKE?',
    ['S_Z_PROGRAMMING_INF_FOR_THE_TUESDAY_DAY_PERIOD'] =  'PGMTUESDAY?',
    ['S_Z_PROGRAMMING_INF_FOR_THE_TUESDAY_EVE_PERIOD'] =  'PGMTUESEVE?',
    ['S_Z_PROGRAMMING_INF_FOR_THE_TUESDAY_SLEEP_PERIOD'] =  'PGMTUESSLP?',
    ['S_Z_PROGRAMMING_INF_FOR_THE_WEDNESDAY_WAKE_PERIOD'] =  'PGMWEDWAKE?',
    ['S_Z_PROGRAMMING_INF_FOR_THE_WEDNESDAY_DAY_PERIOD'] =  'PGMWEDDAY?',
    ['S_Z_PROGRAMMING_INF_FOR_THE_WEDNESDAY_EVE_PERIOD'] =  'PGMWEDVENING?', --ending with all word in docPdf - check it in live - may be their mestake
    ['S_Z_PROGRAMMING_INF_FOR_THE_WEDNESDAY_SLEEP_PERIOD'] =  'PGMWEDSLP?',
    ['S_Z_PROGRAMMING_INF_FOR_THE_THURSDAY_WAKE_PERIOD'] =  'PGMTHURWAKE?',
    ['S_Z_PROGRAMMING_INF_FOR_THE_THURSDAY_DAY_PERIOD'] =  'PGMTHURDAY?',
    ['S_Z_PROGRAMMING_INF_FOR_THE_THURSDAY_EVE_PERIOD'] =  'PGMTHUREVENING?', --ending with all word in docPdf - check it in live may be their mestake
    ['S_Z_PROGRAMMING_INF_FOR_THE_THURSDAY_SLEEP_PERIOD'] =  'PGMTHURSLP?',
    ['S_Z_PROGRAMMING_INF_FOR_THE_FRIDAY_WAKE_PERIOD'] =  'PGMFRIWAKE?',
    ['S_Z_PROGRAMMING_INF_FOR_THE_FRIDAY_DAY_PERIOD'] =  'PGMFRIDAY?',
    ['S_Z_PROGRAMMING_INF_FOR_THE_FRIDAY_EVE_PERIOD'] =  'PGMFRIEVENING?', --ending with all word in docPdf - check it in live may be their mestake
    ['S_Z_PROGRAMMING_INF_FOR_THE_FRIDAY_SLEEP_PERIOD'] =  'PGMFRISLP?',
    ['S_Z_PROGRAMMING_INF_FOR_THE_SATURDAY_WAKE_PERIOD'] =  'PGMSATWAKE?',
    ['S_Z_PROGRAMMING_INF_FOR_THE_SATURDAY_DAY_PERIOD'] =  'PGMSATDAY?',
    ['S_Z_PROGRAMMING_INF_FOR_THE_SATURDAY_EVE_PERIOD'] =  'PGMSATEVENING?', --ending with all word in docPdf - check it in live may be their mestake
    ['S_Z_PROGRAMMING_INF_FOR_THE_SATURDAY_SLEEP_PERIOD'] =  'PGMSATSLP?',
    ['S_Z_PROGRAMMING_INF_FOR_THE_SUNDAY_WAKE_PERIOD'] =  'PGMSUNWAKE?',
    ['S_Z_PROGRAMMING_INF_FOR_THE_SUNDAY_DAY_PERIOD'] =  'PGMSUNDAY?',
    ['S_Z_PROGRAMMING_INF_FOR_THE_SUNDAY_EVE_PERIOD'] =  'PGMSUNEVENING?', --ending with all word in docPdf - check it in live may be their mestake
    ['S_Z_PROGRAMMING_INF_FOR_THE_SUNDAY_SLEEP_PERIOD'] =  'PGMSUNSLP?'
}

--! Array that contains all CONFIGURATION commands  stings (those we will  send to Carrier ) .
Carrier.CONFIGURATION_COMMANDS = {
    ['S_Z_SET_FAN_SETTINGS__AUTO_LOW_MED_HIGH'] =  'FAN!', 
    ['S_SET_THERMOSTAT_MODE__HEAT_COOL_AUTO_OFF_EHEAT'] =  'MODE!',  
    ['S_Z_SET_HOLD_STATUS__ON_OFF'] =  'HOLD!', 
    ['S_Z_SET_UNOCCUPIED_STATUS__ON_OFF'] =  'UNOCC!', 
    ['S_Z_SET_HEAT_SETPOINT__XX_TIME'] =  'HTSP!', 
    ['S_Z_SET_COOL_SETPOINT__XX_TIME'] =  'CLSP!', 
    ['S_SET_DAY__0_6'] =  'DAY!', 
    ['S_SET_TIME__TIME_A_P'] =  'TIME!', 
    ['S_Z_SET_OVERRIDE_TIMER__TIMER'] =  'OTMR!', 
    ['S_SET_ZONE_NUMBER__1_8'] =  'ZONE!', 
    ['S_Z_SET_ZONE_NAME_FOR_ZONE__STRING'] = 'NAME!', 
    ['S_RESET_FILTER_LIFE__0'] =  'FILTRLVL!',
    ['S_RESET_UVLAMP_LIFE__0'] =  'UVLVL!',
    ['S_RESET_HUMIDIFIER_PAD_LIFE__0'] =  'HUMLVL!', 
    ['S_RESET_VENTILATOR_FILTER_LIFE__0'] =  'VENTLVL!',
    ['S_SET_FILTER_REMINDER__ON_OFF'] =  'FILTRRMD!',
    ['S_SET_UVLAMP_REMINDER__ON_OFF'] =  'UVRMD!',
    ['S_SET_HUMIDIFIER_PAD_REMINDER__ON_OFF'] =  'HUMRMD!', 
    ['S_SET_VENTILATION_PAD_REMINDER__ON_OFF'] =  'VENTRMD!',
    ['S_SET_BACKLIGHTING__ON_OFF'] =  'BLIGHT!',
    ['S_SET_VACATION_STATE__XXX'] =  'VACDAYS!', 
    ['S_SET_VACATION_MIN_TEMPERATURE__XX'] =  'VACMINT!', 
    ['S_SET_VACATION_MAX_TEMPERATURE__XX'] =  'VACMAXT!',
    ['S_SET_VACATION_MIN_HUMIDITY__XXX'] =  'VACMINH!',
    ['S_SET_VACATION_MAX_HUMIDITY__XXX'] =  'VACMAXH!',
    ['S_SET_VACATION_FAN_SETTING__AUTO_LOW_MED_HIGH'] =  'VACFAN!', 
    ['S_SET_UNITS_OF_THE_THERMOSTAT__E_M'] =  'CFGEM!',
    ['S_SET_AUTO_CONFIG_FOR_THERMOSTAT__ON_OF'] =  'CFGAUTO!',
    ['S_SET_DEADBAND_OF_THERMOSTAT__2_6'] =  'CFGDEAD!',
    ['S_SET_CYCLES_PER_HOUR_OF_THERMOSTAT__2_6'] =  'CFGCPH!',
    ['S_SET_PROGRAMMABLE_FAN_SETTING__ON_OFF'] =  'CFGFAN!'
}

--! Array that contains all PROGRAMMING commands  stings (those we will  send to Carrier ) .
Carrier.PROGRAMMING_COMMANDS = {
    ['S_SET_NUMBER_ALLOWED_PROGRAMMING_PERIOD__2or4'] =  'CFGPER!',
    ['S_SET_PROGRAMMING_STATE_OF_THERMOSTAT__ON_OFF'] =  'CFGPGM!',
    ['S_Z_SET_PROGRAMMING_INF_FOR_THE_MONDAY_WAKE_PERIOD__TIME_A_P_HEAT_COOL_FAN'] =  'PGMMONWAKE!',  -- S1Z1PGMONWAKE!06:30 A, 70, 72, AUTO
    ['S_Z_SET_PROGRAMMING_INF_FOR_THE_MONDAY_DAY_PERIOD__TIME_A_P_HEAT_COOL_FAN'] =  'PGMMONDAY!', 
    ['S_Z_SET_PROGRAMMING_INF_FOR_THE_MONDAY_EVE_PERIOD__TIME_A_P_HEAT_COOL_FAN'] =  'PGMMONENING!', 
    ['S_Z_SET_PROGRAMMING_INF_FOR_THE_MONDAY_SLEEP_PERIOD__TIME_A_P_HEAT_COOL_FAN'] =  'PGMMONSLP!',
    ['S_Z_SET_PROGRAMMING_INF_FOR_THE_TUESDAY_WAKE_PERIOD__TIME_A_P_HEAT_COOL_FAN'] =  'PGMTUESWAKE!',
    ['S_Z_SET_PROGRAMMING_INF_FOR_THE_TUESDAY_DAY_PERIOD__TIME_A_P_HEAT_COOL_FAN'] =  'PGMTUESDAY!',
    ['S_Z_SET_PROGRAMMING_INF_FOR_THE_TUESDAY_EVE_PERIOD__TIME_A_P_HEAT_COOL_FAN'] =  'PGMTUESENING!',
    ['S_Z_SET_PROGRAMMING_INF_FOR_THE_TUESDAY_SLEEP_PERIOD__TIME_A_P_HEAT_COOL_FAN'] =  'PGMTUESSLP!',
    ['S_Z_SET_PROGRAMMING_INF_FOR_THE_WEDNESDAY_WAKE_PERIOD__TIME_A_P_HEAT_COOL_FAN'] =  'PGMWEDWAKE!',
    ['S_Z_SET_PROGRAMMING_INF_FOR_THE_WEDNESDAY_DAY_PERIOD__TIME_A_P_HEAT_COOL_FAN'] =  'PGMWEDDAY!',
    ['S_Z_SET_PROGRAMMING_INF_FOR_THE_WEDNESDAY_EVE_PERIOD__TIME_A_P_HEAT_COOL_FAN'] =  'PGMWEDVENING!',
    ['S_Z_SET_PROGRAMMING_INF_FOR_THE_WEDNESDAY_SLEEP_PERIOD__TIME_A_P_HEAT_COOL_FAN'] =  'PGMWEDSLP!',
    ['S_Z_SET_PROGRAMMING_INF_FOR_THE_THURSDAY_WAKE_PERIOD__TIME_A_P_HEAT_COOL_FAN'] =  'PGMTHURWAKE!',
    ['S_Z_SET_PROGRAMMING_INF_FOR_THE_THURSDAY_DAY_PERIOD__TIME_A_P_HEAT_COOL_FAN'] =  'PGMTHURDAY!',
    ['S_Z_SET_PROGRAMMING_INF_FOR_THE_THURSDAY_EVE_PERIOD__TIME_A_P_HEAT_COOL_FAN'] =  'PGMTHUREVENING!',
    ['S_Z_SET_PROGRAMMING_INF_FOR_THE_THURSDAY_SLEEP_PERIOD__TIME_A_P_HEAT_COOL_FAN'] =  'PGMTHURSLP!',
    ['S_Z_SET_PROGRAMMING_INF_FOR_THE_FRIDAY_WAKE_PERIOD__TIME_A_P_HEAT_COOL_FAN'] =  'PGMFRIWAKE!',
    ['S_Z_SET_PROGRAMMING_INF_FOR_THE_FRIDAY_DAY_PERIOD__TIME_A_P_HEAT_COOL_FAN'] =  'PGMFRIDAY!',
    ['S_Z_SET_PROGRAMMING_INF_FOR_THE_FRIDAY_EVE_PERIOD__TIME_A_P_HEAT_COOL_FAN'] =  'PGMFRIEVENING!', 
    ['S_Z_SET_PROGRAMMING_INF_FOR_THE_FRIDAY_SLEEP_PERIOD__TIME_A_P_HEAT_COOL_FAN'] =  'PGMFRISLP!',
    ['S_Z_SET_PROGRAMMING_INF_FOR_THE_SATURDAY_WAKE_PERIOD__TIME_A_P_HEAT_COOL_FAN'] =  'PGMSATWAKE!',
    ['S_Z_SET_PROGRAMMING_INF_FOR_THE_SATURDAY_DAY_PERIOD__TIME_A_P_HEAT_COOL_FAN'] =  'PGMSATDAY!',
    ['S_Z_SET_PROGRAMMING_INF_FOR_THE_SATURDAY_EVE_PERIOD__TIME_A_P_HEAT_COOL_FAN'] =  'PGMSATEVENING!', 
    ['S_Z_SET_PROGRAMMING_INF_FOR_THE_SATURDAY_SLEEP_PERIOD__TIME_A_P_HEAT_COOL_FAN'] =  'PGMSATSLP!',
    ['S_Z_SET_PROGRAMMING_INF_FOR_THE_SUNDAY_WAKE_PERIOD__TIME_A_P_HEAT_COOL_FAN'] =  'PGMSUNWAKE!',
    ['S_Z_SET_PROGRAMMING_INF_FOR_THE_SUNDAY_DAY_PERIOD__TIME_A_P_HEAT_COOL_FAN'] =  'PGMSUNDAY!',
    ['S_Z_SET_PROGRAMMING_INF_FOR_THE_SUNDAY_EVE_PERIOD__TIME_A_P_HEAT_COOL_FAN'] =  'PGMSUNEVENING!', 
    ['S_Z_SET_PROGRAMMING_INF_FOR_THE_SUNDAY_SLEEP_PERIOD__TIME_A_P_HEAT_COOL_FAN'] =  'PGMSUNSLP!',
    ['S_RESET_FACTORY_DEFAULTS__RESET'] =  'CFG!'
}

--! Array that contains all SERVICE commands  stings (those we will  send to Carrier ) .
Carrier.SERVICE_COMMANDS = {
    ['S_RETRIVE_DEALER_NAME'] =  'DEALER?',
    ['S_RETRIVE_DEALER_PHONE'] =  'DEALERPH?',
    ['S_SET_DEALER_NAME__STRING'] =  'PGMMONWAKE!',
    ['S_SET_DEALER_PHONE__STRING'] =  'PGMMONDAY!' -- STRINT Max 19 characters
}

--! Initialize main plugin object and child devices (zones).
--!@param id id of the device
--!@member zones objects table
--!@member initMode bool for checking if it is initial state for zones creation 
function Carrier:__init(id)
    Device.__init(self, id)

    self.carrierConfig = CarrierConfig(self)

    self:startSocket() 

    self.initMode = false
    zones = {}

    self.commandsQueue = carrierCommandQueue()

    if (tableSize(self:populateZones()) == 0) then
        self.zoneIDCounter = 1
        self.systemIDCounter = 1
        self.initMode = true
        self:setSystemId(1)
        self:setZoneId(1)
        self:initializeChildDevices()
    else
        self:initCommandQueue()
    end

    self:setSystemId(1)
    self:setZoneId(1)
      
    self:synchronizePropertiesFromActiveZone()     
end

--! Populating zones array and child devices.
--@return zones Array that contain all available zones objects.
function Carrier:populateZones()
    print ('Carrier:populateZones')
    local childDevices = plugin.getChildDevices(self.id)

    for _,dev in pairs(childDevices) do
        if (dev.type == 'com.fibaro.carrierZone') then
            self.childDevices[dev.id] = CarrierZone(self, dev)
            zones['S'..dev.properties.systemId..'Z'..dev.properties.zoneId] = self.childDevices[dev.id]
        end
    end

    return zones
end

--! Creating TPCsocket for communication with S.A.M.
function Carrier:startSocket()
    print ('Carrier:startSocket')
    if(self.sock) then
        self.sock:close()
    end

    self.sock = TCPListener({delimiter = string.char( 0x0D , 0x0A ), readTimeout = 10000, connectTimeout=10000})
    self.sock:registerEvent('dataReceived', function(sock, data) self:onDataReceived(data, id) end)
    self.sock:registerEvent('disconnected', function()  self:onClosed() end)
    self.sock:registerEvent('connected', function() self:onConnected() end)
    self.sock:registerEvent('error', function(sock, error) print("error", error) end)

    if (self.properties.ip ~= '' and self.properties.port ~= '' and 
        self.properties.ip ~= nil and self.properties.port ~= nil) then  
    self.sock:connect(self.properties.ip, tonumber(self.properties.port))
    self.properties.systemId = 1
    else
        print('No ip or port')
    end        
end

--! Initialize process of commandQueue that is responsible for GUI refreshing.
--!@param commandsQueue The commandsQueue to be initialize.
function Carrier:initCommandQueue()
    print ('Carrier:initCommandQueue')
    self.commandsQueue = carrierCommandQueue()
    self:resetCommandsQueue();
    self:startPolling()   
end

--! Clears commands queue, and reinitializes it with all cyclic commands.
function Carrier:resetCommandsQueue()
    print ('Carrier:resetCommandsQueue')

    self.commandsQueue:clear()

    if (self:countTable(zones) > 8) then 
        numberOfSystems = 2
    else
        numberOfSystems = 1
    end

    for key,command in pairs(Carrier.STATUS_COMMANDS) do
        if (key:sub(1,4)  == 'S_Z_') then
            for k,v in pairs(zones) do
                local zoneAndSystemIdentyfier = k
                local cmd = zoneAndSystemIdentyfier..
                            command..
                            self.COMMAND_TERMINATOR
                self.commandsQueue:pushBack(cmd)
            end

        elseif (key:sub(1,10) == 'S_VACATION') then
            for i=1, numberOfSystems do                
                local cmd = 'S'..i..
                            command..
                            self.COMMAND_TERMINATOR
                self.commandsQueue:pushBack(cmd)
            end 

        elseif (key:sub(1,8) == 'S_OUTDOR') then
            for i=1, numberOfSystems do                
                local cmd = 'S'..i..
                            command..
                            self.COMMAND_TERMINATOR
                self.commandsQueue:pushBack(cmd)
            end 
        end
    end
end

--! Counting number of array elements.
--@param table A table to count.
--@return counter Number of table elements.
function Carrier:countTable(table)
    local counter = 0
    for k,v in pairs(table) do
        counter = counter + 1
    end

    return counter
end

--! Checks whether command should be repeated cyclicaly.
--!@param command Command to be checked.
--!@return true/false
function Carrier:isCommandRepeatable(command)
    -- commands with '?' should be repeated cyclicaly
    if string.match(command, '^[a-zA-Z0-9]+?') then
        return true
    end

    return false
end

--! 15 seconds infinite polling loop.
function Carrier:startPolling()
    print ('Carrier:startPolling')
    self.nextCommandTimeoutId = setTimeout(function( ) self:startPolling(); end, 15000)

    if (self.initMode == false) then
        self:sendNextCommand()
    end
end

--! Executes given command - sends it to carrier device
--!@command Command string to be sent
function Carrier:executeCommand(command)
    print('Executing Command: '..command)
    setTimeout(function() self.sock:send(command)end, 100)
end

--! Sends next command from queue. If command is cyclically repeatable, puts it on back of queue
function Carrier:sendNextCommand()
    print ('Carrier:sendNextCommand')
    local command = self.commandsQueue:popFront()

    if (command) then
        if self:isCommandRepeatable(command) then
            self.commandsQueue:pushBack(command)
        end

        self:executeCommand(command)  
    end  
end

--! Test zones for temperature response to indicate that given zone is available in the system.
--! If test pass then zone will be added to zones table in onDataReceived initMode part method.
function Carrier:initializeChildDevices()
        print ('Carrier:initializeChildDevices')
    if (self.initMode == true) then
        print('testing ZONE FOR ABILITY: zoneID:'..self.zoneIDCounter)

        local cmd = 'S'..self.systemIDCounter..
        'Z'..self.zoneIDCounter..
        self.STATUS_COMMANDS['S_Z_ROOM_TEMPERATURE']..          
        self.COMMAND_TERMINATOR
        setTimeout(function() self.sock:send(cmd)end, 100) --response format: S1Z1RT: xxx°F/C
    end
end

--! Search for active zone in zones array.
--@return zone or nil
function Carrier:getActiveZone()
    print ('Carrier:getActiveZone')
    local zoneIdetifierKey = 'S'..self.properties.systemId..'Z'..self.properties.zoneId
    local activeZone = zones[zoneIdetifierKey]

    if (activeZone) then 
        return activeZone
    end 

    return nil   
end

--! Fires when soket connects to the resource.
function Carrier:onConnected()
    print ('Carrier:onConnected')
    Device.updateProperty(self, 'dead', true)
end

--! Fires when soket disconnects from the resource.
function Carrier:onClosed()
    print ('Carrier:onClosed')
    if (self.properties.ip ~= '' and self.properties.port ~= '' and 
        self.properties.ip ~= nil and self.properties.port ~= nil) then  
    time = os.date("*t")
    setTimeout(function() self.sock:connect(self.properties.ip, tonumber(self.properties.port)) end, 3000)
    else
        print('No ip or port added')
    end  
end

--! Method responsible for ui update according to data received from carrier.
--! initMode part is responsible for adding active zones to zones array. 
--@param data Data string received from Carrier.  
function Carrier:onDataReceived(data)
    print ('Carrier:onDataReceived')

    if (self.initMode == true) then
        if (string.find(data, 'NAK') == nil) then
            local stringHeatSeatpoint = data:match(':(.*)')
            local units = string.match(stringHeatSeatpoint, "%a+")
            self:updateProperty('units', units)       

            local dev = plugin.createChildDevice(self.id, 'com.fibaro.carrierZone', 'S'..self.systemIDCounter..' Zone ' .. self.zoneIDCounter)
            local devObject = CarrierZone(self, dev)               
            devObject:updateProperty('zoneId', self.zoneIDCounter)
            devObject:updateProperty('systemId', self.systemIDCounter)
            self.childDevices[dev.id] = devObject  
            zones['S'..dev.properties.systemId..'Z'..dev.properties.zoneId] = self.childDevices[dev.id] 

            print('adding zoneID:'..self.zoneIDCounter..' witch temp: '..stringHeatSeatpoint)
        end

        self.zoneIDCounter = self.zoneIDCounter + 1

        if (self.zoneIDCounter > 8) then
            if (self.systemIDCounter == 1) then
                self.systemIDCounter = 2
                self.zoneIDCounter = 1 
            else 
                self.initMode = false
                self:initCommandQueue()
                return 0
            end 
        end 

        self:initializeChildDevices() 

        return
    else
        print ("onDataReceived data:"..data)

        --Clears timeout to prepare for restart ifinit polling loop. 
        clearTimeout(self.nextCommandTimeoutId)

        local zoneIdetifierKey = data:sub(1,4)

        if (zones[zoneIdetifierKey]) then --This is reponse from zone.
            zoneToRefresh = zones[zoneIdetifierKey]  
        else --This is response from system or zone error.
             --we assure to refresh only current selected system.
            if (data:sub(1,2) == 'S'..self.properties.systemId) then
            else --Data error or reference from  currently not selected system.
                print('!!!!!!!!!!!!!!!!!!polling1')
                self.nextCommandTimeoutId = setTimeout(function( ) self:startPolling(); end, 15000)
                self:sendNextCommand()
                return
            end
        end    

        if (string.find(data, 'HTSP')) then -- HeatSetpoint --response format: S1Z1HTSP: xxx°F/C
            local stringheatSeatpoint = data:match(':(.*)')
            if (stringheatSeatpoint == 'ACK') then --this is configuration command response
                print('configuration command response')
            elseif (string.find(stringheatSeatpoint, 'NAK')) then --this means invalid command (ex. wrong parameter was send to Carrier)
                print('Invalid command')
            else  --this is status command resopnse
                if (stringheatSeatpoint:sub(1,1) == '0') then
                    stringheatSeatpoint = stringheatSeatpoint:sub(2,#stringheatSeatpoint)
                end

                local valueHeatSeatpoint = tonumber(string.match(stringheatSeatpoint, "%d+"))
                local unitsHeatSeatpoint = string.match(stringheatSeatpoint, "%a+")
                zoneToRefresh:updateProperty('ui.Current_Heat_Setpoint_Value_Label.caption', valueHeatSeatpoint..'°'..unitsHeatSeatpoint) 
                zoneToRefresh:updateProperty('currentHeatSetpoint', stringheatSeatpoint )
            end
        
        elseif (string.find(data, 'CLSP')) then -- --response format: S1Z1CLSP: xxx°F/C
            local stringcoolSeatpoint = data:match(':(.*)')
            if (stringcoolSeatpoint == 'ACK') then --this is configuration command response
                print('configuration command response')
            elseif (string.find(stringcoolSeatpoint, 'NAK')) then --this means invalid command (ex. wrong parameter was send to Carrier)
                print('Invalid  command')
            else  -- this is status command resopnse
                if (stringcoolSeatpoint:sub(1,1) == '0') then
                    stringcoolSeatpoint = stringcoolSeatpoint:sub(2,#stringcoolSeatpoint)
                end

                local valueCoolSeatpoint = tonumber(string.match(stringcoolSeatpoint, "%d+"))
                local unitsCoolSeatpoint = string.match(stringcoolSeatpoint, "%a+")
                zoneToRefresh:updateProperty('ui.Current_Cool_Setpoint_Value_Label.caption', valueCoolSeatpoint..'°'..unitsCoolSeatpoint) 
                zoneToRefresh:updateProperty('currentCoolSetpoint', stringcoolSeatpoint )
            end
        
        elseif (string.find(data, 'RT')) then -- --response format: S1Z1RT: xx°F/C
            local stringRoomTemp = data:match(':(.*)')

            if (stringRoomTemp == 'ACK') then --this is configuration command response
                print('this is configuration command response')
            elseif (string.find(stringRoomTemp, 'NAK')) then --this means invalid command (ex. wrong parameter was send to Carrier)
                print('Invalid RT command')
            else  -- this is status command resopnse
                if (stringRoomTemp:sub(1,1) == '0') then
                    stringRoomTemp = stringRoomTemp:sub(2,#stringRoomTemp)
                end

                local valueRoomTemp = tonumber(string.match(stringRoomTemp, "%d+"))
                local unitsRoomTemp = string.match(stringRoomTemp, "%a+")
                zoneToRefresh:updateProperty('ui.Current_Temp_Label.caption', valueRoomTemp..'°'..unitsRoomTemp) 
                zoneToRefresh:updateProperty('currentTemp', stringRoomTemp )
                zoneToRefresh:updateProperty('value', valueRoomTemp )

            end
        
        elseif (string.find(data, 'OVR')) then -- --response format: S1Z1OVR:OFF/ON
            local stringOVRStatus = data:match(':(.*)')
            if (stringOVRStatus == 'ACK') then --this is configuration command response
                print('configuration command response')
            elseif (string.find(stringOVRStatus, 'NAK')) then --this means invalid command (ex. wrong parameter was send to Carrier)
                print('Invalid command')
            else  -- this is status command resopnse

                zoneToRefresh:updateProperty('ui.Override_Timer_value_Label.caption', stringOVRStatus) 
                zoneToRefresh:updateProperty('overrideStatus', stringOVRStatus )
            end
        
        elseif (string.find(data, 'OTMR')) then -- --response format: S1Z1OTMR:03:32
            local stringOVRTimer = data:match(':(.*)')
            if (stringOVRTimer == 'ACK') then --this is configuration command response
                print('configuration command response')
            elseif (string.find(stringOVRTimer, 'NAK')) then --this means invalid command (ex. wrong parameter was send to Carrier)
                print('Invalid command')
            else  -- this is status command resopnse

                local hour = tonumber(string.match(stringOVRTimer, "%d+"))
                local minute = tonumber(stringOVRTimer:match(hour..':(.*)'))
                
                zoneToRefresh:updateProperty('ui.Curretn_Override_Timer_Value_Hour_Label.caption', hour)
                zoneToRefresh:updateProperty('ui.Current_Override_Timer_Value_Minutes_Label.caption', minute) 
                zoneToRefresh:updateProperty('currentOverrideTimer', stringOVRTimer )
            end
        
        elseif (data:sub(5,7) == 'FAN') then -- --response format: S1Z1FAN:AUTO
            local stringFanSettings = data:match(':(.*)')
            if (stringFanSettings == 'ACK') then --this is configuration command response
                print('configuration command response')
            elseif (string.find(stringFanSettings, 'NAK')) then --this means invalid command (ex. wrong parameter was send to Carrier)
                print('Invalid  command')
            else  -- this is status command resopnse
        
                zoneToRefresh:updateProperty('ui.Current_Fan_Speed_Value_Label.caption', stringFanSettings)
                zoneToRefresh:updateProperty('currentFanSettings', stringFanSettings )
            end
        
        elseif (string.find(data, 'HOLD')) then -- --response format: S1Z1HOLD:OFF
            local stringHoldStatus = data:match(':(.*)')
            if (stringHoldStatus == 'ACK') then --this is configuration command response
                print('this is configuration command response')
            elseif (string.find(stringHoldStatus, 'NAK')) then --this means invalid command (ex. wrong parameter was send to Carrier)
                print('Invalid stringHoldStatus command')
            else  -- this is status command resopnse
                
                zoneToRefresh:updateProperty('ui.Current_Hold_Status_Value_Label.caption', stringHoldStatus)
                zoneToRefresh:updateProperty('currentHoldStatus', stringHoldStatus )
            end
        
        elseif (string.find(data, 'UNOCC')) then -- --response format: S1Z1UNOCC:OFF
            local stringUnoccupiedStatus = data:match(':(.*)')
            if (stringUnoccupiedStatus == 'ACK') then --this is configuration command response
                print('configuration command response')
            elseif (string.find(stringUnoccupiedStatus, 'NAK')) then --this means invalid command (ex. wrong parameter was send to Carrier)
                print('Invalid  command')
            else  -- this is status command resopnse
                
                zoneToRefresh:updateProperty('ui.Current_Unoccupied_Status_Value_Label.caption', stringUnoccupiedStatus)
                zoneToRefresh:updateProperty('currentUnoccupiedStatus', stringUnoccupiedStatus )
            end
        
        elseif (string.find(data, 'PGM')) then -- --response format: S1Z1PGMMONWAKE:06:00 A, 16�C, 30�C, AUTO (AUTO if prog function is ON)
            local stringProgramingInfo = data:match('PGM(.*)')
            if (string.find(stringProgramingInfo, 'ACK')) then --this is configuration command response
                print('configuration command response')
            elseif (string.find(stringProgramingInfo, 'NAK')) then --this means invalid command (ex. wrong parameter was send to Carrier)
                print('Invalid  command')
            else  -- this is status command resopnse
                local day = ''
                local period = ''
                local hour = ''
                local minutes = ''
                local timePeriod = ''
                local heatSetpoint = ''
                local coolSetpoint = ''
                local fanSpped = ''

                if (string.find(stringProgramingInfo, 'MON')) then day = 'MON'
                elseif (string.find(stringProgramingInfo, 'TUES')) then day = 'TUES'
                elseif (string.find(stringProgramingInfo, 'WED')) then day = 'WED'
                elseif (string.find(stringProgramingInfo, 'THUR')) then day = 'THUR'
                elseif (string.find(stringProgramingInfo, 'FRI')) then day = 'FRI'
                elseif (string.find(stringProgramingInfo, 'SAT')) then day = 'SAT'
                elseif (string.find(stringProgramingInfo, 'SUN')) then day = 'SUN'
                end

                if (string.find(stringProgramingInfo, 'WAKE')) then period = 'WAKE'
                elseif (string.find(stringProgramingInfo, 'DAY')) then period = 'DAY'
                elseif (string.find(stringProgramingInfo, 'EVE')) then period = 'EVE'
                elseif (string.find(stringProgramingInfo, 'SLP')) then period = 'SLP'
                end

                local time = string.match(stringProgramingInfo,':(.-),')
                hour = string.match(stringProgramingInfo,':(.*):')
                minute = string.match(time,':(%d+)')
                timePeriod = string.match(time,'%a+')

                heatSetpoint, coolSetpoint, fanMode = string.match(stringProgramingInfo,',.(%d+).-(%d+)(.*)')
                fanMode = string.match(fanMode,' (%a+)$')

                print('----day '..day)
                print('----priod '..period)
                print('----hour '..hour)
                print('----minute '..minute..'end')
                print('----timePeriod '..timePeriod)
                print('----heatSetpoint '..heatSetpoint)
                print('----coolSetpoint '..coolSetpoint)

                if (fanMode ~= nil) then
                    print ("fanMode:"..fanMode)    
                else
                    fanMode = ''
                end    

                zoneToRefresh:updateProperty('ui.Current_Programming_Z_Day_Value_Label.caption', day)
                zoneToRefresh:updateProperty('ui.Current_Programming_Z_Period_Value_Label.caption', period)
                zoneToRefresh:updateProperty('ui.Current_Programming_Z_PERIOD_Value_NL.caption', timePeriod)
                zoneToRefresh:updateProperty('ui.Current_Programming_Z_Heat_Setpoint_Value_Label.caption', heatSetpoint)
                zoneToRefresh:updateProperty('ui.Current_Programming_Z_Cool_Setpoint_Value_Label.caption', coolSetpoint)
                zoneToRefresh:updateProperty('ui.Current_Programming_Z_Fan_Speed_Value_Label.caption', fanMode)

                zoneToRefresh:updateProperty('ui.Programming_Z_Day_Value_Label.caption', day)
                zoneToRefresh:updateProperty('ui.Programming_Z_Period_Value_Label.caption', period)
                zoneToRefresh:updateProperty('ui.Programming_Z_PERIOD_Value_NL.caption', timePeriod)
                zoneToRefresh:updateProperty('ui.Programming_Z_Heat_Setpoint_Value_Label.caption', heatSetpoint)
                zoneToRefresh:updateProperty('ui.Programming_Z_Cool_Setpoint_Value_Label.caption', coolSetpoint)
                zoneToRefresh:updateProperty('ui.Programming_Z_Fan_Speed_Value_Label.caption', fanMode)
                zoneToRefresh:updateProperty('ui.Programming_Z_Hour_VL.caption', hour)
                zoneToRefresh:updateProperty('ui.Programming_Z_Minuts_VL.caption', minute)

                zoneToRefresh:updateProperty('programmingDay', day )
                zoneToRefresh:updateProperty('programmingHour', hour )
                zoneToRefresh:updateProperty('programmingMinutes', minute )
                zoneToRefresh:updateProperty('programmingPeriod', period )
                zoneToRefresh:updateProperty('programmingTimePeriod', timePeriod )
                zoneToRefresh:updateProperty('programmingHeatSetpoint', heatSetpoint )
                zoneToRefresh:updateProperty('programmingCoolSetpoint', coolSetpoint )
                zoneToRefresh:updateProperty('fanSettings', fanMode )
            end
        
        elseif (string.find(data, 'VACDAYS') ) then  --response format: S1VACDAYS: 100
            local stringVacDays = data:match(':(.*)')
            if (stringVacDays == 'ACK') then --this is configuration command response
                print('configuration command response')
            elseif (string.find(stringVacDays, 'NAK')) then --this means invalid command (ex. wrong parameter was send to Carrier)
                print('Invalid  command')
            else  -- this is status command resopnse 
                self:updateProperty('ui.Current_Vacation_State_Value_Label.caption', stringVacDays) 
                self:updateProperty('currentVacationDays', stringVacDays )
                if (tonumber(stringVacDays) ~= 0) then
                    self:updateProperty('currentVacationState', 'ON')
                else
                    self:updateProperty('currentVacationState', 'OFF')
                end
            end
        
        elseif (string.find(data, 'VACMINT') ) then  --response format: S1VACMINT:20�C
            local stringVacMinTemp = data:match(':(.*)')
            if (stringVacMinTemp == 'ACK') then --this is configuration command response
                print('configuration command response')
            elseif (string.find(stringVacMinTemp, 'NAK')) then --this means invalid command (ex. wrong parameter was send to Carrier)
                print('Invalid  command')
            else  -- this is status command resopnse 
                if (stringVacMinTemp:sub(1,1) == '0') then
                    stringVacMinTemp = stringVacMinTemp:sub(2,#stringVacMinTemp)
                end
                local valueVacMinTemp = tonumber(string.match(stringVacMinTemp, "%d+"))
                local unitsVacMinTemp = string.match(stringVacMinTemp, "%a+")
                
                self:updateProperty('ui.Current_Vacation_Min_Temp_Value_Label.caption', valueVacMinTemp..'°'..unitsVacMinTemp) 
                self:updateProperty('currentVacationMinTemperature', stringVacMinTemp )
            end
        
        elseif (string.find(data, 'VACMAXT') ) then  --response format: S1VACMAXT:30�C
            local stringVacMaxTemp = data:match(':(.*)')
            if (stringVacMaxTemp == 'ACK') then --this is configuration command response
                print('tconfiguration command response')
            elseif (string.find(stringVacMaxTemp, 'NAK')) then --this means invalid command (ex. wrong parameter was send to Carrier)
                print('Invalid  command')
            else  -- this is status command resopnse 
                if (stringVacMaxTemp:sub(1,1) == '0') then
                    stringVacMaxTemp = stringVacMaxTemp:sub(2,#stringVacMaxTemp)
                end
                local valueVacMaxTemp = tonumber(string.match(stringVacMaxTemp, "%d+"))
                local unitsVacMaxTemp = string.match(stringVacMaxTemp, "%a+")
                
                self:updateProperty('ui.Current_Vacation_Max_Temp_Value_Label.caption', valueVacMaxTemp..'°'..unitsVacMaxTemp) 
                self:updateProperty('currentVacationMaxTemperature', stringVacMaxTemp )
            end
        
        elseif (string.find(data, 'VACMINH') ) then  --response format: S1VACMINH:15%
            local stringVacMinHumidity = data:match(':(.*)')
            if (stringVacMinHumidity == 'ACK') then --this is configuration command response
                print('configuration command response')
            elseif (string.find(stringVacMinHumidity, 'NAK')) then --this means invalid command (ex. wrong parameter was send to Carrier)
                print('Invalid  command')
            else  -- this is status command resopnse 
                if (stringVacMinHumidity:sub(1,1) == '0' and #stringVacMinHumidity>2 ) then
                    stringVacMinHumidity = stringVacMinHumidity:sub(2,#stringVacMinHumidity)
                end

                local valueVacMinHumidity = tonumber(string.match(stringVacMinHumidity, "%d+"))
                self:updateProperty('ui.Current_Vacation_Min_Hum_Value_Label.caption', valueVacMinHumidity..'%') 
                self:updateProperty('currentVacationMinHumidity', valueVacMinHumidity..'%' )
            end
        
        elseif (string.find(data, 'VACMAXH') ) then  --response format: S1VACMAXH:55%
            local stringVacMaxHumidity = data:match(':(.*)')
            if (stringVacMaxHumidity == 'ACK') then --this is configuration command response
                print('configuration command response')
            elseif (string.find(stringVacMaxHumidity, 'NAK')) then --this means invalid command (ex. wrong parameter was send to Carrier)
                print('Invalid command')
            else  -- this is status command resopnse 
                if (stringVacMaxHumidity:sub(1,1) == '0' and #stringVacMaxHumidity>2 ) then
                    stringVacMaxHumidity = stringVacMaxHumidity:sub(2,#stringVacMaxHumidity)
                end

                local valueVacMaxHumidity = tonumber(string.match(stringVacMaxHumidity, "%d+"))
                self:updateProperty('ui.Current_Vacation_Max_Hum_Value_Label.caption', valueVacMaxHumidity..'%') 
                self:updateProperty('currentVacationMaxHumidity', valueVacMaxHumidity..'%' )
            end
        
        elseif (string.find(data, 'VACFAN:') ) then  --response format: S1VACFAN:AUTO
            local stringVacFanSetting = data:match(':(.*)')
            if (stringVacFanSetting == 'ACK') then --this is configuration command response
                print('configuration command response')
            elseif (string.find(stringVacFanSetting, 'NAK')) then --this means invalid command (ex. wrong parameter was send to Carrier)
                print('Invalid command')
            else  -- this is status command resopnse 
                if (stringVacFanSetting == 'MED') then
                     stringVacFanSetting = 'MEDIUM'
                end

                self:updateProperty('ui.Current_Vacation_Fan_Speed_Value_Label.caption', stringVacFanSetting) 
                self:updateProperty('currentVacationFanSetting',stringVacFanSetting)
            end
        
        elseif (string.find(data, 'OAT:') ) then  --response format: S1OAT:xxx F/C
            local stringOutdorTemp = data:match(':(.*)')
            if (stringOutdorTemp == 'ACK') then --this is configuration command response
                print('configuration command response')
            elseif (string.find(stringOutdorTemp, 'NAK')) then --this means invalid command (ex. wrong parameter was send to Carrier)
                print('Invalid command')
            else  -- this is status command resopnse 
                self:updateProperty('ui.Current_Outside_Temp_Value_Label.caption', stringOutdorTemp) 
                self:updateProperty('currentOutsideTemperature',stringOutdorTemp)
            end
        
        elseif (string.find(data, 'NAME:') ) then  --response format: S1Z1NAME:NAME
            local stringName = data:match(':(.*)')
            if (stringName == 'ACK') then --this is configuration command response
                print('configuration command response')
            elseif (string.find(stringName, 'NAK')) then --this means invalid command (ex. wrong parameter was send to Carrier)
                print('Invalid command')
            else  -- this is status command resopnse 
                zoneToRefresh:updateProperty('ui.Current_Name_Valu_Label.caption', stringName) 
                zoneToRefresh:updateProperty('currentZoneName',stringName)
            end
        end
    end

    self:synchronizePropertiesFromActiveZone()
                    print('!!!!!!!!!!!!!!!!!!polling2')

    self.nextCommandTimeoutId = setTimeout(function( ) self:startPolling(); end, 15000)
    self:sendNextCommand()
end

--! Gets properties from active zone that is currently setting to some values.
--! Synchronize this values with data (user interface elements)in main Carrier plugin.
--! Checking if active zone is equal to selecetd zone in main plugin.
function Carrier:synchronizePropertiesFromActiveZone()
    local zoneObject = self:getActiveZone()
    if (zoneObject) then
        for zoneProperty,zoneValue in pairs(zoneObject.properties) do
            if (zoneProperty:sub(1,3) == 'ui.') then
                for selfProperty, selfValue in pairs(self.properties) do
                    if (zoneProperty == selfProperty) then
                        self:updateProperty(selfProperty, zoneValue)
                    end
                end
            end
        end
    end
end

--! Setting currently selected system id, and updating appropriate UI element.
--@param systgemId SystemId that should be set. 
function Carrier:setSystemId(systemId)
    if(tableSize(zones) <=8 and systemId == 2) then
        self:updateProperty('ui.System_Value_Label.caption', 'System '..systemId..' Not ava.')
    else
        self:updateProperty('systemId',systemId)
        self:updateProperty('ui.System_Value_Label.caption', 'System '..systemId)
    end
end

--! Setting currently selected zone id, and updating appropriate UI element.  
--@param zoneId ZoneId that should be set. 
function Carrier:setZoneId(zoneId)
    if (tableSize(zones) < zoneId ) then
        self:updateProperty('ui.Zone_Value_Label.caption', 'Zone '..zoneId..' Not ava.')
    else
        self:updateProperty('zoneId',zoneId)
        self:updateProperty('ui.Zone_Value_Label.caption', 'Zone '..zoneId)
    end
end

--! Bunch of callbacks from configuration buttons actoin. 
--! Sets auto configuration (ON/OF) on Carrier thermostat.
function Carrier:setAutoConfiguration()
    self.carrierConfig:setAutoConfigForThermostat(self.properties.autoConfigurationInput)
end

--! Sets units (E-imperial/M-metric) on Carrier thermostat.
function Carrier:setTempUnit()
    self.carrierConfig:setTempUnit(self.properties.units)
end

--! Sets heating mode (Auto/Cool/Heat/Off/Emergency heat) on Carrier thermostat.
function Carrier:setMode()
    self.carrierConfig:setThermostatMode(self.properties.modeInput)
end

--! Sets currnet day and time on Carrier thermostat.
function Carrier:setDay()
    self.carrierConfig:setDay(self.properties.dayNumberInput)
    self.carrierConfig:setTime(self.properties.hourInput, self.properties.minuteInput, self.properties.periodInput)
end

--! Sets filter usage to 0% on Carrier thermostat.
function Carrier:resetFilter()
    self.carrierConfig:setResetFilterLife('0')
end

--! Sets UV Lamp usage to 0% on Carrier thermostat.
function Carrier:resetUvLamp()
    self.carrierConfig:setResetUvlampLife('0')
end

--! Sets Humidity filter usage to 0% on Carrier thermostat.
function Carrier:resetHumidifier()
    self.carrierConfig:setResetHumidifierPadLife('0')
end

--! Sets Ventilator filter usage to 0% on Carrier thermostat.
function Carrier:resetVentilator()
    self.carrierConfig:setResetVentilatorFilterLife('0')
end

--! Sets all reminders (ON/OFF) on Carrier thermostat.
function Carrier:setReminder()
    self.carrierConfig:setFilterReminder(self.properties.filterReminderInput)
    self.carrierConfig:setUvlampReminder(self.properties.uvlampReminderInput)
    self.carrierConfig:setHumidifierPadReminder(self.properties.humidifierReminderInput)
    self.carrierConfig:setVentilationPadReminder(self.properties.ventilatorPadReminderInput)
end

--! Sets Deadband - (2/3/4/5/6) on Carrier thermostat.
function Carrier:setDeadband()
    self.carrierConfig:setDeadbandOfThermostat(self.properties.deadbandInput)
end

--! Sets allowed number of cycle per hour (2/3/4/5/6) on Carrier thermostat.
function Carrier:setThermostat()
    self.carrierConfig:setCyclesPerHourOfThermostat(self.properties.thermostatInput)
end

--! Sets programing availabilithy (ON/OFF) on Carrier thermostat.
function Carrier:setProgramming()
    self.carrierConfig:setProgrammingStateOfThermostat(self.properties.programmingStateInput)
    self.carrierConfig:setProgrammableFanSetting(self.properties.programableFanInput)
    self.carrierConfig:setNumberAllowedProgrammingPeriod(self.properties.periodProgInput)
end

--! Calculate array elements number.
--@param  array to be count. 
function tableSize(table)
    local count = 0
    for _,_ in pairs(table) do
        count = count + 1
    end

    return count
end
