--! Class responsible for getting and setting data to Carrier.
--! Those methosd are combined with main Carrier plugin class.
class 'CarrierConfig' 

function CarrierConfig:__init(parent)
    self.parent = parent
end

function CarrierConfig:getaAutoConfigurationOfTheThermostat() 
    print('CarrierConfig:getaAutoConfigurationOfTheThermostat')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.STATUS_COMMANDS['S_AUTO_CONFIGURATION_OF_THE_THERMOSTAT']..          
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierConfig:setAutoConfigForThermostat(state)
    print('CarrierConfig:setAutoConfigForThermostat')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.CONFIGURATION_COMMANDS['S_SET_AUTO_CONFIG_FOR_THERMOSTAT__ON_OF']..
                state..         
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierConfig:getTempUnit()
    print('CarrierConfig:getTempUnit')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.STATUS_COMMANDS['S_UNITS_OF_THE_THERMOSTAT']..     
                self.parent.COMMAND_TERMINATOR   
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd) 
end

function CarrierConfig:setTempUnit(unit) 
    print('CarrierConfig:setUnits')

    local unitsToSend = ''
    if (unit == 'C') then
        unitsToSend = 'M'
    elseif (unit == 'F') then
        unitsToSend = 'E'
    end
    
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.CONFIGURATION_COMMANDS['S_SET_UNITS_OF_THE_THERMOSTAT__E_M'].. 
                unitsToSend..         
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierConfig:getMode() 
    print('CarrierConfig:getMode')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.STATUS_COMMANDS['S_MODE']..          
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierConfig:setThermostatMode(mode) 
    print('CarrierConfig:setThermostatMode')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.CONFIGURATION_COMMANDS['S_SET_THERMOSTAT_MODE__HEAT_COOL_AUTO_OFF_EHEAT'].. 
                mode..         
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierConfig:getDay() 
    print('CarrierConfig:getDay')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.STATUS_COMMANDS['S_DAY']..          
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierConfig:setDay(day)   
    local reqDay = day

    if (reqDay < 0) then
        print("unsuported value of day: "..reqDay)
        reqDay = 0
    elseif (reqDay > 6) then
        print("unsuported value of day: "..reqDay)
        reqDay = 6
    end    

    local cmd = 'S'..self.parent.properties.systemId..        
                self.parent.CONFIGURATION_COMMANDS['S_SET_DAY__0_6']..          
                reqDay..
                self.parent.COMMAND_TERMINATOR

    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierConfig:getTime() 
    print('CarrierConfig:getTime')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.STATUS_COMMANDS['S_TIME']..          
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierConfig:setTime(hourParam, minuteParam, periodParam) 
    local hour = hourParam
    local minute = minuteParam
    local period = periodParam
   
    if (hour < 1) then
        print("unsuported value of hour: "..hour)
        hour = 1
    elseif (hour > 12) then
        print("unsuported value of hour: "..hour)
        hour = 12
    end

    if (minute < 0) then
        print("unsuported value of minute: "..minute)
        minute = 0
    elseif (minute > 59) then
        print("unsuported value of minute: "..minute)
        minute = 59
    end
    
    if (hour < 10) then
        hour = '0'..hour
    end
    if (minute < 10) then
        minute = '0'..minute
    end

    local reqTime = hour..':'..minute..' '..period

    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.CONFIGURATION_COMMANDS['S_SET_TIME__TIME_A_P']..          
                reqTime..
                self.parent.COMMAND_TERMINATOR

    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierConfig:getZoneNumber() 
    print('CarrierConfig:getZoneNumber')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.STATUS_COMMANDS['S_ZONE_NUMBER']..          
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierConfig:setZoneNumber(zoneNumber) 
    print('CarrierConfig:setZoneNumber')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.CONFIGURATION_COMMANDS['S_SET_ZONE_NUMBER__1_8'].. 
                zoneNumber..         
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierConfig:getFilterLife() 
    print('CarrierConfig:getFilterLife')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.STATUS_COMMANDS['S_FILTER_LIFE']..          
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierConfig:setResetFilterLife(command) 
    print('CarrierConfig:setResetFilterLife')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.CONFIGURATION_COMMANDS['S_RESET_FILTER_LIFE__0'].. 
                command..         
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierConfig:getUvlapLive() 
    print('CarrierConfig:getUvlapLive')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.STATUS_COMMANDS['S_UVLAP_LIVE']..          
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierConfig:setResetUvlampLife(command) 
    print('CarrierConfig:setResetUvlampLife')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.CONFIGURATION_COMMANDS['S_RESET_UVLAMP_LIFE__0'].. 
                command..         
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierConfig:getHumidifierPadLife() 
    print('CarrierConfig:getHumidifierPadLife')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.STATUS_COMMANDS['S_HUMIDIFIER_PAD_LIFE']..          
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierConfig:setResetHumidifierPadLife(command) 
    print('CarrierConfig:setResetHumidifierPadLife')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.CONFIGURATION_COMMANDS['S_RESET_HUMIDIFIER_PAD_LIFE__0'].. 
                command..         
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierConfig:getResetVentilatorFilterLife() 
    --there is no this get method command in documentation
end

function CarrierConfig:setResetVentilatorFilterLife(command)
    print('CarrierConfig:setResetVentilatorFilterLife')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.CONFIGURATION_COMMANDS['S_RESET_VENTILATOR_FILTER_LIFE__0'].. 
                command..         
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end 

function CarrierConfig:getFilterReminderSetting() 
    print('CarrierConfig:getFilterReminderSetting')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.STATUS_COMMANDS['S_FILTER_REMINDER_SETTING']..          
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end    

function CarrierConfig:setFilterReminder(status) 
    print('CarrierConfig:setFilterReminder')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.CONFIGURATION_COMMANDS['S_SET_FILTER_REMINDER__ON_OFF'].. 
                status..         
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierConfig:getLampReminderSetting() 
    print('CarrierConfig:getLampReminderSetting')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.STATUS_COMMANDS['S_LAMP_REMINDER_SETTING']..          
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierConfig:setUvlampReminder(status) 
    print('CarrierConfig:setUvlampReminder: ')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.CONFIGURATION_COMMANDS['S_SET_UVLAMP_REMINDER__ON_OFF'].. 
                status..         
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierConfig:getHumidifierPadReminderSetting() 
    print('CarrierConfig:getHumidifierPadReminderSetting')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.STATUS_COMMANDS['S_HUMIDIFIER_PAD_REMINDER_SETTING']..          
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierConfig:setHumidifierPadReminder(status) 
    print(status)
    print('CarrierConfig:setHumidifierPadReminder')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.CONFIGURATION_COMMANDS['S_SET_HUMIDIFIER_PAD_REMINDER__ON_OFF'].. 
                status..         
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierConfig:getVentilationPadReminder() 
    --there is no this get method command in documentation
end

function CarrierConfig:setVentilationPadReminder(status)
    print('CarrierConfig:setVentilationPadReminder')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.CONFIGURATION_COMMANDS['S_SET_VENTILATION_PAD_REMINDER__ON_OFF'].. 
                status..         
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierConfig:getVacationState() 
    print('CarrierConfig:getVacationState')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.STATUS_COMMANDS['S_VACATION_STATE']..          
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierConfig:setVacationState(value) 
    --vacation state is an number from 0 to 100 - 0 means vac state is off
    local currentVacDaysString = self.parent.properties.currentVacationDays
    if (currentVacDaysString == '' or currentVacDaysString == nil) then
        print 'must wait for currentVacDaysString proprerty refresh'
        self:getVacationDays()
        return
    end      

    local vacDaysString = self.parent.properties.vacationDays
    if (vacDaysString == '' or vacDaysString == nil or vacDaysString == '---') then
        print 'updating heatSetpoint'
        vacDaysString = currentVacDaysString
    end  

    local currentVacDaysInt = tonumber(vacDaysString)

    currentVacDaysInt = currentVacDaysInt + value;

    if (currentVacDaysInt < 0) then
        print("unsuported value of currentVacDays: "..currentVacDaysInt)
        currentVacDaysInt = 0
    elseif (currentVacDaysInt > 100) then
        print("unsuported value of currentVacDays: "..currentVacDaysInt)
        currentVacDaysInt = 100
    end
    
    if (currentVacDaysInt < 100 and currentVacDaysInt >= 10) then
        vacDaysString = '0'..currentVacDaysInt
    elseif (currentVacDaysInt < 10) then
        vacDaysString = '00'..currentVacDaysInt
    else
        vacDaysString = tostring(currentVacDaysInt)
    end

    local prefix = 'S'..self.parent.properties.systemId..
                    self.parent.CONFIGURATION_COMMANDS['S_SET_VACATION_STATE__XXX']
    local cmd = prefix..         
                vacDaysString..
                self.parent.COMMAND_TERMINATOR

    print('Command: '..cmd)
    self:getVacationDays() -- it will become second command in the quque after next line
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, prefix) --response format: S1Z1HTSP: xxx°F/C

    if (vacDaysString:sub(1,1) == '0') then
        vacDaysString = vacDaysString:sub(2,#vacDaysString)
        if (vacDaysString:sub(1,1) == '0') then
            vacDaysString = vacDaysString:sub(2,#vacDaysString)
        end
    end
    self.parent:updateProperty('ui.Vacation_State_Value_Label.caption', vacDaysString) 
    self.parent:updateProperty('vacationDays', vacDaysString )

    if (currentVacDaysInt ~= 0) then
        self.parent:updateProperty('vacationState', 'ON' )
    else
        self.parent:updateProperty('vacationState', 'OFF' )
    end
end

function CarrierConfig:getVacationDays() 
    print('CarrierConfig:getVacationDays')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.STATUS_COMMANDS['S_VACATION_DAYS']..          
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end      

-- There is no method setVacatonDays in Carrier documentation -- days will be changed if 
-- method  setCacationState provide number grather than 0 else vacation state is set to OFF
function CarrierConfig:getVacationMinimumTemperature() 
    print('CarrierConfig:getVacationMinimumTemperature')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.STATUS_COMMANDS['S_VACATION_MINIMUM_TEMPERATURE']..          
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierConfig:setVacationMinTemperature(value) 
    local currentVacMinTempString = self.parent.properties.currentVacationMinTemperature
    if (currentVacMinTempString == '' or currentVacMinTempString == nil) then
        print 'must wait for currentVacMinTempString proprerty refresh'
        self:getVacationMinimumTemperature()
        return
    end   

    local vacMinTemp = self.parent.properties.vacationMinTemperature
    if (vacMinTemp == '' or vacMinTemp == nil or vacMinTemp == '---') then
        print 'updating vacMinTemp'
        vacMinTemp = currentVacMinTempString
    end  

    print('string: '.. vacMinTemp)
    local valueVacMinTemp = tonumber(string.match(vacMinTemp, "%d+"))
    print('value: '.. valueVacMinTemp)
    local unitsVacMinTemp = string.match(currentVacMinTempString, "%a+")
    print('units: '.. unitsVacMinTemp)

    if (unitsVacMinTemp ~= self.parent.properties.units) then
        print ('need to unify units ')
        self:getVacationMinimumTemperature()
        return
    end

    local temp = valueVacMinTemp + value

    if (unitsVacMinTemp == 'C') then
        if (temp < 0) then
            print("unsuported value of setPoint: "..temp)
            temp = 0
        elseif (temp > 35) then
            print("unsuported value of setPoint: "..temp)
            temp = 35
        end
    else
        if (temp < 32) then
            print("unsuported value of setPoint: "..temp)
            temp = 32
        elseif (temp > 95) then
            print("unsuported value of setPoint: "..temp)
            temp = 95
        end
    end    

    if (temp < 10) then
        temp = '0'..temp
    end

    local prefix = 'S'..self.parent.properties.systemId..
                    self.parent.CONFIGURATION_COMMANDS['S_SET_VACATION_MIN_TEMPERATURE__XX']
    local cmd = prefix..        
                temp..
                self.parent.COMMAND_TERMINATOR

    print('Command: '..cmd)

    self:getVacationMinimumTemperature() -- it will become second command in the quque after next line
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, prefix) --response format: S1Z1HTSP: xxx°F/C
    self.parent:updateProperty('ui.Vacation_Min_Temp_Value_Label.caption', temp..'°'..unitsVacMinTemp) 
    self.parent:updateProperty('vacationMinTemperature', temp..'°'..unitsVacMinTemp )
end

function CarrierConfig:getVacationMaximumTemperature() 
    print('CarrierConfig:getVacationMaximumTemperature')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.STATUS_COMMANDS['S_VACATION_MAXIMUM_TEMPERATURE']..          
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierConfig:setVacationMaxTemperature(value)
    local currentVacMaxTempString = self.parent.properties.currentVacationMaxTemperature
    if (currentVacMaxTempString == '' or currentVacMaxTempString == nil) then
        print 'must wait for currentVacMaxTempString proprerty refresh'
        self:getVacationMaximumTemperature()
        return
    end   

    local vacMaxTemp = self.parent.properties.vacationMaxTemperature
    if (vacMaxTemp == '' or vacMaxTemp == nil or vacMaxTemp == '---') then
        print 'updating vacMaxTemp'
        vacMaxTemp = currentVacMaxTempString
    end  

    print('string: '.. vacMaxTemp)
    local valueVacMaxTemp = tonumber(string.match(vacMaxTemp, "%d+"))
    print('value: '.. valueVacMaxTemp)
    local unitsVacMaxTemp = string.match(currentVacMaxTempString, "%a+")
    print('units: '.. unitsVacMaxTemp)

    if (unitsVacMaxTemp ~= self.parent.properties.units) then
        print ('need to unify units ')
        self:getVacationMaximumTemperature()
        return
    end

    local temp = valueVacMaxTemp + value

    if (unitsVacMaxTemp == 'C') then
        if (temp < 0) then
            print("unsuported value of setPoint: "..temp)
            temp = 0
        elseif (temp > 35) then
            print("unsuported value of setPoint: "..temp)
            temp = 35
        end
    else
        if (temp < 32) then
            print("unsuported value of setPoint: "..temp)
            temp = 32
        elseif (temp > 95) then
            print("unsuported value of setPoint: "..temp)
            temp = 95
        end
    end    

    if (temp < 10) then
        temp = '0'..temp
    end

    local prefix = 'S'..self.parent.properties.systemId..
                self.parent.CONFIGURATION_COMMANDS['S_SET_VACATION_MAX_TEMPERATURE__XX']        
    local cmd = prefix..
                temp..
                self.parent.COMMAND_TERMINATOR

    print('Command: '..cmd)
    self:getVacationMaximumTemperature() -- it will become second command in the quque after next line
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, prefix) 
    self.parent:updateProperty('ui.Vacation_Max_Temp_Value_Label.caption', temp..'°'..unitsVacMaxTemp) 
    self.parent:updateProperty('vacationMaxTemperature', temp..'°'..unitsVacMaxTemp )
end

function CarrierConfig:getVacationMinimumHumidity() 
    print('CarrierConfig:getVacationMinimumHumidity')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.STATUS_COMMANDS['S_VACATION_MINIMUM_HUMIDITY']..          
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierConfig:setVacationMinHumidity(value)
    print('CarrierConfig:setVacationMinHumidity')
    --Vale must be provided with leading zero (xxx)

    local prefix = 'S'..self.parent.properties.systemId..
                    self.parent.CONFIGURATION_COMMANDS['S_SET_VACATION_MIN_HUMIDITY__XXX']
    local cmd = prefix..
                value..         
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self:getVacationMinimumHumidity() -- it will become second command in the quque after next line
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, prefix)

    if (value:sub(1,1) == '0') then
        value = value:sub(2,#value)
        if (value:sub(1,1) == '0') then
            value = value:sub(2,#value)
        end
    end
    self.parent:updateProperty('ui.Vacation_Min_Hum_Value_Label.caption', value..'%') 
    self.parent:updateProperty('vacationMinHumidity', value..'%' )
end

function CarrierConfig:getVacationMaximumHimidity() 
    print('CarrierConfig:getVacationMaximumHimidity')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.STATUS_COMMANDS['S_VACATION_MAXIMUM_HIMIDITY']..          
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end   

function CarrierConfig:setVacationMaxHumidity(value)
    print('CarrierConfig:setVacationMaxHumidity')
    --Vale must be provided with leading zero (xxx)
    local prefix = 'S'..self.parent.properties.systemId..
                    self.parent.CONFIGURATION_COMMANDS['S_SET_VACATION_MAX_HUMIDITY__XXX']
    local cmd = prefix..
                value..         
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self:getVacationMaximumHimidity() -- it will become second command in the quque after next line
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, prefix)

    if (value:sub(1,1) == '0') then
        value = value:sub(2,#value)
    end
    self.parent:updateProperty('ui.Vacation_Max_Hum_Value_Label.caption', value..'%') 
    self.parent:updateProperty('vacationMaxHumidity', value..'%' )
end

function CarrierConfig:getVacationFanSetting() 
    print('CarrierConfig:getVacationFanSetting')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.STATUS_COMMANDS['S_VACATION_FAN_SETTING']..          
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierConfig:setVacationFanSetting(mode)
    print('CarrierConfig:setVacationFanSetting')

    local prefix = 'S'..self.parent.properties.systemId..
                self.parent.CONFIGURATION_COMMANDS['S_SET_VACATION_FAN_SETTING__AUTO_LOW_MED_HIGH'] 
    local cmd = prefix..
                mode..         
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)

    self:getVacationFanSetting()
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, prefix)

    if (mode == 'MED') then
         mode = 'MEDIUM'
    end
    self.parent:updateProperty('ui.Vacation_Fan_Speed_Value_Label.caption', mode) 
    self.parent:updateProperty('vacationFanSetting', mode)
end

function CarrierConfig:getDeadbandOfTheThermostat() 
    print('CarrierConfig:getDeadbandOfTheThermostat')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.STATUS_COMMANDS['S_DEADBAND_OF_THE_THERMOSTAT']..          
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierConfig:setDeadbandOfThermostat(value)
    print('CarrierConfig:setDeadbandOfThermostat')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.CONFIGURATION_COMMANDS['S_SET_DEADBAND_OF_THERMOSTAT__2_6'].. 
                value..         
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierConfig:getCyclesPerHourOfTheTermostat() 
    print('CarrierConfig:getCyclesPerHourOfTheTermostat')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.STATUS_COMMANDS['S_CYCLES_PER_HOUR_OF_THE_TERMOSTAT']..          
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierConfig:setCyclesPerHourOfThermostat(value)
    print('CarrierConfig:setCyclesPerHourOfThermostat')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.CONFIGURATION_COMMANDS['S_SET_CYCLES_PER_HOUR_OF_THERMOSTAT__2_6'].. 
                value..         
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierConfig:getProgrammableFanSetting() 
    print('CarrierConfig:getProgrammableFanSetting')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.STATUS_COMMANDS['S_PROGRAMMABLE_FAN_SETTING']..          
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierConfig:setProgrammableFanSetting(state)
    print('CarrierConfig:setProgrammableFanSetting')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.CONFIGURATION_COMMANDS['S_SET_PROGRAMMABLE_FAN_SETTING__ON_OFF'].. 
                state..         
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
   self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierConfig:getProgrammingStateOfTheThermostat() 
    print('CarrierConfig:getProgrammingStateOfTheThermostat')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.PROGRAMMING_INFO_COMMANDS['S_PROGRAMMING_STATE_OF_THE_THERMOSTAT']..          
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierConfig:setProgrammingStateOfThermostat(state)
    print('CarrierConfig:setProgrammingStateOfThermostat')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.PROGRAMMING_COMMANDS['S_SET_PROGRAMMING_STATE_OF_THERMOSTAT__ON_OFF'].. 
                state..         
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierConfig:getNumberOfPeriodsAllowedForProgramming() 
    print('CarrierConfig:getNumberOfPeriodsAllowedForProgramming')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.PROGRAMMING_INFO_COMMANDS['S_NUMBER_OF_PERIODS_ALLOWED_FOR_PROGRAMMING']..          
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierConfig:setNumberAllowedProgrammingPeriod(value)
    print('CarrierConfig:setNumberAllowedProgrammingPeriod')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.PROGRAMMING_COMMANDS['S_SET_NUMBER_ALLOWED_PROGRAMMING_PERIOD__2or4'].. 
                value..         
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierConfig:resetFactoryDefaults(value)
    print('CarrierConfig:setResetFactoryDefaults')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.PROGRAMMING_COMMANDS['S_RESET_FACTORY_DEFAULTS__RESET'].. 
                value..         
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
    self:updateProperty('resetFactoryDefaults', value)
end 

function CarrierConfig:getDisplayedHumidity() 
    print('CarrierConfig:getDisplayedHumidity')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.STATUS_COMMANDS['S_Z_DISPLAYED_HUMIDITY']..          
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end   

function CarrierConfig:getOutdorTemperature() 
    print('CarrierConfig:getOutdorTemperature')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.STATUS_COMMANDS['S_OUTDOR_TEMPERATURE']..          
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end 

function CarrierConfig:getSystemType() 
    print('CarrierConfig:getSystemType')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.STATUS_COMMANDS['S_SYSTEM_TYPE']..          
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end 

function CarrierConfig:getCurrentProgramPeriod() 
    print('CarrierConfig:getCurrentProgramPeriod')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.STATUS_COMMANDS['S_CURRENT_PROGRAM_PERIOD']..          
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierConfig:getHumidificationTarget() 
    print('CarrierConfig:getHumidificationTarget')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.STATUS_COMMANDS['S_Z1_HUMIDIFICATION_TARGET']..          
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierConfig:getHumidifierState() 
    print('CarrierConfig:getHumidifierState')
    local cmd = 'S'..self.parent.properties.systemId..
                self.parent.STATUS_COMMANDS['S_HUMIDIFIER_STATE']..          
                self.parent.COMMAND_TERMINATOR
    print('Command: '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end