require('common.device')

class 'CarrierZone' (Device)

--! Initialize CarrierZone object
--@param parent Object of main Carrier clas plugin. 
--@param dev device object
function CarrierZone:__init(parent, dev)

    print('Zone Init '..type(dev))
	Device.__init(self, dev)
	self.parent = parent

    --! Properties are setting on init becouse we do not refreshig them in infinit loop queue.
    self:updateProperty('programmingDay', 'MON')
    self:updateProperty('programmingHour', '06')
    self:updateProperty('programmingMinutes', '00')
    self:updateProperty('programmingTimePeriod', 'A')
    self:updateProperty('programmingPeriod', 'WAKE')
    self:updateProperty('programmingHeatSetpoint', '20')
    self:updateProperty('programmingCoolSetpoint', '20')
    self:updateProperty('programmingFanSpped', 'AUTO')

    self:updateProperty('ui.Programming_Z_Day_Value_Label.caption', 'MON')
    self:updateProperty('ui.Programming_Z_Period_Value_Label.caption', 'WAKE')
    self:updateProperty('ui.Programming_Z_PERIOD_Value_NL.caption', 'A')
    self:updateProperty('ui.Programming_Z_Heat_Setpoint_Value_Label.caption', '20')
    self:updateProperty('ui.Programming_Z_Cool_Setpoint_Value_Label.caption', '25')
    self:updateProperty('ui.Programming_Z_Fan_Speed_Value_Label.caption', 'AUTO')
    self:updateProperty('ui.Programming_Z_Hour_VL.caption', '06')
    self:updateProperty('ui.Programming_Z_Minuts_VL.caption', '00')
end

function CarrierZone:getRoomTemperature() 
    local cmd = 'S'..self.properties.systemId..
                'Z'..self.properties.zoneId..
                self.parent.STATUS_COMMANDS['S_Z_ROOM_TEMPERATURE']..          
                self.parent.COMMAND_TERMINATOR

    print('getCurrentTemp CarrierZone: '..self.properties.zoneId..
            ' System: '..self.properties.systemId..' command to send : '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierZone:getHeatSetpoint()
    local cmd = 'S'..self.properties.systemId..
                'Z'..self.properties.zoneId..
                self.parent.STATUS_COMMANDS['S_Z_HEAT_SETPOINT']..          
                self.parent.COMMAND_TERMINATOR

    print('getHeatSetpoint CarrierZone: '..self.properties.zoneId..
            ' System: '..self.properties.systemId..' command to send : '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierZone:setHeatSetpoint(command, hour)
    local currentHeatSetpoint = self.properties.currentHeatSetpoint
    if (currentHeatSetpoint == '' or currentHeatSetpoint == nil) then
        print 'must wait for setHeatSetpoint proprert refresh'
        self:getHeatSetpoint()
        return
    end  

    local heatSetpoint = self.properties.heatSetpoint
    if (heatSetpoint == '' or heatSetpoint == nil or heatSetpoint == '---') then
        print 'updating heatSetpoint'
        heatSetpoint = currentHeatSetpoint
    end  

    print('string: '.. heatSetpoint)
    local valueHeatSeatpoint = tonumber(string.match(heatSetpoint, "%d+"))
    print('value: '.. valueHeatSeatpoint)
    local unitsHeatSeatpoint = string.match(currentHeatSetpoint, "%a+")
    print('units: '.. unitsHeatSeatpoint)

    if (unitsHeatSeatpoint ~= self.parent.properties.units) then
        self:getHeatSetpoint()
        return
    end

    local temp = valueHeatSeatpoint + command

    if (unitsHeatSeatpoint == 'C') then
        if (temp < 0) then
            print("unsuported value of setPoint: "..temp)
            temp = 0
        elseif (temp > 35) then
            print("unsuported value of setPoint: "..temp)
            temp = 35
        end
    else
        if (temp < 32) then
            print("unsuported value of setPoint: "..temp)
            temp = 32
        elseif (temp > 95) then
            print("unsuported value of setPoint: "..temp)
            temp = 95
        end
    end    

    if (temp < 10) then
        temp = '0'..temp
    end

    local prefix =  'S'..self.properties.systemId..
                    'Z'..self.properties.zoneId..
                     self.parent.CONFIGURATION_COMMANDS['S_Z_SET_HEAT_SETPOINT__XX_TIME']

    local cmd = prefix..     
                temp..
                self.parent.COMMAND_TERMINATOR

    print('setHeatSetpoint CarrierZone: '..self.properties.zoneId..
            ' System: '..self.properties.systemId..' command to send : '..cmd)

    self:getHeatSetpoint() -- it will become second command in the quque after next line
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, prefix) 
    self:updateProperty('ui.Heat_Setpoint_Value_Label.caption', temp..'°'..unitsHeatSeatpoint) 
    self:updateProperty('heatSetpoint', temp..'°'..unitsHeatSeatpoint )

end

function CarrierZone:getCoolSetpoint()
    local cmd = 'S'..self.properties.systemId..
                'Z'..self.properties.zoneId..
                self.parent.STATUS_COMMANDS['S_Z_COOL_SETPOINT']..          
                self.parent.COMMAND_TERMINATOR

    print('getCoolSetpoint CarrierZone: '..self.properties.zoneId..
            ' System: '..self.properties.systemId..' command to send : '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd) 
end

function CarrierZone:setCoolSetpoint(command, hour)
    local currentCoolSetpoint = self.properties.currentCoolSetpoint
    if (currentCoolSetpoint == '' or currentCoolSetpoint == nil) then
        print 'must wait for CoolSetpoint refresh'
        self:getCoolSetpoint()
        return
    end  

    local coolSetpoint = self.properties.coolSetpoint
    if (coolSetpoint == '' or coolSetpoint == nil or coolSetpoint == '---') then
        print 'updating coolSetpoint'
        coolSetpoint = currentCoolSetpoint
    end      

    print('string: '.. coolSetpoint)
    local valueCoolSeatpoint = tonumber(string.match(coolSetpoint, "%d+"))
    print('value: '.. valueCoolSeatpoint)
    local unitsCoolSeatpoint = string.match(currentCoolSetpoint, "%a+")
    print('units: '.. unitsCoolSeatpoint)

    if (unitsCoolSeatpoint ~= self.parent.properties.units) then
        print ('+++++++++++++++++++++++++++++++++++++++')
        self:getCoolSetpoint()
        return
    end

    local temp = valueCoolSeatpoint + command

    if (unitsCoolSeatpoint == 'C') then
        if (temp < 0) then
            print("unsuported value of setPoint: "..temp)
            temp = 0
        elseif (temp > 35) then
            print("unsuported value of setPoint: "..temp)
            temp = 35
        end
    else
        if (temp < 32) then
            print("unsuported value of setPoint: "..temp)
            temp = 32
        elseif (temp > 95) then
            print("unsuported value of setPoint: "..temp)
            temp = 95
        end
    end    

    if (temp < 10) then
        temp = '0'..temp
    end

    local  prefix = 'S'..self.properties.systemId..
                    'Z'..self.properties.zoneId..
                    self.parent.CONFIGURATION_COMMANDS['S_Z_SET_COOL_SETPOINT__XX_TIME']

    local cmd = prefix..          
                temp..
                self.parent.COMMAND_TERMINATOR

    print('setCoolSetpoint CarrierZone: '..self.properties.zoneId..
            ' System: '..self.properties.systemId..' command to send : '..cmd)
    self:getCoolSetpoint() -- it will become second command in the quque after next line 
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, prefix) 
    self:updateProperty('ui.Cool_Setpoint_Value_Label.caption', temp..'°'..unitsCoolSeatpoint) 
    self:updateProperty('coolSetpoint', temp..'°'..unitsCoolSeatpoint )
end

function CarrierZone:getOverrideState() 
    local cmd = 'S'..self.properties.systemId..
                'Z'..self.properties.zoneId..
                self.parent.STATUS_COMMANDS['S_Z_OVERRIDE_STATE']..          
                self.parent.COMMAND_TERMINATOR

    print('getOverrideState CarrierZone: '..self.properties.zoneId..
            ' System: '..self.properties.systemId..' command to send : '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierZone:getOverrideTimer() 
    local cmd = 'S'..self.properties.systemId..
                'Z'..self.properties.zoneId..
                self.parent.STATUS_COMMANDS['S_Z_OVERRIDE_TIMER']..          
                self.parent.COMMAND_TERMINATOR

    print('getOverrideTimer CarrierZone: '..self.properties.zoneId..
            ' System: '..self.properties.systemId..' command to send : '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierZone:setOverrideTimer(value, part) 
    local currentOverrideTimer = self.properties.currentOverrideTimer
    if (currentOverrideTimer == '' or currentOverrideTimer == nil) then
        print 'must wait for time proprert refresh'
        self:getOverrideTimer()
        return
    end  

    local overrideTimer = self.properties.overrideTimer
    if (overrideTimer == '' or overrideTimer == nil or overrideTimer == '---') then
        print 'updating overrideTimer'
        overrideTimer = currentOverrideTimer
    end   


    print('string: '.. overrideTimer)
    local hour = tonumber(string.match(overrideTimer, "%d+"))
    local minute = tonumber(overrideTimer:match(hour..':(.*)'))
    print('valueHour: '.. hour)
    print('valueMinte: '.. minute)


    if (part == 'hour') then
        hour = hour + value
        if (hour < 0) then
            print("unsuported value of hour: "..hour)
            hour = 0
        elseif (hour > 23) then
            print("unsuported value of hour: "..hour)
            hour = 23
        end
    else
        minute = minute + value
        if (minute < 0) then
            print("unsuported value of minute: "..minute)
            minute = 0
        elseif (minute > 59) then
            print("unsuported value of minute: "..minute)
            minute = 59
        end
    end    

    if (hour < 10) then
        hour = '0'..hour
    end
    if (minute < 10) then
        minute = '0'..minute
    end

    local reqTime = hour..':'..minute

    local prefix =  'S'..self.properties.systemId..
                    'Z'..self.properties.zoneId..
                    self.parent.CONFIGURATION_COMMANDS['S_Z_SET_OVERRIDE_TIMER__TIMER'] 
    local cmd = prefix..
                reqTime..    
                self.parent.COMMAND_TERMINATOR

    print('setOverrideTimer CarrierZone: '..self.properties.zoneId..
            ' System: '..self.properties.systemId..' command to send : '..cmd)

    self:getOverrideTimer() -- it will become second command in the quque after next line 
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, prefix)
    self:updateProperty('ui.Override_Timer_Value_Hour_Label.caption', hour)
    self:updateProperty('ui.Override_Timer_Value_Minutes_Label.caption', minute)
    self:updateProperty('overrideTimer', reqTime )    
end

function CarrierZone:getFanSettings() 
    local cmd = 'S'..self.properties.systemId..
                'Z'..self.properties.zoneId..
                self.parent.STATUS_COMMANDS['S_Z_FAN_SETTINGS']..          
                self.parent.COMMAND_TERMINATOR

    print('getFanSettings CarrierZone: '..self.properties.zoneId..
            ' System: '..self.properties.systemId..' command to send : '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd) 
end

function CarrierZone:setFanSettings(speed) 
    local prefix =  'S'..self.properties.systemId..
                    'Z'..self.properties.zoneId..
                    self.parent.CONFIGURATION_COMMANDS['S_Z_SET_FAN_SETTINGS__AUTO_LOW_MED_HIGH']
    local cmd = prefix.. 
                speed..         
                self.parent.COMMAND_TERMINATOR

    print('setFanSettings CarrierZone: '..self.properties.zoneId..
            ' System: '..self.properties.systemId..' command to send : '..cmd)
    self:getFanSettings()
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, prefix)
    self:updateProperty('ui.Fan_Speed_Value_Label.caption', speed)
    self:updateProperty('fanSettings', speed )      
end

function CarrierZone:getHoldStatus() 
    local cmd = 'S'..self.properties.systemId..
                'Z'..self.properties.zoneId..
                self.parent.STATUS_COMMANDS['S_Z_HOLD_STATUS']..          
                self.parent.COMMAND_TERMINATOR

    print('getHoldStatus CarrierZone: '..self.properties.zoneId..
            ' System: '..self.properties.systemId..' command to send : '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierZone:setHoldStatus(status) 
    local prefix =  'S'..self.properties.systemId..
                    'Z'..self.properties.zoneId..
                    self.parent.CONFIGURATION_COMMANDS['S_Z_SET_HOLD_STATUS__ON_OFF']
    local cmd = prefix..
                status..         
                self.parent.COMMAND_TERMINATOR

    print('setHoldStatus CarrierZone: '..self.properties.zoneId..
            ' System: '..self.properties.systemId..' command to send : '..cmd)
    self:getHoldStatus()
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, prefix)
    self:updateProperty('ui.Hold_Status_Value_Label.caption', status)
    self:updateProperty('holdStatus', status ) 
end

function CarrierZone:getUnoccupiedStatus() 
    local cmd = 'S'..self.properties.systemId..
                'Z'..self.properties.zoneId..
                self.parent.STATUS_COMMANDS['S_Z_UNOCCUPIED_STATUS']..          
                self.parent.COMMAND_TERMINATOR

    print('getUnoccupiedStatus CarrierZone: '..self.properties.zoneId..
            ' System: '..self.properties.systemId..' command to send : '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierZone:setUnoccupiedStatus(status) 
    local prefix =  'S'..self.properties.systemId..
                    'Z'..self.properties.zoneId..
                    self.parent.CONFIGURATION_COMMANDS['S_Z_SET_UNOCCUPIED_STATUS__ON_OFF']
    local cmd = prefix.. 
                status..         
                self.parent.COMMAND_TERMINATOR

    print('setUnoccupiedStatus CarrierZone: '..self.properties.zoneId..
            ' System: '..self.properties.systemId..' command to send : '..cmd)
    self:getUnoccupiedStatus()
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, prefix)
    self:updateProperty('ui.Unoccupied_Status_Value_Label.caption', status)
    self:updateProperty('unoccupiedStatus', status ) 
end

function CarrierZone:getDisplayedHumidity() 
    local cmd = 'S'..self.properties.systemId..
                'Z'..self.properties.zoneId..
                self.parent.STATUS_COMMANDS['S_Z_DISPLAYED_HUMIDITY']..          
                self.parent.COMMAND_TERMINATOR

    print('getDisplayedHumidity CarrierZone: '..self.properties.zoneId..
          ' System: '..self.properties.systemId..' command to send : '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd) 
end

function CarrierZone:getZoneNumber() 
    return self.properties.zoneId
end

function CarrierZone:getZoneName() 
    local cmd = 'S'..self.properties.systemId..
                'Z'..self.properties.zoneId..
                self.parent.STATUS_COMMANDS['S_Z_ZONE_NAME']..          
                self.parent.COMMAND_TERMINATOR

    print('getZoneName CarrierZone: '..self.properties.zoneId..
            ' System: '..self.properties.systemId..' command to send : '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierZone:getProgramingInfo() 
    local cmd = 'S'..self.properties.systemId..
                'Z'..self.properties.zoneId..
                'PGM'..
                self.properties.programmingDay..
                self.properties.programmingPeriod..
                '?'..
                self.parent.COMMAND_TERMINATOR

    print('getProgramingInfo CarrierZone: '..self.properties.zoneId..
            ' System: '..self.properties.systemId..' command to send : '..cmd)
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, cmd)
end

function CarrierZone:setProgrammingInfo(day, period, hour, minutes, timePeriod, heatSetpoint, coolSetpoint, fanSpped) 
    local dayLocal = '' 
    local periodLocal = '' 
    local hourLocal = ''
    local minutesLocal = '' 
    local timePeriodLocal = '' 
    local heatSetpointLocal = ''
    local coolSetpointLocal = ''
    local fanSppedLocal = ''

    if(day) then
        dayLocal = day
        self:updateProperty('programmingDay', day)
        self:updateProperty('ui.Programming_Z_Day_Value_Label.caption', day)
    else
        dayLocal = self.properties.programmingDay
    end
    if(period) then
        periodLocal = period
        print('period'..period)
        self:updateProperty('programmingPeriod', period)
        self:updateProperty('ui.Programming_Z_Period_Value_Label.caption', period)
    else
        periodLocal = self.properties.programmingPeriod
    end
    if(hour) then 
        programmingHour = tonumber(self.properties.programmingHour)
        programmingHour = programmingHour + hour
        if (programmingHour < 1) then
            print("unsuported value of programmingHour: "..programmingHour)
            programmingHour = 1
        elseif (programmingHour > 12) then
            print("unsuported value of programmingHour: "..hour)
            programmingHour = 12
        end

        if (programmingHour < 10) then
        programmingHour = '0'..programmingHour
        end

        hourLocal = programmingHour
        self:updateProperty('programmingHour', hourLocal)
        self:updateProperty('ui.Programming_Z_Hour_VL.caption', hourLocal)
    else
        hourLocal = self.properties.programmingHour
    end
    if(minutes) then 
        programmingMinutes = tonumber(self.properties.programmingMinutes)
        programmingMinutes = programmingMinutes + minutes

        if (programmingMinutes < 0) then
            print("unsuported value of programmingMinutes: "..programmingMinutes)
            programmingMinutes = 0
        elseif (programmingMinutes > 59) then
            print("unsuported value of programmingMinutes: "..programmingMinutes)
            programmingMinutes = 59
        end
        if (programmingMinutes < 10) then
            programmingMinutes = '0'..programmingMinutes
        end
        minutesLocal = programmingMinutes
        self:updateProperty('programmingMinutes', minutesLocal)
        self:updateProperty('ui.Programming_Z_Minuts_VL.caption', minutesLocal)
    else
        minutesLocal = self.properties.programmingMinutes
    end
    if(timePeriod) then 
        timePeriodLocal = timePeriod
        self:updateProperty('programmingTimePeriod', timePeriodLocal)
        self:updateProperty('ui.Programming_Z_PERIOD_Value_NL.caption', timePeriodLocal)
    else
        timePeriodLocal = self.properties.programmingTimePeriod
    end
    if(heatSetpoint) then 
        programmingHeatSetpoint = tonumber(self.properties.programmingHeatSetpoint)
        programmingHeatSetpoint = programmingHeatSetpoint + heatSetpoint

        if (self.parent.properties.units == 'C') then
        if (programmingHeatSetpoint < 0) then
            print("unsuported value of setPoint: "..programmingHeatSetpoint)
            programmingHeatSetpoint = 0
        elseif (programmingHeatSetpoint > 35) then
            print("unsuported value of setPoint: "..programmingHeatSetpoint)
            programmingHeatSetpoint = 35
        end
    else
        if (programmingHeatSetpoint < 32) then
            print("unsuported value of setPoint: "..programmingHeatSetpoint)
            programmingHeatSetpoint = 32
        elseif (programmingHeatSetpoint > 95) then
            print("unsuported value of setPoint: "..programmingHeatSetpoint)
            programmingHeatSetpoint = 95
        end
    end    

    if (programmingHeatSetpoint < 10) then
        programmingHeatSetpoint = '0'..programmingHeatSetpoint
    end

        heatSetpointLocal = programmingHeatSetpoint
        self:updateProperty('programmingHeatSetpoint', heatSetpointLocal)
        self:updateProperty('ui.Programming_Z_Heat_Setpoint_Value_Label.caption', heatSetpointLocal)
    else
        heatSetpointLocal = self.properties.programmingHeatSetpoint
    end
    if(coolSetpoint) then 
        programmingCoolSetpoint = tonumber(self.properties.programmingCoolSetpoint)
        programmingCoolSetpoint = programmingCoolSetpoint + coolSetpoint

        if (self.parent.properties.units == 'C') then
            if (programmingCoolSetpoint < 0) then
                print("unsuported value of setPoint: "..programmingCoolSetpoint)
                programmingCoolSetpoint = 0
            elseif (programmingCoolSetpoint > 35) then
                print("unsuported value of setPoint: "..programmingCoolSetpoint)
                programmingCoolSetpoint = 35
            end
        else
            if (programmingCoolSetpoint < 32) then
                print("unsuported value of setPoint: "..programmingCoolSetpoint)
                programmingCoolSetpoint = 32
            elseif (programmingCoolSetpoint > 95) then
                print("unsuported value of setPoint: "..programmingCoolSetpoint)
                programmingCoolSetpoint = 95
            end
        end    

        if (programmingCoolSetpoint < 10) then
            programmingCoolSetpoint = '0'..programmingCoolSetpoint
        end

        coolSetpointLocal = programmingCoolSetpoint
        self:updateProperty('programmingCoolSetpoint', coolSetpointLocal)
        self:updateProperty('ui.Programming_Z_Cool_Setpoint_Value_Label.caption', coolSetpointLocal)
    else
        coolSetpointLocal = self.properties.programmingCoolSetpoint
    end
    if(fanSpped) then
        fanSppedLocal = fanSpped
        self:updateProperty('programmingFanSpped', fanSppedLocal)
        self:updateProperty('ui.Programming_Z_Fan_Speed_Value_Label.caption', fanSppedLocal)
    else
        fanSppedLocal = self.properties.programmingFanSpped
    end
end           

function CarrierZone:setProgrammingData()
    local  prefix = 'S'..self.properties.systemId..
                    'Z'..self.properties.zoneId..
                    'PGM'..
                    self.properties.programmingDay..
                    self.properties.programmingPeriod..
                    '!'
    local cmd = prefix..
                self.properties.programmingHour..
                ':'..
                self.properties.programmingMinutes.. 
                ' '..
                self.properties.programmingTimePeriod.. 
                ', '..
                self.properties.programmingHeatSetpoint..
                ', '..
                self.properties.programmingCoolSetpoint

                if (self.properties.programmingFanSpped ~= '' or self.properties.programmingFanSpped ~= nil) then
                    cmd = cmd..', '..self.properties.programmingFanSpped
                end 

                cmd = cmd..self.parent.COMMAND_TERMINATOR

    print('setProgrammingData CarrierZone: '..self.properties.zoneId..
            ' System: '..self.properties.systemId..' command to send : '..cmd)
    self:getProgramingInfo()
    self.parent.commandsQueue:pushFrontAndRemoveOthers(cmd, prefix)
end