--! General command class - represents command to Davis
class 'command'

--! Enum with command statuses
CommandStatus = {}
CommandStatus.initialised = "initialised"
CommandStatus.running = "running"
CommandStatus.finished = "finished"

--! Current command status
command.status = nil
--! Parent object (Davis)
command.parent = nil

--! Initialises class - sets parent reference
--!@param parent parent Davis object
function command:__init(parent)
	self.parent = parent
	self.status = CommandStatus.initialised
end

--! Sends command to Davis weather station
--!@param command string with command to be sent
function command:send(command)
	self.status = CommandStatus.running
	self.parent:send(command)
end

--! Proceses response from Davis station - should be overriden in subclasses
--!@param data string received from Davis station
function command:processResponse(data)
end

--! Sets status to 'finished' and notifies parent object
function command:finish()
	self.status = CommandStatus.finished
	self.parent:commandFinished()
end

function command:toString()
	return "unknown command"
end