require('command')
require('weatherData')

--! Represents 'LOOP' Davis command. After 'LOOP X' is send there would be X responses in 2.5sec interval
class 'loopCommand'(command)

--! Parameter to command - how many response packets would be sent
loopCommand.loopCount = nil
--! Which response was already received (if ==loopCount, than all data is already received)
loopCommand.responseNumber = nil

--! Initialises object, sets parent object (Davis), and loopCount parameter to command
--!@param parent Davis object
--!@param loopCount number - parameter to LOOP command indicating how many packets would be received
function loopCommand:__init(parent, loopCount)
	command.__init(self, parent)
	self.loopCount = loopCount
end

--! Sends loop command, and also adds it back to Davis commands queue
function loopCommand:send()
	if self.parent.properties.pollingInterval ~= nil and self.parent.properties.pollingInterval ~= '' then -- added to prevent crash when user set null pollingInterval
		setTimeout(function() self.parent.commandQueue:pushBack(loopCommand(self.parent, self.loopCount)) end, self.parent.properties.pollingInterval*1000)

		self.responseNumber = 0
		command.send(self, "LOOP " .. self.loopCount .. "\n")
	end
end

function loopCommand:processResponse(data)
	self.parent:bumpCurrentCommand()
	-- Check for 'LOO' - start of data
	for i=3,6 do
		if string.byte(data, i-2) == 0x4c and string.byte(data, i-1) == 0x4f and string.byte(data, i) == 0x4f then
			data = string.sub(data, i-2)	
			break
		end
	end
	
	if (string.len(data) == 95) then
		self.responseNumber = self.responseNumber + 1
		weather = weatherData(data)
		self.parent:updateUI(weather)
	end


	if(self.responseNumber == self.loopCount) then
		-- ! got and processed all expected packets from Davis station
		self:finish()
	end
end

function loopCommand:toString()
	return "LOOP " .. self.loopCount
end