require('common.unitsConverter')

class 'weatherData'

weatherData.barTrend = nil
weatherData.barometer = nil
weatherData.insideTemperature = nil
weatherData.insideHumidity = nil
weatherData.outsideTemperature = nil
weatherData.windSpeed = nil
weatherData.avg10minWindSpeed = nil
weatherData.windDirection = nil
weatherData.extraTemperatures = nil
weatherData.soilTemperatures = nil
weatherData.leafTemperatures = nil
weatherData.outsideHumidity = nil
weatherData.extraHumidities = nil
weatherData.rainRate = nil
weatherData.uv = nil
weatherData.solarRadiation = nil
weatherData.stormRain = nil
weatherData.startDateOfCurrentStorm = nil
weatherData.dayRain = nil
weatherData.monthRain = nil
weatherData.yearRain = nil
weatherData.dayET = nil
weatherData.monthET = nil
weatherData.yearET = nil
weatherData.soilMoistures = nil
weatherData.leafWetnesses = nil
weatherData.forecastIcons = nil
weatherData.timeOfSunrise = nil
weatherData.timeOfSunset = nil

weatherData.barTrendUnit = {["imperial"]="" ,["metric"]="" }
weatherData.barometerUnit = {["imperial"]="hPa" ,["metric"]="hPa" }
weatherData.insideTemperatureUnit = {["imperial"]="°F" ,["metric"]="°C" }
weatherData.insideHumidityUnit = {["imperial"]="%" ,["metric"]="%" }
weatherData.outsideTemperatureUnit = {["imperial"]="°F" ,["metric"]="°C" }
weatherData.windSpeedUnit = {["imperial"]="mph" ,["metric"]="km/h" }
weatherData.avg10minWindSpeedUnit = {["imperial"]="mph" ,["metric"]="km/h" }
weatherData.windDirectionUnit = {["imperial"]="" ,["metric"]="" }
weatherData.extraTemperaturesUnit = {["imperial"]="°F" ,["metric"]="°C" }
weatherData.soilTemperaturesUnit = {["imperial"]="°F" ,["metric"]="°C" }
weatherData.leafTemperaturesUnit = {["imperial"]="°F" ,["metric"]="°C" }
weatherData.outsideHumidityUnit = {["imperial"]="%" ,["metric"]="%" }
weatherData.extraHumiditiesUnit = {["imperial"]="%" ,["metric"]="%" }
weatherData.rainRateUnit = {["imperial"]="in/h" ,["metric"]="cm/h" }
weatherData.uvUnit = {["imperial"]="UV index" ,["metric"]="UV index" }
weatherData.solarRadiationUnit = {["imperial"]="watt/meter²" ,["metric"]="watt/meter²" }
weatherData.stormRainUnit = {["imperial"]="inch" ,["metric"]="inch" }
weatherData.startDateOfCurrentStormUnit = {["imperial"]="" ,["metric"]="" }
weatherData.dayRainUnit = {["imperial"]="in" ,["metric"]="cm" }
weatherData.monthRainUnit = {["imperial"]="in" ,["metric"]="cm" }
weatherData.yearRainUnit = {["imperial"]="in" ,["metric"]="cm" }
weatherData.dayETUnit = {["imperial"]="in" ,["metric"]="cm" }
weatherData.monthETUnit = {["imperial"]="in" ,["metric"]="cm" }
weatherData.yearETUnit = {["imperial"]="in" ,["metric"]="cm" }
weatherData.soilMoisturesUnit = {["imperial"]="cb" ,["metric"]="cb" }
weatherData.leafWetnessesUnit = {["imperial"]="" ,["metric"]="" }
weatherData.forecastIconsUnit = {["imperial"]="" ,["metric"]="" }
weatherData.timeOfSunriseUnit = {["imperial"]="" ,["metric"]="" }
weatherData.timeOfSunsetUnit = {["imperial"]="" ,["metric"]="" }

--!Initialises all weatcher properties from raw data (received from weather station)
--!@param weatherData binary data from weather station
function weatherData:__init(weatherData)
	self:setBarTrend(string.byte(weatherData, 4))
	self.barometer = self:ffff2nil(256*string.byte(weatherData, 9) + string.byte(weatherData, 8))

	if (self.barometer ~= nil) then
		--convert to hPa
		self.barometer = unitsConverter.inch2cm(self.barometer)/100
		self.barometer = self.barometer * 1.3332239
	end

	self.insideTemperature = self:ffff2nil(256*string.byte(weatherData, 11) + string.byte(weatherData, 10))
	if self.insideTemperature then self.insideTemperature = self.insideTemperature/10 end;

	self.insideHumidity = self:ff2nil(string.byte(weatherData, 12))
	self.outsideTemperature = self:ffff2nil(256*string.byte(weatherData, 14) + string.byte(weatherData, 13))
	if self.outsideTemperature then self.outsideTemperature = self.outsideTemperature/10 end;

	self.windSpeed = self:ff2nil(string.byte(weatherData, 15))
	self.avg10minWindSpeed = self:ff2nil(string.byte(weatherData, 16))
	self:setWindDirection(256*string.byte(weatherData, 18) + string.byte(weatherData, 17))
	self.outsideHumidity = self:ff2nil(string.byte(weatherData, 34))
	self.rainRate = self:ff2nil(string.byte(weatherData, 42))
	self.uv = self:ff2nil(string.byte(weatherData, 44))
	self.solarRadiation = self:ffff2nil(256*string.byte(weatherData, 46) + string.byte(weatherData, 45))
	self.stormRain = self:ffff2nil(256*string.byte(weatherData, 48) + string.byte(weatherData, 47))
	self:setStartDateOfCurrentStorm(256*string.byte(weatherData, 48) + string.byte(weatherData, 47))
	self.dayRain = self:ffff2nil(256*string.byte(weatherData, 52) + string.byte(weatherData, 51))
	self.monthRain = self:ffff2nil(256*string.byte(weatherData, 54) + string.byte(weatherData, 53))
	self.yearRain = self:ffff2nil(256*string.byte(weatherData, 56) + string.byte(weatherData, 55))
	self.dayET = self:ffff2nil(256*string.byte(weatherData, 58) + string.byte(weatherData, 57))
	self.monthET = self:ffff2nil(256*string.byte(weatherData, 60) + string.byte(weatherData, 59))
	self.yearET = self:ffff2nil(256*string.byte(weatherData, 62) + string.byte(weatherData, 61))
	self:setExtraTemperatures(string.byte(weatherData, 19), string.byte(weatherData, 20), string.byte(weatherData, 21), string.byte(weatherData, 22), string.byte(weatherData, 23), string.byte(weatherData, 24), string.byte(weatherData, 25))
	self:setExtraHumidities(string.byte(weatherData, 35), string.byte(weatherData, 36), string.byte(weatherData, 37), string.byte(weatherData, 38), string.byte(weatherData, 39), string.byte(weatherData, 40), string.byte(weatherData, 41))
	self:setSoilTemperatures(string.byte(weatherData, 26), string.byte(weatherData, 27), string.byte(weatherData, 28), string.byte(weatherData, 29))
	self:setSoilMoistures(string.byte(weatherData, 63), string.byte(weatherData, 64), string.byte(weatherData, 65), string.byte(weatherData, 66))
	self:setLeafWetnesses(string.byte(weatherData, 67), string.byte(weatherData, 68), string.byte(weatherData, 69), string.byte(weatherData, 70))
	self:setLeafTemperatures(string.byte(weatherData, 30), string.byte(weatherData, 31), string.byte(weatherData, 32), string.byte(weatherData, 33))
	self.forecastIcons = string.byte(weatherData, 90)
	self.timeOfSunrise = self:timeToString(256*string.byte(weatherData, 93) + string.byte(weatherData, 92))
	self.timeOfSunset = self:timeToString(256*string.byte(weatherData, 95) + string.byte(weatherData, 94))
end

--! Returns unchanged argument or nil if it equals 0xffff
--!@param arg numeric argument
--!@return unchanged argument or nil
function weatherData:ffff2nil(arg)
	if arg == 0xffff then
		return nil
	else
		return arg
	end
end

--! Returns unchanged argument or nil if it equals 0xff
--!@param arg numeric argument
--!@return unchanged argument or nil
function weatherData:ff2nil(arg)
	if arg == 0xff then
		return nil
	else
		return arg
	end
end

--! Sets wind direction to N, NE, E, ... based on davis data (0-360 degree)
--!@param wind davis wind direction
function weatherData:setWindDirection(wind)

	if (wind ~= nil) then
		if wind < 23 or wind >= 238 then
			self.windDirection = "N"
		elseif wind < 68 then
			self.windDirection = "NE"
		elseif wind < 113 then
			self.windDirection = "E"
		elseif wind < 158 then
			self.windDirection = "SE"
		elseif wind < 203 then
			self.windDirection = "S"
		elseif wind < 248 then
			self.windDirection = "SW"
		elseif wind < 293 then
			self.windDirection = "W"
		elseif wind < 338 then
			self.windDirection = "NW"
		end
	end
end

--! Sets bar trend based on Davis value to one of the values: ("Falling Rapidly", "Falling Slowly", "Steady", "Rising Slowly", "Rising Rapidly")
--!@param barTrend value received from Davis possible values are: 196, 236, 0, 20, 60
function weatherData:setBarTrend(barTrend)
	self.barTrend = nil
	if  barTrend == 196 then
		self.barTrend = "Falling Rapidly"
	elseif  barTrend == 236 then
		self.barTrend = "Falling Slowly"
	elseif  barTrend == 0 then
		self.barTrend = "Steady"
	elseif  barTrend == 20 then
		self.barTrend = "Rising Slowly"
	elseif  barTrend == 60 then
		self.barTrend = "Rising Rapidly"
	end
end

--! Sets temperatures from extra sensors (up to 7) values in ferenheits offseted by 90
--!@param value1
--!@param value2
--!@param value3
--!@param value4
--!@param value5
--!@param value6
--!@param value7
function weatherData:setExtraTemperatures(value1, value2, value3, value4, value5, value6, value7)
	self.extraTemperatures = {}
	if value1 ~= 255 then
		self.extraTemperatures[1] = value1-90
	end
	if value2 ~= 255 then
		self.extraTemperatures[2] = value2-90
	end
	if value3 ~= 255 then
		self.extraTemperatures[3] = value3-90
	end
	if value4 ~= 255 then
		self.extraTemperatures[4] = value4-90
	end
	if value5 ~= 255 then
		self.extraTemperatures[5] = value5-90
	end
	if value6 ~= 255 then
		self.extraTemperatures[6] = value6-90
	end
	if value7 ~= 255 then
		self.extraTemperatures[7] = value7-90
	end
end

--! Sets humidities from extra sensors (up to 7) values in %
--!@param value1
--!@param value2
--!@param value3
--!@param value4
--!@param value5
--!@param value6
--!@param value7
function weatherData:setExtraHumidities(value1, value2, value3, value4, value5, value6, value7)
	self.extraHumidities = {}
	if value1 ~= 255 then
		self.extraHumidities[1] = value1
	end
	if value2 ~= 255 then
		self.extraHumidities[2] = value2
	end
	if value3 ~= 255 then
		self.extraHumidities[3] = value3
	end
	if value4 ~= 255 then
		self.extraHumidities[4] = value4
	end
	if value5 ~= 255 then
		self.extraHumidities[5] = value5
	end
	if value6 ~= 255 then
		self.extraHumidities[6] = value6
	end
	if value7 ~= 255 then
		self.extraHumidities[7] = value7
	end
end

--! Sets soils temperatures from possible 4 sensors; values in ferenheits offseted by 90
--!@param value1
--!@param value2
--!@param value3
--!@param value4
function weatherData:setSoilTemperatures(value1, value2, value3, value4)
	self.soilTemperatures = {}
	if value1 ~= 255 then
		self.soilTemperatures[1] = value1-90
	end
	if value2 ~= 255 then
		self.soilTemperatures[2] = value2-90
	end
	if value3 ~= 255 then
		self.soilTemperatures[3] = value3-90
	end
	if value4 ~= 255 then
		self.soilTemperatures[4] = value4-90
	end
end

--! Sets soils moistures from possible 4 sensors; values in centibars
--!@param value1
--!@param value2
--!@param value3
--!@param value4
function weatherData:setSoilMoistures(value1, value2, value3, value4)
	self.soilMoistures = {}
	if value1 ~= 255 then
		self.soilMoistures[1] = value1
	end
	if value2 ~= 255 then
		self.soilMoistures[2] = value2
	end
	if value3 ~= 255 then
		self.soilMoistures[3] = value3
	end
	if value4 ~= 255 then
		self.soilMoistures[4] = value4
	end
end

--! Sets soils moistures from possible 4 sensors; values in 0-15 range (0: very dry, 15:very wet)
--!@param value1
--!@param value2
--!@param value3
--!@param value4
function weatherData:setLeafWetnesses(value1, value2, value3, value4)
	self.leafWetnesses = {}
	if value1 ~= 255 then
		self.leafWetnesses[1] = value1
	end
	if value2 ~= 255 then
		self.leafWetnesses[2] = value2
	end
	if value3 ~= 255 then
		self.leafWetnesses[3] = value3
	end
	if value4 ~= 255 then
		self.leafWetnesses[4] = value4
	end
end

--! Sets leaf temperatures from possible 4 sensors; values in ferenheits offseted by 90
--!@param value1
--!@param value2
--!@param value3
--!@param value4
function weatherData:setLeafTemperatures(value1, value2, value3, value4)
	self.leafTemperatures = {}
	if value1 ~= 255 then
		self.leafTemperatures[1] = value1-90
	end
	if value2 ~= 255 then
		self.leafTemperatures[2] = value2-90
	end
	if value3 ~= 255 then
		self.leafTemperatures[3] = value3-90
	end
	if value4 ~= 255 then
		self.leafTemperatures[4] = value4-90
	end
end


--! Converts binary Davis date to readable string
--!@param time time in format: hour*100+min
--!@return string wit time HH:mm (24h format)
function weatherData:timeToString(time)
	local minute = time%100
	local hour = (time-minute)/100

	return string.format("%.2d:%.2d", hour, minute)
end

--! Converts binary Davis date to readable string
--!@param date binary data received from davis. 2 bytes, bit 0-6:year; bit 7-11:day; bit 12-15:month
--!@return string with date dd.mm.yyyy
function weatherData:dateToString(date)
	local year = bit32.band(date, 0x7f) + 2000
	local month = bit32.rshift(bit32.band(date, 0xf80), 7)
	local day = bit32.rshift(bit32.band(date, 0xf000), 12)
	
	-- TODO: change format (??)
	return string.format("%.2d.%.2d.%d", day, month, year)
end

--! Sets start date of current storm
--!@param date binary date from Davis
function weatherData:setStartDateOfCurrentStorm(date)
	if date > 0 then
		self.startDateOfCurrentStorm = self:dateToString(date)
	else
		self.startDateOfCurrentStorm = nil
	end
end

--! Converts table to string (non-nested table)
--!@param t table
--!@return string representation of table
function weatherData:table2String(t)
	if t == nil then
		return "nil"
	end

	if type(t) ~= 'table' then
		return "NOT TABLE !!!"
	end

	local result = "["
	for k,v in pairs(t) do
		result = result .. v .. ", "
	end

	result = string.sub(result, 0, string.len(result) - 2)
	result = result .. "]"

	return result
end

--! Converts all data to metric system (by default data are kept in imperial system)
function weatherData:convertToMetric()
	self.insideTemperature = unitsConverter.farenheit2celsius(self.insideTemperature)
	self.outsideTemperature = unitsConverter.farenheit2celsius(self.outsideTemperature)	

	if (self.extraTemperatures) then
		for k,v in pairs(self.extraTemperatures) do
			self.extraTemperatures[k] = unitsConverter.farenheit2celsius(v)
		end
	end

	if (self.soilTemperatures) then
		for k,v in pairs(self.soilTemperatures) do
			self.soilTemperatures[k] = unitsConverter.farenheit2celsius(v)
		end
	end

	if (self.leafTemperatures) then
		for k,v in pairs(self.leafTemperatures) do
			self.leafTemperatures[k] = unitsConverter.farenheit2celsius(v)
		end
	end

	if self.windSpeed then
		self.windSpeed = unitsConverter.mile2km(self.windSpeed)
	end

	if self.avg10minWindSpeed then
		self.avg10minWindSpeed = unitsConverter.mile2km(self.avg10minWindSpeed)
	end

end

--! Prints all data currently stored
function weatherData:printAllData()
	print('#########################################################')
	print('weatherData.barTrend: ', self.barTrend)
	print('weatherData.barometer: ', self.barometer)
	print('weatherData.insideTemperature: ', self.insideTemperature)
	print('weatherData.insideHumidity: ', self.insideHumidity)
	print('weatherData.outsideTemperature: ', self.outsideTemperature)
	print('weatherData.windSpeed: ', self.windSpeed)
	print('weatherData.avg10minWindSpeed: ', self.avg10minWindSpeed)
	print('weatherData.windDirection: ', self.windDirection)
	print('weatherData.extraTemperatures: ', self:table2String(self.extraTemperatures))
	print('weatherData.soilTemperatures: ', self:table2String(self.soilTemperatures))
	print('weatherData.leafTemperatures: ', self:table2String(self.leafTemperatures))
	print('weatherData.outsideHumidity: ', self.outsideHumidity)
	print('weatherData.extraHumidities: ', self:table2String(self.extraHumidities))
	print('weatherData.rainRate: ', self.rainRate)
	print('weatherData.uv: ', self.uv)
	print('weatherData.solarRadiation: ', self.solarRadiation)
	print('weatherData.stormRain: ', self.stormRain)
	print('weatherData.startDateOfCurrentStorm: ', self.startDateOfCurrentStorm)
	print('weatherData.dayRain: ', self.dayRain)
	print('weatherData.monthRain: ', self.monthRain)
	print('weatherData.yearRain: ', self.yearRain)
	print('weatherData.dayET: ', self.dayET)
	print('weatherData.monthET: ', self.monthET)
	print('weatherData.yearET: ', self.yearET)
	print('weatherData.soilMoistures: ', self:table2String(self.soilMoistures))
	print('weatherData.leafWetnesses: ', self:table2String(self.leafWetnesses))
	print('weatherData.forecastIcons: ', self.forecastIcons)
	print('weatherData.timeOfSunrise: ', self.timeOfSunrise)
	print('weatherData.timeOfSunset: ', self.timeOfSunset)
	print('=============================================================')
end