require('common.device')
require('net.HTTPClient')

class 'Denon' (Device)

Denon.APP_COMMAND_DATA = [[
<?xml version="1.0" encoding="utf-8"?>
<tx>
 <cmd id="1">GetAllZonePowerStatus</cmd>
 <cmd id="1">GetVolumeLevel</cmd>
 <cmd id="1">GetMuteStatus</cmd>
 <cmd id="1">GetSourceStatus</cmd>
</tx>]]

Denon.POLLING_TIME = 1000

function Denon:__init(id)
	Device.__init(self, id)

	self.http = net.HTTPClient({ timeout = 3000 })
	self.xmlParser = require("common.xmlSimple").newParser()

	self:getState()
end

--
--
-- actions
--
-- 
function Denon:turnOn()
	self.http:request('http://' .. self.properties.ip .. '/goform/formiPhoneAppPower.xml?1+PowerOn')
end

function Denon:turnOff()
	self.http:request('http://' .. self.properties.ip .. '/goform/formiPhoneAppPower.xml?1+PowerStandby')
end

function Denon:setVolume(volume)
	volume = string.format('%0.1f', volume)
	self.http:request('http://' .. self.properties.ip .. '/goform/formiPhoneAppVolume.xml?1+' .. volume)
end

function Denon:setMute(state)
	if (state) then
		self.http:request('http://' .. self.properties.ip .. '/goform/formiPhoneAppMute.xml?1+MuteOn')
	else
		self.http:request('http://' .. self.properties.ip .. '/goform/formiPhoneAppMute.xml?1+MuteOff')
	end
end

function Denon:setInput(input)
	self.http:request('http://' .. self.properties.ip .. '/goform/formiPhoneAppDirect.xml?SI' .. input)
end

function Denon:getState()
	self.http:request('http://' .. self.properties.ip .. '/goform/AppCommand.xml', {
		options = {
			method = 'POST',
			data = Denon.APP_COMMAND_DATA
		},
		success = function(params)
			if (not string.find(params.data, "404 Not Found")) then
				local xmlData = self.xmlParser:ParseXmlText(params.data)

				local zone = xmlData['rx']['cmd'][1]['zone1']:value()
				if (zone == 'ON') then
					self:updateProperty('power', true)
				else
					self:updateProperty('power', false)
				end

				local volume = xmlData['rx']['cmd'][2]['volume']:value()
				if (volume == '--') then
					self:updateProperty('volume', -80)
				else
					self:updateProperty('volume', tonumber(volume))
				end

				local mute = xmlData['rx']['cmd'][3]['mute']:value()
				if (mute == 'on') then
					self:updateProperty('mute', true)
				else
					self:updateProperty('mute', false)
				end

				local source = xmlData['rx']['cmd'][4]['source']:value()
				self:updateProperty('input', source)

				setTimeout(function() self:getState() end, Denon.POLLING_TIME)
			end
		end,
		error = function()
			setTimeout(function() self:getState() end, Denon.POLLING_TIME)
		end
	})
end