require('common.device')


class 'DenonHeosGroup' (Device)
   

function DenonHeosGroup:__init(denonHeos, dev)
	Device.__init(self, dev)
	
	self.denonHeos = denonHeos

	self.uiBinding = {
		["mute_Switch"]   = function(eventArgs) self:setMuteState(eventArgs.values[1]) end,
		["play_Button"]   =	function() self:play() end,
		["pause_Button"]  =	function() self:pause() end,
		["next_Button"]   =	function() self:next() end,
		["prev_Button"]   =	function() self:previous() end,
		["volume_Slider"] = function(eventArgs) self:setVolume(eventArgs.values[1]) end,
	}
end

function DenonHeosGroup:updateProperties(groupDetails)
	self:updateProperty('gid', tostring(groupDetails.gid))
	self:setNewName(groupDetails.name)
	self:setNoMedia()
	self:getNowPlayingMedia()
   	self:getVolume()
end

function DenonHeosGroup:setNewName(newName)
	if newName ~= nil then
		newName = 'Group ' .. newName 
		if newName ~= self.name then 
			self:setName(newName)
		end
	end
end

function DenonHeosGroup:setMuteState(value)
	self:updateProperty('ui.mute_Switch.value', value)
	if value then
		self:mute()
	else
		self:unmute()
	end
end

function DenonHeosGroup:mute()
	if self.denonHeos.sock then 
		local command = 'heos://group/set_mute?gid=' .. self.properties.gid .. '&state=on' .. self.denonHeos.delimiter
		self.denonHeos.sock:send(command)		
	end
end

function DenonHeosGroup:unmute()
	if self.denonHeos.sock then 
		local command = 'heos://group/set_mute?gid=' .. self.properties.gid .. '&state=off' .. self.denonHeos.delimiter
		self.denonHeos.sock:send(command)		
	end
end

function DenonHeosGroup:play()
	if self.denonHeos.sock then 
		local playCommand = 'heos://player/set_play_state?pid=' .. self.properties.gid .. '&state=play' .. self.denonHeos.delimiter
		self.denonHeos.sock:send(playCommand)	
	end
end

function DenonHeosGroup:pause()
	if self.denonHeos.sock then 
		local command = 'heos://player/set_play_state?pid=' .. self.properties.gid .. '&state=pause' .. self.denonHeos.delimiter
		self.denonHeos.sock:send(command)		
	end
end

function DenonHeosGroup:next()
	if self.denonHeos.sock then 
		local command = 'heos://player/play_next?pid=' .. self.properties.gid .. self.denonHeos.delimiter
		self.denonHeos.sock:send(command)		
	end
end

function DenonHeosGroup:previous()
	if self.denonHeos.sock then 
		local command = 'heos://player/play_previous?pid=' .. self.properties.gid .. self.denonHeos.delimiter
		self.denonHeos.sock:send(command)		
	end
end

function DenonHeosGroup:setVolume(value)
	self:updateProperty('ui.mute_Switch.value', false) -- always change mute switch to off state when volume changes

	if self.denonHeos.sock then 
		local command = 'heos://group/set_volume?gid=' .. self.properties.gid .. '&level=' .. value .. self.denonHeos.delimiter
		self.denonHeos.sock:send(command)		
	end
end

function  DenonHeosGroup:getVolume()
	if self.denonHeos.sock then 
		local command = 'heos://player/get_volume?pid=' .. self.properties.gid .. self.denonHeos.delimiter
		self.denonHeos.sock:send(command)		
	end
end

function DenonHeosGroup:onDataReceivedFromGetNowPlaying(data)
	self:setNowPlayingLabel(data)
	self:setAlbumImage(data.image_url)
end

function DenonHeosGroup:getNowPlayingMedia()
	local command = 'heos://player/get_now_playing_media?pid=' .. self.properties.gid .. self.denonHeos.delimiter	
	self.denonHeos.sock:send(command)
end

function DenonHeosGroup:getMute()
	local command = 'heos://player/get_mute?pid=' .. self.properties.gid .. self.denonHeos.delimiter	
	self.denonHeos.sock:send(command)
end

function DenonHeosGroup:setNowPlayingLabel(data)
	self:updateProperty("ui.creator_Label.caption", data.artist)
	self:updateProperty("ui.title_Label.caption", data.song)
end

function DenonHeosGroup:setErrorMessage(errorMessage)
	self:updateProperty("ui.creator_Label.caption", 'Error Occurred')
	self:updateProperty("ui.title_Label.caption", errorMessage)
	self:setAlbumImage(nil)
end

function DenonHeosGroup:setVolumeLabel(volumeLevel)
	self:updateProperty('ui.volume_Label.caption', tostring(volumeLevel))
	self:updateProperty('ui.volume_Slider.value', tonumber(volumeLevel))	
	self:updateProperty('volume', tonumber(volumeLevel))
end

function DenonHeosGroup:setMuteSateLabel(muteState)
	self:updateProperty('ui.mute_Switch.value', muteState)
end

function DenonHeosGroup:setAlbumImage(uri)
	if not uri or uri == '' then
		self:updateProperty("ui.albumArt_Image.source", '/plugins/com.fibaro.denonHeosGroup/img/defaultIcon.png')
	else
		self:updateProperty("ui.albumArt_Image.source", uri)
	end
end

function DenonHeosGroup:setNoMedia()
	self:updateProperty('ui.volume_Label.caption', tostring(0))
	self:updateProperty('ui.volume_Slider.value', 0)
	self:updateProperty('volume', 0)
	self:updateProperty("ui.albumArt_Image.source", '/plugins/com.fibaro.denonHeosGroup/img/defaultIcon.png')
	self:updateProperty("ui.creator_Label.caption", "no media")
	self:updateProperty("ui.title_Label.caption", "")
end



