require('common.device')


class 'DenonHeosZone' (Device)
   

function DenonHeosZone:__init(denonHeos, dev)
	Device.__init(self, dev)
	
	self.denonHeos = denonHeos

	self.uiBinding = {
		["mute_Switch"]   = function(eventArgs) self:setMuteState(eventArgs.values[1]) end,
		["play_Button"]   =	function() self:play() end,
		["pause_Button"]  =	function() self:pause() end,
		["next_Button"]   =	function() self:next() end,
		["prev_Button"]   =	function() self:previous() end,
		["volume_Slider"] = function(eventArgs) self:setVolume(eventArgs.values[1]) end,
	}
end

function DenonHeosZone:updateProperties(zoneDetails)
	self:updateProperty('pid', tostring(zoneDetails.pid))
	self:setNewName(zoneDetails.name)
	self:setNoMedia()
	self:getNowPlayingMedia()
   	self:getVolume()
end

function DenonHeosZone:setNewName(newName)
	if newName ~= nil then
		newName = 'Player ' .. newName 
		if newName ~= self.name then 
			self:setName(newName)
		end
	end
end

function DenonHeosZone:setMuteState(value)
	self:updateProperty('ui.mute_Switch.value', value)
	if value then
		self:mute()
	else
		self:unmute()
	end
end

function DenonHeosZone:mute()
	if self.denonHeos.sock then 
		local command = 'heos://player/set_mute?pid=' .. self.properties.pid .. '&state=on' .. self.denonHeos.delimiter
		self.denonHeos.sock:send(command)		
	end
end

function DenonHeosZone:unmute()
	if self.denonHeos.sock then 
		local command = 'heos://player/set_mute?pid=' .. self.properties.pid .. '&state=off' .. self.denonHeos.delimiter
		self.denonHeos.sock:send(command)		
	end
end

function DenonHeosZone:play()
	if self.denonHeos.sock then 
		local playCommand = 'heos://player/set_play_state?pid=' .. self.properties.pid .. '&state=play' .. self.denonHeos.delimiter
		self.denonHeos.sock:send(playCommand)	
	end
end

function DenonHeosZone:pause()
	if self.denonHeos.sock then 
		local command = 'heos://player/set_play_state?pid=' .. self.properties.pid .. '&state=pause' .. self.denonHeos.delimiter
		self.denonHeos.sock:send(command)		
	end
end

function DenonHeosZone:next()
	if self.denonHeos.sock then 
		local command = 'heos://player/play_next?pid=' .. self.properties.pid .. self.denonHeos.delimiter
		self.denonHeos.sock:send(command)		
	end
end

function DenonHeosZone:previous()
	if self.denonHeos.sock then 
		local command = 'heos://player/play_previous?pid=' .. self.properties.pid .. self.denonHeos.delimiter
		self.denonHeos.sock:send(command)		
	end
end

function DenonHeosZone:setVolume(value)
	self:updateProperty('ui.mute_Switch.value', false) -- always change mute switch to off state when volume changes

	if self.denonHeos.sock then 
		local command = 'heos://player/set_volume?pid=' .. self.properties.pid .. '&level=' .. value .. self.denonHeos.delimiter
		self.denonHeos.sock:send(command)		
	end
end

function  DenonHeosZone:getVolume()
	if self.denonHeos.sock then 
		local command = 'heos://player/get_volume?pid=' .. self.properties.pid .. self.denonHeos.delimiter
		self.denonHeos.sock:send(command)		
	end
end

function DenonHeosZone:onDataReceivedFromGetNowPlaying(data)
	self:setNowPlayingLabel(data)
	self:setAlbumImage(data.image_url)
end

function DenonHeosZone:getNowPlayingMedia()
	local command = 'heos://player/get_now_playing_media?pid=' .. self.properties.pid .. self.denonHeos.delimiter	
	self.denonHeos.sock:send(command)
end

function DenonHeosZone:getMute()
	local command = 'heos://player/get_mute?pid=' .. self.properties.pid .. self.denonHeos.delimiter	
	self.denonHeos.sock:send(command)
end

function DenonHeosZone:setNowPlayingLabel(data)
	self:updateProperty("ui.creator_Label.caption", data.artist)
	self:updateProperty("ui.title_Label.caption", data.song)
end

function DenonHeosZone:setErrorMessage(errorMessage)
	self:updateProperty("ui.creator_Label.caption", 'Error Occurred')
	self:updateProperty("ui.title_Label.caption", errorMessage)
	self:setAlbumImage(nil)
end

function DenonHeosZone:setVolumeLabel(volumeLevel)
	self:updateProperty('ui.volume_Label.caption', tostring(volumeLevel))
	self:updateProperty('ui.volume_Slider.value', tonumber(volumeLevel))	
	self:updateProperty('volume', tonumber(volumeLevel))
end

function DenonHeosZone:setMuteSateLabel(muteState)
	self:updateProperty('ui.mute_Switch.value', muteState)
end

function DenonHeosZone:setAlbumImage(uri)


	if not uri or uri == '' then
		self:updateProperty("ui.albumArt_Image.source", '/plugins/com.fibaro.denonHeosZone/img/defaultIcon.png')
	else
		self:updateProperty("ui.albumArt_Image.source", uri)
	end
end

function DenonHeosZone:setNoMedia()
	self:updateProperty('ui.volume_Label.caption', tostring(0))
	self:updateProperty('ui.volume_Slider.value', 0)
	self:updateProperty('volume', 0)
	self:updateProperty("ui.albumArt_Image.source", '/plugins/com.fibaro.denonHeosZone/img/defaultIcon.png')
	self:updateProperty("ui.creator_Label.caption", "no media")
	self:updateProperty("ui.title_Label.caption", "")
end



