class 'ResponseHandler'


function ResponseHandler:__init(denonHeos)
    self.denonHeos = denonHeos

    self.responsHandlers = {
        ['group/get_groups']                  = function(data) self:handleGetGroups(data) end,
        ['player/get_players']                = function(data) self:hendleGetPlayers(data) end,
        ['player/get_now_playing_media']      = function(data) self:handleGetNowPlaying(data) end,
        ['event/player_now_playing_changed']  = function(data) self:handleNowPlayingChanged(data) end,
        ['event/player_volume_changed']       = function(data) self:handlePlayerVolumeChanged(data) end,
        ['player/get_volume']                 = function(data) self:handleGetVolume(data) end,
        ['event/player_playback_error']       = function(data) self:handlePlaybackError(data) end, 
        ['player/get_mute']                   = function(data) self:handleGetMute(data) end,  
        ['event/group_volume_changed']        = function(data) self:handleGroupVolumeChanged(data) end,  
        ['system/register_for_change_events'] = function(data) self:handleRegisterForChanges(data) end, 
        ['event/player_state_changed']        = function(data) self:handlePlayerStateChanged(data) end,
        ['event/player_now_playing_progress'] = function(data) self:handlePlayerNowPlayingProgress(data) end,
    } 
end 
 
function ResponseHandler:handle(data)
    local ok, jsonData = pcall(json.decode, data)
    if ok and jsonData and jsonData.heos.command then         
        if type(self.responsHandlers[jsonData.heos.command]) == 'function' then 
            self.responsHandlers[jsonData.heos.command](jsonData)    
        else
            print("unhandled message: ", jsonData.heos.command)
        end
    end
end  

function ResponseHandler:handleGetGroups(data)
    if data.heos.result == 'success' then
        self.denonHeos:onDataReceivedFromGetGroups(data.payload)        
    else
        self.denonHeos:onDataReceivedFromGetGroups('error')
    end
end

function ResponseHandler:hendleGetPlayers(data) -- player is the same as zone
    if data.heos.result == 'success' then
        self.denonHeos:onDataReceivedFromGetZones(data.payload)
    else
        self.denonHeos:onDataReceivedFromGetZones('error')
    end
end

function ResponseHandler:handleGetNowPlaying(data)
    local zone = self:getZoneByPid(string.match(data.heos.message, "=(.*)"))
    local group = self:getGroupByGid(string.match(data.heos.message, "=(.*)"))

    if data.heos.result == 'success' and zone then
        zone:onDataReceivedFromGetNowPlaying(data.payload)
    elseif zone then
        zone:onDataReceivedFromGetNowPlaying('error')
    end

    if data.heos.result == 'success' and group then
        group:onDataReceivedFromGetNowPlaying(data.payload)
    elseif group then
        group:onDataReceivedFromGetNowPlaying('error')
    end
end

function ResponseHandler:handleNowPlayingChanged(data)
    local zone = self:getZoneByPid(string.match(data.heos.message, "=(.*)"))

    if zone then
        zone:getNowPlayingMedia()
        zone:getVolume()
        zone:getMute()
    end
end

function ResponseHandler:handlePlayerStateChanged(data)
    local zone = self:getZoneByPid(string.match(data.heos.message, "=(.*)&state"))

    if zone then
        zone:getNowPlayingMedia()
        zone:getVolume()
        zone:getMute()
    end
end

function ResponseHandler:handlePlayerNowPlayingProgress(data)
    local zone = self:getZoneByPid(string.match(data.heos.message, "=(.*)&cur_pos"))

    if zone then
        zone:getNowPlayingMedia()
        zone:getVolume()
        zone:getMute()
    end
end

function ResponseHandler:handlePlayerVolumeChanged(data)
    local pid, volumeLevel, muteState = string.match(data.heos.message, 'pid=(.*)&level=(.*)&mute=(.*)')
    local zone = self:getZoneByPid(pid)

    if muteState == 'off' then 
        muteState = false
    elseif muteState == 'on' then
        muteState = true
    end

    if zone then
        zone:setVolumeLabel(volumeLevel)
        zone:setMuteSateLabel(muteState)
    end
end

function ResponseHandler:handleGroupVolumeChanged(data)
    local gid, volumeLevel, muteState = string.match(data.heos.message, 'gid=(.*)&level=(.*)&mute=(.*)')
    local group = self:getGroupByGid(gid)

    if muteState == 'off' then 
        muteState = false
    elseif muteState == 'on' then
        muteState = true
    end

    if group then
        group:setVolumeLabel(volumeLevel)
        group:setMuteSateLabel(muteState)
    end
end

function ResponseHandler:handleGetVolume(data)
    local pid, volumeLevel = string.match(data.heos.message, 'pid=(.*)&level=(.*)')
    local muteState = false

    local zone = self:getZoneByPid(pid)
    local group = self:getGroupByGid(pid)

    if zone then
        zone:setVolumeLabel(volumeLevel)
    end

    if group then
        group:setVolumeLabel(volumeLevel)
    end
end

function ResponseHandler:handleGetMute(data)
    local pid, muteState = string.match(data.heos.message, 'pid=(.*)&state=(.*)')
    local zone = self:getZoneByPid(pid)
    local group = self:getGroupByGid(pid)

    if muteState == 'off' then 
        muteState = false
    elseif muteState == 'on' then
        muteState = true
    end

    if zone then
        zone:setMuteSateLabel(muteState)
    end

    if group then
        group:setMuteSateLabel(muteState)
    end
end

function ResponseHandler:handlePlaybackError(data)
    local pid, errorMessage = string.match(data.heos.message, 'pid=(.*)&error=(.*)')
    local zone = self:getZoneByPid(pid)
    local group = self:getGroupByGid(pid)

    if zone then
        zone:setErrorMessage(errorMessage)
    end

    if group then
        group:setErrorMessage(errorMessage)
    end
end

function ResponseHandler:handleRegisterForChanges(data)
    self.denonHeos:onDataReceivedFromRegisterForChanges(data)
end

function ResponseHandler:getZoneByPid(pid)
    if self.denonHeos.zones then
        for _,zone in pairs(self.denonHeos.zones) do
            if tostring(zone.properties.pid) == tostring(pid) then
                return zone
            end
        end
    end

    return nil
end

function ResponseHandler:getGroupByGid(gid)
    if self.denonHeos.groups then
        for _,group in pairs(self.denonHeos.groups) do
            if tostring(group.properties.gid) == tostring(gid) then
                return group
            end
        end
    end

    return nil
end

function ResponseHandler:countTableElements(table)
    if type(table) == 'table' then
        local counter = 0
        if table then
            for _,_ in pairs(table) do
                counter = counter + 1
            end 
        end
        return counter
    else
        return 0
    end
end















