require('common.device')
require('dscZone')
require('dscPartition')
require('dscInstallationProgress')
require('common/TCPListener')

DEBUG = false


function tableSize(tab)
	local count = 0
	for _,_ in pairs(tab) do
		count = count + 1
	end

	return count
end


debug_response = {}
debug_response["500"] = "Command Acknowledge"
debug_response["501"] = "Command Error"
debug_response["502"] = "System Error"
debug_response["601"] = "Zone Alarm"
debug_response["602"] = "Zone Alarm Restore"
debug_response["603"] = "Zone Tamper"
debug_response["604"] = "Zone Tamper Restore"
debug_response["605"] = "Zone Fault"
debug_response["606"] = "Zone Fault Restore"
debug_response["609"] = "Zone Open"
debug_response["610"] = "Zone Restored"
debug_response["903"] = "LED Status "
debug_response["840"] = "Trouble Status (LED ON) "
debug_response["841"] = "Trouble Status Restore(LED OFF) "
debug_response["897"] = "Keybus Fault Restore"

debug_response["650"] = "Partition Ready (650)"
debug_response["651"] = "Partition Not Ready (651)"
debug_response["652"] = "Partition Armed - Descriptive Mode "
debug_response["653"] = "Partition in Ready to Force Arm "
debug_response["654"] = "Partition in Alarm (654)"
debug_response["655"] = "Partition Disarmed (655)"
debug_response["656"] = "Exit Delay in Progress (656)"
debug_response["657"] = "Entry Delay in Progress (657)"
debug_response["658"] = "Keypad Lock-out (658)"
debug_response["659"] = "Keypad Blanking (659)"
debug_response["660"] = "Command output in progress (660)"
debug_response["670"] = "Invalid Access Code (670)"
debug_response["671"] = "Function Not Available (671)"
debug_response["672"] = "Failed To Arm (672)"
debug_response["673"] = "Partition Busy (673)"
debug_response["900"] = "Code Required (900)"
debug_response["901"] = "LCD Update"
debug_response["904"] = "Beep Status"
debug_response["750"] = "User Opening"
debug_response["751"] = "Special Opening"


debug_systemError = {}
debug_systemError["017"] = "Keybus Busy - Installer Mode"
debug_systemError["029"] = "Not Valid Parameter"
debug_systemError["021"] = "Requested Partition is out of Range"
debug_systemError["030"] = "Keypad Does Not Come Out of Blank Mode"
debug_systemError["023"] = "Partition is Not Armed"
debug_systemError["031"] = "IT-100 is already in Thermostat menu"
debug_systemError["024"] = "Partition is Not Ready to Arm"
debug_systemError["028"] = "Virtual Keypad is Disabled"
debug_systemError["032"] = "IT-100 is Not in Thermostat menu"
debug_systemError["026"] = "User Code Not Required"
debug_systemError["033"] = "No Response from Thermostat (or Escort Module)"

--
-- DSCAlarm
--
class 'DSCAlarm' (Device)

function DSCAlarm:debug(cmd, data)
	if (DEBUG == true) then
		local responseCmd = ""
		if (debug_response[cmd]) then
			responseCmd = debug_response[cmd]
		end

		if (tonumber(cmd) == 502) then
			responseCmd = responseCmd  .. " : \"" .. debug_systemError[data] .. "\" "
		end

		if (tonumber(cmd)>= 601 and tonumber(cmd) <= 610) then
			responseCmd = responseCmd  .. " : \"" .. self.zones[tonumber(data:sub(2))].name .. "\"  (" .. tonumber(data:sub(2)).. ")"
		end

		if (tonumber(cmd)>= 650 and tonumber(cmd) <= 751) then
			responseCmd = responseCmd  .. " : \"" .. self.partitions[tonumber(data:sub(1, 1))].name .. "\"  (" .. tonumber(data:sub(1, 1)).. ")"
		end

		print ("DEBUG >>> Response: " .. cmd .. " <<< : " .. responseCmd)
	end
end

DSCAlarm.COMMANDS = {
	--
	-- zone alarm
	--
	['570'] =	function(dev, data)
					local id = tonumber(data:sub(1,3))
					local label = ''

					-- zone label
					if (id >= 1 and id <= 64) then
						label = data:sub(4)
						print('ZONE ' .. id .. ': ' .. label)
						dev.zones[id]:setName(label)
						dev.zones[id]:setVisible(true)
						dev.installationProgress:makeProgress()
					elseif (id >= 101 and id <= 108) then
						id = id - 100
						label = data:sub(4)
						print('PARTITION ' .. id .. ': ' .. label)
						dev.partitions[id]:setName(label)
						dev.partitions[id]:setVisible(true)
						dev.installationProgress:makeProgress()
					end
				end,
	--
	-- zone alarm
	--
	['601'] =	function(dev, data)
					local id = tonumber(data:sub(2))
					dev.zones[id]:updateProperty('alarm', true)
				end,
	['602'] =	function(dev, data)
					local id = tonumber(data:sub(2))
					dev.zones[id]:updateProperty('alarm', false)
				end,
	--
	-- zone tamper
	--
	['603'] =	function(dev, data)
					local id = tonumber(data:sub(2))
					dev.zones[id]:updateProperty('tamper', true)
				end,
	['604'] =	function(dev, data)
					local id = tonumber(data:sub(2))
					dev.zones[id]:updateProperty('tamper', false)
				end,
	--
	-- zone value
	--
	['609'] =	function(dev, data)
					local id = tonumber(data)
					dev.zones[id]:updateProperty('value', true)
				end,
	['610'] =	function(dev, data)
					local id = tonumber(data)
					dev.zones[id]:updateProperty('value', false)
				end,
	--
	-- partition ready
	--
	['650'] =	function(dev, data)
					local id = tonumber(data:sub(1, 1))
					dev.partitions[id]:updateProperty('armed', false)
					dev.partitions[id]:updateProperty('exitTime', false)
					dev.partitions[id]:updateProperty('alarm', false)
					dev.partitions[id]:updateProperty('ready', true)
				end,
	['651'] =	function(dev, data)
					local id = tonumber(data:sub(1, 1))
					dev.partitions[id]:updateProperty('armed', false)
					dev.partitions[id]:updateProperty('ready', false)
					dev.partitions[id]:updateProperty('exitTime', false)
					dev.partitions[id]:updateProperty('alarm', false)
				end,
	--
	-- partition armed
	--
	['652'] =	function(dev, data)
					local id = tonumber(data:sub(1, 1))
					dev.partitions[id]:updateProperty('armed', true)
					dev.partitions[id]:updateProperty('exitTime', false)
				end,
	--
	-- partition alarm
	--
	['654'] =	function(dev, data)
					local id = tonumber(data)
					dev.partitions[id]:updateProperty('alarm', true)
				end,
	['655'] =	function(dev, data)
					local id = tonumber(data:sub(1, 1))
					dev.partitions[id]:updateProperty('armed', false)
					dev.partitions[id]:updateProperty('exitTime', false)
					dev.partitions[id]:updateProperty('alarm', false)
				end,
	--
	-- exit delay
	--
	['656'] =	function(dev, data)
					id = tonumber(data)
					dev.partitions[id]:updateProperty('exitTime', true)
				end,
	--
	-- entry delay
	--
	['657'] =	function(dev, data)
					id = tonumber(data)
					-- TODO: add in the future
					-- dev.partitions[id]:updateProperty('entryTime', true)
				end
}

function DSCAlarm:__init(id)
	Device.__init(self, id)

	self.zonesCount = 64
	self.partitionsCount = 8

	self.zones = {}
	self.partitions = {}
	self.reload = false -- flag indicating if all devices data should be reloaded

	--
	-- get child devices
	--
	local childDevices = plugin.getChildDevices(self.id)
	for _,dev in pairs(childDevices) do
		if (dev.type == 'com.fibaro.dscZone') then
			self.childDevices[dev.id] = DSCZone(self, dev)
		elseif (dev.type == 'com.fibaro.dscPartition') then
			self.childDevices[dev.id] = DSCPartition(self, dev)
		end
	end

	self:initializeChildDevices()

	-- create connection

	self.sock = TCPListener({delimiter = string.char( 0x0D , 0x0A ), readTimeout = 10000, connectTimeout=10000})
    self.sock:registerEvent('dataReceived', function(sock, data) self:onDataReceived(data, id) end)
    self.sock:registerEvent('disconnected', function()  self:onClosed() end)
    self.sock:registerEvent('connected', function() self:onConnected() end)
    self.sock:registerEvent('error', function(sock, error) print("error", error) end)

    if (self.properties.ip ~= '' and self.properties.port ~= '' and 
        self.properties.ip ~= nil and self.properties.port ~= nil) then  
			
			self.sock:connect(self.properties.ip, tonumber(self.properties.port))

    else
        print('No ip or port')
    end   
end

function DSCAlarm:initializeChildDevices()
	if (tableSize(self.childDevices) ~= self.zonesCount+self.partitionsCount) then
		local steps = 3*(self.zonesCount+self.partitionsCount)-tableSize(self.childDevices)
		self.installationProgress = DSCInstallationProgress(self.id, steps)
		self.installationProgress:createProgressNotification()

		local existentZones = {}
		local existentPartitions = {}
		for k,v in pairs(self.childDevices) do
			if (v.properties.zoneId) then
				existentZones[v.properties.zoneId] = true
			elseif (v.properties.partitionId) then
				existentPartitions[v.properties.partitionId] = true
			end
		end

		for i=1,self.zonesCount do
			if existentZones[i]  ~= true then
				self.installationProgress:makeProgress()
				local dev = plugin.createChildDevice(self.id, 'com.fibaro.dscZone', 'Zone ' .. i)
				local devObject = DSCZone(self, dev)

				self.childDevices[dev.id] = devObject
				devObject:updateProperty('zoneId', i)
			end
		end

		for i=1,self.partitionsCount do
			if existentPartitions[i]  ~= true then
				self.installationProgress:makeProgress()
				local dev = plugin.createChildDevice(self.id, 'com.fibaro.dscPartition', 'Partition ' .. i)
				local devObject = DSCPartition(self, dev)

				self.childDevices[dev.id] = devObject
				devObject:updateProperty('partitionId', i)
			end
		end

		self.reload = true -- reload all data
	end

	for id, dev in pairs(self.childDevices) do
		if (dev.properties.zoneId) then
			self.zones[dev.properties.zoneId] = dev
		elseif (dev.properties.partitionId) then
			self.partitions[dev.properties.partitionId] = dev
		else
			print('[WARNING] Wrong child found (id = ' .. id .. ')')
		end
	end
end

function DSCAlarm:onConnected()

	print("CONNECT")
	self:sendData('001', '')
	if (self.reload == true) then
		self:startReload()
	end
end

function DSCAlarm:onClosed()
	print("reconnect")
	setTimeout(function() self.sock:connect(self.properties.ip, tonumber(self.properties.port)) end, 3000)
end

function DSCAlarm:onDataReceived(data, id)
	if (data) then
		local cmd, data, csum = data:match("^(%d%d%d)(.*)(%x%x)$")
		self:debug(cmd, data)
		
		local func = DSCAlarm.COMMANDS[cmd]

		if (func) then
			func(self,data)
		end
	end
end

function DSCAlarm:reloadConfiguration()
	local maxSteps = 2*(self.zonesCount+self.partitionsCount)
	self.installationProgress = DSCInstallationProgress(self.id, maxSteps)
	self.installationProgress:createProgressNotification()

	self:startReload()
end

function DSCAlarm:startReload()
	for id, dev in pairs(self.childDevices) do
		dev:setVisible(false)
		self.installationProgress:makeProgress()
	end

	self:sendData('002', '')
end

function DSCAlarm:sendData(cmd, data)
	local payload = cmd .. data
	message = string.format("%s%0.2X\r\n", payload, self:__calculateChecksum(payload))
	self.sock:send(message)
end

function DSCAlarm:__calculateChecksum(data)
	local t = 0
	for i = 1, data:len() do
		t = t + data:byte(i)
	end
	return t % 256
end