require('common.device')
require('envisaLinkZone')
require('envisaLinkPartition')
require('envisaLinkInstallationProgress')
require('common/TCPListener')

DEBUG = false

function tableSize(tab)
	local count = 0
	for _,_ in pairs(tab) do
		count = count + 1
	end

	return count
end


debug_response = {}
debug_response["500"] = "Command Acknowledge (500)"
debug_response["501"] = "Command Error (501)"
debug_response["502"] = "System Error (502)"
debug_response["505"] = "Login Interaction (505)"
debug_response["510"] = "Keypad LED State – Partition 1 only (510)"
debug_response["511"] = "Keypad LED FLASH state – Partition 1 only (511)"
debug_response["550"] = "Time/Date Broadcast (550)"
debug_response["560"] = "Ring Detected (560)"
debug_response["561"] = "Indoor Temperature Broadcast (561)"
debug_response["562"] = "Outdoor Temperature Broadcast (562)"
debug_response["601"] = "Zone Alarm (601)"
debug_response["602"] = "Zone Alarm Restore (602)"
debug_response["603"] = "Zone Tamper (603)"
debug_response["604"] = "Zone Tamper Restore (604)"
debug_response["605"] = "Zone Fault (605)"
debug_response["606"] = "Zone Fault Restore (606)"
debug_response["609"] = "Zone Open (609)"
debug_response["610"] = "Zone Restored (610)"
debug_response["615"] = "Envisalink Zone Timer Dump (615)"
debug_response["620"] = "Duress Alarm (620)"
debug_response["621"] = "[F] Key Alarm (621)"
debug_response["622"] = "[F] Key Restore (622)"
debug_response["623"] = "[A] Key Alarm (623)"
debug_response["624"] = "[A] Key Restoral (624)"
debug_response["625"] = "[P] Key Alarm (625)"
debug_response["626"] = "[P] Key Restore (626)"
debug_response["631"] = "2-Wire Smoke/Aux Alarm (631)"
debug_response["632"] = "2-Wire Smoke/Aux Restore (632)"
debug_response["650"] = "Partition Ready (650)"
debug_response["651"] = "Partition Not Ready (651)"
debug_response["652"] = "Partition Armed - Descriptive Mode "
debug_response["653"] = "Partition Ready – Force Arming Enabled 653"
debug_response["654"] = "Partition in Alarm (654)"
debug_response["655"] = "Partition Disarmed (655)"
debug_response["656"] = "Exit Delay in Progress (656)"
debug_response["657"] = "Entry Delay in Progress (657)"
debug_response["658"] = "Keypad Lock-out (658)"
debug_response["659"] = "Partition Failed to Arm (659)"
debug_response["660"] = "PGM Output is in Progress (660)"
debug_response["663"] = "Chime Enabled (663)"
debug_response["664"] = "Chime Disabled (664)"
debug_response["670"] = "Invalid Access Code (670)"
debug_response["671"] = "Function Not Available (671)"
debug_response["672"] = "Failed To Arm (672)"
debug_response["673"] = "Partition Busy (673)"
debug_response["674"] = "System Arming in Progress (674)"
debug_response["680"] = "System in Installers Mode (680)"
debug_response["700"] = "User Closing (700)"
debug_response["701"] = "Special Closing (701)"
debug_response["702"] = "Partial Closing (702)"
debug_response["750"] = "User Opening (750)"
debug_response["751"] = "Special Opening (751)"
debug_response["800"] = "Panel Battery Trouble (800)"
debug_response["801"] = "Panel Battery Trouble Restore (801)"
debug_response["802"] = "Panel AC Trouble (802)"
debug_response["803"] = "Panel AC Restore (803)"
debug_response["806"] = "System Bell Trouble (806)"
debug_response["807"] = "System Bell Trouble Restoral (807)"
debug_response["814"] = "FTC Trouble (814)"
debug_response["816"] = "Buffer Near Full (816)"
debug_response["829"] = "General System Tamper (829)"
debug_response["830"] = "General System Tamper Restore (830)"
debug_response["840"] = "Trouble LED ON (840)"
debug_response["841"] = "Trouble LED OFF (841)"
debug_response["842"] = "Fire Trouble Alarm (842)"
debug_response["843"] = "Fire Trouble Alarm Restore (843)"
debug_response["849"] = "Verbose Trouble Status (849)"
debug_response["900"] = "Code Required (900)"
debug_response["912"] = "Command Output Pressed (912)"
debug_response["921"] = "Master Code Required (921)"
debug_response["922"] = "Installers Code Required (922)"



debug_systemError = {}
debug_systemError["000"] = "No Error (000)"
debug_systemError["001"] = "Receive Buffer Overrun (a command is received while another is still being processed) (001)"
debug_systemError["002"] = "Receive Buffer Overflow (002)"
debug_systemError["003"] = "Transmit Buffer Overflow (003)"
debug_systemError["010"] = "Keybus Transmit Buffer Overrun (010)"
debug_systemError["011"] = "Keybus Transmit Time Timeout (011)"
debug_systemError["012"] = "Keybus Transmit Mode Timeout (012)"
debug_systemError["013"] = "Keybus Transmit Keystring Timeout (013)"
debug_systemError["014"] = "Keybus Interface Not Functioning (the TPI cannot communicate with the security system) (014)"
debug_systemError["015"] = "Keybus Busy (Attempting to Disarm or Arm with user code) (015)"
debug_systemError["016"] = "Keybus Busy – Lockout (The panel is currently in Keypad Lockout – too many disarm attempts) (016)"
debug_systemError["017"] = "Keybus Busy – Installers Mode (Panel is in installers mode, most functions are unavailable)"
debug_systemError["018"] = "Keybus Busy – General Busy (The requested partition is busy) (018)"
debug_systemError["020"] = "Command Syntax Erro (020)"
debug_systemError["021"] = "Command Partition Error (Requested Partition is out of bounds)"
debug_systemError["022"] = "Command Not Supported (022)"
debug_systemError["023"] = "System Not Armed (sent in response to a disarm command)"
debug_systemError["024"] = "System Not Ready to Arm (system is either not-secure, in exit-delay, or already armed)"
debug_systemError["025"] = "Command Invalid Length (025)"
debug_systemError["026"] = "User Code not Required"
debug_systemError["027"] = "Invalid Characters in Command (no alpha characters are allowed except for checksum) (027)"

--
-- envisaLink
--
class 'EnvisaLinkAlarm' (Device)

function EnvisaLinkAlarm:debug(cmd, data)

	if (DEBUG == true) then
		local responseCmd = ""
		if (debug_response[cmd]) then
			responseCmd = debug_response[cmd]
		end

		if (tonumber(cmd) == 502) then
			responseCmd = responseCmd  .. " : \"" .. debug_systemError[data] .. "\" "
		end

		if (tonumber(cmd) == 505) then
        	if (data == "0") then
                responseCmd = "Password provided was incorrect"
            elseif (data == "1") then
            	responseCmd = "Password Correct, session established"
            elseif (data == "2") then
                responseCmd = "Time out. You did not send a password within 10 seconds."
            elseif (data == "3") then
                responseCmd = "Request for password, sent after socket setup."
			end
		end

		if (tonumber(cmd)>= 601 and tonumber(cmd) <= 610) then
			responseCmd = responseCmd  .. " : \"" .. self.zones[tonumber(data:sub(2))].name .. "\"  (" .. tonumber(data:sub(2)).. ")"
		end

		if (tonumber(cmd)>= 650 and tonumber(cmd) <= 751) then
			responseCmd = responseCmd  .. " : \"" .. self.partitions[tonumber(data:sub(1, 1))].name .. "\"  (" .. tonumber(data:sub(1, 1)).. ")"
		end
	end
end



EnvisaLinkAlarm.COMMANDS = {
	
	['505'] =   function(dev, data)
    				if (data == "1") then
                        dev:sendData('001', '')
                    elseif (data == "3") then
                        dev:sendData("005", tostring(dev.properties.envisalinkPassword))
                    end                    
                end,	
	--
	-- zone alarm
	--
	['601'] =	function(dev, data)
					local id = tonumber(data:sub(2))
					dev.zones[id]:updateProperty('alarm', true)
				end,
	['602'] =	function(dev, data)
					local id = tonumber(data:sub(2))
					dev.zones[id]:updateProperty('alarm', false)
				end,
	--
	-- zone tamper
	--
	['603'] =	function(dev, data)
					local id = tonumber(data:sub(2))
					dev.zones[id]:updateProperty('tamper', true)
				end,
	['604'] =	function(dev, data)
					local id = tonumber(data:sub(2))
					dev.zones[id]:updateProperty('tamper', false)
				end,
	--
	-- zone value
	--
	['609'] =	function(dev, data)
					local id = tonumber(data)
					dev.zones[id]:updateProperty('value', true)
				end,
	['610'] =	function(dev, data)
					local id = tonumber(data)
					dev.zones[id]:updateProperty('value', false)
				end,
	--
	-- partition ready
	--
	['650'] =	function(dev, data)
					local id = tonumber(data:sub(1, 1))
					dev.partitions[id]:updateProperty('armed', false)
					dev.partitions[id]:updateProperty('exitTime', false)
					dev.partitions[id]:updateProperty('alarm', false)
					dev.partitions[id]:updateProperty('ready', true)
				end,
	['651'] =	function(dev, data)
					local id = tonumber(data:sub(1, 1))
					dev.partitions[id]:updateProperty('armed', false)
					dev.partitions[id]:updateProperty('ready', false)
					dev.partitions[id]:updateProperty('exitTime', false)
					dev.partitions[id]:updateProperty('alarm', false)
				end,
	--
	-- partition armed
	--
	['652'] =	function(dev, data)
					local id = tonumber(data:sub(1, 1))
					dev.partitions[id]:updateProperty('armed', true)
					dev.partitions[id]:updateProperty('exitTime', false)
				end,
	--
	-- partition alarm
	--
	['654'] =	function(dev, data)
					local id = tonumber(data)
					dev.partitions[id]:updateProperty('alarm', true)
				end,
	['655'] =	function(dev, data)
					local id = tonumber(data:sub(1, 1))
					dev.partitions[id]:updateProperty('armed', false)
					dev.partitions[id]:updateProperty('exitTime', false)
					dev.partitions[id]:updateProperty('alarm', false)
				end,
	--
	-- exit delay
	--
	['656'] =	function(dev, data)
					id = tonumber(data)
					dev.partitions[id]:updateProperty('exitTime', true)
				end,
	['657'] =	function(dev, data)
					id = tonumber(data)
					dev.partitions[id]:updateProperty('exitTime', true)
				end,

	['900'] =	function(dev, data)
					dev:sendData("200", tostring(self.parent.properties.userCode))
				end
}

function EnvisaLinkAlarm:__init(id)
	Device.__init(self, id)

	self.zonesCount = 64
	self.partitionsCount = 8

	self.zones = {}
	self.partitions = {}
	self.reload = true -- flag indicating if all devices data should be reloaded

	--
	-- get child devices
	--
	local childDevices = plugin.getChildDevices(self.id)
	for _,dev in pairs(childDevices) do
		if (dev.type == 'com.fibaro.envisaLinkZone') then
			self.childDevices[dev.id] = EnvisaLinkZone(self, dev)
		elseif (dev.type == 'com.fibaro.envisaLinkPartition') then
			self.childDevices[dev.id] = EnvisaLinkPartition(self, dev)
		end
	end
	self:initializeChildDevices()

	-- create connection

	self.sock = TCPListener({delimiter = string.char( 0x0D , 0x0A ), readTimeout = 10000, connectTimeout=10000})
    self.sock:registerEvent('dataReceived', function(sock, data) self:onDataReceived(data, id) end)
    self.sock:registerEvent('disconnected', function()  self:onClosed() end)
    self.sock:registerEvent('connected', function() self:onConnected() end)
    self.sock:registerEvent('error', function(sock, error) print("error", error) end)

    if (self.properties.ip ~= '' and self.properties.port ~= '' and 
        self.properties.ip ~= nil and self.properties.port ~= nil) then  
			
			self.sock:connect(self.properties.ip, tonumber(self.properties.port))
    end   
end

function EnvisaLinkAlarm:initializeChildDevices()
	if (tableSize(self.childDevices) ~= self.zonesCount+self.partitionsCount) then
		local steps = 3*(self.zonesCount+self.partitionsCount)-tableSize(self.childDevices)
		self.installationProgress = EnvisaLinkInstallationProgress(self.id, steps)
		self.installationProgress:createProgressNotification()

		local existentZones = {}
		local existentPartitions = {}
		for k,v in pairs(self.childDevices) do
			if (v.properties.zoneId) then
				existentZones[v.properties.zoneId] = true
			elseif (v.properties.partitionId) then
				existentPartitions[v.properties.partitionId] = true
			end
		end

		for i=1,self.zonesCount do
			if existentZones[i]  ~= true then
				self.installationProgress:makeProgress()
				local dev = plugin.createChildDevice(self.id, 'com.fibaro.envisaLinkZone', 'Zone ' .. i)
				local devObject = EnvisaLinkZone(self, dev)

				self.childDevices[dev.id] = devObject
				devObject:updateProperty('zoneId', i)
			end
		end

		for i=1,self.partitionsCount do
			if existentPartitions[i]  ~= true then
				self.installationProgress:makeProgress()
				local dev = plugin.createChildDevice(self.id, 'com.fibaro.envisaLinkPartition', 'Partition ' .. i)
				local devObject = EnvisaLinkPartition(self, dev)

				self.childDevices[dev.id] = devObject
				devObject:updateProperty('partitionId', i)
			end
		end

		self.reload = true -- reload all data
	end

	for id, dev in pairs(self.childDevices) do
		if (dev.properties.zoneId) then
			self.zones[dev.properties.zoneId] = dev
		elseif (dev.properties.partitionId) then
			self.partitions[dev.properties.partitionId] = dev
		end
	end
end

function EnvisaLinkAlarm:onConnected()
	if (DEBUG == true) then    
			print ("Connected")
	end
end

function EnvisaLinkAlarm:onClosed()
	setTimeout(function() self.sock:connect(self.properties.ip, tonumber(self.properties.port)) end, 3000)
end

function EnvisaLinkAlarm:onDataReceived(data, id)
	if (data) then
		local cmd, data, csum = data:match("^(%d%d%d)(.*)(%x%x)$")
		local payloadChecksum = string.format("%0.2X",self:__calculateChecksum(cmd .. data))
		
		if ( csum == payloadChecksum) then
			self:debug(cmd, data)
			local func = EnvisaLinkAlarm.COMMANDS[cmd]
			if (func) then
				func(self,data)
			end
		end
	end
end

function EnvisaLinkAlarm:reloadConfiguration()
	local maxSteps = 2*(self.zonesCount+self.partitionsCount)
	self.installationProgress = EnvisaLinkInstallationProgress(self.id, maxSteps)
	self.installationProgress:createProgressNotification()
end

 function EnvisaLinkAlarm:sendData(cmd, data)
 	if(data ~= nil) then
		payload = cmd .. data
	else
		payload = cmd
	end

	local payloadChecksum = self:__calculateChecksum(payload)
	local message = string.format("%s%0.2X\r\n", payload, payloadChecksum)
	
	if (DEBUG == true) then    
			print ("          SEND      Payload: " .. payload .. " Checksum: " .. payloadChecksum)
	end

	self.sock:send(message)
end

function EnvisaLinkAlarm:__calculateChecksum(data)
	local t = 0
	for i = 1, data:len() do
		t = t + data:byte(i)
	end
	return t % 256
end
