
class 'EnvisaLinkInstallationProgress'

function EnvisaLinkInstallationProgress:__init(deviceId, maxSteps)
    self.deviceId = deviceId
    self.maxSteps = maxSteps
    self.progress = 0
    self.active = false
end

function EnvisaLinkInstallationProgress:createProgressNotification()
    self.timeout = setTimeout(function() self:onTimeout() end, 10000)

    local postBody = {["type"] = "SatelConfigurationNotification", ["canBeDeleted"] = false, ["data"] = {["deviceId"]  = self.deviceId, ["progress"] = 0}}
    local postResponse = api.post("/notificationCenter", postBody)

    if postResponse then
        self.active = true
        self.progress = 0
        self.notificationId = postResponse.id
    end
end

function EnvisaLinkInstallationProgress:delete()
    if self.active == false then
        return
    end

    local putBody = {["canBeDeleted"] = true}
    api.put("/notificationCenter/" .. self.notificationId, putBody)
    api.delete("/notificationCenter/" .. self.notificationId)
    self.active = false
end

function EnvisaLinkInstallationProgress:makeProgress()
    if self.active == false then
        return
    end

    clearTimeout(self.timeout)
    self.timeout = setTimeout(function() self:onTimeout() end, 10000)

    local oldPercentage = math.floor(self.progress*100/self.maxSteps)

    self.progress = self.progress + 1

    local newPercentage = math.floor(self.progress*100/self.maxSteps)

    if oldPercentage ~= newPercentage then
        self:setProgress(newPercentage)
    end

    if self.progress == self.maxSteps then
        self:delete()
    end
end

function EnvisaLinkInstallationProgress:setProgress(progress)
    if self.active == false then
        return
    end

    local putBody = {["canBeDeleted"] = false, ["data"] = {["deviceId"]  = self.deviceId, ["progress"] = progress}}
    api.put("/notificationCenter/" .. self.notificationId, putBody)
end 

function EnvisaLinkInstallationProgress:onTimeout()
    self:delete()
end