class 'FibaroIntercomHelper'

function FibaroIntercomHelper:__init(config, callback)
    assert(type(callback) == 'function')

    logger.info("[FibaroIntercomHelper:__init]")

	self.callback = callback
	self.properties = config
	self.sock = JsonRpcOverWebsocket({callTimeout = 10000, secure = true})
    assert(self.sock ~= nil)
    self.sock:registerEvent('connected', function()  self:login() end)
    self:connect()
end

function FibaroIntercomHelper:connect()
    local connectionString = 'wss://' .. self.properties.ip .. ':' .. self.properties.port .. '/wsock'
    logger.info("[FibaroIntercomHelper:connect] " .. connectionString)
   	self.sock:connect(connectionString)
end

function FibaroIntercomHelper:login()
	local methodName = 'com.fibaro.intercom.account.login'
	local params = {['user'] = self.properties.email, ['pass'] = self.properties.localPassword}

	self:sendRequest(methodName, params, function (response) self:getName(response.result.token) end)
end

function FibaroIntercomHelper:getName(token)
	local methodName = 'com.fibaro.intercom.device.info.getName'
	local params = {['token'] = token}

	self:sendRequest(methodName, params, function (response) self.callback(response.result.name) end)
end

function FibaroIntercomHelper:sendRequest(methodName, params, callback)
    self.sock:sendJsonRpcCall(methodName, params,  {
        success = function(response) 
            if callback then
                callback(response)
            end
        end,
        error = function (err)
        	print(err)
        end
    })
end
