require('common.device')
require('net.HTTPClient')
class 'LGTV' (Device)


--! Constructor - creates LGTV object. 
--!@param id id of this device or table with this device data
function LGTV:__init(id)
	Device.__init(self, id)

	self.http = net.HTTPClient()
	self.lg = {}
	self.timeoutID = nil
	self.lg.pairingKey = self.properties.authCode
	self.lg.sessionId = ''
	self.lg.ip = self.properties.ip
	self.lg.port = '8080'
	self.headers = {}
	self.headers['Content-Type'] = 'application/atom+xml'
	self.headers['Accept-Encoding'] = 'identity'
	self.lg.pairingKeyLink = '/udap/api/pairing'
	self.lg.pairingKeyData = '<?xml version=\"1.0\" encoding=\"utf-8\"?><auth><type>AuthKeyReq</type></auth>'
	self.lg.pairingKeyAuthLink = '/udap/api/pairing'
	self.lg.pairingKeyAuthData = '<?xml version=\"1.0\" encoding=\"utf-8\"?><auth><type>AuthReq</type><value>' .. self.lg.pairingKey .. '</value></auth>'
	self.lg.cmdData = '<?xml version=\"1.0\" encoding=\"utf-8\"?><command><name>HandleKeyInput</name><value>cmdcode</value></command>'
	self.lg.cmdLink = '/roap/api/command'
	self:getSessionId()
end

--! send request to lg tv for display authorization key
function LGTV:requestAuthorizationKey()

self.http:request('http://'.. self.lg.ip .. ':' .. tostring(self.lg.port) .. self.lg.pairingKeyLink, {
		options = {
			timeout = 3000,
			method = 'POST',
			headers = self.headers, 
			data = self.lg.pairingKeyData
		},
		success = function(data)
		end,
		error = function(err)
		print (err)
		end
	})
end


--! send request to lg tv for session id
function LGTV:getSessionId()
	self.http:request('http://' .. self.lg.ip .. ':' .. tostring(self.lg.port) .. self.lg.pairingKeyAuthLink, {
		options = {
			timeout = 3000,
			method = 'POST',
			headers = self.headers, 
			data = self.lg.pairingKeyAuthData
		},
		success = function(data)
		end,
		error = function(error)
		end
	})
end


--! send request to lg tv 
--!@param command Command number (string)
function LGTV:handleCommand(command)
	local cmdData = '<?xml version=\"1.0\" encoding=\"utf-8\"?><command><name>HandleKeyInput</name><value>' .. command .. '</value></command>'
	self.http:request('http://' .. self.lg.ip .. ':' .. tostring(self.lg.port) .. self.lg.cmdLink, {
		options = {
			timeout = 3000,
			method = 'POST',
			headers = self.headers, 
			data = cmdData
		},
		success = function(params)
			if params.status and params.status ~= 200 then 
    			self:retryCommand(command)
			end
		end,
		error = function(err)
			self:retryCommand(command)
		end
	})
end

--! Retry request to lg tv if tv does not respond correctly
--!@param command Command number (string)
function LGTV:retryCommand(command)
	if (self.timeoutID) then
		clearTimeout(self.timeoutID)
    end

    self.timeoutID = setTimeout(function()
    	self:getSessionId()
		self:handleCommand(command)
    end, 100)
end

--! turn off tv
function LGTV:POWERButton()
    self:handleCommand('1')
end


--! change audio video
function LGTV:TVRADButton()
    self:handleCommand('410')
end


--! quick menu button
function LGTV:QMENUButton()
    self:handleCommand('405')
end


--! not supported
function LGTV:GUIDEButton()
    self:handleCommand('') --guide
end


--! change display ratio
function LGTV:RATIOButton()
    self:handleCommand('46')
end


--! chnage input source
function LGTV:INPUTButton()
    self:handleCommand('47')
end


--! number button (1)
function LGTV:NUM1Button()
    self:handleCommand('3')
end


--! number button (2)
function LGTV:NUM2Button()
    self:handleCommand('4')
end


--! number button (3)
function LGTV:NUM3Button()
    self:handleCommand('5')
end


--! number button (4)
function LGTV:NUM4Button()
    self:handleCommand('6')
end


--! number button (5)
function LGTV:NUM5Button()
    self:handleCommand('7')
end


--! number button (6)
function LGTV:NUM6Button()
    self:handleCommand('8')
end


--! number button (7)
function LGTV:NUM7Button()
    self:handleCommand('9')
end


--! number button (8)
function LGTV:NUM8Button()
    self:handleCommand('10')
end


--! number button (9)
function LGTV:NUM9Button()
    self:handleCommand('11')
end


--! show list of canals
function LGTV:LISTButton()
    self:handleCommand('50')
end


--! number button (0)
function LGTV:NUM0Button()
    self:handleCommand('2')
end


--! show last watched canal
function LGTV:QVIEWButton()
    self:handleCommand('403')
end


--! rise volume
function LGTV:VOLUMEUPButton()
    self:handleCommand('24')
end


--! decrease volume
function LGTV:VOLUMEDOWNButton()
    self:handleCommand('25')
end


--! favourite canal
function LGTV:FAVButton()
    self:handleCommand('404')
end


--! not supported
function LGTV:APPButton()
    self:handleCommand('') -- włochy
end


--! mute voice
function LGTV:MUTEButton()
    self:handleCommand('26')
end


--! next channel
function LGTV:CHANNELUPButton()
    self:handleCommand('27')
end


--! preveius channel
function LGTV:CHANNELDOWNButton()
    self:handleCommand('28')
end


--! show teletext
function LGTV:TEXTButton()
    self:handleCommand('51')
end


--! channel info
function LGTV:INFOButton()
    self:handleCommand('45')
end


--! text options
function LGTV:TOPTButton()
    self:handleCommand('406')
end


--! show smart menu
function LGTV:SMARTButton()
    self:handleCommand('21')
end


--! up direction
function LGTV:UPButton()
    self:handleCommand('12')
end


--! show my apps menu
function LGTV:MYAPPSButton()
    self:handleCommand('417')
end


--!  left direction
function LGTV:LEFTButton()
    self:handleCommand('14')
end


--! ok button
function LGTV:OKButton()
    self:handleCommand('20')
end


--! right direction
function LGTV:RIGHTButton()
    self:handleCommand('15')
end


--! go back
function LGTV:BACKButton()
    self:handleCommand('23')
end


--! down direction
function LGTV:DOWNButton()
    self:handleCommand('13')
end


--! exit
function LGTV:EXITButton()
    self:handleCommand('412')
end


--! not supported
function LGTV:SETTINGSButton()
    self:handleCommand('') --settings
end


--! turn on record
function LGTV:RECORDButton()
    self:handleCommand('40')
end


--! stop
function LGTV:STOPButton()
    self:handleCommand('35')
end


--! rewind media
function LGTV:FASTREWINDButton()
    self:handleCommand('37')
end


--! play media
function LGTV:PLAYButton()
    self:handleCommand('33')
end


--! pause media
function LGTV:PAUSEButton()
    self:handleCommand('34')
end


--! fastforeward media
function LGTV:FASTFORWARDButton()
    self:handleCommand('36')
end


--! red button
function LGTV:REDButton()
    self:handleCommand('31')
end


--! green button
function LGTV:GREENButton()
    self:handleCommand('30')
end


--! yellow button
function LGTV:YELLOWButton()
    self:handleCommand('32')
end


--! blue button
function LGTV:BLUEButton()
    self:handleCommand('29')
end

