require('common.device')

class 'Activity' (Device)

function Activity:__init(parentHub, dev)
	Device.__init(self, dev)
	self.parentHub = parentHub
	self.canUpdateStatus = true

	self.statusMap = {
		[0] = "i18n_startStopActivitySwitchOff",
		[1] = "i18n_activityCurrentStateStarting",
		[2] = "i18n_activityCurrentStateRunning",
		[3] = "i18n_activityCurrentStatePoweringOff",
		[4] = "i18n_activityCurrentStateOffline",
	}
end

function Activity:updateProperties(activityId, cloudLocatedActivityDetails)
	if cloudLocatedActivityDetails.name then 
		self:setNewName(cloudLocatedActivityDetails.name)		
	end

	if activityId ~= nil then
		activityId = tostring(activityId)
		self:updateProperty('activityId', activityId)
	end
end

function Activity:setNewName(newName)
	if newName ~= nil then
		newName = 'Activity ' .. newName 
		if newName ~= self.name then 
			self:setName(newName)
		end
	end
end

function Activity:start()
	if self.parentHub.harmonyAccount.accessToken and self.parentHub.harmonyAccount.accessToken ~= '' then
		self.canUpdateStatus = false
		
		local headersAccessTokenAuth = {
			['Authorization'] = 'Bearer ' .. self.parentHub.harmonyAccount.accessToken
		}

		local params = {
			url = self.parentHub.harmonyAccount.urls.cloudApi .. 'hub/' .. self.parentHub.properties.hubId .. '/activity/' .. self.properties.activityId .. '/start',
			method = 'POST',
			headers = headersAccessTokenAuth,
			callback =  function(response) self:onDataReceivedFromStartActivity(response) end
		}

		self.parentHub.harmonyAccount:httpRequest(params)
	end
end

function Activity:onDataReceivedFromStartActivity(response)
	self.canUpdateStatus = true
	self.parentHub.harmonyAccount:getCurrentState()
end

function Activity:stop()
	if self.parentHub.harmonyAccount.accessToken and self.parentHub.harmonyAccount.accessToken ~= '' then
		local headersAccessTokenAuth = {
			['Authorization'] = 'Bearer ' .. self.parentHub.harmonyAccount.accessToken
		}

		local params = {
			url = self.parentHub.harmonyAccount.urls.cloudApi .. 'hub/' .. self.parentHub.properties.hubId .. '/activity/' .. self.properties.activityId .. '/end',
			method = 'POST',
			headers = headersAccessTokenAuth,
			callback =  function(response) self:onDataReceivedFromStopActivity(response) end
		}

		self.parentHub.harmonyAccount:httpRequest(params)
	end
end

function Activity:onDataReceivedFromStopActivity(response)
	self.canUpdateStatus = true
	self.parentHub.harmonyAccount:getCurrentState()
end

function Activity:updateStatus(status, switchState)
	if self.canUpdateStatus then
		self:updateProperty('ui.activityCurrentStateValueLabel.caption', self.statusMap[status])
		self:updateProperty('ui.startStopActivitySwitch.value', switchState)
	end
end

function Activity:changeActivityState(stateParam)
	local state = stateParam
	
	if self.parentHub.properties.isOnline == false then
		print('---> Hub is offline - restarting plugin to check Hub availability')		
		return plugin.restart()
	end

	if state == self.properties['ui.startStopActivitySwitch.value']  then
		state = not state
	end
	
	if state == nil then
		state = not self.properties['ui.startStopActivitySwitch.value']
	end 

	self:updateProperty('ui.startStopActivitySwitch.value', state)
	if state == true then
		self:start()
	elseif state == false then
		self:stop()
	end
end  