require('common.device')
require('net.HTTPClient')
require('Hub')


class 'HarmonyAccount' (Device)


function HarmonyAccount:__init(id)
	Device.__init(self, id)
	
	self.http = net.HTTPClient({ timeout = 20000 }) 
	self.authCode = ''
	self.accessToken = ''
	self.hubs = self:getHubDevicesFromExistingHC2Model()

	self.headersApplication = {
		['Content-Type'] = 'application/x-www-form-urlencoded',
	}

	self:initStatuses()
    self:initialMethod()
end

function HarmonyAccount:initStatuses()
  self.connectionStatuses = {
      ["setInit"] = function()
        self:setConnectionStatusLabels('i18n_connectionStatusInitState')
        self:pinButtonDisable()
      end,
      ["setGetPin"] = function()
        self:setConnectionStatusLabels('i18n_connectionStatusInitState')
        self:pinButtonEnable()
      end,
      ["setConnecting"] = function()
        self:setConnectionStatusLabels('i18n_connectionStatusConnecting')
      end,
      ["setConnected"] = function()
        self:setConnectionStatusLabels('i18n_connectionStatusConnected')
        self:pinButtonDisable()
      end,
      ["setTokenGranted"] = function()
        self:setConnectionStatusLabels('i18n_connectionStatusAccesTokenGranted')
        self:pinButtonDisable()
      end,
      ["setConnectionError"] = function()
        self:setConnectionStatusLabels('i18n_connectionStatusConnectionError')
        self:pinButtonEnable()
      end,
      ["setUndefinedError"] = function()
        self:setConnectionStatusLabels('i18n_connectionStatusUndefinedError')
        self:pinButtonEnable()
      end
  }
end

function HarmonyAccount:setConnectionStatusLabels(caption)
  self:updateProperty('ui.connectionStatusLabel.caption', caption)
  self:updateProperty('ui.config.connectionStatusConfigLabel.caption', caption)
end

function HarmonyAccount:pinButtonDisable()
  self:updateProperty('ui.config.link.enabled', false)
  self:updateProperty('ui.config.pincode.enabled', false)
end

function HarmonyAccount:pinButtonEnable()
  self:updateProperty('ui.config.link.enabled', true)
  self:updateProperty('ui.config.pincode.enabled', true)
end

function HarmonyAccount:initialMethod()
	self.connectionStatuses.setInit()

    self.accessToken = self.properties.accessToken or ''
    self.urls = {
    		['accessToken'] = 'https://pluginsauth.fibaro.com/logitech/harmony/getToken.php?code=' .. (self.properties.pincode or ''),
    		['cloudApi'] = 'https://home.myharmony.com/cloudapi/'
    }
    if self.accessToken == '' and self:credentialsExist()  then -- Must obtain new access token by pincode that was used in past in Enter Pincode section
        self.connectionStatuses.setConnecting()
        self:getAccessToken()
    elseif not self:credentialsExist() and self.accessToken == '' then -- First init - notify user to update pin code and press save button
        self.connectionStatuses.setGetPin()
    elseif self.accessToken and self.accessToken ~= '' then -- try use already obtained acces token
    	self.connectionStatuses.setConnecting()
        self:discoverCloudLocatedHubs()
    end
end

function HarmonyAccount:credentialsExist()
    return self.properties.pincode ~= nil and self.properties.pincode ~= ''
end

function HarmonyAccount:getAccessToken()
	local params = {
			url = self.urls.accessToken,
			method = 'GET',
			callback =  function(response) self:onDataReceivedFromGetAccessToken(response) end
		}

	self:httpRequest(params)
end

function HarmonyAccount:onDataReceivedFromGetAccessToken(response)
    local ok, jsonResponse = pcall(json.decode, response.data)

    if ok and jsonResponse then
        if jsonResponse.access_token then
            self.connectionStatuses.setTokenGranted()
            self:updateProperty('accessToken', jsonResponse.access_token)
            self.accessToken = jsonResponse.access_token
            self:discoverCloudLocatedHubs()
        else
            self.connectionStatuses.setUndefinedError()
        end
    else
        self.connectionStatuses.setUndefinedError()
    end
end

function HarmonyAccount:httpRequest(params)	
	self.http:request(params.url, {	
		options={
			headers = params.headers,
			method = params.method,
			data = params.data,
		}, 
		success = function(response)
			if (params.callback) then
				params.callback(response)			
			end
		end,
		error = function (error)
			print('*******************')
			print('* ERROR: '.. error)
			print('*******************')

			if (params.callback) then
				params.callback(error)			
			end					
		end
	})
end

function HarmonyAccount:discoverCloudLocatedHubs()
	self:updateProperty('ui.infoLabel.caption', 'i18n_infoLabelDiscovering')
	if self.accessToken and self.accessToken ~= '' then
		local headersAccessTokenAuth = {
			['Authorization'] = 'Bearer ' .. self.accessToken
		}

		local params = {
			url = self.urls.cloudApi .. 'discover',
			method = 'GET',
			headers = headersAccessTokenAuth,
			callback =  function(response) self:onDataReceivedFromDiscoverCloudLocatedHubs(response) end
		}

		self:httpRequest(params)
	end
end

function HarmonyAccount:onDataReceivedFromDiscoverCloudLocatedHubs(response)
	if not response.status == 200 then
		self:updateProperty('ui.infoLabel.caption', 'i18n_infoLabelHubsDiscoveryFailed')
		self:updateProperty('ui.connectionStatusLabel.caption', 'i18n_connectionStatusCantConnectoToApi')

		return
	end

	local ok, jsonResponse = pcall(json.decode, response.data)
  
	if ok and jsonResponse then
		if jsonResponse.hubs then
			self.hubs = self:getHubDevicesFromExistingHC2Model() -- We doing this way in case if some one delete activity by cross button. Then we do not have notification (won't update internal model) but model should be always updates is sync button is pressed.
			self:synchronizeHubs(jsonResponse.hubs, self.hubs)
		else
			self:updateProperty('ui.infoLabel.caption', 'i18n_infoLabelHubsDiscoveryFailed')
			self:updateProperty('ui.connectionStatusLabel.caption', 'i18n_connectionStatusNotConnected')
		end
	else 
		self:updateProperty('ui.infoLabel.caption', 'i18n_infoLabelHubsDiscoveryFailed')
		self:updateProperty('ui.connectionStatusLabel.caption', 'i18n_connectionStatusNotConnected')
	end
end

function HarmonyAccount:synchronizeHubs(cloudHubs, hc2Hubs) -- sync hubs and their activities on system first start (checks only for hub and activities objects) - sync state of activities is located below
	local hc2HubsVolume = self:countTableElements(hc2Hubs)
	local cloudHubsVolume = self:countTableElements(cloudHubs)

	if cloudHubsVolume > 0 then
		for cloudHubId,cloudHubDeteils in pairs(cloudHubs) do   
			local hc2Hub = self:getObjectFromContainer(cloudHubId, hc2Hubs)       
        	if not hc2Hub then -- if hub from cloud is not in local hubs we will create it
    			self:createHubDevice(cloudHubId, cloudHubDeteils)
    		end
    	end
    end

    if hc2HubsVolume > 0 then 
    	for hc2HubId,hc2Hub in pairs(hc2Hubs) do   
			local cloudHubDeteils = self:getObjectFromContainer(hc2HubId, cloudHubs)       
        	if not cloudHubDeteils then -- if hub from HC is not in cloud hubs we will remove it
    			self:removeHub(hc2Hub)
    		end
    	end
    end

    if cloudHubsVolume > 0 then
    	for cloudHubId,cloudHubDeteils in pairs(cloudHubs) do   
			local hc2Hub = self:getObjectFromContainer(cloudHubId, hc2Hubs)       
        	if hc2Hub then -- if hub from cloud is in local hubs we will update same local id hub with datra form cloud hub and discover its activities
    			hc2Hub:updateProperties(nil, cloudHubDeteils)
    			hc2Hub:discoverCloudLocatedActivities()
    		end
    	end
    end

    setTimeout(function() self:getCurrentState() end, 5000)
end

function HarmonyAccount:getObjectFromContainer(id, container)
	if container then
		id = tostring(id)
		for objectId,object in pairs(container) do
			objectId = tostring(objectId)
			if id == objectId then			
				return object
			end
		end
	end

	return nil
end

function HarmonyAccount:createHubDevice(hubId, hubDeteils)
	local hubName = ''
	if hubDeteils.status and hubDeteils.status == 200 then
		hubName = hubDeteils.response.data.name
	end

	hubId = tostring(hubId)
    local hubDevice = plugin.createChildDevice(self.id, 'com.fibaro.logitechHarmonyHub', 'Hub ' .. hubName)
    local hubDeviceObject = Hub(self, hubDevice)
    hubDeviceObject:updateProperties(hubId, hubDeteils)
    hubDeviceObject:discoverCloudLocatedActivities()
    self.hubs[hubId] = hubDeviceObject                        
end

function HarmonyAccount:removeHubDevices()
	self.hubs = {}
	local childDevices = plugin.getChildDevices(self.id)

    for _,dev in pairs(childDevices) do
        if (dev.type == 'com.fibaro.logitechHarmonyHub') then
        	plugin.deleteDevice(dev.id)
        end
    end
end

function HarmonyAccount:removeHub(hub)
	self.hubs[hub.properties.hubId] = nil
    plugin.deleteDevice(hub.id)
end

function HarmonyAccount:getHubDevicesFromExistingHC2Model()
	local hubDevices = {}

	local childDevices = plugin.getChildDevices(self.id)

    for _,dev in pairs(childDevices) do
        if (dev.type == 'com.fibaro.logitechHarmonyHub') then
            hubDevices[dev.properties.hubId] = Hub(self, dev)
        end
    end

    return hubDevices
end

function HarmonyAccount:syncPlugin()
	self:initialMethods()
end

function HarmonyAccount:countTableElements(table)
	local counter = 0

	if table then
		for _,_ in pairs(table) do
			counter = counter + 1
		end	
	end

	return counter
end

function HarmonyAccount:getCurrentState()
	if self.accessToken and self.accessToken ~= '' then
		local headersAccessTokenAuth = {
			['Authorization'] = 'Bearer ' .. self.accessToken
		}

		local params = {
			url = self.urls.cloudApi .. 'state',
			method = 'GET',
			headers = headersAccessTokenAuth,
			callback =  function(response) self:onDataReceivedFromGetCurrentState(response) end
		}

		self:httpRequest(params)
	end
end

function HarmonyAccount:onDataReceivedFromGetCurrentState(response) 
	self:updateProperty('ui.infoLabel.caption', '')
	self.connectionStatuses.setConnected()

	local ok, jsonResponse = pcall(json.decode, response.data)
  
	if ok and jsonResponse and jsonResponse.hubs and response.status == 200 then
		for hubId, hubDev in pairs(self.hubs) do
			local cloudHubDeteils = self:getObjectFromContainer(hubId, jsonResponse.hubs)

			if cloudHubDeteils.status == 200 then -- hub is online    
				hubDev:updateProperty('isOnline', true) 

				local activitiesStatusLabelString = ""

				for activityId,activityDev in pairs(hubDev.activities) do
					activitiesStatusLabelString = activitiesStatusLabelString .. activityDev.name 
					if activityId == cloudHubDeteils.response.data.currentAvActivity then
						activityDev:updateStatus(cloudHubDeteils.response.data.activityStatus, true)
						activitiesStatusLabelString = activitiesStatusLabelString .. ': ' .. "ON" .. '\n'
					else
						activityDev:updateStatus(0, false)
						activitiesStatusLabelString = activitiesStatusLabelString .. ': ' .. "OFF" .. '\n'
					end
				end

				hubDev:updateActivitiesStatusLabel(activitiesStatusLabelString)
				hubDev:updateProperties(hubId,cloudHubDeteils)
			else --  hub is offline
				hubDev:updateProperty('isOnline', false) 			 

				hubDev:updateActivitiesStatusLabel('---')
				hubDev:updateProperties(hubId,cloudHubDeteils)
				hubDev:setAllActivitiesToOffline()				
			end	
		end
	end
end

function HarmonyAccount:getActivityById(deviceId)
	if self.hubs then
		for hubId, hubDev in pairs(self.hubs) do
			for activityId,activityDev in pairs(hubDev.activities) do
				if deviceId == activityDev.id then
					return activityDev
				end
			end
		end		
	end

	return nil
end


