require('common.device')
require('Activity')

class 'Hub' (Device)

function Hub:__init(harmonyAccount, dev)
	Device.__init(self, dev)
	
	self.harmonyAccount = harmonyAccount
	self.activities = self:getActivityDevicesFromExistingHC2Model()
end

function Hub:updateProperties(hubId, cloudLocatedHubDetails)
	if cloudLocatedHubDetails.status and cloudLocatedHubDetails.status == 200 then -- hub is cloudLocated 
		self:updateProperty('ui.connectionStatusLabelValue.caption', 'i18n_connectionStatusLabelOnline')
		self:setNewName(cloudLocatedHubDetails.response.data.name)		
	else
		self:updateProperty('ui.connectionStatusLabelValue.caption', 'i18n_connectionStatusLabelOffline')
		self:setNewName(cloudLocatedHubDetails.name)		
	end

	if hubId ~= nil then
		hubId = tostring(hubId)
		self:updateProperty('hubId', hubId)
	end
end

function Hub:updateActivitiesStatusLabel(activitiesStatus)
	self:updateProperty('ui.hubActivitiesLabelValue.caption', activitiesStatus)
end

function Hub:setNewName(newName)
	if newName ~= nil then
		newName = 'Hub ' .. newName 
		if newName ~= self.name then
			self:setName(newName)
		end
	end
end

function Hub:discoverCloudLocatedActivities()
	if self.harmonyAccount.accessToken and self.harmonyAccount.accessToken ~= '' then
		local headersAccessTokenAuth = {
			['Authorization'] = 'Bearer ' .. self.harmonyAccount.accessToken
		}

		local params = {
			url = self.harmonyAccount.urls.cloudApi .. 'hub/' .. self.properties.hubId .. '/activity/all',
			method = 'GET',
			headers = headersAccessTokenAuth,
			callback =  function(response) self:onDataReceivedFromDiscoverCloudLocatedActivities(response) end
		}

		self.harmonyAccount:httpRequest(params)
	end
end

function Hub:onDataReceivedFromDiscoverCloudLocatedActivities(response)
	local ok, jsonResponse = pcall(json.decode, response.data)
  
	if ok and jsonResponse then
		if jsonResponse.data.activities then
			self.activities = self:getActivityDevicesFromExistingHC2Model() -- We doing this way in case if some one delete activity by cross button. Then we do not have notification (won't update internal model) but model should be always updates is sync button is pressed.
			self:synchronizeActivities(jsonResponse.data.activities, self.activities)
		else
			print('No activities in json object - Retry DiscoverHubs - Hub Name:', self.name)
		end
	else 
		print('DiscoverActivities faild - Retry Authorization - Hub Name:', self.name)
	end
end

function Hub:synchronizeActivities(cloudActivities, hc2Activities) -- sync activities 
	local hc2ActivitiesVolume = self.harmonyAccount:countTableElements(hc2Activities)
	local cloudActivitiesVolume = self.harmonyAccount:countTableElements(cloudActivities)

	if cloudActivitiesVolume > 0 then
		for cloudActivityId,cloudActivityDeteils in pairs(cloudActivities) do   
			local hc2Activity = self.harmonyAccount:getObjectFromContainer(cloudActivityId, hc2Activities)       
        	if not hc2Activity then -- if Activity from cloud is not in local actibities we will create it
    			self:createActivityDevice(cloudActivityId, cloudActivityDeteils)
    		end
    	end
    end

    if hc2ActivitiesVolume > 0 then 
    	for hc2ActivityId,hc2Activity in pairs(hc2Activities) do   
			local cloudActivityDeteils = self.harmonyAccount:getObjectFromContainer(hc2ActivityId, cloudActivities)       
        	if not cloudActivityDeteils then -- if activity from HC is not in cloud activities we will remove it
    			self:removeActivity(hc2Activity)
    		end
    	end
    end

    if cloudActivitiesVolume > 0 then
    	for cloudActivityId,cloudActivityDeteils in pairs(cloudActivities) do   
			local hc2Activity = self.harmonyAccount:getObjectFromContainer(cloudActivityId, hc2Activities)       
        	if hc2Activity then 
    			hc2Activity:updateProperties(cloudActivityId, cloudActivityDeteils)
    		end
    	end
    end
end

function Hub:createActivityDevice(activityId, activityDeteils)
	local activityName = ''
	if activityDeteils.name then
		activityName = activityDeteils.name
	end

	activityId = tostring(activityId)
    local activityDevice = plugin.createChildDevice(self.id, 'com.fibaro.logitechHarmonyActivity', 'Activity ' .. activityName)
    local activityDeviceObject = Activity(self, activityDevice)
    activityDeviceObject:updateProperties(activityId, activityDeteils)
    self.activities[activityId] = activityDeviceObject  
end

function Hub:removeActivityDevices()
	local childDevices = plugin.getChildDevices(self.id)

    for _,dev in pairs(childDevices) do
        if (dev.type == 'com.fibaro.logitechHarmonyActivity') then
        	plugin.deleteDevice(dev.id)
        end
    end
end

function Hub:removeActivity(activitiy)
	self.activities[activitiy.properties.activityId] = nil
    plugin.deleteDevice(activitiy.id)
end

function Hub:getActivityDevicesFromExistingHC2Model()
	local activityDevices = {}

	local childDevices = plugin.getChildDevices(self.id)

    for _,dev in pairs(childDevices) do
        if (dev.type == 'com.fibaro.logitechHarmonyActivity') then
            activityDevices[dev.properties.activityId] = Activity(self, dev)
        end
    end

    return activityDevices
end 

function Hub:setAllActivitiesToOffline()
	for _,activityDev in pairs(self.activities) do
		activityDev:updateStatus(4, activityDev.properties['ui.startStopActivitySwitch.value'])
	end
end



