require('common.device')
require('net.HTTPClient')


class 'ncPlus' (Device)


function ncPlus:__init(id)
    Device.__init(self, id)

    self.http = net.HTTPClient({timeout = 5000})
    self.uuid = ''

    self:getUuid()
end

function ncPlus:getUuid()
    local url = 'http://' .. self.properties.ip .. ':' .. self.properties.port .. '/upnpdev'

    self.http:request(url, {
        options = {            
            method = 'GET',
        },
        success = function(response)
            self.uuid = string.match(response.data, 'uuid:(.-)<ul>')
            if self.uuid ~= '' then 
                self:setUiLabelsToConnectedState() 
            else
                self:setUiLabelsToErrorState()
            end
        end,
        error = function(error)
            self:setUiLabelsToErrorState()
        end,
    })
end

function ncPlus:sendKey(key, keyType)
    local xmlBody = self:generateXml(keyType or 'keydn', key)
    local url = 'http://' .. self.properties.ip .. ':' .. self.properties.port .. '/upnpfun/ctrl/uuid_' .. self.uuid .. '/04'

    self.http:request(url, {
        options = {
            headers = {
                    ['Content-Type'] = 'text/xml; charset="utf-8"',
                    ['SOAPACTION']   = '"urn:adbglobal.com:service:X_ADB_RemoteControl:1#ProcessInputEvent"',
            },    
            data = xmlBody,
            method = 'POST',
        },
        success = function(response) 
            if not keyType then
                self:sendKey(key, 'keyup')
            end
        end,
        error = function(error) 
            self:setUiLabelsToErrorState()
        end,
    })
end
 
function ncPlus:generateXml(keyType, key)
    local xmlBody = 
[[
<?xml version="1.0" encoding="utf-8"?> 
<s:Envelope xmlns:s="http://schemas.xmlsoap.org/soap/envelope/" s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"> 
<s:Body> 
<u:ProcessInputEvent xmlns:u="urn:adbglobal.com:service:X_ADB_RemoteControl:1"><InputEvent>ev=]] .. keyType .. [[,code=]] .. key .. [[</InputEvent></u:ProcessInputEvent></s:Body> 
</s:Envelope>
]]

    return xmlBody
end

function ncPlus:setUiLabelsToErrorState()
    self:updateProperty('ui.statusLabelValue.caption', 'i18n_statusNotConnected')
end

function ncPlus:setUiLabelsToConnectedState()
    self:updateProperty('ui.statusLabelValue.caption', 'i18n_statusConnected') 
end






