require('common.device')
require('common.unitsConverter')
require('netatmoThermostat')


class 'NetatmoDevice' (Device)

--! Initializes Netatmo Plug/Realy object.
--!@param parent: Id of a parent.
--!@param id: Id of the device.
function NetatmoDevice:__init(parent, dev)
	Device.__init(self, dev)       

    local childDevices = plugin.getChildDevices(self.id)

    for _,devs in pairs(childDevices) do
            print(devs.properties.netatmoID)
            if (devs.type == 'com.fibaro.netatmoThermostat') then
                self.childDevices[devs.properties.netatmoID] = NetatmoThermostat(self, devs)
            end
    end

    self:setVisible(false)
end

--! Adds wireless module (Thermostat) to the Netatmo Plug/Realy object.
--!@param module: Module's defining structure.
function NetatmoDevice:addModule(module)

    if self.childDevices[module._id]==nil then
        local dev = plugin.createChildDevice(self.id,'com.fibaro.netatmoThermostat', 'netatmo Thermostat')
        self.childDevices[module._id]=NetatmoThermostat(self, dev)
        self.childDevices[module._id]:updateProperty('netatmoID', module._id)
    end

end

--! Updates values and UI state basing on measures.
--!@param res: Response from the server with current state of a thermostat.
--!@param module_id: Id of a module.
function NetatmoDevice:updateState(res,module_id)
    local location=api.get("/settings/location")
    if location.temperatureUnit == 'F' then
        self.childDevices[module_id]:updateProperty('currentTemp', res.body.measured.temperature)
        self.childDevices[module_id]:updateProperty('currentSetpointMode', res.body.setpoint.setpoint_mode)
        if res.body.setpoint.setpoint_temp~=nil then
            self.childDevices[module_id]:updateProperty('currentSetpointTemp', res.body.setpoint.setpoint_temp)
            self.childDevices[module_id]:updateProperty('ui.Setpoint_Value_Label.caption', unitsConverter.celsius2farenheit(res.body.setpoint.setpoint_temp)..'°F')
        else
            self.childDevices[module_id]:updateProperty('currentSetpointTemp', res.body.measured.setpoint_temp)  
            self.childDevices[module_id]:updateProperty('ui.Setpoint_Value_Label.caption', unitsConverter.celsius2farenheit(res.body.measured.setpoint_temp)..'°F')
        end

        self.childDevices[module_id]:updateProperty('currentSetpointEndtime', res.body.setpoint.setpoint_endtime)

        self.childDevices[module_id]:updateProperty('ui.Current__MSetpoint_Value_Label.caption', unitsConverter.celsius2farenheit(res.body.measured.setpoint_temp)..'°F')
        if res.body.setpoint.setpoint_temp~=nil then
            self.childDevices[module_id]:updateProperty('ui.Current__Setpoint_Value_Label.caption', unitsConverter.celsius2farenheit(res.body.setpoint.setpoint_temp)..'°F')
             self.childDevices[module_id]:updateProperty('setpointTemp', res.body.setpoint.setpoint_temp)
        else
            self.childDevices[module_id]:updateProperty('setpointTemp',res.body.measured.setpoint_temp)
            self.childDevices[module_id]:updateProperty('ui.Current__Setpoint_Value_Label.caption', unitsConverter.celsius2farenheit(res.body.measured.setpoint_temp)..'°F')
        end
        self.childDevices[module_id]:updateProperty('ui.Current__Temperature_Value_Label.caption', unitsConverter.celsius2farenheit(res.body.measured.temperature)..'°F')
    else
        self.childDevices[module_id]:updateProperty('currentTemp', res.body.measured.temperature)
        self.childDevices[module_id]:updateProperty('currentSetpointMode', res.body.setpoint.setpoint_mode)
        if res.body.setpoint.setpoint_temp~=nil then
            self.childDevices[module_id]:updateProperty('currentSetpointTemp', res.body.setpoint.setpoint_temp)        
            self.childDevices[module_id]:updateProperty('ui.Setpoint_Value_Label.caption', res.body.setpoint.setpoint_temp..'°C')
        else
            self.childDevices[module_id]:updateProperty('currentSetpointTemp', res.body.measured.setpoint_temp)
            self.childDevices[module_id]:updateProperty('ui.Setpoint_Value_Label.caption', res.body.measured.setpoint_temp..'°C')
        end

        self.childDevices[module_id]:updateProperty('currentSetpointEndtime', res.body.setpoint.setpoint_endtime)

        self.childDevices[module_id]:updateProperty('ui.Current__MSetpoint_Value_Label.caption', res.body.measured.setpoint_temp..'°C')
        if res.body.setpoint.setpoint_temp~=nil then
            self.childDevices[module_id]:updateProperty('ui.Current__Setpoint_Value_Label.caption', res.body.setpoint.setpoint_temp..'°C')
            self.childDevices[module_id]:updateProperty('setpointTemp',res.body.setpoint.setpoint_temp)
        else
            self.childDevices[module_id]:updateProperty('ui.Current__Setpoint_Value_Label.caption', res.body.measured.setpoint_temp..'°C')
            self.childDevices[module_id]:updateProperty('setpointTemp',res.body.measured.setpoint_temp)
        end
        self.childDevices[module_id]:updateProperty('ui.Current__Temperature_Value_Label.caption', res.body.measured.temperature..'°C')
    end

    local mode=res.body.setpoint.setpoint_mode
    if mode=="away" then
        self.childDevices[module_id]:updateProperty('ui.Away_Switch.value', true)
        self.childDevices[module_id]:updateProperty('ui.HG_Switch.value', false)
    elseif mode=="hg" then
        self.childDevices[module_id]:updateProperty('ui.HG_Switch.value', true)
        self.childDevices[module_id]:updateProperty('ui.Away_Switch.value', false)
    else
        self.childDevices[module_id]:updateProperty('ui.Away_Switch.value', false)
        self.childDevices[module_id]:updateProperty('ui.HG_Switch.value', false)
    end
    self:updateProperty('lastUpdated',res.body.last_plug_seen)
    self.childDevices[module_id]:updateProperty('lastUpdated', res.body.last_therm_seen)

end


