require('common.device')
require('common/TCPListener')


class 'NuVoZone' (Device)


function NuVoZone:__init(parent, dev, name)
	Device.__init(self, dev)

	self.port = 4747
	self.ip = self.properties.ip
	self.mute = false
	self.parent = parent

	self:setNewName(name)

	self.uiBinding = {
		["mute_Switch"] = 	function(eventArgs)
								self:setMute(eventArgs.values[1])
							end,
		["play_Button"] =	function() self:sendCommand("play") end,
		["pause_Button"] =	function() self:sendCommand("pause") end,
		["next_Button"] =	function() self:sendCommand("next") end,
		["prev_Button"] =	function() self:sendCommand("previous") end,
		["volume_Slider"] = function(eventArgs) self:sendCommand("volume", eventArgs.values[1]) end,

		-- This is for test puposes (until input control is developed) 
		["source_Button"] =	function() nuvo:setAVTransportUri(self.properties['currentSourceURI']) end,
		--["ungroup_Button"] = function() nuvo:getQueue() end, --ungroup() end,
	}

	self.commands = {}	
	self.commands.BROWSE = '{"id" : "req-1", "url" : "/stable/av/", "method" : "browse"}'
	self.commands.NEXT = '{ "id" : "req-2","url" : "nuvoApi:av/next","method" : "invoke"}'
	self.commands.PREVIOUS = '{ "id" : "req-2","url" : "nuvoApi:av/previous","method" : "invoke"}'
	self.commands.PAUSE = '{ "id" : "req-2","url" : "nuvoApi:av/pause","method" : "invoke"}'
	self.commands.PLAY = '{ "id" : "req-2","url" : "nuvoApi:av/play","method" : "invoke"}'
	self.commands.LIKE = '{ "id" : "req-2","url" : "nuvoApi:av/like","method" : "invoke"}'
	self.commands.DISLIKE = '{ "id" : "req-2","url" : "nuvoApi:av/dislike","method" : "invoke"}'
	self.commands.VOLUME_HEAD = '{"id" : "req-5", "url" : "/stable/av/volume", "method" : "updateValue", "params" : {"value" : {"int" :'
	self.commands.VOLUME_TAIL = '}}}'
    self.commands.MUTE = '{"id" : "req-5", "url" : "nuvoApi:av/volume", "method" : "updateValue", "params" : {"value" : {"bool" : true}}}'
    self.commands.UNMUTE = '{"id" : "req-5", "url" : "nuvoApi:av/volume", "method" : "updateValue", "params" : {"value" : {"bool" : false}}}'
	self.commands.SEEK_HEAD = '{"id" : "req-7", "url" : "/stable/av/trackIndex", "method" : "updateValue", "params" : {"value" : {"int" :' -- count from 0
	self.commands.SEEK_TAIL = '}}}'

	self.sock = TCPListener({delimiter = "\n", readTimeout = 2000, connectTimeout=2000})
	self.sock:registerEvent('connected', function() self:onConnected() end)
	self.sock:registerEvent('disconnected', function() self:onClosed() end)
	self.sock:registerEvent('dataReceived', function(x, data) self:onReceived(data) end)

	self.sock:connect(self.ip, self.port)
end

function NuVoZone:setNewName(newName)
	if newName ~= nil then 
		if newName ~= self.name then 
			self:setName(newName)
			self.name = newName
			self.parent:updateZonesTable()
		end
	end
end

function NuVoZone:setMute(value)
	self:updateProperty('ui.mute_Switch.value', value)
	if value then
		self:send(self.commands.MUTE)
	else
		self:send(self.commands.UNMUTE)
	end
end

function NuVoZone:onConnected()
	self:updateProperty('dead', false)
	self.parent:updateZonesTableWithZone(self)
    self:send(self.commands.BROWSE)
end

function NuVoZone:onClosed()
	self:updateProperty('dead', true)
	self.parent:updateZonesTableWithZone(self)
    setTimeout(function() self.sock:connect(self.ip, self.port) end, 3000)
end

function NuVoZone:setNowPlayingLabel(playing)
	self:updateProperty("ui.albumArt_Image.source", playing["icon"])
	self:updateProperty("ui.creator_Label.caption", playing["title"])
	self:updateProperty("ui.title_Label.caption", playing["subtitle"])
end

function NuVoZone:setVolumeLabel(volume)
	if volume then
		self:updateProperty('ui.volume_Label.caption', tostring(volume["volume"]))
		self:updateProperty('ui.volume_Slider.value', volume["volume"])	
		self:updateProperty('volume', volume["volume"])
		self:updateProperty('ui.mute_Switch.value', volume.muted)

		self.parent:refreshGlobalVolume()
	end
end

function NuVoZone:onNoMedia()
	self:updateProperty('ui.volume_Label.caption', tostring(0))
	self:updateProperty('ui.volume_Slider.value', 0)
	self:updateProperty('volume', 0)
	self.volume = 0	
	self:updateProperty("ui.albumArt_Image.source", '/plugins/com.fibaro.nuvoPlayer/img/defaultIcon.png')
	self:updateProperty("ui.creator_Label.caption", "no media")
	self:updateProperty("ui.title_Label.caption", "")

	self.parent:refreshGlobalVolume()
end

function NuVoZone:setVolume(volume)
	if volume then
		self:updateProperty('ui.volume_Label.caption', tostring(volume))
		self:updateProperty('ui.volume_Slider.value', volume)
		self:updateProperty('volume', volume)
		
		self.volume = volume

		self.parent:refreshGlobalVolume()
	end	
end

function NuVoZone:send(command)
	self.sock:send(command)
end

function NuVoZone:sendCommand(command, value)
	if command == 'play' then
		self:send(self.commands.PLAY)
	elseif command == 'pause' then
		self:send(self.commands.PAUSE)
	elseif command == 'next' then
		self:send(self.commands.NEXT)
	elseif command == 'previous' then
		self:send(self.commands.PREVIOUS)
	elseif command == 'volume' then
		if value < 0
		then
			value = 0
		elseif value > 100
		then
			value = 100
		end
		self:send(self.commands.VOLUME_HEAD .. value .. self.commands.VOLUME_TAIL)
	elseif command == 'seek' then
		self:send(self.commands.SEEK_HEAD .. value .. self.commands.SEEK_TAIL)
	end
end

function NuVoZone:onReceived(data)
    data = json.decode(data)
    if data["type"] == "reply" then 
    	if data["result"] ~= nil then
    		self:setNewName(data["result"]["item"]["title"])
    		for i,value in ipairs(data["result"]["children"]) do
    			if value["id"] ~= nil and value["id"] == "volume" then
    				self:setVolume(value["value"]["volume"]["level"])
    			elseif value["id"] ~= nil and value["id"] == "state" then
    				if value["value"]["avState"] == "noMedia" then
    					self:onNoMedia()
    				end
    			elseif value["id"] ~= nil and value["id"] == "info" then
    				self:setNowPlayingLabel(value)
    			end
    		end
    	end
    elseif data["type"] == "event" then
    	for i,value in ipairs(data["event"]) do
    		if value["item"] ~= nil then
    			if value.item.title and value.item.icon and value.item.subtitle and value.item.creator and not value.item.containerType then
    				self:setNowPlayingLabel({
    					['icon'] = value.item.icon,
    					['subtitle'] = value.item.subtitle,
    					['title'] = value.item.title,

    				})
    			end
    			if value["item"]["title"] ~= nil  and value["item"]["type"] ~= nil and value["item"]["type"] == 'container' then
    				self:setNewName(value["item"]["title"])
    				self.parent:updateZonesTable()
				elseif value["item"]["url"] == "nuvoApi:av/volume" then
    				muted = false
    				volume = 0 

    				if value["item"]["value"]["volume"]["mute"] ~= nil and value["item"]["value"]["volume"]["mute"] == true then
    					muted = true
    				end
    				if value["item"]["value"]["volume"]["level"] ~= nil then
    					volume = value["item"]["value"]["volume"]["level"]
    				end
    					self:setVolumeLabel{["volume"] = volume, ["muted"] = muted}
    			elseif value["item"]["value"] ~= nil and value["item"]["value"]["avState"] ~= nil  then
    				if value["item"]["value"]["avState"] == "noMedia" then
    					self:onNoMedia()
    				end
    			end     			    			   			    		
			end			
    	end
    end
end


