require('common.device')
require('common.base64')
require('net.HTTPClient')
cosmo = require("cosmo.cosmo")

class 'Philips' (Device)

Philips.COMMANDS = {
	["KEY_ENTER"] = 'cursorOk',
	["KEY_UP"] = 'cursorUp',
	["KEY_DOWN"] = 'cursorDown',
	["KEY_RIGHT"] = 'cursorRight',
	["KEY_LEFT"] = 'cursorLeft',
	["KEY_HOME"] = 'home',
	["KEY_OPTIONS"] = 'optionsMenu',
	["KEY_RETURN"] = 'back',
	["KEY_1"] = 'numb1',
	["KEY_2"] = 'numb2',
	["KEY_3"] = 'numb3',
	["KEY_4"] = 'numb4',
	["KEY_5"] = 'numb5',
	["KEY_6"] = 'numb6',
	["KEY_7"] = 'numb7',
	["KEY_8"] = 'numb8',
	["KEY_9"] = 'numb9',
	["KEY_0"] = 'numb0',
	["KEY_POWEROFF"] = 'power',
	["KEY_AUDIO"] = 'audio',
	["KEY_SUBTITLE"] = 'subtitle',
	["KEY_YELLOW"] = 'yellowButton',
	["KEY_BLUE"] = 'blueButton',
	["KEY_RED"] = 'redButton',
	["KEY_GREEN"] = 'greenButton',
	["KEY_PLAY"] = 'play',
	["KEY_STOP"] = 'stop',
	["KEY_PAUSE"] = 'pause',
	["KEY_REWIND"] = 'rewind',
	["KEY_FORWARD"] = 'forward',
	["KEY_PREV"] = 'previous',
	["KEY_NEXT"] = 'next',
	["KEY_TOP_MENU"] = 'topMenu',
	["KEY_POP_UP_MENU"] = 'popUpMenu',
	["KEY_EJECT"] = 'eject'
}


Philips.CONTROL_URI = "/smctrl.app"

Philips.UPnP_COMMAND = '{"request":{"requestType":"action","control":"$command","value":"$command","timestamp":"1384971059135"}}'

function Philips:__init(id)
	Device.__init(self, id)
	self.name = 'FibaroRemote'
	self.controlUrl = 'http://' .. self.properties.ip .. ':51000' .. Philips.CONTROL_URI
	self.http = net.HTTPClient({ timeout = 3000 })
end


function Philips:sendKey(code)
	cmd = Philips.COMMANDS[code]

	if (cmd) then
		local body = cosmo.fill(Philips.UPnP_COMMAND, { command = cmd })
		self:__sendRequest(body)
	end
end


function Philips:__sendRequest(command)
	local controlHeaders = {
		["Content-Type"] = 'application/json; charset=UTF-8'
	}
	print(self.controlUrl)
	self.http:request(self.controlUrl, {
		options ={
			headers = controlHeaders,
			method = 'POST',
			data = command
		},
		success = function(param)
			print('Succes STATUS: ' .. param.status)
			print('Succes DATA: ' .. param.data)
		end,
		error = function (error)
			print('*******************')
			print('* ERROR: '.. error)
			print('*******************')
		end
	})
end