colorConverter = {}

colorConverter.colorTriangle = {}
colorConverter.colorTriangle["red"] = {["x"]=0.674, ["y"]=0.322}
colorConverter.colorTriangle["green"] = {["x"]=0.408, ["y"]=0.517}
colorConverter.colorTriangle["blue"] = {["x"]=0.168, ["y"]=0.041}

-- x,y ==> <0,1>
colorConverter.xy2rgb = function(x, y)
    local point = {["x"]=x, ["y"]=y}
    if colorConverter.isPointInLampsReach(point)==false then
       point = colorConverter.getClosestPointInTriangle(point) 
       x = point.x
       y = point.y
    end

    local z = 1.0 - x - y

    local Y = 1.0
    local X = (Y/y) * x
    local Z = (Y/y) * z

    local r = X *  3.2406 + Y * -1.5372 + Z * -0.4986
    local g = X * -0.9689 + Y *  1.8758 + Z *  0.0415
    local b = X *  0.0557 + Y * -0.2040 + Z *  1.0570

    local max = math.max(r,g,b)
    if (max>1) then
        r = r/max
        g = g/max
        b = b/max
    end

    if r <= 0.0031308 then r = 12.92 * r; else r = (1.055) * math.pow(r, (1.0 / 2.4)) - 0.055; end
    if g <= 0.0031308 then g = 12.92 * g; else g = (1.055) * math.pow(g, (1.0 / 2.4)) - 0.055; end
    if b <= 0.0031308 then b = 12.92 * b; else b = (1.055) * math.pow(b, (1.0 / 2.4)) - 0.055; end

    max = math.max(r,g,b)
    if (max>1) then
        r = r/max
        g = g/max
        b = b/max
    end

    r = math.max(0, r)
    g = math.max(0, g)
    b = math.max(0, b)

    return {["r"] = r, ["g"] = g, ["b"] = b}
end

-- r, g, b ==> <0,1>
colorConverter.rgb2xy = function(red, green, blue)
    local r = 0;
    local g = 0;
    local b = 0;
    if red   > 0.04045 then r = math.pow((red   + 0.055) / (1.055), 2.400000095367432) else r =  (red   / 12.92) end
    if green > 0.04045 then g = math.pow((green + 0.055) / (1.055), 2.400000095367432) else g =  (green / 12.92) end
    if blue  > 0.04045 then b = math.pow((blue  + 0.055) / (1.055), 2.400000095367432) else b =  (blue  / 12.92) end

    local X = r * 0.649926 + g * 0.103455 + b * 0.197109;
    local Y = r * 0.234327 + g * 0.743075 + b * 0.022598;
    local Z = r * 0.0000000 + g * 0.053077 + b * 1.035763;
    
    local cx = X / (X + Y + Z);
    local cy = Y / (X + Y + Z);
    
    if cx == 0/0 then
        cx = 0.0;        
    end

    if cy == 0/0 then
        cy = 0.0;
    end

    local point = {["x"]=cx, ["y"]=cy}
    if colorConverter.isPointInLampsReach(point)==false then
       point = colorConverter.getClosestPointInTriangle(point) 
       cx = point.x
       cy = point.y
    end

    return {
        ["x"] = cx,
        ["y"] = cy,
    };
end

colorConverter.getDistanceBetweenTwoPoints = function(point1, point2)
    local dx = point1.x - point2.x;
    local dy = point1.y - point2.y;
    local dist = math.sqrt(dx * dx + dy * dy);

    return dist;
end

colorConverter.crossProduct = function(point1, point2)
    return point1.x * point2.y - point1.y * point2.x;
end

colorConverter.isPointInLampsReach = function(point)
    local v1 = {}
    v1.x = colorConverter.colorTriangle.green.x-colorConverter.colorTriangle.red.x
    v1.y = colorConverter.colorTriangle.green.y-colorConverter.colorTriangle.red.y

    local v2 = {}
    v2.x = colorConverter.colorTriangle.blue.x-colorConverter.colorTriangle.red.x
    v2.y = colorConverter.colorTriangle.blue.y-colorConverter.colorTriangle.red.y

    q = {}
    q.x = point.x-colorConverter.colorTriangle.red.x
    q.y = point.y-colorConverter.colorTriangle.red.y

    s = colorConverter.crossProduct(q, v2) / colorConverter.crossProduct(v1, v2);
    t = colorConverter.crossProduct(v1, q) / colorConverter.crossProduct(v1, v2);
    if ((s >= 0.0) and (t >= 0.0) and (s + t <= 1.0)) then
        return true;
    end
    
    return false;
end

colorConverter.getClosestPointInTriangle = function(point)
    local pAB = colorConverter.getClosestPointOnLine(point, colorConverter.colorTriangle.red, colorConverter.colorTriangle.green);
    local pAC = colorConverter.getClosestPointOnLine(point, colorConverter.colorTriangle.blue, colorConverter.colorTriangle.red);
    local pBC = colorConverter.getClosestPointOnLine(point, colorConverter.colorTriangle.green, colorConverter.colorTriangle.blue);

    local dAB = colorConverter.getDistanceBetweenTwoPoints(point, pAB);
    local dAC = colorConverter.getDistanceBetweenTwoPoints(point, pAC);
    local dBC = colorConverter.getDistanceBetweenTwoPoints(point, pBC);

    local lowest = dAB;
    local closestPoint = pAB;
    if (dAC < lowest) then
        lowest = dAC;
        closestPoint = pAC;
    end
    if (dBC < lowest) then
        lowest = dBC;
        closestPoint = pBC;
    end

    return closestPoint
end

colorConverter.getClosestPointOnLine = function(point, linePointA, linePointB)
    local pointAP = {["x"] = point.x - linePointA.x, ["y"] = point.y - linePointA.y};
    local pointAB = {["x"] = linePointB.x - linePointA.x, ["y"] = linePointB.y - linePointA.y};
    local ab2 = pointAB.x * pointAB.x + pointAB.y * pointAB.y;
    local apAb = pointAP.x * pointAB.x + pointAP.y * pointAB.y;
    local t = apAb / ab2;

    if (t < 0.0) then
      t = 0.0;
    elseif (t > 1.0) then
      t = 1.0;
    end

    local newPoint = {["x"]=linePointA.x + pointAB.x * t, ["y"]=linePointA.y + pointAB.y * t};
    return newPoint;
end
