--
-- Tested on:
-- Product: Pioneer AV Receiver
-- Model number: VSX-828-K
-- Software Version: s9294.4004.1066
--

require('pioneer')
require('common.FIButtonUpDown')
--
-- create main class
--
pioneer = Pioneer(plugin.mainDeviceId)

uiBinding = {
	["mute_Switch"] = 	function(eventArgs)
							print(eventArgs.values[1])
							pioneer:setMute(eventArgs.values[1])
						end,

	["power_Switch"] =	function(eventArgs)
							if (eventArgs.values[1]) then
								pioneer:turnOn()
							else
								pioneer:turnOff()
							end
						end,
	["HDMI4_Button"] = function() pioneer:setInput('HDMI 4') end,
	["HDMI5_Button"] = function() pioneer:setInput('HDMI 5') end,
	["HDMI6_Button"] = function() pioneer:setInput('HDMI 6') end,
	["HDMI7_Button"] = function() pioneer:setInput('HDMI 7') end,

	["CD_Button"] = function() pioneer:setInput('CD') end,
	["DVD_Button"] = function() pioneer:setInput('DVD') end,
	["BD_Button"] = function() pioneer:setInput('BD') end,
	["iPod_Button"] = function() pioneer:setInput('iPod/USB') end,

	["CBL_SAT_Button"] = function() pioneer:setInput('CBL/SAT') end,
	["DVD_BDR_Button"] = function() pioneer:setInput('DVD/BDR') end,
	["TV_SAT_Button"] = function() pioneer:setInput('TV/SAT') end,

	["ADAPTER_PORT_Button"] = function() pioneer:setInput('ADAPTER PORT') end,
	["MEDIA_SERVER_Button"] = function() pioneer:setInput('MEDIA SERVER') end,
	["TUNER_Button"] = function() pioneer:setInput('TUNER') end,
	["INTERNET_RADIO_Button"] = function() pioneer:setInput('INTERNET RADIO') end,
	["FAVORITE_Button"] = function() pioneer:setInput('FAVORITE') end,
	["Minus_Button"] = function() pioneer:setVolume("VD") end,
	["Plus_Button"] = function() pioneer:setVolume("VU") end
}

uiBindingForRepetition = {
	["Minus_Button"] = function() pioneer:setVolume("VD") end,
	["Plus_Button"] = function() pioneer:setVolume("VU") end
}

fiButtonUpDown = FIButtonUpDown({
	repeatAfter=1000,
	refreshRate=100,
	normalBinding=uiBinding,
	repeatBinding=uiBindingForRepetition
})

pioneer.onPropertyChanged = {
	power = function(dev, value)
		if (value == false) then
			dev:updateProperty('ui.input_Label.caption', '---')
			dev:updateProperty('ui.volume_Label.caption', '---')
		else
			dev:updateProperty('ui.input_Label.caption', dev.properties.input)
			dev:updateProperty('ui.volume_Label.caption', tostring(dev.properties.volume))
		end

		dev:updateProperty('ui.power_Switch.value', value)
		dev:refreshState()
	end,

	volume = function(dev, value)
		dev:updateProperty('ui.volume_Label.caption', tostring(value))
	end,

	mute = function(dev, value)
		if (value == true) then
			dev:updateProperty('ui.volume_Label.caption', '---')
		else
			dev:updateProperty('ui.volume_Label.caption', tostring(dev.properties.volume))
		end
		
		dev:updateProperty('ui.mute_Switch.value', value)
	end,

	input = function(dev, value)
		dev:updateProperty('ui.input_Label.caption', value)
	end
}

function onAction(deviceId, action)
	pioneer:callAction(action.actionName, unpack(action.args))
end

function onUIEvent(deviceId, event)
	fiButtonUpDown:onUIEvent(deviceId, event)
end

function configure(deviceId, config)
	if (config.ip) then
		if (pioneer:updateProperty('ip', config.ip)) then
			plugin.restart()
		end
	end
end