require('common.device')
require('net.HTTPClient')


class 'PlanikaFla' (Device)


function PlanikaFla:__init(id)
	Device.__init(self, id)

	self.stateTable = {
		[1] = "READY",
	    [2] = "PLEASE WAIT",
		[3] = "COOLING THE DEVICE",
		[4] = "FILL WITH FUEL",
		[5] = "NO FUEL",
		[6] = "FULL FUEL",
		[252] = "SERVICE MODE",
		[7] = "COOLING HIT",
		[8] = "COOLING TILTED DEVICE",
		[9] = "COOLING THE DEVICE",
		[10] = "NO FUEL - REFUEL WHEN THE DEVICE IS COOLED",
		[20] = "WORKING",
		[60] = "MENU",
		[70] = "MENU",
		[71] = "MENU",
		[71] = "MENU",
		[73] = "MENU",
		[74] = "MENU",
		[75] = "MENU",
		[76] = "MENU",
		[251] = "MENU",
		[80] = "PILOT",
		[100] = "ERROR",
		[19] = "COOLING - CO2",
		[198] = "PILOT ERROR",
		[250] = "DEVICE LOCK",
		[254] = "LOGO"
	}  

	self.commandUri = "/but.php"
	self.currentStateUri = "/odczyty1.var"
	self.currentNameUri = "/nazwa.txt"

	self.http = net.HTTPClient({timeout = 1000})

	self.pollingForStatusTime = 500
	self.pollingCommandId = nil
	self.connectionStatuTimeoutId = nil
	self.errorCounter = 0
	self.canRefreshUi = true

	self:updateStatusAndName()
end

function PlanikaFla:sendRequest(params)
	local url = 'http://' .. self.properties.ip .. params.url

	self.http:request(url, {
		options = {
			method = "GET",
		},
		success = function(response)
			if (params.callback) then
				if response.status and response.status == 200 then
					self.errorCounter = 0
					params.callback(response)
				else
					self.errorCounter = self.errorCounter + 1

					if self.errorCounter > 5 then -- Hack for planika error in Json - ensures the error is permanent
						self:updateUi("ui.CONNECT_TO_LABEL_VALUE.caption", "NOT CONNECTED") 
					end
				end
			end
		end,
		error = function(error)
			print(error)
			self.errorCounter = self.errorCounter + 1

			if self.errorCounter > 5 then
				self:updateUi("ui.CONNECT_TO_LABEL_VALUE.caption", "NOT CONNECTED") 
			end
		end
	})
end

function PlanikaFla:start()
	local params = {
		["url"]	= self.commandUri .. "?id=ButtonStart",
	}

	self:sendRequest(params)
end

function PlanikaFla:stop()
	local params = {
		["url"]	= self.commandUri .. "?id=ButtonON",
	}

	self:sendRequest(params)
end

function PlanikaFla:plus()
	local params = {
		["url"]	= self.commandUri .. "?id=ButtonPlus",
	}

	self:sendRequest(params)
end

function PlanikaFla:minus()
	local params = {
		["url"]	= self.commandUri .. "?id=ButtonMinus",
	}

	self:sendRequest(params)
end

function PlanikaFla:changeLockState(state)
	self:pauseRefreshUi(2)

	local action = ""

	if state then
		action = "?id=ButtonLockON"
	else
		action = "?id=ButtonLockOFF"
	end

	local params = {
		["url"]	= self.commandUri .. action,
	}

	self:sendRequest(params)

	self:updateProperty('ui.LOCK_ON_OFF_SWITCH.value', state)
end

function PlanikaFla:updateStatusAndName()
	if (self.pollingCommandId) then
  	 	clearTimeout(self.pollingCommandId)
	end

	local paramsForStatusRequest = {
		["url"]	= self.currentStateUri,
		["callback"] = function(response) self:statusCallback(response) end,
	}
	self:sendRequest(paramsForStatusRequest)


	local paramsForNameRequest = {
		["url"]	= self.currentNameUri,
		["callback"] = function(response) self:updateNameCallback(response) end,
	}
	self:sendRequest(paramsForNameRequest)

	self:checkConnectionStatus()

    self.pollingCommandId = setTimeout(function( ) self:updateStatusAndName(); end, self.pollingForStatusTime)
end

function PlanikaFla:statusCallback(response)
	local ok, jsonResponse = pcall(json.decode, response.data)
  
	if ok and jsonResponse then
		for k,v in pairs(jsonResponse.odczyty) do
			self:updateUi(k,v)
		end
	else 
		print("error in json", response.data) -- silent error - in case of bad json occurred on every "n" cycles
	end
end

function PlanikaFla:updateNameCallback(response)
	if response.data then
		self:updateUi("name", response.data)
	end
end

function PlanikaFla:updateUi(key, value)
	if self.canRefreshUi then
		if key == "fuel" then
			self:updateProperty("ui.FUEL_LEVEL_LABEL_VALUE.caption", tostring(value))
		elseif key == "ogien" then
			self:updateProperty("ui.FLAME_SIZE_LABEL_VALUE.caption", tostring(value / 2))
		elseif key == "tryb" then
			self:updateProperty("ui.STATUS_LABEL_VALUE.caption", self.stateTable[value])	
			if tostring(value) == "250" then
				self:updateProperty('ui.LOCK_ON_OFF_SWITCH.value', true)
			else
				self:updateProperty('ui.LOCK_ON_OFF_SWITCH.value', false)
			end
		elseif key == "name" then
			self:updateProperty("ui.CONNECT_TO_LABEL_VALUE.caption", value)
		else
			self:updateProperty(key, value)
			if type(value) == "string" then
				self:updateProperty("ui.STATUS_LABEL_VALUE.caption", value)
			end
		end
	end
end

function PlanikaFla:getBinarySwitches()
	local binarySwitchesList = {{["value"] = 0, ["label"] = "---"}}

	local devicesList = api.get('/devices')

	if devicesList and #devicesList > 0 then		
		for k,v in pairs(devicesList) do
			if type(v) == "table" and (v.baseType == "com.fibaro.binarySwitch" or v.type == "com.fibaro.binarySwitch") then
				table.insert(binarySwitchesList, {["value"] = tonumber(v.id), ["label"] = tostring(v.name)})
			end
		end
	end

	return binarySwitchesList
end

function PlanikaFla:checkConnectionStatus()
	if self.properties.timeToDisconnect < 1 then
		print ("timeToDisconnect is to small")
		self:updateProperty("timeToDisconnect", 1)
	end

	if self.properties["ui.CONNECT_TO_LABEL_VALUE.caption"] == "NOT CONNECTED" and not self.connectionStatuTimeoutId then
		self.connectionStatuTimeoutId = setTimeout(function( ) self:turnOffAssignedBinarySwitch() end, self.properties.timeToDisconnect * 1000 * 60 )
	elseif self.properties["ui.CONNECT_TO_LABEL_VALUE.caption"] ~= "NOT CONNECTED" and self.connectionStatuTimeoutId then
	 	clearTimeout(self.connectionStatuTimeoutId)

 		self.connectionStatuTimeoutId = nil
 	end
end

function PlanikaFla:turnOffAssignedBinarySwitch()
	api.get("/callAction?deviceID=" .. tonumber(self.properties.selectedSwitch) .. "&name=" .. "turnOff")
end

-- Hack for planika slow refresh - removes double blink for our switch control
function PlanikaFla:pauseRefreshUi(seconds)
	self.canRefreshUi = false
	setTimeout(function () self.canRefreshUi = true end, seconds * 1000)
end



