require('common.device')
require('common.TCPListener')


class 'RussoundMCA88XZone' (Device)
   

function RussoundMCA88XZone:__init(russoundController, dev)
	Device.__init(self, dev)
 	
	self.russoundMCA88X = russoundController.russoundMCA88X
	self.russoundController = russoundController
	self.delimiter = '\r\n'
	self.currentSource = '' 
	self.queue = {}

	self.uiBinding = {
		['mute_Switch']    = function(eventArgs) self:setMuteState(eventArgs.values[1]) self.russoundMCA88X:updateSystemLabels() end,
		['play_Button']    = function() self:play() 	self.russoundMCA88X:updateSystemLabels() end,
		['pause_Button']   = function() self:pause() self.russoundMCA88X:updateSystemLabels() end,
		['next_Button']    = function() self:next() self.russoundMCA88X:updateSystemLabels() end,
		['prev_Button']    = function() self:previous() self.russoundMCA88X:updateSystemLabels() end,
		['volume_Slider']  = function(eventArgs) self:setVolume(eventArgs.values[1]) self.russoundMCA88X:updateSystemLabels() end,
		['power_Switch']   = function() self:setPower() self.russoundMCA88X:updateSystemLabels() end,
		['source1_Button'] = function() self:activateSource(1) self.russoundMCA88X:updateSystemLabels() end,
		['source2_Button'] = function() self:activateSource(2) self.russoundMCA88X:updateSystemLabels() end,
		['source3_Button'] = function() self:activateSource(3) self.russoundMCA88X:updateSystemLabels() end,
		['source4_Button'] = function() self:activateSource(4) self.russoundMCA88X:updateSystemLabels() end,
		['source5_Button'] = function() self:activateSource(5) self.russoundMCA88X:updateSystemLabels() end,
		['source6_Button'] = function() self:activateSource(6) self.russoundMCA88X:updateSystemLabels() end,
		['source7_Button'] = function() self:activateSource(7) self.russoundMCA88X:updateSystemLabels() end,
		['source8_Button'] = function() self:activateSource(8) self.russoundMCA88X:updateSystemLabels() end,
	}			

	self:updateResponseHandlers()
	self:setNoMedia()
end 

function RussoundMCA88XZone:updateResponseHandlers()
	self.responsHandlers = {
		[self.properties.id .. '.name='] 	      		  = function(name) self:setNewName(name) self.russoundMCA88X:updateSystemLabels() end, 
		[self.properties.id .. '.volume='] 	      		  = function(volumeValue) self:setVolumeLabel(volumeValue) self.russoundMCA88X:updateSystemLabels() end, 
		[self.properties.id .. '.status=']	      		  = function(status) self:setPowerSwitch(status) self.russoundMCA88X:updateSystemLabels() end,
		[self.properties.id .. '.mute=']          		  = function(muteState) self:setMuteLabel(muteState) self.russoundMCA88X:updateSystemLabels() end,
		[self.properties.id .. '.partyMode=']     		  = function(status) self:setPartyMode(status) self.russoundMCA88X:updateSystemLabels() end,
		[self.properties.id .. '.currentSource='] 		  = function(sourceNumber) self:setCurrentSourceLabel(sourceNumber) self.russoundMCA88X:updateSystemLabels() end,
		['S[' .. self.currentSource .. '].coverArtURL=']  = function(url) self:setAlbumImage(url) self.russoundMCA88X:updateSystemLabels() end,
		['S[' .. self.currentSource .. '].artistName=']	  = function(artist) self:setArtistLabel(artist) self.russoundMCA88X:updateSystemLabels() end,
		['S[' .. self.currentSource .. '].albumName=']	  = function(album) self:setAlbumLabel(album) self.russoundMCA88X:updateSystemLabels() end,
		['S[' .. self.currentSource .. '].songName=']	  = function(songName) self:setSongNameLabel(songName) self.russoundMCA88X:updateSystemLabels() end,
		['S[' .. self.currentSource .. '].mode=']         = function(modeString) self:setPartyMode(modeString) self.russoundMCA88X:updateSystemLabels() end,
		['S[1].name=']    								  = function(name) self:setSourceButtonName(1, name) self.russoundMCA88X:updateSystemLabels() end,
		['S[2].name=']               			 		  = function(name) self:setSourceButtonName(2, name) self.russoundMCA88X:updateSystemLabels() end,
		['S[3].name=']               			 		  = function(name) self:setSourceButtonName(3, name) self.russoundMCA88X:updateSystemLabels() end,
		['S[4].name=']               			 		  = function(name) self:setSourceButtonName(4, name) self.russoundMCA88X:updateSystemLabels() end,
		['S[5].name=']               			 		  = function(name) self:setSourceButtonName(5, name) self.russoundMCA88X:updateSystemLabels() end,
		['S[6].name=']               			 		  = function(name) self:setSourceButtonName(6, name) self.russoundMCA88X:updateSystemLabels() end,
		['S[7].name=']               			 		  = function(name) self:setSourceButtonName(7, name) self.russoundMCA88X:updateSystemLabels() end,
		['S[8].name=']               			 		  = function(name) self:setSourceButtonName(8, name) self.russoundMCA88X:updateSystemLabels() end,
		['S[' .. self.currentSource .. '].playlistName='] = function(name) self:setPlayListNameLabel(name) self.russoundMCA88X:updateSystemLabels() end,
		['S[' .. self.currentSource .. '].channelName=']  = function(name) self:setChannelNameLabel(name) self.russoundMCA88X:updateSystemLabels() end,
	}
end

function RussoundMCA88XZone:updateZoneSettings(zoneDetails, zoneId)
 	self:updateProperty('id', tostring(zoneId))
 	self:updateResponseHandlers()
end

function RussoundMCA88XZone:setSourceButtonName(sourceNumber, name)
	self:updateProperty('ui.source' .. tostring(sourceNumber) .. '_Button.caption', 'S' .. tostring(sourceNumber) .. ' ' .. name)
end

function RussoundMCA88XZone:activateSource(sourceNumber)
	if self.russoundController.sock then
		local command = 'EVENT ' .. self.properties.id .. '!KeyRelease SelectSource ' .. tostring(sourceNumber) .. self.delimiter
		self.russoundController.sock:send(command)	
	end
end

function RussoundMCA88XZone:setCurrentSourceLabel(sourceNumber)
	self.currentSource = tostring(sourceNumber)
	self:updateResponseHandlers()
	self:updateProperty('ui.sourceValue_Label.caption',  self.properties['ui.source' .. tostring(sourceNumber) .. '_Button.caption'])
end

function RussoundMCA88XZone:onDataReceived(command, param)
	if command and param then         
        if type(self.responsHandlers[command]) == 'function' then 
            self.responsHandlers[command](param)            
        end
    end  
end

function RussoundMCA88XZone:updateFavoriteUI(validFavoriteTable)
	if self:needUpdatePlaylist(validFavoriteTable) then
		local toMerge = {}
		for id, name in pairs(validFavoriteTable or {}) do
			table.insert(toMerge, {
				["id"] = tonumber(id), 
				["name"] = name,		
				["artist"] = '', 		
				["album"] = '',
			})
		end

		self.queue = toMerge

		if #self.queue > 0 then
			self:updateProperty('ui.playlist.value', self.queue, true)
		end 
	end
end

function RussoundMCA88XZone:needUpdatePlaylist(favoritesFromDevice)
	if self.russoundMCA88X:countTableElements(favoritesFromDevice) ~= self.russoundMCA88X:countTableElements(self.properties['ui.playlist.value']) then 
		return true
	end

	for favoriteId, favoriteName in pairs(favoritesFromDevice or {}) do			
		for _,playlistObject in pairs(self.properties['ui.playlist.value'] or {}) do
			if tonumber(playlistObject.id) == tonumber(favoriteId) and playlistObject.name ~= favoriteName then -- we found new entry that should be repleaced witch existing one in playlist
				return true
			end
		end
	end

	return false
end

function RussoundMCA88XZone:setNewName(newName)
	if newName ~= nil then 
		if newName ~= self.name then 
			self:setName(newName)
		end
	end
end

function RussoundMCA88XZone:setMuteLabel(value)
	local muteState
	
	if tostring(value) == 'ON' then
		muteState = true
	elseif tostring(value) == "OFF" then
		muteState = false
	end

	self:updateProperty('ui.mute_Switch.value', muteState)

	return muteState
end

function RussoundMCA88XZone:setMuteState(value)
	local muteState = self:setMuteStateLabel(value)
	
	if muteState then
		self:mute()
	else
		self:unmute()
	end
end

function RussoundMCA88XZone:mute()
	if self.russoundController.sock then
		local command = 'CEVENT ' .. self.properties.id .. '!KeyHold Mute 150' .. self.delimiter
		self.russoundController.sock:send(command)	
	end
end

function RussoundMCA88XZone:unmute()
	if self.russoundController.sock then
		local command = 'EVENT ' .. self.properties.id .. '!KeyRelease Mute' .. self.delimiter
		self.russoundController.sock:send(command)	
	end
end

function RussoundMCA88XZone:play()
	if self.russoundController.sock then
		local command = 'EVENT ' .. self.properties.id .. '!KeyRelease Play' .. self.delimiter
		self.russoundController.sock:send(command)	
	end
end

function RussoundMCA88XZone:pause()
	if self.russoundController.sock then
		local command = 'EVENT ' .. self.properties.id .. '!KeyRelease Pause' .. self.delimiter
		self.russoundController.sock:send(command)	
	end		  
end

function RussoundMCA88XZone:next()
	if self.russoundController.sock then
		local command = 'EVENT ' .. self.properties.id .. '!KeyRelease Next' .. self.delimiter
		self.russoundController.sock:send(command)	
	end
end

function RussoundMCA88XZone:previous()
	if self.russoundController.sock then
		local command = 'EVENT ' .. self.properties.id .. '!KeyRelease Previous' .. self.delimiter
		self.russoundController.sock:send(command)	
	end
end

function RussoundMCA88XZone:setVolume(value)
	if self.russoundController.sock then
		local volumeValue =  math.floor(tonumber(value / 2))
		local command = 'EVENT ' .. self.properties.id .. '!KeyPress Volume ' .. volumeValue .. self.delimiter
		self.russoundController.sock:send(command)
	end
end

function RussoundMCA88XZone:setPower()
	if self.russoundController.sock then
		local command = 'EVENT ' .. self.properties.id .. '!KeyRelease Power' .. self.delimiter
		self.russoundController.sock:send(command)
	end
end

function RussoundMCA88XZone:setPowerSwitch(state)
	local switchState
	
	if tostring(state) == 'ON' then
		switchState = true
	elseif tostring(state) == "OFF" then
		switchState = false
	end

	self:updateProperty('ui.power_Switch.value', switchState)
end

function RussoundMCA88XZone:setArtistLabel(name)
	if name == nil or name == '' then  
		name = '' 
	end
	self:updateProperty("ui.creator_Label.caption", name)
end

function RussoundMCA88XZone:setSongNameLabel(name)
	if name == nil or name == '' then  
		name = '' 
	end
	self:updateProperty("ui.title_Label.caption",  name)
end

function RussoundMCA88XZone:setAlbumLabel(name)
	if name == nil or name == '' then  
		name = '' 
	end
	self:updateProperty("ui.title_Label.caption",  name)
end

function RussoundMCA88XZone:setPlayListNameLabel(name)
	if name == nil or name == '' then 
		name = '' 
	end
	self:updateProperty("ui.playListNameLabel.caption", name)
end

function RussoundMCA88XZone:setChannelNameLabel(name)
	if name == nil or name == '' then  
		name = '' 
	end
	self:updateProperty("ui.channelNameLabel.caption", name)
end

function RussoundMCA88XZone:setVolumeLabel(volumeLevel)
	local volumeLevel = tonumber(volumeLevel) * 2
	self:updateProperty('ui.volume_Label.caption', tostring(volumeLevel))
	self:updateProperty('ui.volume_Slider.value', tonumber(volumeLevel))	
	self:updateProperty('volume', tonumber(volumeLevel))
end

function RussoundMCA88XZone:setMuteStateLabel(muteState)
	self:updateProperty('ui.mute_Switch.value', muteState)
end

function RussoundMCA88XZone:setAlbumImage(uri)
	if not uri or uri == '' then
		self:updateProperty("ui.albumArt_Image.source", '/plugins/com.fibaro.russoundMCA88XZone/img/defaultIcon.png')
	else
		self:updateProperty("ui.albumArt_Image.source", uri)
	end
end

function RussoundMCA88XZone:setPartyMode(status)
	local partyStatus 
	if status == 'ON' or status == 'Party' then
		partyStatus = true
	elseif status == 'OFF' then
		partyStatus = false
	end

	self:updateProperty('partyMode', partyStatus)
end

function RussoundMCA88XZone:setNoMedia()
	self:updateProperty('ui.volume_Label.caption', tostring(0))
	self:updateProperty('ui.volume_Slider.value', 0)
	self:updateProperty('volume', 0)
	self:updateProperty("ui.albumArt_Image.source", '/plugins/com.fibaro.russoundXSourceZone/img/defaultIcon.png')
	self:setSongNameLabel('No media')
	self:setAlbumLabel(nil)
	self:setPlayListNameLabel(nil)
	self:setChannelNameLabel(nil)
end

function RussoundMCA88XZone:queueSortOrder(from, to)
	if from < to then
		to = to - 1 -- Hack for BUI playlist 
	end

	local valTo = self.queue[to]

	self.queue[to] = self.queue[from]
	self.queue[from] = valTo

	self:updateProperty('ui.playlist.value', self.queue, true)
end

function RussoundMCA88XZone:removeFromPlaylist(index)
	pcall(function()	 	
		self.russoundController.sock:send('EVENT C[1].Z[1]!deleteSystemFavorite ' .. self.queue[index].id .. self.delimiter)
		
		table.remove(self.queue, index)
		
		if #self.queue > 0 then
			self:updateProperty('ui.playlist.value', self.queue, true)
		else
			self:updateProperty('ui.playlist.value', json.array(), true)
		end	
	end)	
end

function RussoundMCA88XZone:playFromQueue(id)
	pcall(function()
		self:updateProperty('ui.playlist.currentId', 100)

		self.russoundController.sock:send('EVENT C[1].Z[1]!restoreSystemFavorite ' .. self.queue[id].id .. self.delimiter)	
	end)
end

function RussoundMCA88XZone:playFromFavorite(id)
	pcall(function()
		self.russoundController.sock:send('EVENT C[1].Z[1]!restoreSystemFavorite ' .. id .. self.delimiter)
	end)	
end
