require('common.device')
require('common.TCPListener')


class 'RussoundX5Zone' (Device)
   

function RussoundX5Zone:__init(russoundX5, dev)
	Device.__init(self, dev)
 	
	self.russoundX5 = russoundX5
	self.delimiter = '\r\n'

	self.uiBinding = {
		['mute_Switch']    = function(eventArgs) self:setMuteState(eventArgs.values[1]) self.russoundX5:updateSystemLabels() end,
		['play_Button']    = function() self:play() 	self.russoundX5:updateSystemLabels() end,
		['pause_Button']   = function() self:pause() self.russoundX5:updateSystemLabels() end,
		['next_Button']    = function() self:next() self.russoundX5:updateSystemLabels() end,
		['prev_Button']    = function() self:previous() self.russoundX5:updateSystemLabels() end,
		['volume_Slider']  = function(eventArgs) self:setVolume(eventArgs.values[1]) self.russoundX5:updateSystemLabels() end,
		['power_Switch']   = function() self:setPower() self.russoundX5:updateSystemLabels() end,
		['source1_Button'] = function() self:activateSource(1) self.russoundX5:updateSystemLabels() end,
		['source2_Button'] = function() self:activateSource(2) self.russoundX5:updateSystemLabels() end,
		['source3_Button'] = function() self:activateSource(3) self.russoundX5:updateSystemLabels() end,
		['source4_Button'] = function() self:activateSource(4) self.russoundX5:updateSystemLabels() end,
		['source5_Button'] = function() self:activateSource(5) self.russoundX5:updateSystemLabels() end,
		['source6_Button'] = function() self:activateSource(6) self.russoundX5:updateSystemLabels() end,
		['source7_Button'] = function() self:activateSource(7) self.russoundX5:updateSystemLabels() end,
		['source8_Button'] = function() self:activateSource(8) self.russoundX5:updateSystemLabels() end,
	}			

	self.responsHandlers = {
		['C[1].Z[1].name='] 	     = function(name) self:setNewName(name) self.russoundX5:updateSystemLabels() end, 
		['C[1].Z[1].volume='] 	     = function(volumeValue) self:setVolumeLabel(volumeValue) self.russoundX5:updateSystemLabels() end, 
		['C[1].Z[1].status=']	     = function(status) self:setPowerSwitch(status) self.russoundX5:updateSystemLabels() end,
		['C[1].Z[1].mute=']          = function(muteState) self:setMuteLabel(muteState) self.russoundX5:updateSystemLabels() end,
		['C[1].Z[1].partyMode=']     = function(status) self:setPartyMode(status) self.russoundX5:updateSystemLabels() end,
		['C[1].Z[1].currentSource='] = function(sourceName) self:setCurrentSourceLabel(sourceName) self.russoundX5:updateSystemLabels() end,
		['S[1].coverArtURL=']	     = function(url) self:setAlbumImage(url) self.russoundX5:updateSystemLabels() end,
		['S[1].artistName=']	     = function(artist) self:setArtistLabel(artist) self.russoundX5:updateSystemLabels() end,
		['S[1].albumName=']		     = function(album) self:setAlbumLabel(album) self.russoundX5:updateSystemLabels() end,
		['S[1].songName=']		     = function(songName) self:setSongNameLabel(songName) self.russoundX5:updateSystemLabels() end,
		['S[1].mode=']               = function(modeString) self:setPartyMode(modeString) self.russoundX5:updateSystemLabels() end,
		['S[1].name=']               = function(name) self:setSourceButtonName(1, name) self.russoundX5:updateSystemLabels() end,
		['S[2].name=']               = function(name) self:setSourceButtonName(2, name) self.russoundX5:updateSystemLabels() end,
		['S[3].name=']               = function(name) self:setSourceButtonName(3, name) self.russoundX5:updateSystemLabels() end,
		['S[4].name=']               = function(name) self:setSourceButtonName(4, name) self.russoundX5:updateSystemLabels() end,
		['S[5].name=']               = function(name) self:setSourceButtonName(5, name) self.russoundX5:updateSystemLabels() end,
		['S[6].name=']               = function(name) self:setSourceButtonName(6, name) self.russoundX5:updateSystemLabels() end,
		['S[7].name=']               = function(name) self:setSourceButtonName(7, name) self.russoundX5:updateSystemLabels() end,
		['S[8].name=']               = function(name) self:setSourceButtonName(8, name) self.russoundX5:updateSystemLabels() end,
		['C[1].type=']				 = function(type) self:onDataReceivedFromCheckDeviceType(type) end,
	}

	self:setNoMedia()
end 

function RussoundX5Zone:updateZoneSettings(zoneDetails, zoneIp, activeSocket)
 	self:updateProperty('ip', tostring(zoneIp))

    self.sock = activeSocket
    self.sock:registerEvent('dataReceived', function(sock, data) self:onDataReceived(sock, data) end)
    self.sock:registerEvent('disconnected', function()  self:onClosed(zoneIp) end)
    self.sock:registerEvent('connected', function() self:onConnected(zoneIp) end)
    self.sock:registerEvent('error', function(error) print("error", error, zoneIp) end)

    self:checkDeviceType()
end

function RussoundX5Zone:checkDeviceType()
	if self.sock then
		self.sock:send('GET C[1].type'.. self.delimiter)
	end
end

function RussoundX5Zone:onDataReceivedFromCheckDeviceType(devType)
	if devType == 'Xstream-X5' then
		self:setWatchZone('ON')
	    self:setWatchAllSources('ON')
	else
        self.russoundX5:removeZoneDevice(self)
	end
end

function RussoundX5Zone:setWatchZone(stateParam)
	if self.sock then
		local state = stateParam or 'ON'
		self.sock:send('WATCH C[1].Z[1] ' .. state .. self.delimiter)	
	end	
end


function RussoundX5Zone:setWatchAllSources(stateParam)
	if self.sock then
		local state = stateParam or 'ON'
		self.sock:send('WATCH S[1] ' .. state .. self.delimiter)	
		self.sock:send('WATCH S[2] ' .. state .. self.delimiter)
		self.sock:send('WATCH S[3] ' .. state .. self.delimiter)
		self.sock:send('WATCH S[4] ' .. state .. self.delimiter)
		self.sock:send('WATCH S[5] ' .. state .. self.delimiter)
		self.sock:send('WATCH S[6] ' .. state .. self.delimiter)
		self.sock:send('WATCH S[7] ' .. state .. self.delimiter)
		self.sock:send('WATCH S[8] ' .. state .. self.delimiter)
	end	
end

function RussoundX5Zone:setSourceButtonName(sourceNumber, name)
	self:updateProperty('ui.source' .. tostring(sourceNumber) .. '_Button.caption', 'S' .. tostring(sourceNumber) .. ' ' .. name)
end

function RussoundX5Zone:activateSource(sourceNumber)
	if self.sock then
		local command = 'EVENT C[1].Z[1]!KeyRelease SelectSource ' .. tostring(sourceNumber) .. self.delimiter
		self.sock:send(command)	
	end
end

function RussoundX5Zone:setCurrentSourceLabel(sourceName)
	self:updateProperty('ui.sourceValue_Label.caption',  self.properties['ui.source' .. tostring(sourceName) .. '_Button.caption'])
end

function RussoundX5Zone:onConnected(ip)
	print('RussoundX5Zone:onConnected(ip)', ip) 
end

function RussoundX5Zone:onDataReceived(sock,  data)
 	local command = string.match(data, 'N (.-)"') or string.match(data, 'S (.-)"')
	local param = string.match(data, '"(.*)"')

	if command and param then         
        if type(self.responsHandlers[command]) == 'function' then 
        	print("handled message: ", command, param)
            self.responsHandlers[command](param)    
        else
            print("unhandled message: ", command, param)
        end
    end  
end

function RussoundX5Zone:onClosed(ip)
	print('RussoundX5Zone:onClosed(ip)', ip)
end

function RussoundX5Zone:setNewName(newName)
	if newName ~= nil then 
		if newName ~= self.name then 
			self:setName(newName)
		end
	end
end

function RussoundX5Zone:setMuteLabel(value)
	local muteState
	
	if tostring(value) == 'ON' then
		muteState = true
	elseif tostring(value) == "OFF" then
		muteState = false
	end

	self:updateProperty('ui.mute_Switch.value', muteState)

	return muteState
end

function RussoundX5Zone:setMuteState(value)
	local muteState = self:setMuteStateLabel(value)
	
	if muteState then
		self:mute()
	else
		self:unmute()
	end
end

function RussoundX5Zone:mute()
	if self.sock then
		local command = 'CEVENT C[1].Z[1]!KeyHold Mute 150' .. self.delimiter
		self.sock:send(command)	
	end
end

function RussoundX5Zone:unmute()
	if self.sock then
		local command = 'EVENT C[1].Z[1]!KeyRelease Mute' .. self.delimiter
		self.sock:send(command)	
	end
end

function RussoundX5Zone:play()
	if self.sock then
		local command = 'EVENT C[1].Z[1]!KeyRelease Play' .. self.delimiter
		self.sock:send(command)	
	end
end

function RussoundX5Zone:pause()
	if self.sock then
		local command = 'EVENT C[1].Z[1]!KeyRelease Pause' .. self.delimiter
		self.sock:send(command)	
	end		  
end

function RussoundX5Zone:next()
	if self.sock then
		local command = 'EVENT C[1].Z[1]!KeyRelease Next' .. self.delimiter
		self.sock:send(command)	
	end
end

function RussoundX5Zone:previous()
	if self.sock then
		local command = 'EVENT C[1].Z[1]!KeyRelease Previous' .. self.delimiter
		self.sock:send(command)	
	end
end

function RussoundX5Zone:setVolume(value)
	if self.sock then
		local volumeValue =  math.floor(tonumber(value / 2))
		local command = 'EVENT C[1].Z[1]!KeyPress Volume ' .. volumeValue .. self.delimiter
		self.sock:send(command)
	end
end

function RussoundX5Zone:setPower()
	if self.sock then
		local command = 'EVENT C[1].Z[1]!KeyRelease Power' .. self.delimiter
		self.sock:send(command)
	end
end

function RussoundX5Zone:setPowerSwitch(state)
	local switchState
	
	if tostring(state) == 'ON' then
		switchState = true
	elseif tostring(state) == "OFF" then
		switchState = false
	end

	self:updateProperty('ui.power_Switch.value', switchState)
end

function RussoundX5Zone:setArtistLabel(artist)
	self:updateProperty("ui.creator_Label.caption", artist)
end

function RussoundX5Zone:setSongNameLabel(title)
	self:updateProperty("ui.title_Label.caption", title)
end

function RussoundX5Zone:setAlbumLabel(album)
	self:updateProperty("ui.title_Label.caption", album)
end

function RussoundX5Zone:setVolumeLabel(volumeLevel)
	local volumeLevel = tonumber(volumeLevel) * 2
	self:updateProperty('ui.volume_Label.caption', tostring(volumeLevel))
	self:updateProperty('ui.volume_Slider.value', tonumber(volumeLevel))	
	self:updateProperty('volume', tonumber(volumeLevel))
end

function RussoundX5Zone:setMuteStateLabel(muteState)
	self:updateProperty('ui.mute_Switch.value', muteState)
end

function RussoundX5Zone:setAlbumImage(uri)
	if not uri or uri == '' then
		self:updateProperty("ui.albumArt_Image.source", '/plugins/com.fibaro.russoundX5Zone/img/defaultIcon.png')
	else
		self:updateProperty("ui.albumArt_Image.source", uri)
	end
end

function RussoundX5Zone:setPartyMode(status)
	local partyStatus 
	if status == 'ON' or status == 'Party' then
		partyStatus = true
	elseif status == 'OFF' then
		partyStatus = false
	end

	self:updateProperty('partyMode', partyStatus)
end

function RussoundX5Zone:setNoMedia()
	self:updateProperty('ui.volume_Label.caption', tostring(0))
	self:updateProperty('ui.volume_Slider.value', 0)
	self:updateProperty('volume', 0)
	self:updateProperty("ui.albumArt_Image.source", '/plugins/com.fibaro.russoundX5Zone/img/defaultIcon.png')
	self:updateProperty("ui.creator_Label.caption", "no media")
	self:updateProperty("ui.title_Label.caption", "")
end
