require('common.device')
require('RussoundXSourceZone')


class 'RussoundController' (Device)
   

function RussoundController:__init(russoundXSource, dev)
	Device.__init(self, dev)
 	
	self.russoundXSource = russoundXSource
	self.delimiter = '\r\n'
	self.zones = {}	
	self.onlineZones = {}	

	self.requiredZoneNumber = 200

	self.numberOfSystemFavorite = 32
	self.numberOfValidSystemFavorite = 0
	self.systemFavoriteTable = {}
	self.tableOfValidSystemFavorites = {}
	self.favoriteRefreshTime = 5000

	self:setVisible(false)

	self.zonesNumberPerControllerType = { -- uncomment desired key-value in this table to simultaneously support one of the commented types 
		-- ['MCA-C6'] 	   = 8, 
		-- ['Xstream-X5'] = 1,
		['XSource'] = 1,
		-- ['MCA-88X']	   = 8,
	}

	self.sourcesNumberPerControllerType = { 
		['MCA-C6'] 	   = 8,  
		['Xstream-X5'] = 8,
		['XSource'] = 1,
		['MCA-88X']	   = 8,
	}
end 

function RussoundController:updateSettings(controllerIp, activeSocket)
 	self:updateProperty('ip', tostring(controllerIp))
    self.sock = activeSocket
    self.sock:registerEvent('dataReceived', function(sock, data) self:onDataReceived(sock, data) end)
    self.sock:registerEvent('disconnected', function()  self:onClosed(controllerIp) end)
    self.sock:registerEvent('connected', function() self:onConnected(controllerIp) end)
    self.sock:registerEvent('error', function(error) print("error", error, controllerIp) end)
    self:checkDeviceType()
end

function RussoundController:checkDeviceType()
	if self.sock then
		self.sock:send('GET C[1].type'.. self.delimiter)
	end
end

function RussoundController:onDataReceivedFromCheckDeviceType(devType)
	if self.zonesNumberPerControllerType[devType] then
		self:updateProperty('model', devType)
		self:setWatchZone('ON')
		self:getSystemFavorite()
	else
        self.russoundXSource:removeControllerDevice(self)
	end
end

function RussoundController:setWatchZone(stateParam)
	if self.sock then
		local state = stateParam or 'ON'
		for i = 1, self.zonesNumberPerControllerType[self.properties.model] do
			self.sock:send('WATCH C[1].Z[' .. i .. '] ' .. state .. self.delimiter)	
		end
	end	
end

function RussoundController:getSystemFavorite()
	for i = 1, self.numberOfSystemFavorite do
		self.sock:send('GET system.favorite[' .. i .. '].valid' .. self.delimiter)
	end	
end

function RussoundController:onConnected(ip)
	print('RussoundController:onConnected(ip)', ip) 
end

function RussoundController:onDataReceived(sock,  data)
	-- init state
 	if self.russoundXSource:countTableElements(self.zones) < (self.zonesNumberPerControllerType[self.properties.model] or 1) then -- "or 1" in case devType isn't in zonesNumberPerControllerType table
 		local zoneId = string.match(data, '(C%[%d%].Z%[%d%])')
 		local zoneNumber, zoneName = string.match(data, 'N C%[1%].Z%[(%d)%].name="(.*)"')
		local devType = string.match(data, 'S C%[1%].type="(.*)"')

		if zoneNumber and zoneName and zoneId then         
	        self:updateOnlineZonesTable(zoneNumber, zoneName, zoneId)
	    elseif devType then
		   	self:onDataReceivedFromCheckDeviceType(devType)
	   	end  
	-- running state
	else
		local command = string.match(data, 'N (.-)"') or string.match(data, 'S (.-)"')
		local param = string.match(data, '"(.*)"')
		local favoriteNumberValid, valid = string.match(data, 'S System.favorite%[(%d+)%].valid="(.*)"')
		local favoriteNumberName, favoriteName = string.match(data, 'S System.favorite%[(%d+)%].name="(.*)"')

		
		-- FAVORITES
		if favoriteNumberValid then
			self:generateFavoritTable(favoriteNumberValid, valid)
		end

		if favoriteNumberName and favoriteName then
			self:generateFovoriteValidTable(favoriteNumberName, favoriteName)
		end
		
		-- COMMANDS AND DATA
		if command and param then 
			for _,zone in pairs(self.zones) do
				zone:onDataReceived(command, param)			
			end
		end
	end
end

function RussoundController:generateFavoritTable(favoriteNumberValid, valid)
	self.systemFavoriteTable[favoriteNumberValid] = valid

	if self.russoundXSource:countTableElements(self.systemFavoriteTable) >= self.numberOfSystemFavorite then
		for id, isValid in pairs(self.systemFavoriteTable or {}) do
			if id and isValid == 'TRUE' then
				self.numberOfValidSystemFavorite = self.numberOfValidSystemFavorite + 1
				self:getFavoriteName(id)
			end
		end
	end
end

function RussoundController:generateFovoriteValidTable(favoriteNumber, favoriteName)
	self.tableOfValidSystemFavorites[favoriteNumber] = favoriteName

	if self.russoundXSource:countTableElements(self.tableOfValidSystemFavorites) >= self.numberOfValidSystemFavorite then
		self:syncZonesWitchFavoritValidTable(self.tableOfValidSystemFavorites)
	end
end

function RussoundController:syncZonesWitchFavoritValidTable(table)
	for _,zone in pairs(self.zones or {}) do
		zone:updateFavoriteUI(table)
	end

	if self.getSystemFavoriteTimeoutId then
		clearTimeout(self.getSystemFavoriteTimeoutId)
	end
	
	self.getSystemFavoriteTimeoutId = setTimeout(function()
		self.systemFavoriteTable = {}
		self.tableOfValidSystemFavorites = {}
		self.numberOfValidSystemFavorite = 0
		self:getSystemFavorite()
	end, self.favoriteRefreshTime) 
end

function RussoundController:getFavoriteName(favoriteNumber)
	self.sock:send('GET system.favorite[' .. favoriteNumber .. '].name' .. self.delimiter)
end

function RussoundController:onClosed(ip)
	print('RussoundController:onClosed(ip)', ip)
	plugin.restart() 
end

function RussoundController:updateOnlineZonesTable(zoneNumber, zoneName, zoneId)
	self.onlineZones[zoneId] = {['name'] = zoneName, ['number'] = zoneNumber}

	if self.russoundXSource:countTableElements(self.onlineZones) == self.zonesNumberPerControllerType[self.properties.model] then
		self.zones = self:getZonesFromExistingHC2Model() -- In case if some one delete zone by cross button. Then we do not have notification (won't update internal model) but the model should be always updates if sync button is pressed.
		self:synchronizeZones(self.onlineZones, self.zones)

		self:setWatchZone('ON')	-- we do it again to previously created self.zones to update their state
		self:setWatchAllSources()	
	end
end

function RussoundController:synchronizeZones(onlineZones, hcZones)
	local onlineZonesVolume = self.russoundXSource:countTableElements(onlineZones)
	local hcZonesVolume = self.russoundXSource:countTableElements(hcZones)
	local createdZones = {}

	if onlineZonesVolume > 0 then
		for onlineZoneId,onlineZoneDetails in pairs(onlineZones) do  
			local hcZone = self:getZoneByIdFromLocalZones(hcZones, onlineZoneId)  
        	if not hcZone then -- if onlne zone from LAN is not in local hcZones we will create it
    			local id, zone = self:createZoneDevice(onlineZoneDetails, onlineZoneId)
    			createdZones[id] = zone
    		end
    	end
    end
    	
    if hcZonesVolume > 0 then 
    	for _,hcZone in pairs(hcZones) do 
			local cloudZoneDetails = self:getZoneByIdFromOnlineZones(onlineZones, hcZone.properties.id)       
        	if not cloudZoneDetails then -- if zone from HC is not in online zones we will remove it
    			self:removeZoneDevice(hcZone) 
    		end
    	end
    end
 
    if onlineZonesVolume > 0 then
    	for onlineZoneId,onlineZoneDetails in pairs(onlineZones) do  
			local hcZone = self:getZoneByIdFromLocalZones(hcZones, onlineZoneId)         
        	if hcZone then -- if zone from local LAN is in local zones we will update same local id zone with datra form online zone
    			hcZone:updateZoneSettings(onlineZoneDetails, onlineZoneId)
    		end
    	end
    end

    self.zones = self:mergeTables(hcZones, createdZones) -- to prevent double zone update in createZone and in loop above

	self.russoundXSource.zones[self.id] = self.zones
	self.russoundXSource:updateSystemLabels()
end 

function RussoundController:setWatchAllSources(stateParam)
	if self.sock then
		local state = stateParam or 'ON'
		for i = 1, self.sourcesNumberPerControllerType[self.properties.model] do
			self.sock:send('WATCH S[' .. i .. '] ' .. state .. self.delimiter)				
		end
	end	
end

function RussoundController:getZonesFromExistingHC2Model()
	local zones = {}

	local childDevices = plugin.getChildDevices(self.id)

    for _,dev in pairs(childDevices) do
        if (dev.type == 'com.fibaro.russoundXSourceZone') then
            zones[dev.properties.id] = RussoundXSourceZone(self, dev)
        end
    end

    return zones
end

function RussoundController:mergeTables(table1, table2)
	for k,v in pairs(table2) do
		table1[k] = v
	end

	return table1
end

function RussoundController:getZoneByIdFromLocalZones(hcZones, id)
	if type(hcZones) == 'table' then
		return hcZones[id] 
	end
	
	return nil
end

function RussoundController:getZoneByIdFromOnlineZones(onlineZones, id)
	if type(onlineZones) == 'table' then
		return onlineZones[id]		
	end
	
	return nil
end

function RussoundController:createZoneDevice(onlineZoneDetails, id)
	if onlineZoneDetails and id then
		local zoneDevice = plugin.createChildDevice(self.id, 'com.fibaro.russoundXSourceZone', onlineZoneDetails.name)
	    local zoneDeviceObject = RussoundXSourceZone(self, zoneDevice)
	    zoneDeviceObject:updateZoneSettings(onlineZoneDetails, id)
	    return id, zoneDeviceObject   
    end                     
end

function RussoundController:removeZoneDevice(zone)
	self.zones[zone.properties.id] = nil
    plugin.deleteDevice(zone.id)
end




