require('common.device')
require('RussoundXZone4Zone')


class 'RussoundController' (Device)
   

function RussoundController:__init(russoundXZone4, dev)
	Device.__init(self, dev)
 	
	self.russoundXZone4 = russoundXZone4
	self.delimiter = '\r\n'
	self.zones = {}	
	self.onlineZones = {}	

	self:setVisible(false)  

	self.zonesNumberPerControllerType = { -- uncomment desired key-value in this table to simultaneously support one of the commented types 		
		['XZone4'] = 4, 
	}
end 

function RussoundController:updateSettings(controllerIp, activeSocket)
 	self:updateProperty('ip', tostring(controllerIp))
    self.sock = activeSocket
    self.sock:registerEvent('dataReceived', function(sock, data) self:onDataReceived(sock, data) end)
    self.sock:registerEvent('disconnected', function()  self:onClosed(controllerIp) end)
    self.sock:registerEvent('connected', function() self:onConnected(controllerIp) end)
    self.sock:registerEvent('error', function(error) print("error", error, controllerIp) end)
    self:checkDeviceType()
end

function RussoundController:checkDeviceType()
	if self.sock then
		self.sock:send('GET C[1].type'.. self.delimiter)
	end 
end

function RussoundController:onDataReceivedFromCheckDeviceType(devType)
	if self.zonesNumberPerControllerType[devType] then
		self:updateProperty('model', devType)
		self:updateOnlineZonesTable()
	else
        self.russoundXZone4:removeControllerDevice(self)
	end
end
 


function RussoundController:onConnected(ip)
	print('----> CONTROLLER onConnected(ip)', ip) 
end

function RussoundController:onDataReceived(sock,  data)
	local devType = string.match(data, 'S C%[1%].type="(.*)"')

	if devType then
	   	self:onDataReceivedFromCheckDeviceType(devType)
   	end  	
end

function RussoundController:onClosed(ip)
	print('----> CONTROLLER onClosed(ip)', ip)
	plugin.restart() 
end

function RussoundController:updateOnlineZonesTable()
	for i=1,self.zonesNumberPerControllerType[self.properties.model] do
		self.onlineZones['C[1].Z[' .. i .. ']'] = {['name'] = 'Zone ' .. i, ['portNumber'] = '962' .. i}
	end

	self.zones = self:getZonesFromExistingHC2Model() -- In case if some one delete zone by cross button. Then we do not have notification (won't update internal model) but the model should be always updates if sync button is pressed.
	self:synchronizeZones(self.onlineZones, self.zones)
end

function RussoundController:synchronizeZones(onlineZones, hcZones)
	local onlineZonesVolume = self.russoundXZone4:countTableElements(onlineZones)
	local hcZonesVolume = self.russoundXZone4:countTableElements(hcZones)
	local createdZones = {}

	if onlineZonesVolume > 0 then
		for onlineZoneId,onlineZoneDetails in pairs(onlineZones) do  
			local hcZone = self:getZoneByIdFromLocalZones(hcZones, onlineZoneId)  
        	if not hcZone then -- if onlne zone from LAN is not in local hcZones we will create it
    			local id, zone = self:createZoneDevice(onlineZoneDetails, onlineZoneId, self)
    			createdZones[id] = zone
    		end
    	end
    end
    	
    if hcZonesVolume > 0 then 
    	for _,hcZone in pairs(hcZones) do 
			local cloudZoneDetails = self:getZoneByIdFromOnlineZones(onlineZones, hcZone.properties.id)       
        	if not cloudZoneDetails then -- if zone from HC is not in online zones we will remove it
    			self:removeZoneDevice(hcZone) 
    		end
    	end
    end
 
    if onlineZonesVolume > 0 then
    	for onlineZoneId,onlineZoneDetails in pairs(onlineZones) do  
			local hcZone = self:getZoneByIdFromLocalZones(hcZones, onlineZoneId)         
        	if hcZone then -- if zone from local LAN is in local zones we will update same local id zone with datra form online zone
    			hcZone:updateZoneSettings(onlineZoneDetails, onlineZoneId, self)
    		end
    	end
    end

    self.zones = self:mergeTables(hcZones, createdZones) -- to prevent double zone update in createZone and in loop above

	self:updateSystemLabels()
end 

function RussoundController:updateSystemLabels()
	self.russoundXZone4.zones[self.id] = self.zones
	self.russoundXZone4:updateSystemLabels()
end


function RussoundController:getZonesFromExistingHC2Model()
	local zones = {}

	local childDevices = plugin.getChildDevices(self.id)

    for _,dev in pairs(childDevices) do
        if (dev.type == 'com.fibaro.russoundXZone4Zone') then
            zones[dev.properties.id] = RussoundXZone4Zone(self, dev)
        end
    end

    return zones
end

function RussoundController:mergeTables(table1, table2)
	for k,v in pairs(table2) do
		table1[k] = v
	end

	return table1
end

function RussoundController:getZoneByIdFromLocalZones(hcZones, id)
	if type(hcZones) == 'table' then
		return hcZones[id] 
	end
	
	return nil
end

function RussoundController:getZoneByIdFromOnlineZones(onlineZones, id)
	if type(onlineZones) == 'table' then
		return onlineZones[id]		
	end
	
	return nil
end

function RussoundController:createZoneDevice(onlineZoneDetails, id)
	if onlineZoneDetails and id then
		local zoneDevice = plugin.createChildDevice(self.id, 'com.fibaro.russoundXZone4Zone', onlineZoneDetails.name)
	    local zoneDeviceObject = RussoundXZone4Zone(self, zoneDevice)
	    zoneDeviceObject:updateZoneSettings(onlineZoneDetails, id, self)
	    return id, zoneDeviceObject   
    end                     
end

function RussoundController:removeZoneDevice(zone)
	self.zones[zone.properties.id] = nil
    plugin.deleteDevice(zone.id)
end

function RussoundController:generateFavoritTable(favoriteNumberValid, valid)
	self.systemFavoriteTable[favoriteNumberValid] = valid

	if self.russoundXZone4:countTableElements(self.systemFavoriteTable) >= self.numberOfSystemFavorite then
		for id, isValid in pairs(self.systemFavoriteTable or {}) do
			if id and isValid == 'TRUE' then
				self.numberOfValidSystemFavorite = self.numberOfValidSystemFavorite + 1
				self:getFavoriteName(id)
			end
		end
	end
end

function RussoundController:generateFovoriteValidTable(favoriteNumber, favoriteName)
	self.tableOfValidSystemFavorites[favoriteNumber] = favoriteName

	if self.russoundXZone4:countTableElements(self.tableOfValidSystemFavorites) >= self.numberOfValidSystemFavorite then
		self:syncZonesWitchFavoritValidTable(self.tableOfValidSystemFavorites)
	end
end

function RussoundController:syncZonesWitchFavoritValidTable(table)
	for _,zone in pairs(self.zones or {}) do
		zone:updateFavoriteUI(table)
	end

	if self.getSystemFavoriteTimeoutId then
		clearTimeout(self.getSystemFavoriteTimeoutId)
	end
	
	self.getSystemFavoriteTimeoutId = setTimeout(function()
		self.systemFavoriteTable = {}
		self.tableOfValidSystemFavorites = {}
		self.numberOfValidSystemFavorite = 0
		self:getSystemFavorite()
	end, self.favoriteRefreshTime) 
end

function RussoundController:getFavoriteName(favoriteNumber)
	self.sock:send('GET system.favorite[' .. favoriteNumber .. '].name' .. self.delimiter)
end


