require('common.device')
require('common.TCPListener')


class 'RussoundXZone4Zone' (Device)
   
 
function RussoundXZone4Zone:__init(russoundController, dev)
	Device.__init(self, dev)
 	
	self.russoundXZone4 = russoundController.russoundXZone4
	self.russoundController = russoundController
	self.delimiter = '\r\n'
	self.currentSource = '' 
	self.queue = {}
	self.sock = nil  

	self.numberOfSystemFavorite = 32
	self.numberOfValidSystemFavorite = 0
	self.systemFavoriteTable = {}
	self.tableOfValidSystemFavorites = {}
	self.favoriteRefreshTime = 5000

	self.uiBinding = {
		['mute_Switch']    = function(eventArgs) self:setMuteState(eventArgs.values[1]) self.russoundController:updateSystemLabels() end,
		['play_Button']    = function() self:play() 	self.russoundController:updateSystemLabels() end,
		['pause_Button']   = function() self:pause() self.russoundController:updateSystemLabels() end,
		['next_Button']    = function() self:next() self.russoundController:updateSystemLabels() end,
		['prev_Button']    = function() self:previous() self.russoundController:updateSystemLabels() end,
		['volume_Slider']  = function(eventArgs) self:setVolume(eventArgs.values[1]) self.russoundController:updateSystemLabels() end,
		['power_Switch']   = function() self:setPower() self.russoundController:updateSystemLabels() end,
		['source1_Button'] = function() self:activateSource(1) self.russoundController:updateSystemLabels() end,
		['source2_Button'] = function() self:activateSource(2) self.russoundController:updateSystemLabels() end,
		['source3_Button'] = function() self:activateSource(3) self.russoundController:updateSystemLabels() end,
		['source4_Button'] = function() self:activateSource(4) self.russoundController:updateSystemLabels() end,
		['source5_Button'] = function() self:activateSource(5) self.russoundController:updateSystemLabels() end,
		['source6_Button'] = function() self:activateSource(6) self.russoundController:updateSystemLabels() end,
		['source7_Button'] = function() self:activateSource(7) self.russoundController:updateSystemLabels() end,
		['source8_Button'] = function() self:activateSource(8) self.russoundController:updateSystemLabels() end,
	}			
 
	self:updateResponseHandlers()
	self:setNoMedia()
end 
 
function RussoundXZone4Zone:updateResponseHandlers()
	self.responsHandlers = {
		['C[1].Z[1].name='] 	      		  			  = function(name) self:setNewName(name) self.russoundController:updateSystemLabels() end, 
		['C[1].Z[1].volume='] 	      		  			  = function(volumeValue) self:setVolumeLabel(volumeValue) self.russoundController:updateSystemLabels() end, 
		['C[1].Z[1].status=']	      		  			  = function(status) self:setPowerSwitch(status) self.russoundController:updateSystemLabels() end,
		['C[1].Z[1].mute=']          		  			  = function(muteState) self:setMuteLabel(muteState) self.russoundController:updateSystemLabels() end,
		['C[1].Z[1].partyMode=']     		  			  = function(status) self:setPartyMode(status) self.russoundController:updateSystemLabels() end,
		['C[1].Z[1].currentSource='] 		  			  = function(sourceNumber) self:setCurrentSourceLabel(sourceNumber) self.russoundController:updateSystemLabels() end,
		['S[' .. self.currentSource .. '].coverArtURL=']  = function(url) self:setAlbumImage(url) self.russoundController:updateSystemLabels() end,
		['S[' .. self.currentSource .. '].artistName=']	  = function(artist) self:setArtistLabel(artist) self.russoundController:updateSystemLabels() end,
		['S[' .. self.currentSource .. '].albumName=']	  = function(album) self:setAlbumLabel(album) self.russoundController:updateSystemLabels() end,
		['S[' .. self.currentSource .. '].songName=']	  = function(songName) self:setSongNameLabel(songName) self.russoundController:updateSystemLabels() end,
		['S[' .. self.currentSource .. '].mode=']         = function(modeString) self:setPartyMode(modeString) self.russoundController:updateSystemLabels() end,
		['S[1].name=']    								  = function(name) self:setSourceButtonName(1, name) self.russoundController:updateSystemLabels() end,
		['S[2].name=']               			 		  = function(name) self:setSourceButtonName(2, name) self.russoundController:updateSystemLabels() end,
		['S[3].name=']               			 		  = function(name) self:setSourceButtonName(3, name) self.russoundController:updateSystemLabels() end,
		['S[4].name=']               			 		  = function(name) self:setSourceButtonName(4, name) self.russoundController:updateSystemLabels() end,
		['S[5].name=']               			 		  = function(name) self:setSourceButtonName(5, name) self.russoundController:updateSystemLabels() end,
		['S[6].name=']               			 		  = function(name) self:setSourceButtonName(6, name) self.russoundController:updateSystemLabels() end,
		['S[7].name=']               			 		  = function(name) self:setSourceButtonName(7, name) self.russoundController:updateSystemLabels() end,
		['S[8].name=']               			 		  = function(name) self:setSourceButtonName(8, name) self.russoundController:updateSystemLabels() end,
		['S[' .. self.currentSource .. '].playlistName='] = function(name) self:setPlayListNameLabel(name) self.russoundController:updateSystemLabels() end,
		['S[' .. self.currentSource .. '].channelName=']  = function(name) self:setChannelNameLabel(name) self.russoundController:updateSystemLabels() end,
	}
end
 
function RussoundXZone4Zone:onConnected(ip)
	print('----> ZONE onConnected(ip)', ip, self.name) 
	self:setWatchZone()   
 	self:setWatchAllSources() 
 	self:getSystemFavorite()
end


--------------------------------------FAVORITE
function RussoundXZone4Zone:getSystemFavorite()
	for i = 1, self.numberOfSystemFavorite do
		self.sock:send('GET system.favorite[' .. i .. '].valid' .. self.delimiter)
	end	

	if self.getSystemFavoriteTimeoutId then
		clearTimeout(self.getSystemFavoriteTimeoutId)
	end
	
	self.getSystemFavoriteTimeoutId = setTimeout(function()
		self.systemFavoriteTable = {}
		self.tableOfValidSystemFavorites = {}
		self.numberOfValidSystemFavorite = 0
		self:getSystemFavorite()
	end, self.favoriteRefreshTime) 
end

function RussoundXZone4Zone:generateFavoritTable(favoriteNumberValid, valid)
	self.systemFavoriteTable[favoriteNumberValid] = valid

	if self.russoundXZone4:countTableElements(self.systemFavoriteTable) >= self.numberOfSystemFavorite then
		for id, isValid in pairs(self.systemFavoriteTable or {}) do
			if id and isValid == 'TRUE' then
				self.numberOfValidSystemFavorite = self.numberOfValidSystemFavorite + 1
				self:getFavoriteName(id)
			end
		end
	end
end

function RussoundXZone4Zone:generateFovoriteValidTable(favoriteNumber, favoriteName)
	self.tableOfValidSystemFavorites[favoriteNumber] = favoriteName

	if self.russoundXZone4:countTableElements(self.tableOfValidSystemFavorites) >= self.numberOfValidSystemFavorite then
		self:updateFavoriteUI(self.tableOfValidSystemFavorites)
	end
end
  
function RussoundXZone4Zone:getFavoriteName(favoriteNumber)
	self.sock:send('GET system.favorite[' .. favoriteNumber .. '].name' .. self.delimiter)
end


----------------------------------------------

function RussoundXZone4Zone:onDataReceived(sock,  data)
	local command = string.match(data, 'N (.-)"') or string.match(data, 'S (.-)"')
	local param = string.match(data, '"(.*)"')	

	if command and param then         
        if type(self.responsHandlers[command]) == 'function' then 
            self.responsHandlers[command](param)            
        end
    end  	

    -- FAVORITES
	local favoriteNumberValid, valid = string.match(data, 'S System.favorite%[(%d+)%].valid="(.*)"')
	local favoriteNumberName, favoriteName = string.match(data, 'S System.favorite%[(%d+)%].name="(.*)"')
	if favoriteNumberValid then
		self:generateFavoritTable(favoriteNumberValid, valid)
	end

	if favoriteNumberName and favoriteName then
		self:generateFovoriteValidTable(favoriteNumberName, favoriteName)
	end 
end

function RussoundXZone4Zone:onClosed(ip)
	print('----> ZONE onClosed(ip)', ip, self.name)
end
 
function RussoundXZone4Zone:setWatchZone(stateParam)
	if self.sock then
		local state = stateParam or 'ON'
		self.sock:send('WATCH C[1].Z[1] ' .. state .. self.delimiter)	
	end	
end

function RussoundXZone4Zone:setWatchAllSources(stateParam) 
	if self.sock then
		local state = stateParam or 'ON' 
		self.sock:send('WATCH S[1] ' .. state .. self.delimiter)				
	end	
end


function RussoundXZone4Zone:updateZoneSettings(zoneDetails, zoneId, controller)
	self:updateProperty('ip', tostring(controller.properties.ip))
	self:updateProperty('id', tostring(zoneId))

	if not self.sock then 
	    self.sock = TCPListener({delimiter = string.char( 0x0D , 0x0A ), readTimeout = 10000, connectTimeout = 10000})
	    self.sock:registerEvent('dataReceived', function(sock, data) self:onDataReceived(sock, data) end)
	    self.sock:registerEvent('disconnected', function()  self:onClosed(controller.properties.ip) end)
	    self.sock:registerEvent('connected', function() self:onConnected(controller.properties.ip) end)
	    self.sock:registerEvent('----> ZONE error', function(error) print("error", error, controller.properties.ip) end)
	 
	    self.sock:connect(controller.properties.ip, tonumber(zoneDetails.portNumber))     
	end

 	self:updateResponseHandlers()
end

function RussoundXZone4Zone:setSourceButtonName(sourceNumber, name)
	self:updateProperty('ui.source' .. tostring(sourceNumber) .. '_Button.caption', 'S' .. tostring(sourceNumber) .. ' ' .. name)
end

function RussoundXZone4Zone:activateSource(sourceNumber)
	if self.sock then
		local command = 'EVENT ' .. 'C[1].Z[1]' .. '!KeyRelease SelectSource ' .. tostring(sourceNumber) .. self.delimiter
		self.sock:send(command)	
	end
end

function RussoundXZone4Zone:setCurrentSourceLabel(sourceNumber)
	self.currentSource = tostring(sourceNumber)
	self:updateResponseHandlers()
	self:updateProperty('ui.sourceValue_Label.caption',  self.properties['ui.source' .. tostring(sourceNumber) .. '_Button.caption'])
end

function RussoundXZone4Zone:updateFavoriteUI(validFavoriteTable)
	if self:needUpdatePlaylist(validFavoriteTable) or self.russoundXZone4:countTableElements(self.queue) < 1 then
		local toMerge = {}
		for id, name in pairs(validFavoriteTable or {}) do
			table.insert(toMerge, {
				["id"] = tonumber(id), 
				["name"] = name,		
				["artist"] = '', 		
				["album"] = '',
			})
		end 

		self.queue = toMerge

	
		if #self.queue > 0 then
			self:updateProperty('ui.playlist.value', self.queue, true)
		end 
	end
end

function RussoundXZone4Zone:needUpdatePlaylist(favoritesFromDevice)
	if self.russoundXZone4:countTableElements(favoritesFromDevice) ~= self.russoundXZone4:countTableElements(self.properties['ui.playlist.value']) then 
		return true
	end

	for favoriteId, favoriteName in pairs(favoritesFromDevice or {}) do			
		for _,playlistObject in pairs(self.properties['ui.playlist.value'] or {}) do
			if tonumber(playlistObject.id) == tonumber(favoriteId) and playlistObject.name ~= favoriteName then -- we found new entry that should be repleaced witch existing one in playlist
				return true
			end
		end
	end

	return false
end

function RussoundXZone4Zone:setNewName(newName)
	if newName ~= nil then 
		if newName ~= self.name then 
			self:setName(newName)
			self.name = newName
		end
	end
end

function RussoundXZone4Zone:setMuteLabel(value)
	local muteState
	
	if tostring(value) == 'ON' then
		muteState = true
	elseif tostring(value) == "OFF" then
		muteState = false
	end

	self:updateProperty('ui.mute_Switch.value', muteState)

	return muteState
end

function RussoundXZone4Zone:setMuteState(value)
	local muteState = self:setMuteStateLabel(value)
	
	if muteState then
		self:mute()
	else
		self:unmute()
	end
end

function RussoundXZone4Zone:mute()
	if self.sock then
		local command = 'CEVENT ' .. 'C[1].Z[1]' .. '!KeyHold Mute 150' .. self.delimiter
		self.sock:send(command)	
	end
end

function RussoundXZone4Zone:unmute()
	if self.sock then
		local command = 'EVENT ' .. 'C[1].Z[1]' .. '!KeyRelease Mute' .. self.delimiter
		self.sock:send(command)	
	end
end

function RussoundXZone4Zone:play()
	if self.sock then
		local command = 'EVENT ' .. 'C[1].Z[1]' .. '!KeyRelease Play' .. self.delimiter
		self.sock:send(command)	
	end
end

function RussoundXZone4Zone:pause()
	if self.sock then
		local command = 'EVENT ' .. 'C[1].Z[1]' .. '!KeyRelease Pause' .. self.delimiter
		self.sock:send(command)	
	end		  
end

function RussoundXZone4Zone:next()
	if self.sock then
		local command = 'EVENT ' .. 'C[1].Z[1]' .. '!KeyRelease Next' .. self.delimiter
		self.sock:send(command)	
	end
end

function RussoundXZone4Zone:previous()
	if self.sock then
		local command = 'EVENT ' .. 'C[1].Z[1]' .. '!KeyRelease Previous' .. self.delimiter
		self.sock:send(command)	
	end
end

function RussoundXZone4Zone:setVolume(value)
	if self.sock then
		local volumeValue =  math.floor(tonumber(value / 2))
		local command = 'EVENT ' .. 'C[1].Z[1]' .. '!KeyPress Volume ' .. volumeValue .. self.delimiter
		self.sock:send(command)
	end
end

function RussoundXZone4Zone:setPower()
	if self.sock then
		local command = 'EVENT ' .. 'C[1].Z[1]' .. '!KeyRelease Power' .. self.delimiter
		self.sock:send(command)
	end
end

function RussoundXZone4Zone:setPowerSwitch(state)
	local switchState
	
	if tostring(state) == 'ON' then
		switchState = true
	elseif tostring(state) == "OFF" then
		switchState = false
	end

	self:updateProperty('ui.power_Switch.value', switchState)
end

function RussoundXZone4Zone:setArtistLabel(name)
	if name == nil or name == '' then  
		name = '' 
	end
	self:updateProperty("ui.creator_Label.caption", name)
end

function RussoundXZone4Zone:setSongNameLabel(name)
	if name == nil or name == '' then  
		name = '' 
	end
	self:updateProperty("ui.title_Label.caption",  name)
end

function RussoundXZone4Zone:setAlbumLabel(name)
	if name == nil or name == '' then  
		name = '' 
	end
	self:updateProperty("ui.title_Label.caption",  name)
end

function RussoundXZone4Zone:setPlayListNameLabel(name)
	if name == nil or name == '' then 
		name = '' 
	end
	self:updateProperty("ui.playListNameLabel.caption", name)
end

function RussoundXZone4Zone:setChannelNameLabel(name)
	if name == nil or name == '' then  
		name = '' 
	end
	self:updateProperty("ui.channelNameLabel.caption", name)
end

function RussoundXZone4Zone:setVolumeLabel(volumeLevel)
	local volumeLevel = tonumber(volumeLevel) * 2
	self:updateProperty('ui.volume_Label.caption', tostring(volumeLevel))
	self:updateProperty('ui.volume_Slider.value', tonumber(volumeLevel))	
	self:updateProperty('volume', tonumber(volumeLevel))
end

function RussoundXZone4Zone:setMuteStateLabel(muteState)
	self:updateProperty('ui.mute_Switch.value', muteState)
end

function RussoundXZone4Zone:setAlbumImage(uri)
	if not uri or uri == '' then
		self:updateProperty("ui.albumArt_Image.source", '/plugins/com.fibaro.russoundXZone4Zone/img/defaultIcon.png')
	else
		self:updateProperty("ui.albumArt_Image.source", uri)
	end
end

function RussoundXZone4Zone:setPartyMode(status)
	local partyStatus 
	if status == 'ON' or status == 'Party' then
		partyStatus = true
	elseif status == 'OFF' then
		partyStatus = false
	end

	self:updateProperty('partyMode', partyStatus)
end

function RussoundXZone4Zone:setNoMedia()
	self:updateProperty('ui.volume_Label.caption', tostring(0))
	self:updateProperty('ui.volume_Slider.value', 0)
	self:updateProperty('volume', 0)
	self:updateProperty("ui.albumArt_Image.source", '/plugins/com.fibaro.russoundXZone4Zone/img/defaultIcon.png')
	self:setSongNameLabel('No media')
	self:setAlbumLabel(nil)
	self:setPlayListNameLabel(nil)
	self:setChannelNameLabel(nil)
end

function RussoundXZone4Zone:queueSortOrder(from, to)
	if from < to then
		to = to - 1 -- Hack for BUI playlist 
	end

	local valTo = self.queue[to]

	self.queue[to] = self.queue[from]
	self.queue[from] = valTo

	self:updateProperty('ui.playlist.value', self.queue, true)
end

function RussoundXZone4Zone:removeFromPlaylist(index)
	pcall(function()	 	
		self.sock:send('EVENT C[1].Z[1]!deleteSystemFavorite ' .. self.queue[index].id .. self.delimiter)

		table.remove(self.queue, index)
		
		if #self.queue > 0 then
			self:updateProperty('ui.playlist.value', self.queue, true)
		else
			self:updateProperty('ui.playlist.value', json.array(), true)
		end	
	end)	
end

function RussoundXZone4Zone:playFromQueue(id)
	pcall(function()
		self:updateProperty('ui.playlist.currentId', 100)
		self.sock:send('EVENT C[1].Z[1]!restoreSystemFavorite ' .. self.queue[id].id .. self.delimiter)	
	end)
end

function RussoundXZone4Zone:playFromFavorite(id)
	pcall(function()
		self.sock:send('EVENT C[1].Z[1]!restoreSystemFavorite ' .. id .. self.delimiter)
	end)	
end


