require('common.device')
require('common.base64')
require('common/TCPListener')
cosmo = require("cosmo.cosmo")
require('common.upnpDevice')
class 'Samsung' (UPnPDevice)

Samsung.START_PACKET =	string.char( 0x00, 0x13, 0x00, 0x69, 0x70, 0x68, 0x6f,
									 0x6e, 0x65, 0x2e, 0x69, 0x61, 0x70, 0x70,
									 0x2e, 0x73, 0x61, 0x6d, 0x73, 0x75, 0x6e,
									 0x67 )

Samsung.PORT = 55000
Samsung.RENDERING_CONTROL = '/smp_15_'
Samsung.AVTransport = '/smp_22_'


function Samsung:__init(id)
	UPnPDevice.__init(self, id)

	-- create connection
	self:prepareTcpSocket()
	self.shouldSendKey = false
end

function Samsung:prepareTcpSocket()
	self.sock = TCPListener({delimiter = "\n\r", readTimeout = 2000, connectTimeout=2000})
	self.sock:registerEvent('connected', function() self:onConnected() end)
	self.sock:registerEvent('disconnected', function() self:onClosed() end)

	self.sock:connect(self.properties.ip, Samsung.PORT)
end

function Samsung:onConnected()
	self:authenticate()
end

function Samsung:onClosed()
	self.shouldSendKey = false
	setTimeout(function() self.sock:connect(self.properties.ip, Samsung.PORT) end, 3000)
end

--
--
-- actions
--
-- 
function Samsung:authenticate()
	local payload = string.char(0x64, 0x00) .. self:__encodeString(self.properties.ip) .. self:__encodeString('a') .. self:__encodeString('b')
	local packet = self.START_PACKET .. self:__calculateSize(payload) .. payload
	self.sock:send(packet)
	self.shouldSendKey = true
end

function Samsung:sendKey(key)
	if self.shouldSendKey then
		local payload = string.char(0x00, 0x00, 0x00) .. self:__encodeString(key)
		local packet = self.START_PACKET .. self:__calculateSize(payload) .. payload
		self.sock:send(packet)
	end
end

function Samsung:turnOff()
	self.sock:send('KEY_POWEROFF')
end

function Samsung:runMedia(url)
	self:stop()
	setTimeout(function() self:setAVTransportUri(url) end, 300)
	setTimeout(function() 	self:play() end, 600)
	setTimeout(function() 	self:setVolume(30) end, 900)
end

--
-- internal util functions
--
function Samsung:__calculateSize(data)
	local size = string.len(data)
	local b0 = size % 255
	local b1 = (size - b0) / 255 

	return string.reverse(string.char(b1, b0))
end

function Samsung:__encodeString(data)
	local strEncoded = to_base64(data)
	local sizeEncoded = self:__calculateSize(strEncoded)
	return  sizeEncoded .. strEncoded
end