require('common.device')

class 'SatelPartition' (Device)

function SatelPartition:__init(parent, dev)
    Device.__init(self, dev)
    self.parent = parent

    self.onPropertyChanged = {}
    self.onPropertyChanged.exitTimeMoreThan10 = function() self:exitTimeUpdated() end
    self.onPropertyChanged.exitTimeLessThan10 = function() self:exitTimeUpdated() end
end


function SatelPartition:get32BitId()
    local id = 0x80000000

    for i=2, self.properties.partitionId do
        id = id/2
    end
    return id
end

function SatelPartition:exitTimeUpdated()
    if (self.properties.exitTimeMoreThan10 == true) then
        self:updateProperty("exitTime", 2)
    elseif (self.properties.exitTimeLessThan10 == true) then
        self:updateProperty("exitTime", 1)
    else
        self:updateProperty("exitTime", 0)
    end
end

function SatelPartition:updateProperty(property, value)
    if (property == "armed") then
        if (value ~= self.properties[property]) then
            for key, zone in pairs(self.parent.zones) do
                if zone.properties.armed ~= value and zone.properties.disabled == false and zone.properties.satelPartitionID==self.properties.partitionId then
                    zone:updateProperty('armed', value)
                end
            end
        end
    end
    Device.updateProperty(self, property, value)
end

function SatelPartition:arm()
    self.parent:sendData(string.char(0x80) .. self.parent:getPinAsBytes() .. self.parent:to32bits(self.properties.partitionId))
end

function SatelPartition:disarm()
    self.parent:sendData(string.char(0x84) .. self.parent:getPinAsBytes() .. self.parent:to32bits(self.properties.partitionId))
    self:clearAlarm()
end

function SatelPartition:clearAlarm()
    self.parent:sendData(string.char(0x85) .. self.parent:getPinAsBytes() .. self.parent:to32bits(self.properties.partitionId))
end

function SatelPartition:reload()
    self.parent:sendData(string.char(0xEE, 0x00, self.properties.partitionId))
end
