require('luaunit/luaunit')

require('../satelAlarm')

TestSatel = {} 
    function TestSatel:setUp()
        self.satel = SatelAlarm(1)
    end

    function TestSatel:testCrc()
        local crc = self.satel:calculateCrc(string.char(0x09))
        assertEquals( crc,  0xd7eb)

        crc = self.satel:calculateCrc(string.char(0x1c))
        assertEquals( crc,  0xd7fe)

        crc = self.satel:calculateCrc(string.char(0xe0, 0x12, 0x34, 0xff, 0xff))
        assertEquals( crc,  0x8a9b)
    end

    function TestSatel:test_to128bits()
        local bitmask

        bitmask = self.satel:to128bits(1)
        assertEquals(bitmask, string.char(0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00))

        bitmask = self.satel:to128bits(7)
        assertEquals(bitmask, string.char(0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00))

        bitmask = self.satel:to128bits(8)
        assertEquals(bitmask, string.char(0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00))

        bitmask = self.satel:to128bits(9)
        assertEquals(bitmask, string.char(0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00))

        bitmask = self.satel:to128bits(56)
        assertEquals(bitmask, string.char(0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00))

        bitmask = self.satel:to128bits(53)
        assertEquals(bitmask, string.char(0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00))

        bitmask = self.satel:to128bits(128)
        assertEquals(bitmask, string.char(0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80))
    end

    function TestSatel:test_to32bits()
        local bitmask

        bitmask = self.satel:to32bits(1)
        assertEquals(bitmask, string.char(0x01, 0x00, 0x00, 0x00))

        bitmask = self.satel:to32bits(8)
        assertEquals(bitmask, string.char(0x80, 0x00, 0x00, 0x00))

        bitmask = self.satel:to32bits(9)
        assertEquals(bitmask, string.char(0x00, 0x01, 0x00, 0x00))

        bitmask = self.satel:to32bits(19)
        assertEquals(bitmask, string.char(0x00, 0x00, 0x04, 0x00))

        bitmask = self.satel:to32bits(32)
        assertEquals(bitmask, string.char(0x00, 0x00, 0x00, 0x80))

    end

    function TestSatel:test_idListTo32Bits()
        local partitionList
        local result

        partitionList = "1"
        result = self.satel:idListTo32Bits(partitionList)      
        assertEquals(result, string.char(0x01, 0x00, 0x00, 0x00))

        partitionList = "1, 32"
        result = self.satel:idListTo32Bits(partitionList)      
        assertEquals(result, string.char(0x01, 0x00, 0x00, 0x80))

        partitionList = "8"
        result = self.satel:idListTo32Bits(partitionList)      
        assertEquals(result, string.char(0x80, 0x00, 0x00, 0x00))

        partitionList = "1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16"
        result = self.satel:idListTo32Bits(partitionList)      
        assertEquals(result, string.char(0xff, 0xff, 0x00, 0x00))

        partitionList = "32,15,9,27,3,13"
        result = self.satel:idListTo32Bits(partitionList)      
        assertEquals(result, string.char(0x04, 0x51, 0x00, 0x84))
    end

    function TestSatel:test_idListTo128Bits()
        local partitionList
        local result

        partitionList = "1"
        result = self.satel:idListTo128Bits(partitionList)      
        assertEquals(result, string.char(0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00))

        partitionList = "1, 32"
        result = self.satel:idListTo128Bits(partitionList)      
        assertEquals(result, string.char(0x01, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00))

        partitionList = "8"
        result = self.satel:idListTo128Bits(partitionList)      
        assertEquals(result, string.char(0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00))

        partitionList = "1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16"
        result = self.satel:idListTo128Bits(partitionList)      
        assertEquals(result, string.char(0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00))

        partitionList = "32,15,9,27,3,13"
        result = self.satel:idListTo128Bits(partitionList)      
        assertEquals(result, string.char(0x04, 0x51, 0x00, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00))

        partitionList = "128"
        result = self.satel:idListTo128Bits(partitionList)      
        assertEquals(result, string.char(0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80))

        partitionList = "121"
        result = self.satel:idListTo128Bits(partitionList)      
        assertEquals(result, string.char(0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01))
    end


LuaUnit:run()