--
-- includes
--
require('sonos')

--
-- create main class
--
sonos = Sonos(plugin.mainDeviceId)


uiBinding = {
	["mute_Switch"] = 	function(eventArgs)
							sonos:updateProperty("ui.mute_Switch.value", eventArgs.values[1])
							sonos:setMute(eventArgs.values[1])
						end,

	["play_Button"] =	function() sonos:play() end,
	["pause_Button"] =	function() sonos:pause() end,
	["stop_Button"] =	function() sonos:stop() end,
	["next_Button"] =	function() sonos:next() end,
	["prev_Button"] =	function() sonos:prev() end,

	["volume_Slider"] = function(eventArgs) sonos:setVolume(eventArgs.values[1]) end,

-- This is for test puposes (until input control is developed) 
	["source_Button"] =	function() sonos:setAVTransportUri(sonos.properties['currentSourceURI']) end,
	--["ungroup_Button"] = function() sonos:getQueue() end, --ungroup() end,	
}

sonos.onPropertyChanged = {
	volume = function(dev, value)
		dev:updateProperty('ui.volume_Label.caption', tostring(value))
		dev:updateProperty('ui.volume_Slider.value', value)
	end,

	mute = function(dev, value)
		dev:updateProperty(dev, 'ui.mute_Switch.value', value)
	end,

	position = function(dev, value)
		dev:updateProperty('ui.position_Label.caption', tostring(value))
	end,

	creator = function(dev, value)
		dev:updateProperty('ui.creator_Label.caption', tostring(value))
	end,

	title = function(dev, value)
		dev:updateProperty('ui.title_Label.caption', tostring(value))
	end,

	state = function(dev, value)
		dev:updateProperty('ui.state_Label.caption', tostring(value))
	end,

	albumArtURI = function(dev, value)
		dev:updateProperty('ui.albumArt_Image.source', tostring(value))
	end
}

function onAction(deviceId, action)
	sonos:callAction(action.actionName, unpack(action.args))
end

function onUIEvent(deviceId, event)
	if event.eventType == 'playlistSort' then
		local values = event.values
		sonos:queueSortOrder(values[1].from, values[1].to)
	elseif event.eventType == 'removeSong' then
		local values = event.values
		sonos:removeFromPlaylist(values[1].id)
	elseif event.eventType == 'urlSourceChange' then
		local values = event.values
		sonos:playFromUri(values[1].uri)
	elseif event.eventType == 'currentIdUpdate' then
		local values = event.values
		sonos:playFromQueue(values[1].id)
	end

	local callback = uiBinding[event.elementName]
	
	if (callback) then
		callback(event)
	end
end

function configure(deviceId, config)
	local restart = false
	if (config.ip) then
		if (sonos:updateProperty('ip', config.ip)) then
			restart = true
		end
	end
	
	if (config.currentSourceURI) then
		
		if (sonos:updateProperty('currentSourceURI', config.currentSourceURI)) then
			sonos:updateProperty('ui.source_Label.caption', config.currentSourceURI)
			restart = true
			
		end
	end

	if restart then
		plugin.restart()
	end
end
