require('common.upnpDevice')
require('sonosUPnP')

--[[
Sources for sonos van be encoded url pointing to e.g. mp3 file, playlist files(e.g. m3u), or radio streams
Radio streams must be prefixed by x-rincon-mp3radio:// e.g.:
	x-rincon-mp3radio://polskieradio.pl/st/program1M.asx
]]
class 'Sonos' (UPnPDevice)

Sonos.PORT = 1400
Sonos.RENDERING_CONTROL = '/MediaRenderer/RenderingControl/Control'
Sonos.AVTransport = '/MediaRenderer/AVTransport/Control'

Sonos.VOLUME_POLLING_TIME = 500
Sonos.MUTE_POLLING_TIME = 1000
Sonos.CURRENT_STATE_POLLING_TIME = 1000
Sonos.TRACK_INFO_POLLING_TIME = 1000

function Sonos:__init(id)
	UPnPDevice.__init(self, id)
	
	self:initializeUPnP({
		ip = self.properties.ip, 
		port = Sonos.PORT,
		RenderingControl = Sonos.RENDERING_CONTROL,
		AVTransport = Sonos.AVTransport,
		timeout = 3000
	})

	self:monitorVolume(Sonos.VOLUME_POLLING_TIME)
	self:monitorMute(Sonos.MUTE_POLLING_TIME)
	self:monitorCurrentState(Sonos.CURRENT_STATE_POLLING_TIME)
	self:monitorTrackInfo(Sonos.TRACK_INFO_POLLING_TIME)
	self.upnp:getPlayerInfo()
	reload = function() self.upnp:getCurTrackInfo() end
	reload2 = function() self.upnp:getQueue() end
	self:interval(1000, reload)
	self:interval(10000, reload2)
end

function Sonos:interval( time, callback )
    callback()

    self.timeoutId = setTimeout(function() self:interval(time, callback) end, time)
end 

function Sonos:initializeUPnP(config)
	self.upnp = SonosUPnP(self, config)
end

function Sonos:chooseInput( uri, urimetadata )
	self:setAVTransportUri(uri, urimetadata)
end

function Sonos:ungroup( )
	self.upnp:ungroup()
end

function Sonos:getQueue()
	self.upnp:getQueue()
end

function Sonos:clearPlaylist()
	self.upnp:clearPlaylist()
end

function Sonos:getCurTrackInfo()
	self.upnp:getCurTrackInfo()
end

function Sonos:removeFromPlaylist(index)
	self.upnp:removeFromPlaylist(index)
end

function Sonos:playFromUri(uri)
	self.upnp:playFromUri(uri)
end

function Sonos:playFromQueue(index)
	self.upnp:playFromQueue(index)
end

function Sonos:queueSortOrder(from, to)
	self.upnp:queueSortOrder(from, to)
end

function Sonos:addToPlaylist(uri)
	self.upnp:addToPlaylist(uri)
end

