require('common.upnp')
local lom = require ("lxp.lom")
local xpath = require ("xpath")
require('net.HTTPClient')
local cosmo = require('cosmo.cosmo')
require("json")

--[[
Sources for sonos van be encoded url pointing to e.g. mp3 file, playlist files(e.g. m3u), or radio streams
Radio streams must be prefixed by x-rincon-mp3radio:// e.g.:
	x-rincon-mp3radio://polskieradio.pl/st/program1M.asx
]]
class 'SonosUPnP' (UPnP)

SonosUPnP.UNGROUP_ACTION = 'urn:schemas-upnp-org:service:AVTransport:1#BecomeCoordinatorOfStandaloneGroup"'
SonosUPnP.UNGROUP_BODY = [[<u:BecomeCoordinatorOfStandaloneGroup xmlns:u="urn:schemas-upnp-org:service:AVTransport:1"><InstanceID>0</InstanceID></u:BecomeCoordinatorOfStandaloneGroup>]]

SonosUPnP.GroupManagement = {}
SonosUPnP.GroupManagement.ADD_MEMBER_ACTION = "urn:schemas-upnp-org:service:GroupManagement:1#AddMember"
SonosUPnP.GroupManagement.ADD_MEMBER_TEMPLATE = "<ns0:AddMember><MemberId>$memberID</MemberId></ns0:AddMember>"

SonosUPnP.GetCurTrackAction = "urn:schemas-upnp-org:service:AVTransport:1#GetPositionInfo"
SonosUPnP.GetCurTrackBody = "<u:GetPositionInfo xmlns:u=\"urn:schemas-upnp-org:service:AVTransport:1\"><InstanceID>0</InstanceID><Channel>Master</Channel></u:GetPositionInfo>"


SonosUPnP.QueueBody = "<u:Browse xmlns:u=\"urn:schemas-upnp-org:service:ContentDirectory:1\"><ObjectID>Q:0</ObjectID><BrowseFlag>BrowseDirectChildren</BrowseFlag><Filter>dc:title,res,dc:creator,upnp:artist,upnp:album,upnp:albumArtURI</Filter><StartingIndex>0</StartingIndex><RequestedCount>10000</RequestedCount><SortCriteria></SortCriteria></u:Browse>"
SonosUPnP.BrowseAction = "urn:schemas-upnp-org:service:ContentDirectory:1#Browse"
SonosUPnP.ContentDirectory = "/MediaServer/ContentDirectory/Control"

SonosUPnP.PLAY_FROM_QUEUE_RESPONSE = '<s:Envelope xmlns:s="http://schemas.xmlsoap.org/soap/envelope/" s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"><s:Body><u:SetAVTransportURIResponse xmlns:u="urn:schemas-upnp-org:service:AVTransport:1"></u:SetAVTransportURIResponse></s:Body></s:Envelope>'

SonosUPnP.SET_TRANSPORT_ACTION = '"urn:schemas-upnp-org:service:AVTransport:1#SetAVTransportURI"'
SonosUPnP.PLAY_FROM_QUEUE_BODY_TEMPLATE = [[
<u:SetAVTransportURI xmlns:u="urn:schemas-upnp-org:service:AVTransport:1">
    <InstanceID>0</InstanceID>
    <CurrentURI>$uri</CurrentURI>
    <CurrentURIMetaData></CurrentURIMetaData>
</u:SetAVTransportURI>
]]

SonosUPnP.SEEK_ACTION = '\"urn:schemas-upnp-org:service:AVTransport:1#Seek\"'
SonosUPnP.SEEK_TRACK_BODY_TEMPLATE = [[
<u:Seek xmlns:u="urn:schemas-upnp-org:service:AVTransport:1">
<InstanceID>0</InstanceID>
<Unit>TRACK_NR</Unit>
<Target>$track</Target>
</u:Seek>
]]

SonosUPnP.ORDER_ACTION = '\"urn:schemas-upnp-org:service:AVTransport:1#ReorderTracksInQueue\"'
SonosUPnP.ORDER_BODY_TEMPLATE = [[
<u:ReorderTracksInQueue xmlns:u="urn:schemas-upnp-org:service:AVTransport:1">
<InstanceID>0</InstanceID>
<StartingIndex>$from</StartingIndex>
<NumberOfTracks>1</NumberOfTracks>
<InsertBefore>$to</InsertBefore>
</u:ReorderTracksInQueue>
]]

SonosUPnP.PLAY_URI_BODY_TEMPLATE = '<u:SetAVTransportURI xmlns:u="urn:schemas-upnp-org:service:AVTransport:1"><InstanceID>0</InstanceID><CurrentURI>$uri</CurrentURI><CurrentURIMetaData> </CurrentURIMetaData></u:SetAVTransportURI>'

SonosUPnP.ADD_TO_QUEUE_ACTION = 'urn:schemas-upnp-org:service:AVTransport:1#AddURIToQueue'
SonosUPnP.ADD_TO_QUEUE_BODY_TEMPLATE = '<u:AddURIToQueue xmlns:u="urn:schemas-upnp-org:service:AVTransport:1"><InstanceID>0</InstanceID><EnqueuedURI>$uri</EnqueuedURI><EnqueuedURIMetaData></EnqueuedURIMetaData><DesiredFirstTrackNumberEnqueued>0</DesiredFirstTrackNumberEnqueued><EnqueueAsNext>1</EnqueueAsNext></u:AddURIToQueue>'

SonosUPnP.REMOVE_FROM_QUEUE_ACTION = 'urn:schemas-upnp-org:service:AVTransport:1#RemoveTrackFromQueue'
SonosUPnP.REMOVE_FROM_QUEUE_BODY_TEMPLATE = '<u:RemoveTrackFromQueue xmlns:u="urn:schemas-upnp-org:service:AVTransport:1"><InstanceID>$instance</InstanceID><ObjectID>$objid</ObjectID><UpdateID>$updateid</UpdateID></u:RemoveTrackFromQueue>'

SonosUPnP.CLEAR_QUEUE_ACTION = '\"urn:schemas-upnp-org:service:AVTransport:1#RemoveAllTracksFromQueue\"'
SonosUPnP.CLEAR_QUEUE_BODY = '<u:RemoveAllTracksFromQueue xmlns:u="urn:schemas-upnp-org:service:AVTransport:1"><InstanceID>0</InstanceID></u:RemoveAllTracksFromQueue>'


function SonosUPnP:__init(device, config
)	self.http = net.HTTPClient()
	self.device = device
	UPnP.__init(self, config)
	self.queue = {}	
end

function SonosUPnP:ungroup()
	self:__sendRequest(self.AVTRANSPORT_URI, self.UNGROUP_ACTION, self.UNGROUP_BODY, function(data) print(data) end, function(data) print ("ERROR");print(data) end)
end

function SonosUPnP:setAVTransportUri(uri, metadata)
	local body = cosmo.fill(UPnP.SET_AV_TRANSPORT_TEMPLATE, { uri = uri, metadata = metadata })
	self:__sendRequest(self.AVTRANSPORT_URI, UPnP.SET_AV_TRANSPORT_ACTION, body, function(data) print(data) end, function(data) print ("ERROR");print(data) end)
end

function SonosUPnP:getQueue()
	self:__sendRequest(self.ContentDirectory, self.BrowseAction, self.QueueBody, function(data) self:parseResponse(data) end, function(data) print ("ERROR");print(data) end)
end

function clearTable(tab)
	for k,v in pairs(tab) do
  		tab[k] = nil
	end
end

function SonosUPnP:parseResponse(data)
	local queueXML

	clearTable(self.queue)
	local xml_tree = lom.parse(data.data)
	local result = xpath.selectNodes(xml_tree, '//Result')

	for k,v in pairs(result[1]) do
		if k == 1 then 
			queueXML = v
		end
	end
	local xml_tree2 = lom.parse(queueXML)
	local result2 = xpath.selectNodes(xml_tree2, '//item')
	counter = 0
	for k,v in pairs(result2) do	
		counter = counter + 1

		if counter <= 400 then --UGLY TEMPORARY HACK
			unit = {}

			for k2,v2 in pairs(v) do
				unit["id"] = counter
				if k2 == 2 then
					unit["icon"] = v2[1]
				end
				if k2 == 3 then
					unit["name"] = v2[1]
				end
				if k2 == 5 then
					unit["artist"] = v2[1]
				end
				if k2 == 6 then
					unit["album"] = v2[1]
				end	
			end		
			table.insert(self.queue, unit)
		end
	end	

	if(#self.queue > 0)
	then
		self.device:updateProperty('ui.playlist.value', self.queue, true)
	else
		self.device:updateProperty('ui.playlist.value', json.array(), true)
		self:clearSongInfo()
	end 

end

function SonosUPnP:playerInfo(data)
	local xml_tree = lom.parse(data)
	local ZoneName = xpath.selectNodes(xml_tree, '//ZoneName')
	local LocalUID = xpath.selectNodes(xml_tree, '//LocalUID')
	local SerialNumber = xpath.selectNodes(xml_tree, '//SerialNumber')
	local SoftwareVersion = xpath.selectNodes(xml_tree, '//SoftwareVersion')
	local HardwareVersion = xpath.selectNodes(xml_tree, '//HardwareVersion')
	local MACAddress = xpath.selectNodes(xml_tree, '//MACAddress')
	
	self.device:updateProperty('ZoneName', ZoneName[1][1])
	self.device:updateProperty('LocalUID', LocalUID[1][1])
	self.device:updateProperty('SerialNumber', SerialNumber[1][1])
	self.device:updateProperty('SoftwareVersion', SoftwareVersion[1][1])
	self.device:updateProperty('HardwareVersion', HardwareVersion[1][1])
	self.device:updateProperty('MACAddress', MACAddress[1][1])
end

function SonosUPnP:getPlayerInfo()
	self.http:request('http://'.. self.device.properties.ip .. ':1400/status/zp', {
	success = function(data)
	self:playerInfo(data.data)
	end,
	error = function(err)
	print (err)
	end
	})
end

function SonosUPnP:playFromQueue(id)
	self:clearSongInfo()
	local uri = 'x-rincon-queue:' .. self.device.properties.LocalUID .. '#0'
	local body = cosmo.fill(self.PLAY_FROM_QUEUE_BODY_TEMPLATE, { uri = uri })
	self:__sendRequest(self.AVTRANSPORT_URI, self.SET_TRANSPORT_ACTION, body, function(data) self:playFromQueueCallback(data, id) end, function(data) print ("ERROR");print(data) end)
end

function SonosUPnP:playFromQueueSeek(id)
	local body = cosmo.fill(self.SEEK_TRACK_BODY_TEMPLATE, { track = id})
	self:__sendRequest(self.AVTRANSPORT_URI, self.SEEK_ACTION, body, function(data) self.device:play()  end, function(data) print ("ERROR");print(data) end)
end

function SonosUPnP:playFromQueueCallback(data, id)
	if data.data == self.PLAY_FROM_QUEUE_RESPONSE then
		self:playFromQueueSeek(id)
	end
end

function SonosUPnP:clearPlaylist()
	self:__sendRequest(self.AVTRANSPORT_URI, self.CLEAR_QUEUE_ACTION, self.CLEAR_QUEUE_BODY, function(data) self:getQueue()  end, function(data) print ("ERROR");print(data) end)
end


function SonosUPnP:addToPlaylist(uri)
	local uri = self:formatUriString(uri)

	local body = cosmo.fill(self.ADD_TO_QUEUE_BODY_TEMPLATE, { uri = 'x-rincon-mp3radio://' .. uri})
	self:__sendRequest(self.AVTRANSPORT_URI, self.ADD_TO_QUEUE_ACTION, body, function(data) self:getQueue()  end, function(data) print ("ERROR");print(data) end)
end

function SonosUPnP:removeFromPlaylist(index)
	instance = '0'
	updid = '0'
    objid = 'Q:0/' .. tostring(index)

	local body = cosmo.fill(self.REMOVE_FROM_QUEUE_BODY_TEMPLATE, { instance=instance, objid=objid, updateid=updid})
	self:__sendRequest(self.AVTRANSPORT_URI, self.REMOVE_FROM_QUEUE_ACTION, body, function(data) self:getQueue()  end, function(data) print ("ERROR");print(data) end)
end

function SonosUPnP:getCurTrackInfo()
	self:__sendRequest(self.AVTRANSPORT_URI, self.GetCurTrackAction, self.GetCurTrackBody, function(data) self:getCurTrackInfoPosition(data.data) end, function(data) print ("ERROR");print(data) end)
end

function SonosUPnP:getCurTrackInfoPosition(data)
	local xml_tree = lom.parse(data)
	local curPosition = xpath.selectNodes(xml_tree, '//Track')

	self.device:updateProperty('ui.playlist.currentId',  curPosition[1][1]) 
end

function SonosUPnP:queueSortOrder(from, to)
	local body = cosmo.fill(self.ORDER_BODY_TEMPLATE, { from = from, to = to })
	self:__sendRequest(self.AVTRANSPORT_URI, self.ORDER_ACTION, body, function(data)  self:getQueue() end, function(data) print ("ERROR");print(data) end)
end

function SonosUPnP:playFromUri(uri)
	local uri = self:formatUriString(uri)

	local body = cosmo.fill(self.PLAY_URI_BODY_TEMPLATE, { uri = 'x-rincon-mp3radio://' .. uri })
	self:__sendRequest(self.AVTRANSPORT_URI, self.SET_TRANSPORT_ACTION, body, 
		function(data) 
			self.device:play() 
			self:clearSongInfo()
			self:setUiBasedOnRadioStream(uri)
		end, 
		function(data) print ("ERROR");print(data) end)
end

function SonosUPnP:clearSongInfo()
	self.device:updateProperty('albumArtURI', "/plugins/com.fibaro.sonosSpeaker/img/defaultIcon.png")
	self.device:updateProperty('position', "0:00:00/0:00:00")
	self.device:updateProperty('title', "")
	self.device:updateProperty('creator', "")
end

function SonosUPnP:formatUriString(uri)
	if string.find(uri, 'http://') then
		uri =  string.gsub(uri, "http://", "")
	else string.find(uri, 'www.') 
		uri =  string.gsub(uri, "www.", "")
	end

	return uri
end

function SonosUPnP:setUiBasedOnRadioStream(uri)
	local splitedUri = self:split(uri, "/")

	if #splitedUri > 1 then
		self.device:updateProperty('title', splitedUri[#splitedUri])
		self.device:updateProperty('creator', splitedUri[1]:gsub(':(.*)',''))
	else
		self.device:updateProperty('title', uri)
		self.device:updateProperty('creator', "")
	end
end

function SonosUPnP:split(inputstr, sep)
        if sep == nil then
                sep = "%s"
        end
        local t={} ; i=1
        for str in string.gmatch(inputstr, "([^"..sep.."]+)") do
                t[i] = str
                i = i + 1
        end
        return t
end






