require('common.device')
require('common.base64')
require('net.HTTPClient')
cosmo = require("cosmo.cosmo")
require('common.wol')

class 'Sony' (Device)

Sony.COMMANDS = {
	["KEY_ENTER"] = 'AAAAAwAAHFoAAAA9Aw==',
	["KEY_UP"] = 'AAAAAwAAHFoAAAA5Aw==',
	["KEY_DOWN"] = 'AAAAAwAAHFoAAAA6Aw==',
	["KEY_RIGHT"] = 'AAAAAwAAHFoAAAA8Aw==',
	["KEY_LEFT"] = 'AAAAAwAAHFoAAAA7Aw==',
	["KEY_HOME"] = 'AAAAAwAAHFoAAABCAw==',
	["KEY_OPTIONS"] = 'AAAAAwAAHFoAAAA/Aw==',
	["KEY_RETURN"] = 'AAAAAwAAHFoAAABDAw==',
	["KEY_1"] = 'AAAAAwAAHFoAAAAAAw==',
	["KEY_2"] = 'AAAAAwAAHFoAAAABAw==',
	["KEY_3"] = 'AAAAAwAAHFoAAAACAw==',
	["KEY_4"] = 'AAAAAwAAHFoAAAADAw==',
	["KEY_5"] = 'AAAAAwAAHFoAAAAEAw==',
	["KEY_6"] = 'AAAAAwAAHFoAAAAFAw==',
	["KEY_7"] = 'AAAAAwAAHFoAAAAGAw==',
	["KEY_8"] = 'AAAAAwAAHFoAAAAHAw==',
	["KEY_9"] = 'AAAAAwAAHFoAAAAIAw==',
	["KEY_0"] = 'AAAAAwAAHFoAAAAJAw==',
	["KEY_POWEROFF"] = 'AAAAAwAAHFoAAAAVAw==',
	["KEY_DISPLAY"] = 'AAAAAwAAHFoAAABBAw==',
	["KEY_AUDIO"] = 'AAAAAwAAHFoAAABkAw==',
	["KEY_SUBTITLE"] = 'AAAAAwAAHFoAAABjAw==',
	["KEY_FAVORITES"] = 'AAAAAwAAHFoAAABeAw==',
	["KEY_YELLOW"] = 'AAAAAwAAHFoAAABpAw==',
	["KEY_BLUE"] = 'AAAAAwAAHFoAAABmAw==',
	["KEY_RED"] = 'AAAAAwAAHFoAAABnAw==',
	["KEY_GREEN"] = 'AAAAAwAAHFoAAABoAw==',
	["KEY_PLAY"] = 'AAAAAwAAHFoAAAAaAw==',
	["KEY_STOP"] = 'AAAAAwAAHFoAAAAYAw==',
	["KEY_PAUSE"] = 'AAAAAwAAHFoAAAAZAw==',
	["KEY_REWIND"] = 'AAAAAwAAHFoAAAAbAw==',
	["KEY_FORWARD"] = 'AAAAAwAAHFoAAAAcAw==',
	["KEY_PREV"] = 'AAAAAwAAHFoAAABXAw==',
	["KEY_NEXT"] = 'AAAAAwAAHFoAAABWAw==',
	["KEY_REPLY"] = 'AAAAAwAAHFoAAAB2Aw==',
	["KEY_ADVANCE"] = 'AAAAAwAAHFoAAAB1Aw==',
	["KEY_ANGLE"] = 'AAAAAwAAHFoAAABlAw==',
	["KEY_TOP_MENU"] = 'AAAAAwAAHFoAAAAsAw==',
	["KEY_POP_UP_MENU"] = 'AAAAAwAAHFoAAAApAw==',
	["KEY_EJECT"] = 'AAAAAwAAHFoAAAAWAw==',
	["KEY_KARAOKE"] = 'AAAAAwAAHFoAAABKAw==',
	["KEY_SEN"] = 'AAAAAwAAHFoAAABMAw==',
	["KEY_NETFLIX"] = 'AAAAAwAAHFoAAABLAw==',
	["KEY_3D_MODE"] = 'AAAAAwAAHFoAAABNAw=='
}

Sony.APPS = {
	["YOUTUBE"] = "/apps/com.sony.iptv.type.ytleanback",
	["MUSIC_EXPLORER"] = "/apps/com.sony.musicexplorer",
	["MUSIC_PLAYER"] = "/apps/com.sony.musicplayer",
	["VIDEO_EXPLORER"] = "/apps/com.sony.videoexplorer",
	["VIDEO_PLAYER"] = "/apps/com.sony.videoplayer"
}

Sony.REGISTER_URI = "/register?name=$name&registrationType=initial&deviceId=TVSideView%3A00-22-4d-a4-3b-72&wolSupport=true"
Sony.CONTROL_URI = "/upnp/control/IRCC"
Sony.STATUS_URI = "/getStatus"
Sony.SYSTEM_INFO_URI = "/getSystemInformation"

Sony.UPnP_COMMAND = '<?xml version="1.0"?><s:Envelope xmlns:s="http://schemas.xmlsoap.org/soap/encoding/"><s:Body><u:X_SendIRCC xmlns:u="urn:schemas-sony-com:service:IRCC:1"><IRCCCode>$command</IRCCCode></u:X_SendIRCC></s:Body></s:Envelope>'

Sony.UPnP_COMMAND = [[<?xml version="1.0"?>
<s:Envelope xmlns:s="http://schemas.xmlsoap.org/soap/envelope/" s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/">
  <s:Body>
    <u:X_SendIRCC xmlns:u="urn:schemas-sony-com:service:IRCC:1">
      <IRCCCode>$command</IRCCCode>
    </u:X_SendIRCC>
  </s:Body>
</s:Envelope>]]

Sony.HEADERS = {
	['X-CERS-DEVICE-INFO'] = 'Android4.1.2/TVSideViewForAndroid2.0.2/GT-I9300',
	['X-CERS-DEVICE-ID'] = 'TVSideView:00-22-4d-a4-3b-72',
	['User-Agent'] = 'Dalvik/1.6.0 (Linux; U; Android 4.1.2; GT-I9300 Build/JZO54K)'
}

function Sony:__init(id)
	Device.__init(self, id)
	self.name = 'FibaroRemote'
	self.controlUrl = 'http://' .. self.properties.ip .. ':50001' .. Sony.CONTROL_URI
	self.registerUrl = 'http://' .. self.properties.ip .. ':50002' .. cosmo.fill(Sony.REGISTER_URI, { name = self.name })  
	self.statusUrl = 'http://' .. self.properties.ip .. ':50002' .. Sony.STATUS_URI
	self.appUrl = 'http://' .. self.properties.ip .. ':50202'
	self.sysetmInfoUrl = 'http://' .. self.properties.ip .. ':50002' .. Sony.SYSTEM_INFO_URI

	self.http = net.HTTPClient({ timeout = 3000 })

	self.wakeOnLan = WakeOnLan()

	if (self.properties.authCode) then
		if (self.properties.authCode ~= "") then
			self:authorize(self.properties.authCode)
		end
	end

	self:getMac()
end

function Sony:sendKey(code)
	local cmd = Sony.COMMANDS[code]
	local body = cosmo.fill(Sony.UPnP_COMMAND, { command = cmd })
	self:__sendRequest(body)
end

function Sony:__sendRequest(command)
	local controlHeaders = {
		["soapaction"] = '"urn:schemas-sony-com:service:IRCC:1#X_SendIRCC"',
		["content-type"] = 'text/xml; charset=UTF-8'
	}

	self.http:request(self.controlUrl, {
		options = {
			headers = controlHeaders,
			method = 'POST',
			data = command
		},
		success = function(param)
			print('Succes STATUS: ', param.status)
			print('Succes DATA: ', param.data)
		end,
		error = function (error)
			print('*******************')
			print('* ERROR: ', error)
			print('*******************')
		end
	})

	if (self.properties.mac == "") then
		self:getMac()
	end
end

function Sony:getStatus()
	local statusHeaders = {
		["soapaction"] = '"urn:schemas-sony-com:service:IRCC:1#X_SendIRCC"',
		["content-type"] = 'text/xml; charset=UTF-8'
	}

	self.http:request(self.statusUrl, {
		options = {
			headers = Sony.HEADERS,
			method = 'GET'
		},
		success = function(param)
			print('Succes STATUS: ', param.status)
			print('Succes DATA: ', param.data)
		end,
		error = function (error)
			print('*******************')
			print('* ERROR: ', error)
			print('*******************')
		end
	})
end

function Sony:requestAuthorizationKey()
	Sony.HEADERS['Authorization'] = nil
	print(self.registerUrl)
	self.http:request(self.registerUrl, {
		options = {
			headers = Sony.HEADERS,
			method = 'GET'
		},
		success = function(param)
			print('Succes STATUS: ', param.status)
			print('Succes DATA: ', param.data)
		end,
		error = function (error)
			print('*******************')
			print('* ERROR: ', error)
			print('*******************')
		end
	})
end

function Sony:authorize(authorizationKey)
	local code = to_base64(':' .. authorizationKey)
	local authenticationHeaders = self.HEADERS
	authenticationHeaders['Authorization'] = 'Basic ' .. code

	self.http:request(self.registerUrl, {
		options = {
			headers = authenticationHeaders,
			method = 'GET'
		},
		success = function(param)
			print('Succes STATUS: ', param.status)
			print('Succes DATA: ', param.data)
		end,
		error = function (error)
			print('*******************')
			print('* ERROR: ', error)
			print('*******************')
		end
	})
end

function Sony:runApp(appName)
	if(Sony.APPS[appName] ~= nil) then
		local fullAppUrl = self.appUrl .. Sony.APPS[appName]
		Sony.HEADERS['Content-Length'] = '0'

		self.http:request(fullAppUrl, {
			options = {
				headers = Sony.HEADERS,
				method = 'POST'
			},
			success = function(param)
				print('Succes STATUS: ', param.status)
				print('Succes DATA: ', param.data)
			end,
			error = function (error)
				print('*******************')
				print('* ERROR: ', error)
				print('*******************')
			end
		})
	end
end

function Sony:getMac()
	print('Sony:getMac')
	print('getMac URL: ', self.sysetmInfoUrl)
	self.http:request(self.sysetmInfoUrl, {
		options = {
			method = 'GET'
		},
		success = function(param)
			print('Succes STATUS: ', param.status)
			print('Succes DATA: ', param.data)
			if (param.status == 200) then
				local mac = param.data:match('<functionItem field="MAC" value="(.*)"/>')
				self:updateProperty('macAddress', mac)
			end
		end,
		error = function (error)
			print('*******************')
			print('* ERROR: ', error)
			print('*******************')
		end
	})
end

function Sony:power()
	if (self.properties.ip ~= nil and self.properties.macAddress ~= nil and 
		self.properties.ip ~= '' and self.properties.macAddress ~= '') then

		self:sendKey("KEY_POWEROFF")
		self.wakeOnLan:wake(self.properties.macAddress)
	end
end