require('common.device')
require('common.base64')
require('net.HTTPClient')
require('common.wol')

cosmo = require("cosmo.cosmo")

pollingTime = 600

class 'Sony' (Device)

local function stringTrim(str)
    return (string.gsub(str, "^%s*(.-)%s*$", "%1"))
end
 
local function stringExplode(str, sep)
    local pos, t = 1, {}
    if #sep == 0 or #str == 0 then return end
    for s, e in function() return string.find(str, sep, pos) end do
        table.insert(t, stringTrim(string.sub(str, pos, s-1)))
        pos = e+1
    end
    table.insert(t, stringTrim(string.sub(str, pos)))
    return t
end

Sony.COMMANDS = {
	["KEY_ENTER"] = 'AAAAAQAAAAEAAABlAw==',
	["KEY_UP"] = 'AAAAAQAAAAEAAAB0Aw==',
	["KEY_DOWN"] = 'AAAAAQAAAAEAAAB1Aw==',
	["KEY_RIGHT"] = 'AAAAAQAAAAEAAAAzAw==',
	["KEY_LEFT"] = 'AAAAAQAAAAEAAAA0Aw==',
	["KEY_HOME"] = 'AAAAAQAAAAEAAABgAw==',
	["KEY_OPTIONS"] = 'AAAAAgAAAJcAAAA2Aw==',
	["KEY_RETURN"] = 'AAAAAgAAAJcAAAAjAw==',
	["KEY_1"] = 'AAAAAQAAAAEAAAAAAw==',
	["KEY_2"] = 'AAAAAQAAAAEAAAABAw==',
	["KEY_3"] = 'AAAAAQAAAAEAAAACAw==',
	["KEY_4"] = 'AAAAAQAAAAEAAAADAw==',
	["KEY_5"] = 'AAAAAQAAAAEAAAAEAw==',
	["KEY_6"] = 'AAAAAQAAAAEAAAAFAw==',
	["KEY_7"] = 'AAAAAQAAAAEAAAAGAw==',
	["KEY_8"] = 'AAAAAQAAAAEAAAAHAw==',
	["KEY_9"] = 'AAAAAQAAAAEAAAAIAw==',
	["KEY_0"] = 'AAAAAQAAAAEAAAAJAw==',
	["KEY_11"] = 'AAAAAQAAAAEAAAAKAw==',
	["KEY_12"] = 'AAAAAQAAAAEAAAALAw==',
	["KEY_POWEROFF"] = 'AAAAAQAAAAEAAAAvAw==',
	["KEY_POWERON"] = 'AAAAAQAAAAEAAAAuAw==',

	["KEY_DISPLAY"] = 'AAAAAQAAAAEAAAA6Aw==',
	["KEY_AUDIO"] = 'AAAAAQAAAAEAAAAXAw==',
	["KEY_SUBTITLE"] = 'AAAAAgAAAJcAAAAoAw==',
	["KEY_FAVORITES"] = 'AAAAAgAAAHcAAAB2Aw==',
	["KEY_YELLOW"] = 'AAAAAgAAAJcAAAAnAw==', 
	["KEY_BLUE"] = 'AAAAAgAAAJcAAAAkAw==',
	["KEY_RED"] = 'AAAAAgAAAJcAAAAlAw==',
	["KEY_GREEN"] = 'AAAAAgAAAJcAAAAmAw==',
	["KEY_PLAY"] = 'AAAAAgAAAJcAAAAaAw==',
	["KEY_STOP"] = 'AAAAAgAAAJcAAAAYAw==',
	["KEY_PAUSE"] = 'AAAAAgAAAJcAAAAZAw==',
	["KEY_REWIND"] = 'AAAAAgAAAJcAAAAbAw==',
	["KEY_FORWARD"] = 'AAAAAgAAAJcAAAAcAw==',
	["KEY_PREV"] = 'AAAAAgAAAJcAAAA8Aw==',
	["KEY_NEXT"] = 'AAAAAgAAAJcAAAA9Aw==',
	["KEY_REC"] = 'AAAAAgAAAJcAAAAgAw==',
	["KEY_TOP_MENU"] = 'AAAAAgAAABoAAABgAw==',
	["KEY_POP_UP_MENU"] = 'AAAAAgAAABoAAABhAw==',
	["KEY_EJECT"] = 'AAAAAgAAAJcAAABIAw==',
	["KEY_SEN"] = 'AAAAAgAAABoAAAB9Aw==',
	["KEY_NETFLIX"] = 'AAAAAgAAABoAAAB8Aw==',
	["KEY_3D_MODE"] = 'AAAAAgAAAHcAAABNAw==',
	["KEY_INPUT"] = 'AAAAAQAAAAEAAAAlAw==',
	["KEY_GUIDE"] = 'AAAAAQAAAAEAAAAOAw==',
	["KEY_IMANUAL"] = 'AAAAAgAAABoAAAB7Aw==',
	["KEY_EPG"] = 'AAAAAgAAAKQAAABbAw==',
	["KEY_VOLUP"] = 'AAAAAQAAAAEAAAASAw==',
	["KEY_VOLDOWN"] = 'AAAAAQAAAAEAAAATAw==',
	["KEY_MUTE"] = 'AAAAAQAAAAEAAAAUAw==',
	["KEY_CHUP"] = 'AAAAAQAAAAEAAAAQAw==',
	["KEY_CHDOWN"] = 'AAAAAQAAAAEAAAARAw==',
	["KEY_S_ENTER"] = 'AAAAAQAAAAEAAAALAw==',
	["KEY_ClosedCaption"] = 'AAAAAgAAAKQAAAAQAw==',
	["KEY_Analog"] = 'AAAAAgAAAHcAAAANAw==',
	["KEY_Analog2"] = 'AAAAAQAAAAEAAAA4Aw==',
	["KEY_Teletext"] = 'AAAAAQAAAAEAAAA/Aw==',
	["KEY_TV_RADIO"] = 'AAAAAgAAABoAAABXAw==',
	["KEY_InternetWidgets"] = 'AAAAAgAAABoAAAB6Aw==',
	["KEY_InternetVideo"] = 'AAAAAgAAABoAAAB5Aw==',
	["KEY_ProgramDescription"] = 'AAAAAgAAAJcAAAAWAw==',
	["KEY_acTVila"] = 'AAAAAgAAABoAAAByAw==',
	["KEY_Media"] = 'AAAAAgAAAJcAAAA4Aw==',
	["KEY_SyncMenu"] = 'AAAAAgAAABoAAABYAw==',
	["KEY_Wide"] = 'AAAAAgAAAKQAAAA9Aw==',
	["KEY_RECORD"] = 'AAAAAgAAAJcAAAAgAw=='
}

Sony.REGISTER_URI = "/sony/accessControl"
Sony.CONTROL_URI = "/sony/IRCC"
Sony.DMR_URI = '/dmr.xml'

Sony.REGITSER_COMMAND = '{"id":13,"method":"actRegister","version":"1.0","params":[{"clientid":"FIBARO-REMOTE","nickname":"FIBARO REMOTE"},[{"clientid":"FIBARO-REMOTE","value":"yes","nickname":"FIBARO REMOTE","function":"WOL"}]]}'

Sony.UPnP_COMMAND = [[<?xml version="1.0"?>
<s:Envelope xmlns:s="http://schemas.xmlsoap.org/soap/envelope/" s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/">
  <s:Body>
    <u:X_SendIRCC xmlns:u="urn:schemas-sony-com:service:IRCC:1">
      <IRCCCode>$command</IRCCCode>
    </u:X_SendIRCC>
  </s:Body>
</s:Envelope>]]

function Sony:__init(id)
	Device.__init(self, id)
	
	self.wakeOnLan = WakeOnLan()
	self.name = 'FibaroRemote'
	self.controlUrl = 'http://' .. self.properties.ip .. Sony.CONTROL_URI
	self.registerUrl = 'http://' .. self.properties.ip .. Sony.REGISTER_URI
	self.dmrUrl = 'http://' .. self.properties.ip .. ":52323" .. Sony.DMR_URI
	self.authCookie = ''

	self.http = net.HTTPClient({timeout = 1000})

	if (self.properties.authCode) then
		if (self.properties.authCode ~= "") then
			self:authorize(self.properties.authCode)
		end
	end

	self:checkPower()
end

function Sony:sendKey(code)
	local cmd = Sony.COMMANDS[code]
	local body = cosmo.fill(Sony.UPnP_COMMAND, { command = cmd })
	self:__sendRequest(body)
end

function Sony:__sendRequest(command)
	local controlHeaders = {
		["soapaction"] = '"urn:schemas-sony-com:service:IRCC:1#X_SendIRCC"',
		["content-type"] = 'text/xml; charset=UTF-8',
		["Cookie"] =  self.authCookie
	}
	print(self.controlUrl)
	self.http:request(self.controlUrl, {
		options = {
			headers = controlHeaders,
			method = 'POST',
			data = command
		},
		success = function(params)
			print(params.status)
			print(params.data)
		end,
		error = function(error)
			print(error)
		end
	})
end

function Sony:requestAuthorizationKey()
	self.http:request(self.registerUrl, {
		--headers = Sony.HEADERS,
		options = {
			method = 'POST',
			data = Sony.REGITSER_COMMAND
		},
		success = function(params)
			print(params.status)
			print(params.data)
		end,
		error = function(err)
			print(err)
		end
	})

end

function Sony:authorize(authorizationKey)
	local code = to_base64(':' .. authorizationKey)

	local authenticationHeaders = {
		["Authorization"] = 'Basic ' .. code
	}

	self.http:request(self.registerUrl, {
		options = {
			method = 'POST',
			data = Sony.REGITSER_COMMAND,
			headers = authenticationHeaders
		},
		success = function(params)
			print(params.status)
			print(params.data)
			for k,v in pairs(params.headers) do
				if k == 'Set-Cookie' then
					self.authCookie = v
					break
				end
			end
		end,
		error = function(err)
			print(err)
		end
	})
end

function Sony:power()	
	self.http:request(self.dmrUrl, {

		success = function(params)
			print(params.status)
			self:sendKey("KEY_POWEROFF")
		end,

		error = function(err)
			self:wakeUp()
		end
	})

end

function Sony:checkPower()
	self.http:request(self.dmrUrl, {

		success = function(params)
			sony:updateProperty('power', "1")
			--sony:updateProperty('ui.POWER_Button.value', true)
			if (self.properties.macAddress == "") then

				local mac = params.data:match("<UDN>uuid:(.*)</UDN>")

				local uuid = stringExplode(mac,"-")

				if(uuid[5] ~= nil) then
					mac = uuid[5]
					if(mac:len() == 12) then
						mac = mac:sub(1,2) .. "-" .. mac:sub(3,4) .. "-" .. mac:sub(5,6) .. "-" .. mac:sub(7,8) .. "-" .. mac:sub(9,10) .. "-" .. mac:sub(11,12) 
						self:updateProperty('macAddress', mac)
					end
				end
			end
		end,

		error = function(err)
			sony:updateProperty('power', "0")
			--sony:updateProperty('ui.POWER_Button.value', false)
		end
	})

	setTimeout(function() self:checkPower() end, pollingTime)
end

function Sony:wakeUp()
	print(self.properties.ip)
	if(self.properties.macAddress ~= nil) then
		self.wakeOnLan:wake(self.properties.macAddress)
	end
end
