require('common.device')
require('net.HTTPClient')
require('common.unitsConverter')

class 'yahooWeather' (Device)

function yahooWeather:__init(id)
    Device.__init(self, id) -- Calls super class (Device) constructor.
    self.http = net.HTTPClient({ timeout = 10000 })  -- Creates an instance of HTTPClient class.
    self.pollingCommandId = nil
    self.pollingForStatusTime = 5* 60 * 1000 -- 5 minutes
    self.pollingWaitOnErrorTime = 5000

    self:getYahooWeather()
end

function yahooWeather:getYahooWeather()
    local settingsLocation = api.get('/settings/location')
    local coords = settingsLocation.latitude .. "," .. settingsLocation.longitude
    
    url = "https://query.yahooapis.com/v1/public/yql?q=select%20item.condition%2Cwind%2Catmosphere%2Citem.lat%2Citem.long%20from%20weather.forecast%20where%20woeid%20in%20(select%20woeid%20from%20geo.places(1)%20where%20text%3D%22(" .. coords ..  ")%22)&format=json"

    local params = {
        url = url,
        callback =  function(response) self:onDataReceivedFromGetYahooWeather(response) end,
    }
    self:httpRequest(params)
end

function yahooWeather:urlencode(str)
  if (str) then
    str = string.gsub (str, "\n", "\r\n")
    str = string.gsub (str, "([^%w %-%_%.%~])",
        function (c) return string.format ("%%%02X", string.byte(c)) end)
    str = string.gsub (str, " ", "+")
  end
  return str
end

function yahooWeather:onDataReceivedFromGetYahooWeather(response)
    local jsonData = {}
    local ok, jsonData = pcall(function() return json.decode(response.data) end)

    if ok and jsonData and jsonData.query and jsonData.query.results then
        pcall(function()
            if jsonData.query.results.channel[1] ~= nil then
                self:updateProperty('ConditionCode', tonumber(jsonData.query.results.channel[1].item.condition.code))
                self:updateProperty('ConditionCodeConverted', tonumber(jsonData.query.results.channel[1].item.condition.code))
                self:updateProperty('Humidity', tonumber(jsonData.query.results.channel[1].atmosphere.humidity))
                self:updateProperty('Temperature', unitsConverter.farenheit2celsius(tonumber(jsonData.query.results.channel[1].item.condition.temp)))
                self:updateProperty('WeatherCondition', jsonData.query.results.channel[1].item.condition.text)
                self:updateProperty('Wind', tonumber(jsonData.query.results.channel[1].wind.speed))
                self:updateProperty('Pressure', tonumber(jsonData.query.results.channel[1].atmosphere.pressure))
                self:setCurentWeatherLabel()
            elseif jsonData.query.results.channel.item ~= nil then
                self:updateProperty('ConditionCode', tonumber(jsonData.query.results.channel.item.condition.code))
                self:updateProperty('ConditionCodeConverted', tonumber(jsonData.query.results.channel.item.condition.code))
                self:updateProperty('Humidity', tonumber(jsonData.query.results.channel.atmosphere.humidity))
                self:updateProperty('Temperature', unitsConverter.farenheit2celsius(tonumber(jsonData.query.results.channel.item.condition.temp)))
                self:updateProperty('WeatherCondition', jsonData.query.results.channel.item.condition.text)
                self:updateProperty('Wind', tonumber(jsonData.query.results.channel.wind.speed))
                self:updateProperty('Pressure', tonumber(jsonData.query.results.channel.atmosphere.pressure))
                self:setCurentWeatherLabel()
            else
                self:onErrorReceivedFromGetYahooWeather()
            end
        end)
    end

    if jsonData and jsonData.query and jsonData.query.count and tonumber(jsonData.query.count) == 0 then
            self:onErrorReceivedFromGetYahooWeather()
    end

    if self.pollingCommandIdOnDataReceived then
        clearTimeout(self.pollingCommandIdOnDataReceived)
    end
    self.pollingCommandIdOnDataReceived = setTimeout(function() self:getYahooWeather() end, self.pollingForStatusTime)
end

function yahooWeather:onErrorReceivedFromGetYahooWeather()
    if self.pollingCommandIdOnError then
        clearTimeout(self.pollingCommandIdOnError)
    end

    self.pollingCommandIdOnError = setTimeout(function() self:getYahooWeather() end, self.pollingWaitOnErrorTime)

end

function yahooWeather:httpRequest(params)
    self.http:request(params.url,{
        options = {
            method = params.method
        },
        success = function(response)
            if (params.callback) then
                params.callback(response)
            end
        end,
        error = function(error)
            self:onErrorReceivedFromGetYahooWeather()
        end
    })
end

function yahooWeather:round(num, idp)
    idp = idp or 1
    if idp~=0 and num%1~=0 then
        local mult = 10^idp
        r= math.floor(num * mult + 0.5) / mult
        if r%1==0 then
            return math.floor(r)
        else
            return r
        end
    else
        return math.floor(tonumber(num) + 0.5)
    end
end

function yahooWeather:setCurentWeatherLabel()
        local settingsLocation = api.get('/settings/location')
        local settingsInfo = api.get('/settings/info')
        local humidityValueLabel = tostring(self:round(self.properties.Humidity)) .. "%"
        local tempValueLabel = ""
        local windValueLabel = ""

        if settingsLocation.temperatureUnit == "F" then
            tempValueLabel = tostring(self:round(unitsConverter.celsius2farenheit(self.properties.Temperature))) .. '°F'
        else
            tempValueLabel = tostring(self:round(self.properties.Temperature)) .. '°C'
        end

        if settingsLocation.windUnit == "mph" then
            windValueLabel = tostring(self:round(unitsConverter.km2mile(self.properties.Wind))) .. 'mph'
        else
            windValueLabel = tostring(self:round(self.properties.Wind)) .. 'km/h'
        end

        if settingsInfo.defaultLanguage == 'pl' then
            self:updateProperty('ui.Current_Weather_Label.caption',  'Temp.: ' .. tempValueLabel .. '\n\nWilg.: '  .. humidityValueLabel .. '\n\nWiatr: '   .. windValueLabel)
        else
            self:updateProperty('ui.Current_Weather_Label.caption',  'Temp.: ' .. tempValueLabel .. '\n\nHumidity: '  .. humidityValueLabel .. '\n\nWind: '   .. windValueLabel)
        end
end
