#!/bin/bash

#
#
# PREVENT FROM PARALLEL EXECUTION
#
#

source /opt/fibaro/scripts/config

LOCKFILE="/var/lock/`basename $0`"
RESULTFILE="/opt/fibaro/recoveryStatus"
SPACEFILE="/opt/fibaro/recoverySpace"
LOCKFD=99

# PRIVATE
_lock()             { flock -$1 $LOCKFD; }
_no_more_locking()  { _lock u; _lock xn && rm -f $LOCKFILE; }
_prepare_locking()  { eval "exec $LOCKFD>\"$LOCKFILE\""; trap _no_more_locking EXIT; }

# ON START
_prepare_locking

# PUBLIC
exlock_now()        { _lock xn; }  # obtain an exclusive lock immediately or fail
exlock()            { _lock x; }   # obtain an exclusive lock
shlock()            { _lock s; }   # obtain a shared lock
unlock()            { _lock u; }   # drop a lock

#
#
# MAIN SCRIPT
#
#

set_status()        { echo -ne "$1\t`date +%s`" > $RESULTFILE; }
set_space()        { echo -ne "$1\t`date +%s`" > $SPACEFILE; }

# aquire exclusive lock
exlock_now || exit 1
set_status Checking

status="Corrupted"

# Bring recovery drive online:
if [ "$1" != "--no-mount" ]; then
    recoveryOn
fi

if [[ `mount | grep "/home/fghc2/recovery"` ]]
then
        # calculate checksum
        calculated=`md5sum /home/fghc2/recovery/system/image.gz 2>/dev/null | cut -d ' ' -f 1`
        expected=`cat /home/fghc2/recovery/system/checksum`

        # generate output
        if [ "$calculated" == "$expected" ]  && [ ! -z "$calculated" ]
        then
          status="Ok"
        fi

        freeSpace=`df -m /home/fghc2/recovery | tail -1 | awk '{print $4}'`
        dbSize=`ls -s --block-size=1M /opt/fibaro/db | awk '{print $1}'`
        requiredSpace=`perl -e "print $dbSize*1.2"`

        spaceAvailable=`perl -e "print $requiredSpace < $freeSpace"`

        if [ "$spaceAvailable" != "1" ]; then
            set_space "NoSpaceLeft"
        else
            set_space "Ok"
        fi
else
        status="NotAvailable"
fi

set_status $status

# Brind recovery drive down:
if [ "$1" != "--no-mount" ]; then
    recoveryOff
fi
