#!/usr/bin/php
<?php

$db = new PDO('sqlite:/opt/fibaro/db');

function getIdListOfZwaveDevices()
{
  global $db;

  $statement = $db->prepare('SELECT Id FROM NEW_Device WHERE Device_Type = "com.fibaro.zwaveDevice"');
  $statement->execute();

  $dbResult = $statement->fetchAll();

  $result = array();
  foreach ($dbResult as $dbRow)
    array_push($result, $dbRow['Id']);

  return $result;
}

function getInternalData($deviceId)
{
  global $db;

  $statement = $db->prepare('SELECT InternalData FROM NEW_Internal WHERE Device_Id = :deviceId');
  $statement->bindParam(':deviceId', $deviceId);
  $statement->execute();

  $result = $statement->fetch();
  return json_decode($result['InternalData'], true);
}

function getProperty($deviceId, $propertyName)
{
  global $db;

  $statement = $db->prepare('SELECT Value FROM NEW_Property WHERE Device_Id = :deviceId AND Name = :propertyName');
  $statement->bindParam(':deviceId', $deviceId);
  $statement->bindParam(':propertyName', $propertyName);
  $statement->execute();

  $result = $statement->fetch();
  return json_decode($result['Value'], true);
}

function setInternalData($deviceId, $internalData)
{
  global $db;

  $internalDataJson = json_encode($internalData);

  $statement = $db->prepare('UPDATE NEW_Internal SET InternalData = :internalData WHERE Device_Id = :deviceId');
  $statement->bindParam(':deviceId', $deviceId);
  $statement->bindParam(':internalData', $internalDataJson);
  $statement->execute();
}

function &getValueByName(&$internalData, $endpointId, $valueName)
{
  foreach ($internalData['node']['endPoint'][$endpointId]['value'] as &$value)
  {
    if ($value['data']['nameValue'] == $valueName)
    {
      return $value['data']['value'];
    }
  }

  throw new Exception("Value named '" . $valueName . "' couldn't be found in given internal data.");
}

function createAssociationGroupObject($associationGroup)
{
  $result = array(
    'nodes'     => $associationGroup['devices'],
    'maxNode'   => 0,
    'id'        => $associationGroup['groupID']
  );

  if (isset($associationGroup['associationGrpInfo']))
  {
    $info = $associationGroup['associationGrpInfo'];

    if (isset($info['profile']))
      $result['profile'] = $info['profile'];

    if (isset($info['name']))
      $result['name'] = $info['name'];

    if (isset($info['commands']))
      $result['commands'] = $info['commands'];
  }

  return $result;
}

function fixInternalData($deviceId)
{
  try
  {
    $internalData = getInternalData($deviceId);
    $associationView = getProperty($deviceId, 'associationView');

    $associationViewInternal = &getValueByName($internalData, 0, 'associationView');

    $index = 0;
    $associationViewInternalFixed = array();
    
    foreach ($associationView as $associationViewGroup)
    {
      $group = createAssociationGroupObject($associationViewGroup);

      // check if it's possible to extract 'maxNode' value from internal data (otherwise leave value '0')
      if (isset($associationViewInternal[$index]['maxNode']))
        $group['maxNode'] = $associationViewInternal[$index]['maxNode'];
      else
        echo("Warning: 'maxNode' value couldn't be found for group id = " . $index . ", default value (0) will be used\n");

      array_push($associationViewInternalFixed, $group);
      $index++;
    }

    $associationViewInternal = $associationViewInternalFixed;

    echo('Setting fixed internal data for devId = ' . $deviceId . "\n");
    setInternalData($deviceId, $internalData);
  }
  catch (Exception $e)
  {
    echo('Unexpected error: ' . $e->getMessage() . "\n");
  }
}

//
// MAIN
//
$zwaveDevicesIdList = getIdListOfZwaveDevices();
foreach ($zwaveDevicesIdList as $deviceId)
  fixInternalData($deviceId);
