#!/bin/bash

function getInfo()
{
  ip=`ifconfig eth0 | grep "inet addr" | tr -s ' ' ':' | cut -d ':' -f 4`
  mask=`ifconfig eth0 | grep "inet addr" | tr -s ' ' ':' | cut -d ':' -f 8`
  gw=`route -n | grep ^0.0.0.0 | tr -s ' ' | cut -d ' ' -f 2`
  dns=`cat /etc/resolv.conf | grep "nameserver" | head -n 1 | cut -d ' ' -f 2`
  dhcp=`cat /etc/network/interfaces | grep "iface eth0 inet dhcp" | wc -l`

  echo -ne "$dhcp|$ip|$mask|$gw|$dns"
}

function showHelp()
{

  echo "Usage:
  net.sh get
  net.sh set static IP MASK GW DNS
  net.sh set dhcp
  net.sh reload"

}

function setConfig()
{
  if [[ "$1" == "static" ]]
  then

    ifconfig eth0 $2 netmask $3 up &> /dev/null
    route del default &> /dev/null
    route add default gw $4 eth0 &> /dev/null

    # configure network adapter
    echo -ne "auto lo\niface lo inet loopback\n\n" > /etc/network/interfaces
    echo -ne "auto eth0\niface eth0 inet static\n" >> /etc/network/interfaces
    echo -ne "address $2\n" >> /etc/network/interfaces
    echo -ne "netmask $3\n" >> /etc/network/interfaces
    echo -ne "gateway $4" >> /etc/network/interfaces

    # save shared network config
    echo -ne "type=static\n" > /opt/fibaro/network.conf
    echo -ne "address=$2\n" >> /opt/fibaro/network.conf
    echo -ne "mask=$3\n" >> /opt/fibaro/network.conf
    echo -ne "gateway=$4\n" >> /opt/fibaro/network.conf
    echo -ne "dns=$5\n" >> /opt/fibaro/network.conf

    # configure dns
    echo -ne "nameserver $5" > /etc/resolv.conf

    # add secondary dns
    if [[ "$4" != "8.8.8.8" ]]
    then 
      echo -ne "\nnameserver 8.8.8.8" >> /etc/resolv.conf
    fi

  elif [[ "$1" == "dhcp" ]]
  then

    dhclient eth0
    echo -ne "auto lo\niface lo inet loopback\n\n" > /etc/network/interfaces
    echo -ne "auto eth0\niface eth0 inet dhcp" >> /etc/network/interfaces

    # save shared network config
    echo -ne "type=dhcp" > /opt/fibaro/network.conf

  else

    showHelp
    exit 0

  fi

  getInfo
}

if [[ "$1" == "get" ]]
then

  getInfo

elif [[ "$1" == "set" ]]
then

  setConfig $2 $3 $4 $5 $6

elif [[ "$1" == "reload" ]]
then

  if [[ -e /opt/fibaro/network.conf ]]
  then
    source /opt/fibaro/network.conf
    setConfig $type $address $mask $gateway $dns
  fi

else

  showHelp

fi
