. /opt/fibaro/scripts/update/future/config.sh

MAXIMUM_SIZE=100 # in MB, maximum size of migrated data

# colours
RED='\e[1;31m'
GREEN='\e[1;32m'
BLUE='\e[0;34m'
NORMAL='\e[0m'

_error()
{
#    echo -ne "$RED  ERROR: $1 $NORMAL\n" 1>&2 Does not work with shell
    echo -e "ERROR: $1 \n" 1>&2
    exit $2
}

_log()
{
#  echo -ne "{BLUE}$1${NORMAL}\n" # Does not work with shell
    echo -e "$1\n"
}

check_user_privileges()
{
    if [  "$(id -u)" != "0" ] ; then
        _error "This script must be run as root." 2
    fi
}

computeFileSize()
{
    local REQUIRED_SPACE=${1}
    local FILE=${2}
    if [ -e ${FILE} ] ; then
        local FILE_SIZE=`ls -s --block-size=1M ${FILE} | awk '{print $1}'`
        REQUIRED_SPACE=$(( REQUIRED_SPACE + FILE_SIZE ))
    fi
    echo ${REQUIRED_SPACE}
}

computeDirSize()
{
    local REQUIRED_SPACE=${1}
    local FILE=${2}
    if [ -e ${FILE} ] ; then
        local FILE_SIZE=`du -s --block-size=1M ${FILE} | awk '{print $1}'`
        REQUIRED_SPACE=$(( REQUIRED_SPACE + FILE_SIZE ))
    fi
    echo ${REQUIRED_SPACE}
}

assert_required_migration_size()
{
    local REQUIRED_SPACE=0

    REQUIRED_SPACE=$(computeFileSize ${REQUIRED_SPACE} ${CONFIG_DB_PATH})
    REQUIRED_SPACE=$(computeFileSize ${REQUIRED_SPACE} ${CONFIG_HISTORY_DB_PATH})
    REQUIRED_SPACE=$(computeDirSize ${REQUIRED_SPACE} ${SCENE_DIR})
    REQUIRED_SPACE=$(computeDirSize ${REQUIRED_SPACE} ${NETCONF_FILE})
    REQUIRED_SPACE=$(computeDirSize ${REQUIRED_SPACE} ${ICON_DIR})
    REQUIRED_SPACE=$(computeDirSize ${REQUIRED_SPACE} ${VICON_DIR})

    if [ ${REQUIRED_SPACE} -gt ${MAXIMUM_SIZE} ] ; then
        _error "No enough space for migration. Req: ${REQUIRED_SPACE}M" 3
    fi
}

check_available_space()
{
    local REQUIRED_SPACE=0

    REQUIRED_SPACE=$(computeFileSize ${REQUIRED_SPACE} ${CONFIG_DB_PATH})
    REQUIRED_SPACE=$(computeFileSize ${REQUIRED_SPACE} ${CONFIG_HISTORY_DB_PATH})
    REQUIRED_SPACE=$(computeDirSize ${REQUIRED_SPACE} ${SCENE_DIR})
    REQUIRED_SPACE=$(computeDirSize ${REQUIRED_SPACE} ${NETCONF_FILE})

    local FREE_SPACE=`df -m ${BACKUP_DEFAULT_DIR} | tail -1 | awk '{print $4}'`

    if [ ${FREE_SPACE} -lt ${REQUIRED_SPACE} ] ; then
        _error "No enough space left on the device. Req: ${REQUIRED_SPACE}M Av: ${FREE_SPACE}M" 3
    fi
}
