#!/bin/bash

APP_NAME="voip_manager"
SIP_CONF_FILE="/etc/asterisk/sip.conf"
EXTEN_CONF_FILE="/etc/asterisk/extensions.conf"
MOH_CONF_FILE="/etc/asterisk/musiconhold.conf"
ZAPATA_CONF_FILE="/etc/asterisk/zapata.conf"

ALL_USERS_EXTEN="exten => everyone, 1, Dial("

removeQuotes()
{
  str=$1 

  if [ "${str:0:1}" == '"' ] && [ "${str:${#str}-1:${#str}}" == '"' ]
  then
    str="${str:1:${#str}-2}"
  fi
  echo $str
}

addSipUser ()
{
   local sipUserID="$1"
   local sipUserPassword="$2"
   local sipDisplayName="$3"

   echo "
[$sipUserID]
type=friend
host=dynamic
username=$sipUserID
secret=$sipUserPassword
callerid=$sipDisplayName
context=default" >> $SIP_CONF_FILE
}

getDevices ()
{
  # search for all voip devices (16 - HC_user, 19 - helios_IP, 20 - helios_user, 22 - mobileDevice)
  local query='SELECT id FROM NEW_Device WHERE id IN (SELECT Device_Id FROM NEW_Interface WHERE InterfaceName="voip")'
  local i=0

  for line in `sqlite3 /opt/fibaro/db "$query"`
  do
    echo $line
    devicesArray[$i]=$line
    ((i++))
  done
}

showHelp ()
{
  echo "
DESCRIPTION: Reload VOIP configuration according to entries in database.

USAGE: $APP_NAME reload

EXIT CODES:
  0 - successfull execution
  1 - help message
  2 - must have root priviliges
"
  exit 1
}

#########################
# MAIN                  #
#########################

if [ ! $# -eq 1 ] || [ "$1" != "reload" ]
then
  showHelp
fi

# musiconholf.conf
echo "
[default]
mode=files
directory=/usr/share/asterisk/moh" > $MOH_CONF_FILE

# zapata.conf
echo "
[channels]
musiconhold=default" > $ZAPATA_CONF_FILE

# sip.conf header
echo "
[general]
bindaddr=0.0.0.0
videosupport=yes
disallow=all
allow=ulaw
allow=alaw
allow=gsm
allow=h264
context=default
canreinvite=no
port=5060

tcpenable=yes
transport=udp,tcp

" > $SIP_CONF_FILE

# get devices id list
echo "Searching for devices ..."
getDevices

for dev in "${devicesArray[@]}"
do
  sipUserName=`sqlite3 /opt/fibaro/db "SELECT Value FROM NEW_Property WHERE Device_Id = $dev AND Name = 'sipUserID'"`
  sipUserPassword=`sqlite3 /opt/fibaro/db "SELECT Value FROM NEW_Property WHERE Device_Id = $dev AND Name = 'sipUserPassword'"`
  sipDisplayName=`sqlite3 /opt/fibaro/db "SELECT Value FROM NEW_Property WHERE Device_Id = $dev AND Name = 'sipDisplayName'"`

  deviceType=`sqlite3 /opt/fibaro/db "SELECT Device_Type FROM NEW_Device WHERE Id = $dev AND Device_Type = 'com.fibaro.fibaroIntercom'"`

  #string values in db have quotes that needs to be removed
  sipUserPassword=`removeQuotes $sipUserPassword`
  sipDisplayName=`removeQuotes $sipDisplayName`
  sipUserName=`removeQuotes $sipUserName`


  # if userName is not empty
  if [ "$sipUserName" != "" ]
  then
    
    echo "VOIP client found:"
    echo "  deviceID = $dev"
    echo "  userID = $sipUserName"
    echo "  userPassword = $sipUserPassword"
    echo "  displayName = $sipDisplayName "
    echo "  deviceType = $deviceType"  

    addSipUser "$sipUserName" "$sipUserPassword" "$sipDisplayName"


    if [ "$sipUserName" != "1223334444" ] && [ "$deviceType" == "" ]
    then

      ALL_USERS_EXTEN+="SIP/$sipUserName&"
      
    fi
    
  fi

  echo -e "\n\n"

 
done

# extensions.conf
ALL_USERS_EXTEN="${ALL_USERS_EXTEN%?},60)"

echo "
[default]
exten => _HELIOS-X!, 1, AGI(intercom.php);

exten => _X!, 1, Dial(SIP/\${EXTEN},,m);

${ALL_USERS_EXTEN};

" > $EXTEN_CONF_FILE

# reload asterisk
echo "Reloading asterisk configuration ..."
asterisk -r -x reload

exit 0
