require('actionHandlers.ActionFactory')
require('actionHandlers.SchedulerFactory')
require('common.pcall_with_log')


logger.setLevel(logger.INFO)
logger.info('****************************** Starting ActionHandler ******************************')

if (type(ActionHandler) ~= "table") then
    ActionHandler = {}
end

local schedulerThirtyTimeoutId = nil
local schedulerOneTimeoutId = nil
local schedulerThirtyTimeout = 30*60*1000 -- 30 min
local schedulerOneTimeout = 60*1000 -- 1 min

 
--! Global function that is used for handling an action called from HCServer. 
--@ param: actionParams - a table that contains action parameters.
function onAction(deviceId, action)
  if (deviceId==1 and action.actionName=='ping') then
    return 'pong'
  end
  
  local status, result =  pcall_with_log(function() return onActionImpl(deviceId, action) end)
  return result
end

function onActionImpl(deviceId, action)  
    logger.info('Received action [deviceId=' .. deviceId .. '][actionName=' .. action.actionName .. ']')

    local actionToExec = ActionFactory.getAction(deviceId, action)
    local isOk = nil
    local returnCode = nil

    if actionToExec then
        isOk, returnCode = pcall(function() actionToExec:execute() end)
        if not isOk then
            logger.error('Unexpected error: ' .. (returnCode or '<empty>'))
        end 
    end

    logger.info('Action [deviceId=' .. deviceId .. '][actionName=' .. action.actionName .. '] completed')
    return  isOk
end

function runScheduler(schedulerName)
    logger.info("**************************************************************************")
    logger.info('Received scheduled task: ' .. schedulerName)

    local status, scheduler = pcall_with_log(function() return SchedulerFactory.getScheduler(schedulerName) end)

    if status and scheduler then
        local ok, returnCode = pcall(function() scheduler:execute() end)
        if not ok then
            logger.error('Unexpected error: ' .. (returnCode or '<empty>'))
        end 
    end
    logger.info('scheduled task: ' .. schedulerName .. ' finished')
    logger.info("**************************************************************************")
end

function scheduleOne()
    runScheduler('SchedulerOne')
    if schedulerOneTimeoutId then
        clearTimeout(schedulerOneTimeoutId)
    end
    schedulerOneTimeoutId = setTimeout(function() scheduleOne() end, schedulerOneTimeout)
end

function scheduleThirty()
    runScheduler('SchedulerThirty')
    if schedulerThirtyTimeoutId then
        clearTimeout(schedulerThirtyTimeoutId)
    end
    schedulerThirtyTimeoutId = setTimeout(function() scheduleThirty() end, schedulerThirtyTimeout)
end

scheduleOne()
scheduleThirty()





