require('actionHandlers.HTTPAction')
require('common.pcall_with_log')


class 'CameraMoveLeftAction' (HTTPAction)


function CameraMoveLeftAction:__init(deviceId, params)
    HTTPAction.__init(self, deviceId, params)

    self.type = 'CameraMoveLeftAction'
    self.targetCamera = self:getTargetCameraDevice()
    self.moveTypeUrl = self:initMoveTypeUrl()
    self.stopUrl = self:initStopUrl()
end

function CameraMoveLeftAction:getTargetCameraDevice()
    local hcDevices = api.get('/devices?type=' .. IP_CAMERA_TYPE)

    for _,dev in pairs(hcDevices) do
        if dev.id == self.deviceId and dev.type == IP_CAMERA_TYPE then 
            return dev
        end
    end

    return nil
end

function CameraMoveLeftAction:initMoveTypeUrl()
    if self.targetCamera then
        local left = self:addslashes(self:trim(self.targetCamera.properties.moveLeftPath))
        if left ~= '' then
            local url = 'http://'
            if self.targetCamera.properties.username ~= '' or self.targetCamera.properties.password ~= '' then 
                url = url .. self.targetCamera.properties.username .. ':' .. self.targetCamera.properties.password .. '@'
            end
            return url .. self.targetCamera.properties.ip .. '/' .. left
        end
    end

    return nil
end

function CameraMoveLeftAction:initStopUrl()
    if self.targetCamera then
        local stop = self:addslashes(self:trim(self.targetCamera.properties.moveStopPath))
        if (self.targetCamera.properties.sendStopAfterMove == 'on' or tonumber(self.targetCamera.properties.sendStopAfterMove) == 1) and stop ~= '' then 
            local url = 'http://'
            if self.targetCamera.properties.username ~= '' or self.targetCamera.properties.password ~= '' then 
                url = url .. self.targetCamera.properties.username .. ':' .. self.targetCamera.properties.password .. '@'
            end

            return url .. self.targetCamera.properties.ip .. '/' .. stop
        end
    end
end

function CameraMoveLeftAction:execute()
    self:executeMoveAction()
end

function CameraMoveLeftAction:executeMoveAction() 
    local authHeaders = nil

    if self.targetCamera.properties.username ~= '' or self.targetCamera.properties.password ~= '' then 
        authHeaders = {
            ['Authorization'] = 'Basic '.. to_base64(self.targetCamera.properties.username .. ':'.. self.targetCamera.properties.password),
        }
    end

    local httpRequestParams = {
        url = self.moveTypeUrl,
        method = 'GET',
        headers = authHeaders,
        callback =  function(response) self:onDataReceivedFromExecuteMoveAction(response) end,
    }

    self:httpRequest(httpRequestParams)
end

function CameraMoveLeftAction:onDataReceivedFromExecuteMoveAction(response)
    print('****response***  onDataReceivedFromExecuteMoveAction:')
    tprint(response)

    if self.stopUrl then
        self:executeStopAction()
    end
end

function CameraMoveLeftAction:executeStopAction() 
    local authHeaders = nil

    if self.targetCamera.properties.username ~= '' or self.targetCamera.properties.password ~= '' then 
        authHeaders = {
            ['Authorization'] = 'Basic '.. to_base64(self.targetCamera.properties.username .. ':'.. self.targetCamera.properties.password),
        }
    end 

    local httpRequestParams = {
        url = self.stopUrl,
        method = 'GET',
        headers = authHeaders,
        callback =  function(response) self:onDataReceivedFromExecuteStopAction(response) end,
    }

    self:httpRequest(httpRequestParams)
end

function CameraMoveLeftAction:onDataReceivedFromExecuteStopAction(response)
    print('****response***  onDataReceivedFromExecuteStopAction:')
    tprint(response)
end





