require('net.HTTPClient')
require('actionHandlers.Action')
require('common.pcall_with_log')


class 'HTTPAction' (Action)


function HTTPAction:__init(deviceId, params)
  Action.__init(self, deviceId, params)
  self.type = 'HTTPAction'
  self.http = net.HTTPClient({ timeout = 20000 }) 
end

function HTTPAction:httpRequest(params) 
  self.http:request(params.url, {
    options={
      headers = params.headers,
      method = params.method,
      data = params.data,
    }, 
    success = function(response)
      pcall_with_log(function()
        if (params.callback) then
          params.callback(response)
        end
      end)
    end,
    error = function (error)
      logger.error('Error HTTPAction [url=' .. (params.url or '<empty>') .. ']: ' .. error)

      pcall_with_log(function()
        if (params.callback) then
          params.callback(error)
        end
      end)
    end
  })
end

function HTTPAction:trim(s)
  return (s:gsub("^%s*(.-)%s*$", "%1"))
end

-- Backslash-escape special characters:
function HTTPAction:addslashes(s)
  -- Double quote, single quote, and backslash (per the PHP
  -- manual):
  s = string.gsub(s, "(['\"\\])", "\\%1")
  -- The null character gets turned into a pair of printing
  -- characters by PHP addslashes.  Let's do the same:
  return (string.gsub(s, "%z", "\\0"))
end -- addslashes

function tprint (tbl, indent)
  if type(tbl) == 'table' then
    if not indent then indent = 0 end
    for k, v in pairs(tbl) do
    formatting = string.rep("  ", indent) .. k .. ": "
    if type(v) == "table" then
      print(formatting)
      tprint(v, indent+1)
    elseif type(v) == 'boolean' then
      print(formatting .. tostring(v))    
    else
      print(formatting, v)
    end
    end
  end
end


function HTTPAction:urlencode(str)
  if (str) then
    str = string.gsub (str, "\n", "\r\n")
    str = string.gsub (str, "([^%w %-%_%.%~])",
        function (c) return string.format ("%%%02X", string.byte(c)) end)
    str = string.gsub (str, " ", "+")
  end

  return str
end
