require('actionHandlers.HTTPAction')
require('common.validator')
require('common.base64')


class 'SendPhotoToEmailAction' (HTTPAction)


TRANS_TABLE = {
        ['br'] = 'Imagem da Câmera',
        ['de'] = 'Kamera Bild',
        ['en'] = 'Camera image',
        ['et'] = 'Kaamera pilt',
        ['fr'] = 'Image de la caméra',
        ['it'] = 'Immagine Videocamera',
        ['lv'] = 'Kameras attēls',
        ['nl'] = 'Camera beeld',
        ['pl'] = 'Obraz z kamery',
        ['pt'] = 'Imagem da Câmara',
        ['ro'] = 'Imagine camera',
        ['sv'] = 'Kamerabild',
    }

function SendPhotoToEmailAction:__init(deviceId, params)
    HTTPAction.__init(self, deviceId, params)

    self.type = 'SendPhotoToEmailAction'
    self.subject = TRANS_TABLE[self.settingsInfo.defaultLanguage] or TRANS_TABLE['en']
    self:initEmail()
    self.hcNumber = self.settingsInfo.serialNumber
    self.date = os.date("%Y-%m-%d %H:%M:%S") -- 2015-08-06 14:11:47
    self.sourceCamera = self:getSourceCamera()
    self.cameraName = self.sourceCamera.name
end

function SendPhotoToEmailAction:initEmail()
    self.email = self:addslashes(self:trim(self.actionParams[1]))
end

function SendPhotoToEmailAction:getSourceCamera()
    local sourceCamera = api.get('/devices/' .. tostring(self.deviceId))
    return sourceCamera
end

function SendPhotoToEmailAction:execute()
    self:getCameraImage()
end

function SendPhotoToEmailAction:getCameraImage()


    if self.sourceCamera or self.sourceCamera.type == IP_CAMERA_TYPE or self.sourceCamera.baseType == IP_CAMERA_TYPE then 
        local imageUrl = ''
        if self.sourceCamera.type == FIBARO_INTERCOM then
            imageUrl = 'http://' .. self.sourceCamera.properties.ip .. self.sourceCamera.properties.jpgPath
        elseif self.sourceCamera.properties.username ~= '' or self.sourceCamera.properties.password ~= '' then
            imageUrl = 'http://' .. self.sourceCamera.properties.username .. ':' .. self.sourceCamera.properties.password .. '@' .. self.sourceCamera.properties.ip .. '/' .. self.sourceCamera.properties.jpgPath
        else 
            imageUrl = 'http://' .. self.sourceCamera.properties.ip .. '/' .. self.sourceCamera.properties.jpgPath
        end

        local authHeaders = {
            ['Authorization'] = 'Basic '.. to_base64(self.sourceCamera.properties.username .. ':'.. self.sourceCamera.properties.password),
        }

        local httpRequestParams = {
                url = imageUrl,
                method = 'GET',
                headers = authHeaders,
                callback =  function(response) self:onDataReceivedFromGetCameraImage(response) end,
            }

        self:httpRequest(httpRequestParams)
    end
end

function SendPhotoToEmailAction:onDataReceivedFromGetCameraImage(response)
    if response and response.data then
        self:sendCameraImageToEmail(response.data)
    end
end

function SendPhotoToEmailAction:sendCameraImageToEmail(image)
    local  postData = self:generatePostData(image)

    local controlHeaders = {
        ['Content-Type'] = 'multipart/form-data; boundary=----------------------------7b35ec58b451',
    }

    local httpRequestParams = {
        url = 'https://storage.fibaro.com/save_photo3.php',    
        callback =  function(response) self:onDataReceivedFromSendCameraImageToEmail(response) end,
        data = postData,
        method = 'POST',
        headers = controlHeaders,
    }

    self:httpRequest(httpRequestParams)
end

function SendPhotoToEmailAction:onDataReceivedFromSendCameraImageToEmail(response)
    print('****response****  onDataReceivedFromSendCameraImageToEmail:')
    tprint(response)
end

function SendPhotoToEmailAction:generatePostData(image)
    local postData = 
[[
------------------------------7b35ec58b451
Content-Disposition: form-data; name="Photo"; filename="/tmp/1438863107.jpg"
Content-Type: application/octet-stream

]] .. image .. '\n' .. [[
------------------------------7b35ec58b451
Content-Disposition: form-data; name="Subject"

]] .. self.subject .. '\n' .. [[
------------------------------7b35ec58b451
Content-Disposition: form-data; name="Email"

]] .. self.email .. '\n' .. [[
------------------------------7b35ec58b451
Content-Disposition: form-data; name="PK_AccessPoint"

]] .. self.hcNumber .. '\n' .. [[
------------------------------7b35ec58b451
Content-Disposition: form-data; name="HW_Key"

]] .. self.hcKey .. '\n' .. [[
------------------------------7b35ec58b451
Content-Disposition: form-data; name="Date"

]] .. self.date .. '\n' .. [[
------------------------------7b35ec58b451
Content-Disposition: form-data; name="CameraName"

]] .. self.cameraName .. '\n' .. [[
------------------------------7b35ec58b451--
]]

    return postData
end






