require('actionHandlers.HTTPAction')


class 'SendPushAction' (HTTPAction)


function SendPushAction:__init(deviceId, params)
    HTTPAction.__init(self, deviceId, params)

    self.type = 'SendPushAction'
    self:initMessage()
    self:initTokens()
end

function SendPushAction:initMessage(message)
    if not message then
        self.message = self:addslashes(self:trim(self.actionParams[1]))
    else
        self.message = message
    end
end

function SendPushAction:initTokens()
    local hcDevices = api.get('/devices?type=' .. IOS_DEVICE_TYPE)
    local targetMobileDevice = nil
    self.iosTokens = ''
    self.androidTokens = ''

    for _,dev in pairs(hcDevices) do
        if dev.type == IOS_DEVICE_TYPE and dev.properties.Push == true then 
            if dev.id == self.deviceId then 
                targetMobileDevice = dev
            end
        end
    end

    if targetMobileDevice then 
        local deviceToken = targetMobileDevice.properties.DeviceToken

        if string.len(deviceToken) == 64 then -- this is iOS Device <ugly>
            self.iosTokens = deviceToken
        else -- this is Android Device
            self.androidTokens = deviceToken
        end
    end
end

function SendPushAction:execute()
    local hcNumber = self.settingsInfo.serialNumber
    local controlHeaders = {
            ['Content-Type'] = 'application/x-www-form-urlencoded',
    }
    local httpRequestParams = {}

    local from = self.settingsInfo.hcName

    if self.iosTokens ~= '' then 
        local postData = 'PK_AccessPoint=' .. hcNumber .. '&HW_Key=' .. self.hcKey .. '&Message=' .. self:urlencode(self.message) .. '&Tokens=' .. self.iosTokens .. "&from=" .. from

         httpRequestParams = {
            url = 'https://testpush.fibaro.com/testpush.php',
            method = 'POST',
            headers = controlHeaders,
            callback =  function(response) self:onDataReceivedFromExecute(response) end,
            data = postData,
        }

        self:httpRequest(httpRequestParams)
    end

    if self.androidTokens ~= '' then
        local jsonMessage = '{"serialNumber": "' .. hcNumber ..  '", "type": "standard", "title": "Push", "body": "' .. self.message .. '", "from":"' .. from .. '"}'
        local postData = 'PK_AccessPoint=' .. hcNumber .. '&HW_Key=' .. self.hcKey .. '&Message=' .. self:urlencode(jsonMessage) .. '&Tokens=' .. self.androidTokens  .. '&from=' .. from
                
        httpRequestParams = {
            url = 'https://androidpush.fibaro.com/sendPush3.php',
            method = 'POST',
            headers = controlHeaders,
            callback =  function(response) self:onDataReceivedFromExecute(response) end,
            data = postData,
        }

        self:httpRequest(httpRequestParams)
    end
end


function SendPushAction:onDataReceivedFromExecute(response)
    print('****response****  onDataReceivedFromExecute:')
    tprint(response)
end
