require('common.device')

--! Class responsible for three events types (mouseDown with repeat functionality, mouseUp, cancel)
class "FIButtonUpDown"

--! Initialize FIButtonUpDownClass.
--!@param params: Array with two int values: repeat after and refresh rate.
function FIButtonUpDown:__init(params)
    print(type(params.normalBinding))
	self.repeatAfter = params.repeatAfter
	self.refreshRate = params.refreshRate
    self.uiBinding = params.normalBinding
    self.repeatBinding = params.repeatBinding
    self.repeatTimeoutTriggerId = nil
    self.refreshRateId = nil
end

--! Fires up on each (up, down, cancel) UI control events.
--@param deviceId: Id of the device assigned to plugin.
--@param event: Array containing eventtype (mouseup, mousedown, cancel) and UI control name.
function FIButtonUpDown:onUIEvent(deviceId, event)
	print('FIButtonUpDown:onUIEvent')

    if (event.eventType == 'onReleased') then
    	self:mouseUp(event)
    elseif (event.eventType == 'onPressed') then
        self:mouseDown(event)
    elseif (event.eventType == 'cancel') then
    	self:cancelEvents()
    end
end 

--! Fires up on mousedown event. Starts timer counting to self.repeatAfter and then (if not interrupted on cancel method) fires repeatEvent.
--@param event: Array containing eventtype (mouseup, mousedown, cancel) and UI control name.
function FIButtonUpDown:mouseDown(event)
	print('FIButtonUpDown:mouseDown')

    self.repeatTimeoutTriggerId = setTimeout(function( ) self:repeatEvent(event); end, self.repeatAfter)
end

--! Fires up on mouseup event. Cancels active run loops and fires bind method. 
--@param event: Array containing eventtype (mouseup, mousedown, cancel) and UI control name.
function FIButtonUpDown:mouseUp(event)
	print('FIButtonUpDown:mouseUp')

	self:cancelEvents()
	
    print('onUIEvent: ' .. event.elementName .. ' EventType: ' .. event.eventType)

    local callback = self.uiBinding[event.elementName]

    if (callback) then
        callback(event)
    end
end

--! Fires up on cancel event. Cancels active run loops
--@param event: Array contained eventtype (mouseup, mousedown, cancel) and UI control name.
function FIButtonUpDown:cancelEvents(event)
	print('FIButtonUpDown:cancelEvents')

	if (self.repeatTimeoutTriggerId) then
		clearTimeout(self.repeatTimeoutTriggerId)
	end

	if (self.refreshRateId) then
		clearTimeout(self.refreshRateId)
	end
end

--! Repeating event if mouse is pressed more than self.repeatAfter param.
--@param event: Array contained eventtype (mouseup, mousedown, cancel) and UI control name.
function FIButtonUpDown:repeatEvent(event)
	print('onUIEvent: ' .. event.elementName .. ' EventType: ' .. event.eventType)
    local callback = self.repeatBinding[event.elementName]

    if (callback) then
        callback(event)
    end

    self.refreshRateId = setTimeout(function( ) self:repeatEvent(event); end, self.refreshRate)
end



