class 'Translator' 


function Translator:__init(pluginViewName)
	local filesPath = '/var/www/plugins/' .. (pluginViewName or '')  .. '/translation/'
    self.translationsTable  = self:getTranslationFilesContentTable(filesPath)    
end

function Translator:getTranslationFilesContentTable(directory)
    local translationsTable = {}
    local pfile = io.popen('ls -a "' .. directory .. '"')
    
    for filename in pfile:lines() do        
        translationsTable[filename] = self:getTranslatioFileContent(directory, filename)
    end

    pfile:close()

    return translationsTable
end

function Translator:getTranslatioFileContent(directory, filename)
    local fileHandler = io.open(directory .. filename, "r")

    if fileHandler then
        local translationContent = fileHandler:read("*all")
        fileHandler:close()
        return translationContent
    else
        return nil
    end
end

function Translator:getCurrentSystemLanguage()
	local settingsInfo = api.get('/settings/info')
	return settingsInfo.defaultLanguage
end

function Translator:getTranslatedStringForKey(key)
	local translationFileContent = self:getCurrentTranslatioFileContentFromTable()
	
	if translationFileContent then
		return string.match(translationFileContent, key .. '="(.-)"')
	else
		return key
	end
end

function Translator:getCurrentTranslatioFileContentFromTable()
    return self.translationsTable['translation_' .. self:getCurrentSystemLanguage() .. '.txt'] or self.translationsTable['translation_en.txt'] 
end


-------- How to use:
-- self.translator = Translator('fibaroIntercom')
-- print('translation: ', self.translator:getTranslatedStringForKey('i18n_intercomStatusNotConfigured'))





