require("common.plugin")

class 'Device'

function Device:__init(arg)
    local dev = nil

    if (type(arg) == 'number') then
        dev = api.get("/devices/" .. arg)
    elseif(type(arg) == 'table') then
        dev = arg
    else
        error('argument #1: expected number or table')
    end

    self.id = dev.id
    self.name = dev.name
    self.properties = dev.properties
    self.type = dev.type
    
    self.onPropertyChanged = {}
    self.childDevices = {}
end

--! Update property
--@ param: name - property name
--@ param: value - property value
--@ param: forceUpdate - force updtade peroperty when needed
function Device:updateProperty(name, value, forceUpdate)
    local propertyType = type(self.properties[name])
    forceUpdate = forceUpdate or (propertyType == 'table')

    if (self.properties[name] ~= value or  forceUpdate == true) then
        self.properties[name] = value

        local data = {}
        data["deviceId"] = self.id
        data["propertyName"] = name
        data["value"] = value

        api.post("/plugins/updateProperty", data)

        if (type(self.onPropertyChanged[name]) == 'function') then
            self.onPropertyChanged[name](self, value)
        end

        return true
    else
        return false
    end
end

function Device:addInterfaces(values)
    assert(type(values) == "table")
    self:updatetInterfaces("add",values)
end

function Device:deleteInterfaces(values)
    assert(type(values) == "table")
    self:updatetInterfaces("delete",values)
end

function Device:updatetInterfaces(action, values)
    local data = {}
    data["action"] = action
    data["deviceId"] = self.id
    data["interfaces"] = values

    api.post("/plugins/interfaces", data)
end

function Device:setName(name)
    api.put("/devices/" .. self.id,  {["name"]=name})
end

function Device:setEnabled(enabled)
    api.put("/devices/" .. self.id, {["enabled"]=enabled} )
end

function Device:setVisible(visible)
    api.put("/devices/" .. self.id, {["visible"]=visible} )
end

function Device:callAction(name, ...)
    if (type(self[name]) == 'function') then 
        return self[name](self, ...)
    else
        print('[WARNING] Class does not have ' .. name .. ' function defined - action ignored')
    end
end
