if (type(plugin) ~= "table") then
    plugin = {}
end

--! Gets device data besed on id
--! @param deviceId 
--! @return table with device attributes
function plugin.getDevice(deviceId)
    return api.get("/devices/" .. deviceId)
end

--! Delete device besed on id
--! @param deviceId 
--! @return operation status
function plugin.deleteDevice(deviceId)
    return api.delete("/devices/" .. deviceId)
end

--! Returns single property of device
--! @param deviceId
--! @param propertyName
--! @return value of the property
function plugin.getProperty(deviceId, propertyName)
    local dev = api.get("/devices/" .. deviceId)
    return dev.properties[propertyName]    
end

--! Returns table with all children
--! @param deviceId
--! @return table containing data of all children
function plugin.getChildDevices(deviceId)
    return api.get("/devices?parentId=" .. deviceId)

end

--! Creates new child device, and returns it's data
--! @param parentId
--! @param type child device type (com.fibaro.xyz)
--! @name name for child device
--! @return array with created device data
function plugin.createChildDevice(parentId, type, name)
    local message = {["parentId"]=parentId, ["type"]=type, ["name"]=name}
    return api.post("/plugins/createChildDevice", message)
end

--! Creates new child device, and returns it's data
--! @param parentId
--! @param type child device type (com.fibaro.xyz)
--! @name name for child device
--! @return array with created device data
--! @initialProperties 
function plugin.createChildDevice(parentId, type, name, initialProperties)
    local message = {["parentId"]=parentId, ["type"]=type, ["name"]=name, ["initialProperties"]=initialProperties}
    return api.post("/plugins/createChildDevice", message)
end

--! Creates new child device, and returns it's data
--! @param parentId
--! @param type child device type (com.fibaro.xyz)
--! @name name for child device
--! @return array with created device data
--! @initialProperties 
--! @initialInterfaces
function plugin.createChildDevice(parentId, type, name, initialProperties, initialInterfaces)
    local message = {["parentId"]=parentId, ["type"]=type, ["name"]=name, ["initialProperties"]=initialProperties, ["initialInterfaces"]=initialInterfaces}
    return api.post("/plugins/createChildDevice", message)
end

--! Restarts LuaEnvironment process that runs plugin
function plugin.restart()
    setTimeout(
        function()
            local message = {["deviceId"]=plugin.mainDeviceId}
            api.post("/plugins/restart", message) 
        end, 
        1
    )
end