require('common.device')
require('common.upnp')

class 'UPnPDevice' (Device)

--
-- Base class for controlling upnp devices
--
-- id: deviceId
-- config: configuration for UPnP connection
-- {
--	   ip = <ip>,
--     port = <port>,
--     RenderingControl = <urlPath>,
--     AVTransport = <urlPath>,
--     timeout = <timeout>
-- }
--
function UPnPDevice:__init(id)
	Device.__init(self, id)
end

function UPnPDevice:initializeUPnP(config)
	self.upnp = UPnP(config)
end

function UPnPDevice:play()
	self.upnp:play()
end

function UPnPDevice:pause()
	self.upnp:pause()
end

function UPnPDevice:stop()
	self.upnp:stop()
end

function UPnPDevice:prev()
	self.upnp:prev()
end

function UPnPDevice:next()
	self.upnp:next()
end

function UPnPDevice:setVolume(vol)
	self.upnp:setVolume(vol)
end

function UPnPDevice:setMute(m)
	self.upnp:setMute(m)
end

function UPnPDevice:setAVTransportUri(uri)
	self.upnp:setAVTransportUri(uri)
end

function UPnPDevice:setMainTVSource(source, id)
	self.upnp:setMainTVSource(source, id)
end

function UPnPDevice:monitorVolume(pollingTime)
	self.upnp:getVolume(function(vol)
		self:updateProperty('volume', vol)
		setTimeout(function() self:monitorVolume(pollingTime) end, pollingTime)
	end, function() setTimeout(function() self:monitorVolume(pollingTime) end, pollingTime) end)
end

function UPnPDevice:monitorMute(pollingTime)
	self.upnp:getMute(function(m)
		self:updateProperty('mute', m)
		setTimeout(function() self:monitorMute(pollingTime) end, pollingTime)
	end, function() setTimeout(function() self:monitorMute(pollingTime) end, pollingTime) end)
end

function UPnPDevice:monitorCurrentState(pollingTime)
	self.upnp:getTransportInfo(function(s)
		self:updateProperty('state', s)
		setTimeout(function() self:monitorCurrentState(pollingTime) end, pollingTime)
	end, function() setTimeout(function() self:monitorCurrentState(pollingTime) end, pollingTime) end)

end

function UPnPDevice:monitorTrackInfo(pollingTime)
	self.upnp:getPositionInfo(
		function(data)
			if (data.totalTime == '0:00:00') then
				self:updateProperty('position', '')
			else
				self:updateProperty('position', data.relTime .. '/' .. data.totalTime)
			end

			if data.creator ~= nil and data.creator ~= "" then
				self:updateProperty('creator', data.creator)
			end

			if data.title ~= nil and data.title ~= "" then
				self:updateProperty('title', data.title)
			end

			if data.albumArtURI ~= nil and data.albumArtURI ~= "" then
				self:updateProperty('albumArtURI', 'http://' .. self.properties.ip .. ':1400' .. albumArtURI)
			end

			setTimeout(function() self:monitorTrackInfo(pollingTime) end, pollingTime)
		end,

		function()
			setTimeout(function() self:monitorTrackInfo(pollingTime) end, pollingTime)
		end
	)
end


