local function createJsonRpcResponse(id, result)
	return json.encode({
		['jsonrpc'] = '2.0',
		['id'] = id,
		['result'] = result or json.null()
	})
end

local methods = {
	setGlobalVariable = function(params)
		_G[params.name] = params.value
	end,

	getGlobalVariable = function(params)
		return _G[params.name]
	end,

	callGlobalFunction = function(params)
		local status
		local result 
		status, result = pcall( function() return _G[params.name](unpack(params.params)) end)

        if not status then print("ERROR: ", result) error(result) end
        
        return result
	end,

	executeString = function(params)
		loadstring(params.cmd)()
	end
}

function handleJsonRpc(data)
	local obj = json.decode(data)
	local handler = methods[obj.method]

	if (handler) then
		local result = handler(obj.params)
		return createJsonRpcResponse(obj.id, result)
	else
		return json.encode({
			jsonrpc = '2.0',
			error = {
				code = -32601,
				message = 'Method not found'
			},
			id = obj.id
		})
	end
end
