const os = require( 'os' );
const fs = require( 'fs' );

const testsPath = process.env.TEST_TYPE === 'integration' ?
	'./test/e2e/integration/' :
	'./test/e2e/local/';

global.port = parseInt( 8090 + (Math.random() * 100) );
global.url = 'localhost';

let webdriver = 'localhost';

if( fs.existsSync( 'integration.json' ) ) {
	let integrationConfig = JSON.parse( fs.readFileSync( 'integration.json' ) );
	url = integrationConfig.proxyHost;
	webdriver = integrationConfig.webDriverHost;
}

if( process.env.BUILD_AGENT === 'bamboo' ) {
	url = os
		.networkInterfaces()
		.eth0
		.find( address => address.family === 'IPv4' )
		.address;
	webdriver = 'selenium-server.fibaro.com';
}

console.log( 'HC proxy URL', url );
console.log( 'HC proxy port', port );
console.log( 'WebDriver URL', webdriver );

exports.config = {
	'tests': testsPath + '**/*Feature.js',
	'timeout': 10000,
	'output': './build/reports/tests',
	'helpers': {
		'WebDriverIO': {
			'url': 'http://' + url + ':' + port,
			'host': webdriver,
			'browser': 'chrome',
			'restart': true,
			'windowSize': 'maximize',
			'timeouts': {
				'script': 60000,
				'page load': 10000,
				'implicit': 0
			}
		},
		'HeatingPanelHelper': {
			'require': './test/e2e/includes/heatingPanelHelper.js'
		},
		'LoginHelper': {
			'require': './test/e2e/includes/loginHelper.js'
		},
		'VariableHelper': {
			'require': './test/e2e/includes/variableHelper.js'
		},
		'SprinklesPanelHelper': {
			'require': './test/e2e/includes/sprinklesPanelHelper.js'
		},
		'CoolingPanelHelper': {
			'require': './test/e2e/includes/coolingPanelHelper.js'
		},
		'HumidityPanelHelper': {
			'require': './test/e2e/includes/humidityPanelHelper.js'
		},
		'RoomHelper': {
			'require': './test/e2e/includes/roomHelper.js'
		},
		'ReloadHelper': {
			'require': './test/e2e/includes/reload.js'
		},
		'VisibilityHelper': {
			'require': './test/e2e/includes/visibility.js'
		},
		'DiagnosticApiHelper': {
			'require': './test/e2e/includes/diagnostics.js'
		},
		'DevicesApiHelper': {
			'require': './test/e2e/includes/devices.js'
		},
		'SettingsApiHelper': {
			'require': './test/e2e/includes/settings.js'
		},
		'UserHelper': {
			'require': './test/e2e/includes/userHelper.js'
		},
		'Waitress': {
			'require': './test/e2e/includes/waitress.js',
			'debug': true
		},
		'LocalisationHelper': {
			'require': './test/e2e/includes/localisation.js'
		},
		'MockDevice': {
			'require': './test/e2e/includes/mockDevice.js'
		},
		'Configuration': {
			'require': './test/e2e/includes/configuration.js'
		},
		'VoIPHelper': {
			'require': './test/e2e/includes/voip.js'
		},
		'NotificationHelper': {
			'require': './test/e2e/includes/notification.js'
		},
		'SceneHelper': {
			'require': './test/e2e/includes/scene.js'
		}
	},
	'include': {
		'I': './test/e2e/includes/actor.js',
		'Environment': './test/e2e/includes/environment.js',
		'Request': './test/e2e/includes/request.js',
		'Loader': './test/e2e/pages/main/Loader.js',
		'ConfigurationGeneral': './test/e2e/pages/configuration/General.js',
		'ConfigurationMain': './test/e2e/pages/configuration/Main.js',
		'ConfigurationAccessControl': './test/e2e/pages/configuration/AccessControl.js',
		'ConfigurationGatewayConnections': './test/e2e/pages/configuration/Gateway.js',
		'ConfigurationDiagnostic': './test/e2e/pages/configuration/Diagnostic.js',
		'ConfigurationZWave': './test/e2e/pages/configuration/ZWave.js',
		'ConfigurationNotificationCenter': './test/e2e/pages/configuration/NotificationCenter.js',
		'ConfigurationLocation': './test/e2e/pages/configuration/Location.js',
		'ConfigurationLAN': './test/e2e/pages/configuration/LAN.js',
		'ConfigurationBackupRecovery': './test/e2e/pages/configuration/BackupRecovery.js',
		'DashboardMenu': './test/e2e/pages/main/DashboardMenu.js',
		'Login': './test/e2e/pages/main/Login.js',
		'TopMenu': './test/e2e/pages/main/TopMenu.js',
		'YourHouse': './test/e2e/pages/yourHouse/YourHouse.js',
		'Rooms': './test/e2e/pages/rooms/Rooms.js',
		'Devices': './test/e2e/pages/devices/Devices.js',
		'DevicesConfiguration': './test/e2e/pages/devices/Configuration.js',
		'KeyFobConfiguration': './test/e2e/pages/devices/KeyFob.js',
		'FGT001Configuration': './test/e2e/pages/devices/FGT001.js',
		'IconsConfiguration': './test/e2e/pages/devices/Icons.js',
		'Scenes': './test/e2e/pages/scenes/Scenes.js',
		'SceneRooms': './test/e2e/pages/scenes/blocks/SceneRooms.js',
		'SceneCentralSceneEvent': './test/e2e/pages/scenes/blocks/SceneCentralSceneEvent.js',
		'SceneDoorLock': './test/e2e/pages/scenes/blocks/SceneDoorLock.js',
		'ScenesMagic': './test/e2e/pages/scenes/Magic.js',
		'Consumption': './test/e2e/pages/consumption/Consumption.js',
		'PluginsMain': './test/e2e/pages/plugins/Main.js',
		'PluginsInstalled': './test/e2e/pages/plugins/Installed.js',
		'PluginsAvailable': './test/e2e/pages/plugins/Available.js',
		'PanelsMain': './test/e2e/pages/panels/Main.js',
		'PanelsDashboard': './test/e2e/pages/panels/Dashboard.js',
		'PanelsSms': './test/e2e/pages/panels/Sms.js',
		'PanelsAlarm': './test/e2e/pages/panels/Alarm.js',
		'PanelsHeating': './test/e2e/pages/panels/Heating.js',
		'PanelsCooling': './test/e2e/pages/panels/Cooling.js',
		'PanelsHumidity': './test/e2e/pages/panels/Humidity.js',
		'PanelsSprinklers': './test/e2e/pages/panels/Sprinklers.js',
		'PanelsVariables': './test/e2e/pages/panels/Variables.js',
		'PanelsEvent': './test/e2e/pages/panels/Event.js',
		'PanelsNotifications': './test/e2e/pages/panels/Notifications.js',
		'PanelsLocalisation': './test/e2e/pages/panels/Localisation.js',
		'PanelsLinkedDevicesDashboard': './test/e2e/pages/panels/linkedDevices/Dashboard.js',
		'PanelsLinkedDevicesMain': './test/e2e/pages/panels/linkedDevices/Main.js',
		'PanelsLinkedDevicesAddNewMain': './test/e2e/pages/panels/linkedDevices/AddMain.js',
		'PanelsLinkedDevicesHeating': './test/e2e/pages/panels/linkedDevices/Heating.js',
		'PanelsLinkedDevicesCooling': './test/e2e/pages/panels/linkedDevices/Cooling.js',
		'PanelsLinkedDevicesHumidity': './test/e2e/pages/panels/linkedDevices/Humidity.js',
		'PanelsLinkedDevicesVideoGate': './test/e2e/pages/panels/linkedDevices/VideoGate.js',
		'PanelsVoipDashboard': './test/e2e/pages/panels/voip/Dashboard.js',
		'PanelsVoipAddClient': './test/e2e/pages/panels/voip/AddClient.js',
		'RightMenu': './test/e2e/pages/main/RightMenu.js',
		'WizardWelcome': './test/e2e/pages/wizard/Welcome.js',
		'WizardSummary': './test/e2e/pages/wizard/Summary.js',
		'WizardAddUser': './test/e2e/pages/wizard/AddUser.js',
		'WizardUserConfirm': './test/e2e/pages/wizard/UserConfirm.js',
		'WizardHcName': './test/e2e/pages/wizard/HcName.js',
		'WizardAlarm': './test/e2e/pages/wizard/Alarm.js',
		'WizardPin': './test/e2e/pages/wizard/Pin.js',
		'WizardFinish': './test/e2e/pages/wizard/Finish.js',
		'VirtualDevices': './test/e2e/pages/devices/VirtualDevices.js'
	},
	'bootstrap': testsPath + 'bootstrap.js',
	'teardown': testsPath + 'teardown.js',
	'mocha': {
		'reporterOptions': {
			'mochaFile': './build/reports/tests/INTEGRATION-TESTS.xml'
		}
	},
	'name': 'web'
};
